CONST_RAM equ h'60'

 __config b'00000111001100'          ; Programmer-Configuration
;           ++---------------- #13-#12 BG1:BG0: Bandgap Calibration bits
;             ||||||||||||        >00 = Lowest bandgap voltage
;             ||||||||||||         11 = Highest bandgap voltage
;             +++------------- #11-#9 Unimplemented: Read as 0
;                +------------ #8: CPD: Data Code Protection bit (2)
;                 ||||||||        >1 = Data memory code protection is disabled
;                 ||||||||         0 = Data memory code protection is enabled
;                 +----------- #7: CP: Code Protection bit
;                  |||||||        >1 = Program Memory code protection is disabled
;                  |||||||         0 = Program Memory code protection is enabled
;                  +---------- #6: BODEN: Brown-out Detect Enable bit
;                   ||||||        >1 = BOD enabled
;                   ||||||         0 = BOD disabled
;                   +--------- #5: MCLRE: GP3/MCLR pin function select
;                    |||||         1 = GP3/MCLR pin function is MCLR
;                    |||||        >0 = GP3/MCLR pin function is digital I/O
;                    +-------- #4: PWRTE: Power-up Timer Enable bitd
;                     ||||         1 = PWRT disabled
;                     ||||        >0 = PWRT enabled
;                     +------- #3: WDTE: Watchdog Timer Enable bit
;                      |||        >1 = WDT enabled
;                      |||         0 = WDT disabled
;                      +++---- #2-#0: FOSC2:FOSC0: Oscillator Selection bits
;                                 >100 = INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin,
;                                        I/O function on GP5/OSC1/CLKIN

; ********************************************************
; **  Konfiguration COMPARATOR                          **
; ********************************************************
        movlw   B'00000111'
;                 +--------- #7: Unimplemented: Read as ?0?
;                  +-------- #6: COUT: Comparator Output bit
;                   |||| |       When CINV=0  1 = VIN+ > VIN-
;                   |||| |                    0 = VIN+ < VIN-
;                   |||| |       When CINV=1  0 = VIN+ > VIN-
;                   |||| |                    1 = VIN+ < VIN-
;                   +------- #5: Unimplemented: Read as ?0?
;                    +------ #4: CINV: Comparator Output Inversion bit
;                     || |       1 = Output inverted
;                     || |       0 = Output not inverted
;                     +----- #3: CIS: Comparator Input Switch bit
;                      | |       When CM2:CM0 = 110 or 101:
;                      | |       1 = VIN- connects to CIN+
;                      | |       0 = VIN- connects to CIN-
;                      +++-- #2-#0: CM2:CM0: Comparator Mode bits
;                               >111 = Comparator Off

        movwf CMCON                     ; CMCON-Reg. konfigurieren

; ****************************************************************
; **  Konfiguration PCON: POWER CONTROL REGISTER (ADDRESS: 8Eh) **
; ****************************************************************
;        movlw   B'00010000'
;                 ++-------- #7-#6: Unimplemented: Read as ?0?
;                   +------- #5: ULPWUE: Ultra Low Power Wake-up Enable bit
;                    |||||       1 = Ultra Low Power Wake-up enabled
;                    |||||      >0 = Ultra Low Power Wake-up disabled
;                    +------ #4: SBODEN: Software BOD Enable bit(1)
;                     ||||      >1 = BOD enabled 
;                     ||||           (BODEN<1:0> = 01 in Configuration Word for this bit to control the BOD.)
;                     ||||       0 = BOD disabled
;                     ++---- #3-#2: Unimplemented: Read as ?0?
;                       +--- #1: POR: Power-on Reset STATUS bit
;                        |       1 = No Power-on Reset occurred
;                        |      >0 = A Power-on Reset occurred (must be set in software after a Power-on Reset occurs)
;                        +-- #0: BOD: Brown-out Detect STATUS bit
;                                1 = No Brown-out Detect occurred
;                               >0 = A Brown-out Detect occurred 
;                                    (must be set in software after a Brown-out Detect occurs)

;        bsf STATUS,RP0                 ; -> BANK 1
;        movwf PCON                     ; PCON-Reg. konfigurieren
;        bank_0                         ; -> BANK 0

; ********************************************************
; **  Konfiguration AD-Ports                            **
; ********************************************************
        movlw   B'00000000'
;                 ++++++++-- #7-#0: ANS3:ANS0: Analog Select bits
;                                (Between analog or digital function on pins AN<7:0>, respectively.)
;                                0 = Digital I/O; pin is assigned to port or special function
;                                1 = Analog input; pin is assigned as analog input (1)
;                                    0 = ANS7: Digital 
;                                    0 = ANS6: Digital
;                                    0 = ANS5: Digital
;                                    0 = ANS4: Digital
;                                    0 = ANS3: Digital
;                                    0 = ANS2: Digital
;                                    0 = ANS1: Digital
;                                    0 = ANS0: Digital

        bsf STATUS,RP0                 ; -> BANK 1
        movwf ANSEL                    ; ANSEL-Register
        nop
        bank_0                         ; -> BANK 0

; ********************************************************
; **  Konfiguration TIMER / WDT / Pull-Ups              **
; ********************************************************
        movlw   B'11001000'
;                 +--------- #7: GPPU: PORTA Pull-up Enable bit
;                  ||||| |      >1 = PORTA pull-ups are disabled
;                  ||||| |       0 = PORTA pull-ups are enabled by individual port latch values
;                  +-------- #6: INTEDG: Interrupt Edge Select bit
;                   |||| |      >1 = Interrupt on rising edge of GP2/INT pin
;                   |||| |       0 = Interrupt on falling edge of GP2/INT pin
;                   +------- #5: T0CS: TMR0 Clock Source Select bit
;                    ||| |       1 = Transition on GP2/T0CKI pin
;                    ||| |      >0 = Internal instruction cycle clock (CLKOUT)
;                    +------ #4: T0SE: TMR0 Source Edge Select bit
;                     || |       1 = Increment on high-to-low transition on GP2/T0CKI pin
;                     || |      >0 = Increment on low-to-high transition on GP2/T0CKI pin
;                     +----- #3: PSA: Prescaler Assignment bit
;                      | |      >1 = Prescaler is assigned to the WDT
;                      | |       0 = Prescaler is assigned to the TIMER0 module
;                      +++-- #2-#0: PS2:PS0: Prescaler Rate Select bits
;                                 Bit |TMR0 Rate|WDT Rate
;                                 ----+---------+--------
;                               > 000 | 1 : 2   | 1 : 1
;                                 001 | 1 : 4   | 1 : 2
;                                 010 | 1 : 8   | 1 : 4
;                                 011 | 1 : 16  | 1 : 8
;                                 100 | 1 : 32  | 1 : 16
;                                 101 | 1 : 64  | 1 : 32
;                                 110 | 1 : 128 | 1 : 64
;                                 111 | 1 : 256 | 1 : 128

        bsf STATUS,RP0                 ; -> BANK 1
        movwf OPTION_REG               ; OPTION-Reg. konfigurieren
        bank_0                         ; -> BANK 0

#ifdef timer1_is_ir ;TMR1 IR verwendet fr PWM, Prescaler 1:1
        movlw   B'00000001'
#endif

#ifndef timer1_is_ir ;ohne TMR1 IR
        movlw   B'00110001'
#endif
;                 +--------- #7: Unimplemented: Read as ?0?
;                  +-------- #6: TMR1GE: Timer1 Gate Enable bit
;                   |||| |      >If TMR1ON = 0: This bit is ignored
;                   |||| |       If TMR1ON = 1:
;                   |||| |       1 = Timer1 is on if T1G pin is low
;                   |||| |       0 = Timer1 is on
;                   ++------ #5-#4: T1CKPS1:T1CKPS0: Timer1 Input Clock Prescale Select bits
;                    ||| |      >11 = 1:8 Prescale Value
;                    ||| |       10 = 1:4 Prescale Value
;                    ||| |       01 = 1:2 Prescale Valueue
;                    ||| |       00 = 1:1 Prescale Value
;                     +----- #3: T1OSCEN: LP Oscillator Enable Control bit
;                      | |       If INTOSC without CLKOUT oscillator is active:
;                      | |       1 = LP oscillator is enabled for Timer1 clock
;                      | |      >0 = LP oscillator is off
;                      | |       Else: This bit is ignored
;                      +---- #2: T1SYNC: Timer1 External Clock Input Synchronization Control bit
;                       ||      >if TMR1CS = 0: This bit is ignored. Timer1 uses the internal clock.
;                       +--- #1: TMR1CS: Timer1 Clock Source Select bit
;                        |       1 = External clock from T1OSO/T1CKI pin (on the rising edge)
;                        |      >0 = Internal clock (FOSC/4)
;                        +-- #0: TMR1ON: Timer1 On bit
;                               >1 = Enables Timer1
;                                0 = Stops Timer1

        movwf T1CON                    ; T1CON-Reg. konfigurieren

; ********************************************************
; **  Konfiguration INTERRUPT                           **
; ********************************************************
#ifdef timer1_is_ir ;TMR1 IR verwendet fr PWM
        movlw   B'00000001'
#endif

#ifndef timer1_is_ir ;ohne TMR1 IR
        movlw   B'00000000'
#endif
;                 +--------- #7: EEIE: EE Write Complete Interrupt Enable Bit
;                  |||||||       1 = Enables the EE write complete interrupt
;                  |||||||      >0 = Disables the EE write complete interrupt
;                  +-------- #6: ADIE: A/D Converter Interrupt Enable bit (PIC12F675 only)
;                                1 = Enables the A/D converter interrupt
;                               >0 = Disables the A/D converter interrupt
;                   +------- #5: Unimplemented: Read as ?0?
;                    +------ #4: Unimplemented: Read as ?0?
;                     +----- #3: CMIE: Comparator Interrupt Enable bit
;                                1 = Enables the comparator interrupt
;                               >0 = Disables the comparator interrupt
;                      +---- #2: Unimplemented: Read as ?0?
;                       +--- #1: Unimplemented: Read as ?0?
;                        +-- #0: TMR1IE: TMR1 Overflow Interrupt Enable bit
;                             (>)1 = Enables the TMR1 overflow interrupt
;                               >0 = Disables the TMR1 overflow interrupt

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PIE1                     ; PIE1-Reg. konfigurieren
        bank_0                         ; -> BANK 0

        movlw   B'00010000'
;                 +--------- #7: GIE: Global Interrupt Enable bit
;                  |||||||       1 = Enables all un-masked interrupts
;                  |||||||      >0 = Disables all interrupts, enable later
;                  +-------- #6: PEIE: Peripherial Interrupt Enable bit
;                   ||||||       1 = Enables all unmasked peripherial interrupts
;                   ||||||      >0 = Disables all peripherial interrupts
;                   +------- #5: T0IE: TMR0 Overflow Interrupt Enable bit
;                    |||||       1 = Enables the TMR0 interrupt
;                    |||||      >0 = Disables the TMR0 interrupt
;                    +------ #4: INTE: GP2/INT External Interrupt Enable bit
;                     ||||      >1 = Enables the GP2/INT external interrupt
;                     ||||       0 = Disables the GP2/INT external interrupt
;                     +----- #3: RAIE: PortA Change Interrupt Enable bit
;                      |||       1 = Enables the portA change interrupt
;                      |||      >0 = Disables the portA change interrupt
;                      +---- #2: T0IF: TMR0 overflow interrupt flag bit
;                       ||       1 = TMR0 has overflowed (must be cleared in software)
;                       ||      >0 = TMR0 did not overflow
;                       +--- #1: INTF: GP2/INT External Interrupt Flag bit
;                        |       1 = The GP2/INT external interrupt occurred (must be cleared in software)
;                        |      >0 = The GP2/INT external interrupt did not occur
;                        +-- #0: RAIF: PortA Change Interrupt Flag bit
;                                1 = When at least one of the PORTA pins changed state
;                                    (must be cleared in software)
;                               >0 = None of the PORTA pins have changed state

        movwf INTCON                   ; INTCON-Reg. konfigurieren
