; ********************************************************
; ** Version: V3.12 vom 18/03/2008                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003-2020 Sven Brandt
;Timing:
T_FMAX      equ d'110'                 ; max. valid time from flag to flag
                                       ; T_fmax*1us (for MM Motorola only)
 
; ----- Saving STATUS and W ----------------------
        movwf W_TEMP                   ; Saving W
        swapf STATUS,W                 ; STATUS -> W
        movwf STATUS_TEMP              ; Saving STATUS
        bcf STATUS,RP0                 ; -> BANK 0
 
; ----- IR IB0-Change (positive Edge) ------------
ir_2    btfss INTCON,INTF              ; RA2/INT external interrupt occurred?
         goto posret                   ; - no (should not happen)
        bcf INTCON,INTF                ; - yes, clear IR-Flag
 
        btfsc R_FLAG,F_IDLE            ; if IDLE-Flag
         goto READ_FIRST               ; ... then go to Read the 1st Bit 
 
        btfss R_FLAG,F_READ            ; if not READ-Flag
         goto posret                   ; ...then jump out of interrupt

        ; *** check timing ***
        ; typically:
        ; TMR0 = d'68'   if turnout command
        ; TMR0 = d'172'  if loco command
        movlw T_FMAX
        subwf TMR0,W                   ; W = TMR0 - W = TMR0 - T_FMAX
        btfsc STATUS,C                 ; if C=1, -> TMR0 >= T_FMAX
         goto ir_2f                    ; ... then jump to Interrupt end
        goto  READ_NOTFIRST            ; ... else jump to Read next Bit 
 
 
READ_FIRST; *** if Bit #1 ( no timing-check) ***
        nop
        goto  $+1
        goto  $+1
        goto  $+1
READ_NOTFIRST   ; *** if Bit #2-18 ( after timing-check) ***
        goto  $+1
        goto  $+1
        goto  $+1
        goto  $+1
        goto  $+1
        goto  $+1
        goto  $+1
 
        bcf R_FLAG,F_IDLE
        bsf R_FLAG,F_READ              ; change from IDLE to READ
 
#ifdef test_isr
   bsf PORTA,4
#endif ; test_isr
        ; *** Read Bit ***
        bcf R_DEKADR_1,1               ; clear all Bits >#1
        bcf STATUS,C                   ; clear CARRY
        ; up to here we have had 38 cycles since interrupt trigger
        btfsc PORTA,2                  ; check signal
         bsf STATUS,C
        rlf R_DEKADR_3,F               ; shift bits
        rlf R_DEKADR_2,F               ; ...into the 3
        rlf R_DEKADR_1,F               ; ...receive register
        ; *** Count Bits ***
        decfsz R_BITCOUNT,F            ; if < Bit#18,
         goto posret                   ; ...then jump
 
        ; *** all 18 Bits read ***
#ifdef test_isr
   bsf PORTA,5
#endif ; test_isr
        bsf R_FLAG,F_CMD               ; FLAG for valid command
        movf  R_DEKADR_3,W
        movwf R_DEKADR_3_L
        movf  R_DEKADR_2,W
        movwf R_DEKADR_2_L
        movf  R_DEKADR_1,W
        movwf R_DEKADR_1_L
 
ir_2f   bcf R_FLAG,F_READ              ; new IDLE necessary
 
; ----- Restore STATUS and W ---------------------
posret
#ifdef test_isr
   bcf PORTA,4
   bcf PORTA,5
#endif ; test_isr
        clrf TMR0                      ; clear Timer
        bcf INTCON,T0IF                ; clear Timer Overflow Flag
        swapf STATUS_TEMP,W            ; saved STATUS in W
        movwf STATUS                   ; w -> STATUS
        SWAPF W_TEMP,F                 ; W_TEMP -> W, without..
        SWAPF W_TEMP,W                 ; .. changing Z-Flag
 
        retfie
