; ********************************************************
; ********************************************************
; ** Version: V3.47 vom 11/11/2010                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05/06/07/08/09/10/15 Sven Brandt

; ********************************************************
; History:
; V3.61 - alle Zeiten aus EEprom
; V3.47 - Wert fr Phasenlnge aus EEprom
; V3.41 - sterreichversion angepasst Roland Zagler in MODE2
; V3.40 - Fading einstellbar 

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: LED_AMPEL          **
;** state manager ampel         **
;*********************************
;*********************************
;*********************************
;*********************************
statemachine_ampel

T_GE        equ d'05'          ; ->EEprom (1..255) duration Phase #06 x0.2 sec (Hauptstrasse auf GELB)
T_RG        equ d'05'          ; ->EEprom (1..255) duration Phase #03 x0.2 sec (Hauptstrasse ROT/GELB)
T_RR        equ d'15'          ; ->EEprom (1..255) duration Phase #01 x0.2 sec (Phase ROT/ROT)
T_VR        equ d'10'          ; ->EEprom (1..255) duration Phase #02 x0.2 sec (Fussgnger Vorlauf Rot)
T_VG        equ d'10'          ; ->EEprom (1..255) duration Phase #02 x0.2 sec (Fussgnger Vorlauf Grn)
T_G1        equ d'60'          ; ->EEprom (1..255) duration Phase #04 x0.2 sec (Gruen Hauptstr.)
T_G2        equ d'20'          ; ->EEprom (1..255) duration Phase #14 x0.2 sec (Gruen Nebenstr.)

T_66        equ d'01'          ; (1..255) duration Phase #66 x0.2 sec (Nacht)
T_67        equ d'01'          ; (1..255) duration Phase #66 x0.2 sec (Nacht)
T_68        equ d'01'          ; (1..255) duration Phase #66 x0.2 sec (Nacht)
T_69        equ d'10'          ; (1..255) duration Phase #66 x0.2 sec (Nacht)
T_88        equ d'20'          ; (1..255) duration Phase #88 x0.2 sec
T_99        equ d'10'          ; (1..255) duration Phase #99 x0.2 sec
T_S0        equ d'10'          ; (1..255) duration Start#0 x0.2 sec
T_S1        equ d'01'          ; (1..255) duration Start#1 x0.2 sec
T_S2        equ d'10'          ; (1..255) duration Start#2 x0.2 sec
T_S3        equ d'10'          ; (1..255) duration Start#3 x0.2 sec
T_GB        equ d'01'          ; (1..255) duration Blinking x0.2 sec

;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms


;   ------------------------------------
;   --- Taster -> Auswahl des Status ---
;   ------------------------------------
#ifdef nodecoder
        btfss PORTA,3                  ; if Button press = LOW
         movlw b'00000010'
        btfsc PORTA,3                  ; if Button no press = HIGH
         movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
#endif

;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
#ifndef test
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
#endif
        clrf TMR1L                     ; .. then reset to ZERO
        clrf TMR1H                     ; .. then reset to ZERO

        movf R_COUNT,F                 ; wenn R_COUNT = ZERO, dann ist
        btfsc STATUS,Z                 ; .. PWM abgeschlossen,
         goto ampel_state              ; .. jetzt State-Machine

        incfsz R_COUNT,F               ; PWM fertig ?
         goto ampel_1                  ; .. wenn nicht, jump

        call pwm_finish
        goto ampel_state

ampel_1 ;**** LED steuern ****
        ; R_PWMVALUE wird bei jedem Step vergrssert. Es wird R_ADD_1 und R_ADD_2 im Wechsel addieret,
        ; .. um einen grossen Einstellbereich fr das Fading zu erhalten
        ; Bei berlauf wird auf FF gefreezed
        btfss R_COUNT,0                ; R_ADD_1 und R_ADD_2 im Wechsel addieren 
         movf R_ADD_1,W                ; R_ADD_1 und R_ADD_2 im Wechsel addieren 
        btfsc R_COUNT,0                ; R_ADD_1 und R_ADD_2 im Wechsel addieren 
         movf R_ADD_2,W                ; R_ADD_1 und R_ADD_2 im Wechsel addieren 
        addwf R_PWMVALUE,F             ; .. PWM erhhen
        btfss STATUS,C                 ; .. dann 
         goto ampel_pwm
        movlw h'FF'                    ; wenn berlauf 
        movwf R_PWMVALUE               ; .. Register = FF
        call pwm_finish
        ;return
        
ampel_pwm
        ; ***** PWM-Time Einschaltpunkt *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bsf PORTA,4                   ; LED_1 on
        btfsc R_FADEIN1,1              ; = fade in LED2
         bsf PORTA,5                   ; LED_2 on
        btfsc R_FADEIN1,2              ; = fade in LED3
         bsf PORTA,0                   ; LED_3 on
        btfsc R_FADEIN1,3              ; = fade in LED4
         bsf PORTA,1                   ; LED_4 on
        btfsc R_FADEIN1,4              ; = fade in LED5
         bsf PORTC,1                   ; LED_5 on
        btfsc R_FADEIN1,5              ; = fade in LED6
         bsf PORTC,2                   ; LED_6 on
        btfsc R_FADEIN2,0              ; = fade in LED7
         bsf PORTC,0                   ; LED_7 on
        btfsc R_FADEIN2,1              ; = fade in LED8
         bsf PORTC,5                   ; LED_8 on
        btfsc R_FADEIN2,2              ; = fade in LED9
         bsf PORTC,4                   ; LED_9 on
        btfsc R_FADEIN2,3              ; = fade in LED10
         bsf PORTC,3                   ; LED_10 on

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto ampel_2                  ; ... then JUMP

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bcf PORTA,4                   ; LED_1 off
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bcf PORTA,5                   ; LED_2 off
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bcf PORTA,0                   ; LED_3 off
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bcf PORTA,1                   ; LED_4 off
        btfsc R_FADEOUT1,4             ; = fade out LED5
         bcf PORTC,1                   ; LED_5 off
        btfsc R_FADEOUT1,5             ; = fade out LED6
         bcf PORTC,2                   ; LED_6 off
        btfsc R_FADEOUT2,0             ; = fade out LED7
         bcf PORTC,0                   ; LED_7 off
        btfsc R_FADEOUT2,1             ; = fade out LED8
         bcf PORTC,5                   ; LED_8 off
        btfsc R_FADEOUT2,2             ; = fade out LED9
         bcf PORTC,4                   ; LED_9 off
        btfsc R_FADEOUT2,3             ; = fade out LED10
         bcf PORTC,3                   ; LED_10 off

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

ampel_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto ampel_2                  ; ... then JUMP_2

        ;***** PWM-Time abgelaufen *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bcf PORTA,4                   ; LED_1 off
        btfsc R_FADEIN1,1              ; = fade in LED2
         bcf PORTA,5                   ; LED_2 off
        btfsc R_FADEIN1,2              ; = fade in LED3
         bcf PORTA,0                   ; LED_3 off
        btfsc R_FADEIN1,3              ; = fade in LED4
         bcf PORTA,1                   ; LED_4 off
        btfsc R_FADEIN1,4              ; = fade in LED5
         bcf PORTC,1                   ; LED_5 off
        btfsc R_FADEIN1,5              ; = fade in LED6
         bcf PORTC,2                   ; LED_6 off
        btfsc R_FADEIN2,0              ; = fade in LED7
         bcf PORTC,0                   ; LED_7 off
        btfsc R_FADEIN2,1              ; = fade in LED8
         bcf PORTC,5                   ; LED_8 off
        btfsc R_FADEIN2,2              ; = fade in LED9
         bcf PORTC,4                   ; LED_9 off
        btfsc R_FADEIN2,3              ; = fade in LED10
         bcf PORTC,3                   ; LED_10 off

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bsf PORTA,4                   ; LED_1 on
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bsf PORTA,5                   ; LED_2 on
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bsf PORTA,0                   ; LED_3 on
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bsf PORTA,1                   ; LED_4 on
        btfsc R_FADEOUT1,4             ; = fade out LED5
         bsf PORTC,1                   ; LED_5 on
        btfsc R_FADEOUT1,5             ; = fade out LED6
         bsf PORTC,2                   ; LED_6 on
        btfsc R_FADEOUT2,0             ; = fade out LED7
         bsf PORTC,0                   ; LED_7 on
        btfsc R_FADEOUT2,1             ; = fade out LED8
         bsf PORTC,5                   ; LED_8 on
        btfsc R_FADEOUT2,2             ; = fade out LED9
         bsf PORTC,4                   ; LED_9 on
        btfsc R_FADEOUT2,3             ; = fade out LED10
         bsf PORTC,3                   ; LED_10 on

        return





ampel_state
#ifndef test
        decfsz R_DELAY2,F              ; extra-Verzgerung
         return                        ; auf ca. 0.2 sec
        movlw d'70'
        movwf R_DELAY2
#endif

        decfsz R_DELAY,F               ; if R_DELAY = 0
         return                        ; .. then change R_STATE 

        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'
        movwf R_PWMVALUE

phase1  ; *** Phase Rot/Rot
        movlw d'01'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase2                   ; .. then OK -> jump

        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten


        ; *** load RG (Lnge ROT/ROT) ***
        movlw EE_ADR_T_RR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'02'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO

















         
phase2  ; *** Phase: Fussgnger Haupt auf GRN
        movlw d'02'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase3                   ; .. then OK -> jump

        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten


        ; *** load VG (Lnge Fussgnger Vorlauf Grn) ***
        movlw EE_ADR_T_VG              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'03'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase3  ; *** Phase: Hauptstrasse auf ROT/GELB
        movlw d'03'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase4                   ; .. then OK -> jump

        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten



        ; *** load RG (Lnge ROT/GELB) ***
        movlw EE_ADR_T_RG              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'04'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase4  ; *** Phase: Hauptstrasse auf GRN
        movlw d'04'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase71                  ; .. then OK -> jump

        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten

        ; *** load G1 (Lnge Grn-Phase) ***
        movlw EE_ADR_T_G1               ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T1
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs

        btfsc R_STATUS,2                ; wenn "GRN" (R_STATUS#2), dann
         movlw d'01'                    ; minimale Verzoegerungszeit
        movwf R_DELAY

        movlw d'04'
        movwf R_BL_CT                   ; Anzahl Blinken (nur fr fr AUT-Mode)

        movlw d'05'                     ; D-Mode
        btfsc R_MODE,1                  ; nur AUT-Mode
         movlw d'71'                    ; .. then state 71
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO










phase71  ; *** Phase (nur AUT): Fussgnger Hauptstrasse GRN blinken
        movlw d'71'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase72                  ; .. then OK -> jump

        btfsc R_STATUS,2               ; wenn "GRN" (R_STATUS#2), dann
         clrf R_COUNT                  ; .. keine PWM
        btfsc R_STATUS,2               ; wenn "GRN" (R_STATUS#2), dann
         return                        ; .. nichts machen, in diesem State bleiben

        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'72'
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO

phase72  ; *** Phase (nur AUT): Fussgnger Hauptstrasse GRN blinken
        movlw d'72'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase5                   ; .. then OK -> jump
        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'05'                    ; letzter Blinker
        decfsz R_BL_CT,1               ; Blinkzhler verringern
         movlw d'71'                   ; nochmals blinken
        movwf R_PHASE                  
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO







phase5  ; *** Phase: Fussgnger Haupt auf ROT
        movlw d'05'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase73                  ; .. then OK -> jump

        movlw d'01'                    ; minimale Verzoegerungszeit
        movwf R_DELAY

        btfsc R_MODE,1                 ; AUT-Mode
         goto phase5_a                 ; .. then jump

        btfsc R_STATUS,2               ; wenn "GRN" (R_STATUS#2), dann
         clrf R_COUNT                  ; .. keine PWM
        btfsc R_STATUS,2               ; wenn "GRN" (R_STATUS#2), dann
         clrf R_PWMVALUE               ; .. keine PWM
        btfsc R_STATUS,2               ; wenn "GRN" (R_STATUS#2), dann
         return                        ; .. nichts machen, in diesem State bleiben

        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten

        ; *** load VR (Lnge Fussgnger Vorlauf Rot) ***
        movlw EE_ADR_T_VR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                        
        movlw d'06'                     ; wenn Betriebsart
        btfsc R_STATUS,1                ;  "NACHT" (R_STATUS#1), dann
         movlw d'68'                    ; d'68', sonst d'06'
                                        
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO



phase5_a  ; *** Phase: Fussgnger Haupt auf ROT (AUT-Mode)
        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten

        ; *** load VR (Lnge Fussgnger Vorlauf Rot) ***
        movlw EE_ADR_T_VR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                       
        movlw d'73'                     ; wenn Betriebsart
        btfsc R_STATUS,1                ;  "NACHT" (R_STATUS#1), dann
         movlw d'68'                    ; d'68', sonst d'73'
                                       
        movwf R_PHASE                   ; naechster STATE
                                       
        movlw d'04'                    
        movwf R_BL_CT                   ; Anzahl Blinken
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO















             


phase73  ; *** Phase: Hauptstrasse GRN blinken
        movlw d'73'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase74                  ; .. then OK -> jump
        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'74'
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO

phase74  ; *** Phase: Hauptstrasse GRN blinken
        movlw d'74'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase6                   ; .. then OK -> jump
        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'06'                    ; letzter Blinker
        decfsz R_BL_CT,1               ; Blinkzhler verringern
         movlw d'73'                   ; nochmals blinken
        movwf R_PHASE                  
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase6  ; *** Phase: Hauptstrasse auf GELB
        movlw d'06'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase11                   ; .. then OK -> jump

        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten


        ; *** load GE (Lnge GELB) ***
        movlw EE_ADR_T_GE              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                        
        movlw d'11'                     
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase11  ; *** Phase ROT/ROT
        movlw d'11'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase12                   ; .. then OK -> jump

        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten


        ; *** load RR (Lnge ROT/ROT) ***
        movlw EE_ADR_T_RR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                        
        movlw d'12'                     
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase12  ; *** Phase: Fussgnger Neben auf GRN
        movlw d'12'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase13                  ; .. then OK -> jump

        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten
        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten


        ; *** load VG (Lnge Fussgnger Vorlauf Grn) ***
        movlw EE_ADR_T_VG              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                        
        movlw d'13'                     
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase13  ; *** Phase: Nebenstrasse auf ROT/GELB
        movlw d'13'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase14                  ; .. then OK -> jump

        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten



        ; *** load RG (Lnge ROT/GELB) ***
        movlw EE_ADR_T_RG              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY
                                        
        movlw d'14'                     
        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO


















phase14  ; *** Phase: Nebenstrasse auf GRN
        movlw d'14'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase75                  ; .. then OK -> jump

        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten

        ; *** load TG (Lnge Grn-Phase 2) ***
        movlw EE_ADR_T_G2              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs

        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         movlw d'01'                   ; minimale Verzoegerungszeit
        movwf R_DELAY

        movlw d'04'
        movwf R_BL_CT                  ; Anzahl Blinken (nur fr fr AUT-Mode)

        movlw d'15'                    ; D-Mode
        btfsc R_MODE,1                 ; nur AUT-Mode
         movlw d'75'                   ; .. then state 75
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO










phase75  ; *** Phase (nur AUT): Fussgnger Nebenstrasse GRN blinken
        movlw d'75'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase76                  ; .. then OK -> jump

        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         clrf R_COUNT                  ; .. keine PWM
        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         return                        ; .. nichts machen, in diesem State bleiben

        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'76'
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO

phase76  ; *** Phase (nur AUT): Fussgnger Nebenstrasse GRN blinken
        movlw d'76'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase15                  ; .. then OK -> jump
        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY


        movlw d'15'                    ; letzter Blinker
        decfsz R_BL_CT,1               ; Blinkzhler verringern
         movlw d'75'                   ; nochmals blinken
        movwf R_PHASE                  
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO






phase15  ; *** Phase: Fussgnger Neben auf ROT
        movlw d'15'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase77                  ; .. then OK -> jump

        movlw d'01'                    ; minimale Verzoegerungszeit
        movwf R_DELAY

        btfsc R_MODE,1                 ; AUT-Mode
         goto phase15_a                ; .. then jump

        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         clrf R_COUNT                  ; .. keine PWM
        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         clrf R_PWMVALUE               ; .. keine PWM
        btfsc R_STATUS,3               ; wenn "ROT" (R_STATUS#3), dann
         return                        ; .. nichts machen, in diesem State bleiben

        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        ; *** load VR (Lnge Fussgnger Vorlauf Rot) ***
        movlw EE_ADR_T_VR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'16'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO






phase15_a ; *** Phase: Fussgnger Neben auf ROT (AUT-Mode)
       bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        ; *** load VR (Lnge Fussgnger Vorlauf Rot) ***
        movlw EE_ADR_T_VR              ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'77'
        movwf R_PHASE         ; naechster STATE

        movlw d'04'
        movwf R_BL_CT         ; Anzahl Blinken
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO





















phase77  ; *** Phase: Nebenstrasse GRN blinken
        movlw d'77'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase78                  ; .. then OK -> jump
        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'78'
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO


phase78  ; *** Phase: Nebenstrasse GRN blinken
        movlw d'78'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase16                  ; .. then OK -> jump
        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten

        movlw T_GB                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'16'                    ; letzter Blinker
        decfsz R_BL_CT,1               ; Blinkzhler verringern
         movlw d'77'                   ; nochmals blinken
        movwf R_PHASE                  
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO












phase16  ; *** Phase: Nebenstrasse auf GELB
        movlw d'16'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase66                  ; .. then OK -> jump

        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten

        ; *** load GE (Lnge GELB) ***
        movlw EE_ADR_T_GE             ; get EE Address
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Address: EE_ADR_T2
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_DELAY

        movlw d'01'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase66  ; *** Nacht - gelb an
        movlw d'66'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase67                  ; .. then OK -> jump

;        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
        btfsc R_MODE,1                  ; nur AUT-Mode
         bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
;        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
;        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
;        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
;        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
;        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

;        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten
;        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten
;        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten
;        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_66            ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'67'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase67  ; *** Nacht - gelb aus
        movlw d'67'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase68                  ; .. then OK -> jump

;        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
;        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
;        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
;        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
;        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
;        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
;        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
;        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

;        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
        btfsc R_MODE,1                  ; nur AUT-Mode
         bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten
;        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten
;        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten
;        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_67            ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'66'           ; wenn Betriebsart
        btfss R_STATUS,1      ;  "NACHT" (R_STATUS#1), dann
         movlw d'69'          ; d'66', sonst d'69'

        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO










phase68  ; *** Nacht - gelb aus
        movlw d'68'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase69                  ; .. then OK -> jump

;        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
;        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
;        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
;        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
;        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
;        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
;        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
;        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

;        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten
        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten
        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten
;        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten
        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten
        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_68            ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'66'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase69  ; *** Nacht - gelb aus
        movlw d'69'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase88                  ; .. then OK -> jump

;        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
;        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
;        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
;        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
;        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

;        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten
;        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten
;        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten
;        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_69            ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'88'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase88  ; *** erste Phase nach Start
        movlw d'88'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase99                  ; .. then OK -> jump

;        bsf R_FADEIN1,0                ; ROT   Hauptstrasse einschalten
;        bsf R_FADEIN1,1                ; GELB  Hauptstrasse einschalten
;        bsf R_FADEIN1,2                ; GRUEN Hauptstrasse einschalten
        bsf R_FADEIN1,3                ; ROT   Nebenstrasse einschalten
;        bsf R_FADEIN1,4                ; GELB  Nebenstrasse einschalten
;        bsf R_FADEIN1,5                ; GRUEN Nebenstrasse einschalten
;        bsf R_FADEIN2,0                ; FUSSG-ROT   Hauptstrasse einschalten
;        bsf R_FADEIN2,1                ; FUSSG-GRUEN Hauptstrasse einschalten
;        bsf R_FADEIN2,2                ; FUSSG-ROT   Nebenstrasse einschalten
;        bsf R_FADEIN2,3                ; FUSSG-GRUEN Nebenstrasse einschalten

;        bsf R_FADEOUT1,0               ; ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT1,1               ; GELB  Hauptstrasse abschalten
;        bsf R_FADEOUT1,2               ; GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT1,3               ; ROT   Nebenstrasse abschalten
        bsf R_FADEOUT1,4               ; GELB  Nebenstrasse abschalten
;        bsf R_FADEOUT1,5               ; GRUEN Nebenstrasse abschalten
;        bsf R_FADEOUT2,0               ; FUSSG-ROT   Hauptstrasse abschalten
;        bsf R_FADEOUT2,1               ; FUSSG-GRUEN Hauptstrasse abschalten
;        bsf R_FADEOUT2,2               ; FUSSG-ROT   Nebenstrasse abschalten
;        bsf R_FADEOUT2,3               ; FUSSG-GRUEN Nebenstrasse abschalten

        movlw T_88                      ; Verzoegerungszeit
        movwf R_DELAY                  
                                       
        movlw d'04'                    
        movwf R_BL_CT                   ; Anzahl Blinken

        movlw d'06'
        btfsc R_MODE,1                  ; nur AUT-Mode
         movlw d'73'                    ; .. then state 73

        movwf R_PHASE                   ; naechster STATE
        return                          ; Return, jetzt muss erst R_DELAY wieder auf ZERO













phase99 ; keine gltige Phase eingestellt
        movlw d'01'
        movwf R_PHASE         ; naechster STATE
        return                ; Return, jetzt muss erst R_DELAY wieder auf ZERO

;*******************************************
;** Routine: pwm_finish                   **
;** switch outputs to static              **
;*******************************************
pwm_finish
        ; PWM fertig, ab jetzt statische LEDs
        ; wenn FADE-IN gewesen, dann entsprechendes Port EIN
        btfsc R_FADEIN1,0      ; = fade in LED1
         bsf PORTA,4           ; ROT   fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,1      ; = fade in LED2
         bsf PORTA,5           ; GELB  fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,2      ; = fade in LED3
         bsf PORTA,0           ; GRUEN fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,3      ; = fade in LED4
         bsf PORTA,1           ; ROT   fuer Nebenstrasse = EIN
        btfsc R_FADEIN1,4      ; = fade in LED5
         bsf PORTC,1           ; GELB  fuer Nebenstrasse = EIN
        btfsc R_FADEIN1,5      ; = fade in LED6
         bsf PORTC,2           ; GRUEN fuer Nebenstrasse = EIN
        btfsc R_FADEIN2,0      ; = fade in LED7
         bsf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = EIN
        btfsc R_FADEIN2,1      ; = fade in LED8
         bsf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = EIN
        btfsc R_FADEIN2,2      ; = fade in LED9
         bsf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = EIN
        btfsc R_FADEIN2,3      ; = fade in LED10
         bsf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = EIN
        clrf R_FADEIN1
        clrf R_FADEIN2

        ; wenn FADE-OUT gewesen, dann entsprechendes Port AUS
        btfsc R_FADEOUT1,0     ; = fade out LED1
         bcf PORTA,4           ; ROT   fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,1     ; = fade out LED2
         bcf PORTA,5           ; GELB  fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,2     ; = fade out LED3
         bcf PORTA,0           ; GRUEN fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,3     ; = fade out LED4
         bcf PORTA,1           ; ROT   fuer Nebenstrasse = AUS
        btfsc R_FADEOUT1,4     ; = fade out LED5
         bcf PORTC,1           ; GELB  fuer Nebenstrasse = AUS
        btfsc R_FADEOUT1,5     ; = fade out LED6
         bcf PORTC,2           ; GRUEN fuer Nebenstrasse = AUS
        btfsc R_FADEOUT2,0     ; = fade out LED7
         bcf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = AUS
        btfsc R_FADEOUT2,1     ; = fade out LED8
         bcf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = US
        btfsc R_FADEOUT2,2     ; = fade out LED9
         bcf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = AUS
        btfsc R_FADEOUT2,3     ; = fade out LED10
         bcf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = AUS
        clrf R_FADEOUT1
        clrf R_FADEOUT2
        return