; ********************************************************
; ** Version: V3.26 vom 27/03/2009                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05/08/09 Sven Brandt

; ********************************************************
; History:
; V3.26 - Dumm-Mode Hinzu
;       - noch "Nachblinken" nach Schrankenffnung
; ********************************************************

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: LED_ANDREAS        **
;** state manager andreas       **
;*********************************
;*********************************
;*********************************
;*********************************
statemachine_andreas 

T_01        equ d'05'         ; (1..255) duration Phase #01 x0.2 sec (Gelb)
T_02        equ d'03'         ; (1..255) duration Phase #02 x0.2 sec (Rot)
T_03        equ d'03'         ; (1..255) duration Phase #03 x0.2 sec
T_04        equ d'03'         ; (1..255) duration Phase #04 x0.2 sec
T_05        equ d'03'         ; (1..255) duration Phase #05 x0.2 sec
T_06        equ d'03'         ; (1..255) duration Phase #06 x0.2 sec
T_07        equ d'01'         ; (1..255) duration Phase #06 x0.2 sec
T_08        equ d'05'         ; (1..255) duration blinking after opening

;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms


;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
#ifndef test
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
#endif
        clrf TMR1L                     ; .. then reset to ZERO
        clrf TMR1H                     ; .. then reset to ZERO

        movf R_COUNT,F                 ; wenn R_COUNT = ZERO, dann ist
        btfsc STATUS,Z                 ; .. PWM abgeschlossen,
         goto andreas_state            ; .. jetzt State-Machine

;   -------------------------
;   --- PWM-Generation ------
;   -------------------------
        incfsz R_COUNT,F               ; PWM fertig ?
         goto andreas_1                ; .. wenn nicht, jump

        ; PWM fertig, ab jetzt statische LEDs
        ; wenn FADE-IN gewesen, dann entsprechendes Port EIN
        btfsc R_FADEIN1,0      ; = fade in LED1
         bsf PORTA,4           ; 
        btfsc R_FADEIN1,1      ; = fade in LED2
         bsf PORTA,5           ; 
        btfsc R_FADEIN1,2      ; = fade in LED3
         bsf PORTA,0           ; 
        btfsc R_FADEIN1,3      ; = fade in LED4
         bsf PORTA,1           ; 
        clrf R_FADEIN1

        ; wenn FADE-OUT gewesen, dann entsprechendes Port AUS
        btfsc R_FADEOUT1,0     ; = fade out LED1
         bcf PORTA,4           ; 
        btfsc R_FADEOUT1,1     ; = fade out LED2
         bcf PORTA,5           ; 
        btfsc R_FADEOUT1,2     ; = fade out LED3
         bcf PORTA,0           ; 
        btfsc R_FADEOUT1,3     ; = fade out LED4
         bcf PORTA,1           ; 
        clrf R_FADEOUT1
        goto andreas_state

andreas_1 ;**** LED steuern ****
        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

        ; ***** PWM-Time Einschaltpunkt *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bsf PORTA,4                   ; LED_1 on
        btfsc R_FADEIN1,1              ; = fade in LED2
         bsf PORTA,5                   ; LED_2 on
        btfsc R_FADEIN1,2              ; = fade in LED3
         bsf PORTA,0                   ; LED_3 on
        btfsc R_FADEIN1,3              ; = fade in LED4
         bsf PORTA,1                   ; LED_4 on

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto andreas_2                ; ... then JUMP

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bcf PORTA,4                   ; LED_1 off
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bcf PORTA,5                   ; LED_2 off
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bcf PORTA,0                   ; LED_3 off
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bcf PORTA,1                   ; LED_4 off

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

andreas_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto andreas_2                ; ... then ANDREAS_2

        ;***** PWM-Time abgelaufen *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bcf PORTA,4                   ; LED_1 off
        btfsc R_FADEIN1,1              ; = fade in LED2
         bcf PORTA,5                   ; LED_2 off
        btfsc R_FADEIN1,2              ; = fade in LED3
         bcf PORTA,0                   ; LED_3 off
        btfsc R_FADEIN1,3              ; = fade in LED4
         bcf PORTA,1                   ; LED_4 off

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bsf PORTA,4                   ; LED_1 on
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bsf PORTA,5                   ; LED_2 on
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bsf PORTA,0                   ; LED_3 on
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bsf PORTA,1                   ; LED_4 on

        return


;   -------------------------
;   --- State Machine -------
;   -------------------------
andreas_state

phase0  ; *** Phase Pause
        movlw d'00'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto blink                    ; .. then OK -> jump

        btfsc R_STATUS,0               ; wenn "EIN" (R_STATUS#0), dann
         goto phase0_on                ; .. springe

        ; * Phase0 bleibt stabil, kein Einschalten
        ; * wenn noch LEDs an sind, diese ausblenden.
        btfsc PORTA,4                  ; wenn LED1 ist EIN
         bsf R_FADEOUT1,0              ; LED1  abschalten
        movlw d'01'                    ; 
        btfsc PORTA,4                  ; wenn LED1 ist EIN
         movwf R_COUNT                 ; PWM wieder starten mit '1'
        btfsc PORTA,5                  ; wenn LED2 ist EIN
         bsf R_FADEOUT1,1              ; ..LED2  abschalten
        movlw d'01'                    ; 
        btfsc PORTA,5                  ; wenn LED2 ist EIN
         movwf R_COUNT                 ; PWM wieder starten mit '1'
        return                         ; R_PHASE bleibt unverndert
       
phase0_on
        ; * Phase 0 wird verlassen, erste LED einblenden, State wechseln
        movlw d'01'
        movwf R_DELAY                  ; minimale Verzoegerungszeit

        bsf R_FADEIN1,0                ; LED1  einschalten
        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'

        movlw d'01'                    ; 
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO



blink  ; *** Wechselblinken generieren
       ; wird immer aufgerufen wenn R_PHASE <> d'00' (ZERO)
        btfsc PORTA,4
         goto blink_2
blink_1; *** Blinkphase 1
;        btfsc R_STATUS,0               ; wenn "EIN"
         bsf R_FADEIN1,0               ; LED1  einschalten
        btfsc PORTA,5                  ; wenn LED2 ist EIN
         bsf R_FADEOUT1,1              ; ..LED2  abschalten
        goto blink_99
blink_2; *** Blinkphase 2
;        btfsc R_STATUS,0               ; wenn "EIN"
         bsf R_FADEIN1,1               ; LED2  einschalten
         bsf R_FADEOUT1,0              ; LED1  abschalten
blink_99
        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'

        decfsz R_DELAY,F               ; if R_DELAY = 0 (Verzgerung zwischen den States)
         return                        ; .. then change R_STATE 






phase1  ; *** Phase Gelb
        movlw d'01'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase2                   ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
        bsf R_FADEIN1,3                ; LED 3 (Ampel GELB) einschalten
;        bsf PORTC,1                   ; Schranke-Ausgang #1 ein
;        bsf PORTC,2                   ; Schranke-Ausgang #2 ein
;        bsf PORTC,0                   ; Schranke-Ausgang #3 ein
;        bsf PORTC,5                   ; Schranke-Ausgang #4 ein
;        bsf PORTC,4                   ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_01                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'02'
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        ;clrf PORTC                     ; alle Schranken-Ausgnge aus
        clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        ;bsf R_FADEOUT1,3               ; LED 3 (Ampel GELB) abschalten
        return



         
phase2  ; *** Phase: Rot ein / GELB aus / Schranke #1 
        movlw d'02'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase3                   ; .. then OK -> jump

        bsf R_FADEIN1,2                ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
;        bsf PORTC,2                   ; Schranke-Ausgang #2 ein
;        bsf PORTC,0                   ; Schranke-Ausgang #3 ein
;        bsf PORTC,5                   ; Schranke-Ausgang #4 ein
;        bsf PORTC,4                   ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
        bsf R_FADEOUT1,3               ; LED 3 (Ampel GELB) abschalten

        movlw T_02                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'03'
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,3               ; LED 3 (Ampel GELB) abschalten
        return




phase3  ; *** Phase: Schranke #2
        movlw d'03'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase4                   ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
        bsf PORTC,2                    ; Schranke-Ausgang #2 ein
;        bsf PORTC,0                   ; Schranke-Ausgang #3 ein
;        bsf PORTC,5                   ; Schranke-Ausgang #4 ein
;        bsf PORTC,4                   ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_03                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'04'
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        ;clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,2               ; LED 3 (Ampel ROT) abschalten
        return




phase4  ; *** Phase: Schranke #3
        movlw d'04'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase5                   ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
        bsf PORTC,2                    ; Schranke-Ausgang #2 ein
        bsf PORTC,0                    ; Schranke-Ausgang #3 ein
;        bsf PORTC,5                   ; Schranke-Ausgang #4 ein
;        bsf PORTC,4                   ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_04                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'05'
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        ;clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,2               ; LED 3 (Ampel ROT) abschalten
        return




phase5  ; *** Phase: Schranke #4
        movlw d'05'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase6                   ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
        bsf PORTC,2                    ; Schranke-Ausgang #2 ein
        bsf PORTC,0                    ; Schranke-Ausgang #3 ein
        bsf PORTC,5                    ; Schranke-Ausgang #4 ein
;        bsf PORTC,4                   ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_05                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'06'                    ; 
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        ;clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,2               ; LED 3 (Ampel ROT) abschalten
        return




phase6  ; *** Phase: Schranke #5
        movlw d'06'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase7                   ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
        bsf PORTC,2                    ; Schranke-Ausgang #2 ein
        bsf PORTC,0                    ; Schranke-Ausgang #3 ein
        bsf PORTC,5                    ; Schranke-Ausgang #4 ein
        bsf PORTC,4                    ; Schranke-Ausgang #5 ein
;        bsf PORTC,3                   ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_06                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'07'
        movwf R_PHASE                  ; naechster STATE
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        ;clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,2               ; LED 3 (Ampel ROT) abschalten
        return




phase7  ; *** Phase Schranke #6
        movlw d'07'                    ;
        subwf R_PHASE,W
        btfss STATUS,Z                 ; if not the same
         goto phase99                  ; .. then OK -> jump

;        bsf R_FADEIN1,2               ; LED 3 (Ampel ROT) einschalten
;        bsf R_FADEIN1,3               ; LED 3 (Ampel GELB) einschalten
        bsf PORTC,1                    ; Schranke-Ausgang #1 ein
        bsf PORTC,2                    ; Schranke-Ausgang #2 ein
        bsf PORTC,0                    ; Schranke-Ausgang #3 ein
        bsf PORTC,5                    ; Schranke-Ausgang #4 ein
        bsf PORTC,4                    ; Schranke-Ausgang #5 ein
        bsf PORTC,3                    ; Schranke-Ausgang #6 ein

;        bsf R_FADEOUT1,2              ; LED 3 (Ampel ROT) abschalten
;        bsf R_FADEOUT1,3              ; LED 3 (Ampel GELB) abschalten

        movlw T_07                     ; Verzoegerungszeit
        movwf R_DELAY

        movlw d'07'
        movwf R_PHASE                  ; naechster STATE (bleibt bei 7)
        btfsc R_STATUS,0               ; wenn "EIN"
         return                        ; ..Return, jetzt muss erst R_DELAY wieder auf ZERO

        ; wenn "AUS"...
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        ;clrf R_FADEIN1
        ;clrf R_FADEOUT1
        clrf R_PHASE                   ; IDLE-State
        bsf R_FADEOUT1,2               ; LED 3 (Ampel ROT) abschalten
        return




phase99 ; keine gltige Phase eingestellt
        movlw d'00'
        movwf R_PHASE                  ; naechster STATE
        return                         ; Return, jetzt muss erst R_DELAY wieder auf ZERO


