; ********************************************************
; ** Version: V3.61 vom 05/03/2015                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2005-2015 Sven Brandt

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: statemachine_haus  **
;** Blinksmdus                  **
;** needs <    us               **
;*********************************
;*********************************
;*********************************
;*********************************
statemachine_haus

; **********************************************
; **** Blink-Mode  (Haus)                   ****
; **********************************************
        ; im LEERlauf (Idle) Register trotzem Shuffle (erzeugt Varianz beim Einschalten)
        movf R_FLAGS,F
        btfsc STATUS,Z                 ; .. wenn R_FLAGS == ZERO (d.h. Z=TRUE), then
         call shuffle_register         ; .. SHUFFLE Register

        ; nur Ablauf, wenn eins der Flags (STARTUP/RUNNING/SHUTDOWN) gesetzt ist
        movlw b'00000111'              ; check state
        andwf R_FLAGS,W                ; if not this mode
        btfsc STATUS,Z                 ; .. then
         return                        ; .. return

;   -------------------------
;   --- create 1.2 ms-Raster ---
;   -------------------------
#ifndef simulation
        movf TMR1H,F
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
#endif
        clrf TMR1H                     ; .. then reset to ZERO

#ifdef test
    call toggle_led_1
#endif

        movlw CONST_FADE_01            ;
        movwf TMR1L;                   ; fr schnelleres Blinken / Fading

;   -----------------------------
;   --- Generate ON/OFF Flags ---
;   -----------------------------
        ; ONOFF OLD 0 0 1 1    -> 1 1 0 0  (COMF)
        ; ONOFF     0 1 0 1    -> 0 1 0 1  (unverndert)
        ; FADEIN    0 1 0 0    -> 0 1 0 0  (AND)
        comf R_ONOFF1_OLD,W            ; alte ONOFF Bits invertieren
        andwf R_ONOFF1,W               ; .. AND mit aktuellen ONOFF Werten
        movwf R_FADEIN1                ; .. Ergebnis

        comf R_ONOFF2_OLD,W            ; alte ONOFF Bits invertieren
        andwf R_ONOFF2,W               ; .. AND mit aktuellen ONOFF Werten
        movwf R_FADEIN2                ; .. Ergebnis

        ; ONOFF OLD 0 0 1 1    -> 0 0 1 1  (unverndert)
        ; ONOFF     0 1 0 1    -> 1 0 1 0  (COMF)
        ; FADEOUT   0 0 1 0    -> 0 0 1 0  (AND)
        comf R_ONOFF1,W                ; aktuelle ONOFF Bits invertieren
        andwf R_ONOFF1_OLD,W           ; .. AND mit alten ONOFF Werten
        movwf R_FADEOUT1               ; .. Ergebnis

        comf R_ONOFF2,W                ; aktuelle ONOFF Bits invertieren
        andwf R_ONOFF2_OLD,W           ; .. AND mit alten ONOFF Werten
        movwf R_FADEOUT2               ; .. Ergebnis

;   -----------------------------
;   --- PWM oder StateMachine? --
;   -----------------------------
        movf R_COUNT,F                 ; wenn R_COUNT = ZERO, dann ist
        btfsc STATUS,Z                 ; .. PWM abgeschlossen,
         goto haus_state               ; .. jetzt State-Machine

        ;**** PWM nur in Mode 1 ****
        movlw h'FF'
        btfss R_MODE,0                 ; wenn nicht Mode 1:
         movwf R_COUNT                 ; ... keine PWM -> R_COUNT auf ZERO

        incfsz R_COUNT,F               ; PWM fertig ?
         goto pwm                      ; .. wenn nicht, jump

        call pwm_finish                ; PWM wurde gerade beendet, einmaliger Aufruf bei Ende der PWM

haus_state
        ; *** Verzgerung ***     
        decfsz R_VERZ2,F
         return
        movlw CONST_BREMS
        movwf R_VERZ2

#ifdef test
    call toggle_led_2
#endif

        decfsz R_VERZ,F
         return
        movf R_MAKESLOW,W
        movwf R_VERZ

#ifdef test
    call toggle_led_2
#endif

        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'
        movwf R_PWMVALUE

        ; Shuffle Register
        call shuffle_register

led01   ; *** AN/AUS fr Ausgang 01 *** 
        btfss R_SHUFFLE1,0
         goto led02

        btfss R_SHUFFLE3,0
         bsf R_ONOFF1,0                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE3,1
         bcf R_ONOFF1,0                ; ..wenn Ergebnis nicht ZERO, dann AUS

led02   ; *** AN/AUS fr Ausgang 02 *** 
        btfss R_SHUFFLE1,2
         goto led03

        btfss R_SHUFFLE3,2
         bsf R_ONOFF1,1                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE3,3
         bcf R_ONOFF1,1                ; ..wenn Ergebnis nicht ZERO, dann AUS
 
led03   ; *** AN/AUS fr Ausgang 03 *** 
        btfss R_SHUFFLE1,3
         goto led04

        btfss R_SHUFFLE3,4
         bsf R_ONOFF1,2                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE3,5
         bcf R_ONOFF1,2                ; ..wenn Ergebnis nicht ZERO, dann AUS

led04   ; *** AN/AUS fr Ausgang 04 *** 
        btfss R_SHUFFLE1,4
         goto led05

        btfss R_SHUFFLE3,6
         bsf R_ONOFF1,3                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE3,7
         bcf R_ONOFF1,3                ; ..wenn Ergebnis nicht ZERO, dann AUS

led05   ; *** AN/AUS fr Ausgang 05 *** 
        btfss R_SHUFFLE1,5
         goto led06

        btfss R_SHUFFLE4,0
         bsf R_ONOFF1,4                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE4,1
         bcf R_ONOFF1,4                ; ..wenn Ergebnis nicht ZERO, dann AUS

led06   ; *** AN/AUS fr Ausgang 06 *** 
        btfss R_SHUFFLE1,6
         goto led07

        btfss R_SHUFFLE4,2
         bsf R_ONOFF1,5                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE4,3
         bcf R_ONOFF1,5                ; ..wenn Ergebnis nicht ZERO, dann AUS

led07   ; *** AN/AUS fr Ausgang 07 ***   
        btfss R_SHUFFLE1,7
         goto led08

        btfss R_SHUFFLE4,4
         bsf R_ONOFF1,6                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE4,5
         bcf R_ONOFF1,6                ; ..wenn Ergebnis nicht ZERO, dann AUS

led08   ; *** AN/AUS fr Ausgang 08 *** 
        btfss R_SHUFFLE2,0
         goto led09

        btfss R_SHUFFLE4,6
         bsf R_ONOFF1,7                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE4,7
         bcf R_ONOFF1,7                ; ..wenn Ergebnis nicht ZERO, dann AUS

led09   ; *** AN/AUS fr Ausgang 09 ***   
        btfss R_SHUFFLE2,1
         goto led10

        btfss R_SHUFFLE2,6
         bsf R_ONOFF2,0                ; ..wenn Ergebnis nicht ZERO, dann AN

        btfss R_SHUFFLE2,7
         bcf R_ONOFF2,0                ; ..wenn Ergebnis nicht ZERO, dann AUS

led10   ; *** AN/AUS fr Ausgang 10 ***   
        btfsc F_STARTUP
         bsf R_ONOFF2,1
        btfsc F_RUNNING
         bsf R_ONOFF2,1

        btfsc F_SHUTDOWN
         bcf R_ONOFF2,1
         
led11   ; Shutdown timeout: alle LEDs nach TimeOut abschalten
        btfss F_SHUTDOWN
         goto skip_shutdown_counter
        decfsz R_CNT_SHUTDOWN,F
         goto skip_shutdown_counter

        clrf R_ONOFF1
        clrf R_ONOFF1
skip_shutdown_counter

        ; *** Eingeschaltete Ausgnge (soll) zhlen ***
        clrf R_ANZOUT
        btfsc R_ONOFF1,0
         incf R_ANZOUT,F
        btfsc R_ONOFF1,1
         incf R_ANZOUT,F
        btfsc R_ONOFF1,2
         incf R_ANZOUT,F
        btfsc R_ONOFF1,3
         incf R_ANZOUT,F
        btfsc R_ONOFF1,4
         incf R_ANZOUT,F
        btfsc R_ONOFF1,5
         incf R_ANZOUT,F
        btfsc R_ONOFF1,6
         incf R_ANZOUT,F
        btfsc R_ONOFF1,7
         incf R_ANZOUT,F
        btfsc R_ONOFF2,0
         incf R_ANZOUT,F

        ; *** Eingeschaltete Ausgnge (ist) zhlen ***
        clrf R_ANZOUT_OLD
        btfsc R_ONOFF1_OLD,0
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,1
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,2
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,3
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,4
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,5
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,6
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF1_OLD,7
         incf R_ANZOUT_OLD,F
        btfsc R_ONOFF2_OLD,0
         incf R_ANZOUT_OLD,F

        ; *** Anzahl vernderter Ausgnge zhlen ***
        clrf R_ANZCNG
        movf R_ONOFF1,W
        xorwf R_ONOFF1_OLD,W
        movwf R_TEMP
        btfsc R_TEMP,0
         incf R_ANZCNG,F
        btfsc R_TEMP,1
         incf R_ANZCNG,F
        btfsc R_TEMP,2
         incf R_ANZCNG,F
        btfsc R_TEMP,3
         incf R_ANZCNG,F
        btfsc R_TEMP,4
         incf R_ANZCNG,F
        btfsc R_TEMP,5
         incf R_ANZCNG,F
        btfsc R_TEMP,6
         incf R_ANZCNG,F
        btfsc R_TEMP,7
         incf R_ANZCNG,F

        movf R_ONOFF2,W
        xorwf R_ONOFF2_OLD,W
        movwf R_TEMP
        btfsc R_TEMP,0
         incf R_ANZCNG,F
        btfsc R_TEMP,1
         incf R_ANZCNG,F

        decfsz R_ANZCNG,F              ; REJECT, wenn mehr als 1 Ausgang gendert worden ist
         goto check_changes_rejected

        btfsc F_STARTUP                ;
         goto check_startup
        btfsc F_RUNNING                ;
         goto check_running
        btfsc F_SHUTDOWN               ;
         goto check_shutdown
        return

check_startup
       ; *** HIER: Funktion ist AKTIV, Mindestzahl noch nicht erreicht (StartUp)
       ; *** Anzahl der eingeschalteten Ausgnge grsser als vorher ?? **
        movf R_MINOUT,W                ; get R_MINOUT into W-Register
        subwf R_ANZOUT,W               ; W = R_ANZOUT - W
        btfsc STATUS,C                 ; if C=1, -> R_ANZOUT > R_MINOUT
         goto check_startup_running    ; .. jetzt RUNNING Status

        movf R_ANZOUT_OLD,W            ; get R_ANZOUT_OLD into W-Register
        subwf R_ANZOUT,W               ; W = R_ANZOUT - W
        btfss STATUS,C                 ; if C=0, -> R_ANZOUT < R_ANZOUT_OLD
         goto check_changes_rejected   ; ... then REJECTED

        return                         ; .. return (d.h. nderung OK)

check_startup_running ; mindestanzahl R_MINOUT erreicht, ab jetzt RUNNING Status
        bcf F_STARTUP
        bsf F_RUNNING
        return                         ; .. return (d.h. nderung OK)

check_running
       ; *** Anzahl der eingeschalteten Ausgnge zu klein?? **
        movf R_MINOUT,W                ; get R_MINOUT into W-Register
        subwf R_ANZOUT,W               ; W = R_ANZOUT - W
        btfss STATUS,C                 ; if C=0, -> R_ANZOUT < R_MINOUT
         goto check_changes_rejected   ; ... then REJECTED

        return                         ; .. return (d.h. nderung OK)

check_shutdown
        ; *** Anzahl der eingeschalteten Ausgnge kleiner als vorher ?? **
        movf R_ANZOUT,W                ; get R_ANZOUT into W-Register
        subwf R_ANZOUT_OLD,W           ; W = R_ANZOUT_OLD - W
        btfss STATUS,C                 ; if C=0, -> R_ANZOUT_OLD < R_ANZOUT
         goto check_changes_rejected   ; ... then REJECTED

        movlw CONST_SHUTDOWN
        movwf R_CNT_SHUTDOWN

        return                         ; .. return (d.h. nderung OK)


check_changes_rejected ; nderungen nicht akzeptiert -> restore old values -> no change
        movf R_ONOFF1_OLD,W
        movwf R_ONOFF1
        movf R_ONOFF2_OLD,W
        movwf R_ONOFF2

        btfsc F_SHUTDOWN               ; Im ShutDown ..
         bcf R_ONOFF2,1                ; ... Ausgang 10 trotzdem abschalten
        btfsc F_STARTUP                ; Im ShutDown ..
         bsf R_ONOFF2,1                ; ... Ausgang 10 immer EIN
        btfsc F_RUNNING                ; Im Running ..
         bsf R_ONOFF2,1                ; ... Ausgang 10 immer EIN

        return                         ; 

pwm     ;**** LED steuern ****
        ; R_PWMVALUE wird bei jedem Step vergrssert.
        ; Bei berlauf wird auf FF gefreezed
        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

pwm_on
        ; ***** PWM-Time Einschaltpunkt *****
#ifndef test
        btfsc R_FADEIN1,0              ; = fade in LED1
         bsf LED_01                   ; LED_1 on
        btfsc R_FADEIN1,1              ; = fade in LED2
         bsf LED_02                    ; LED_2 on
        btfsc R_FADEIN1,2              ; = fade in LED3
         bsf LED_03                    ; LED_3 on
        btfsc R_FADEIN1,3              ; = fade in LED4
         bsf LED_04                    ; LED_4 on
#endif;  test
        btfsc R_FADEIN1,4              ; = fade in LED5
         bsf LED_05                    ; LED_5 on
        btfsc R_FADEIN1,5              ; = fade in LED6
         bsf LED_06                    ; LED_6 on
        btfsc R_FADEIN1,6              ; = fade in LED7
         bsf LED_07                    ; LED_7 on
        btfsc R_FADEIN1,7              ; = fade in LED8
         bsf LED_08                    ; LED_8 on
        btfsc R_FADEIN2,0              ; = fade in LED9
         bsf LED_09                    ; LED_9 on
        btfsc R_FADEIN2,1              ; = fade in LED10
         bsf LED_10                    ; LED_10 on

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto pwm_off                  ; ... then JUMP

#ifndef test
        btfsc R_FADEOUT1,0             ; = fade out LED1
         bcf LED_01                    ; LED_1 off
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bcf LED_02                    ; LED_2 off
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bcf LED_03                    ; LED_3 off
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bcf LED_04                    ; LED_4 off
#endif;  test
        btfsc R_FADEOUT1,4             ; = fade out LED5
         bcf LED_05                    ; LED_5 off
        btfsc R_FADEOUT1,5             ; = fade out LED6
         bcf LED_06                    ; LED_6 off
        btfsc R_FADEOUT1,6             ; = fade out LED7
         bcf LED_07                    ; LED_7 off
        btfsc R_FADEOUT1,7             ; = fade out LED8
         bcf LED_08                    ; LED_8 off
        btfsc R_FADEOUT2,0             ; = fade out LED9
         bcf LED_09                    ; LED_9 off
        btfsc R_FADEOUT2,1             ; = fade out LED10
         bcf LED_10                    ; LED_10 off

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

pwm_off ; ** warten auf PWM Off-Time **
        movf TMR1H,F
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto pwm_off                  ; ... then to PWM_OFF

        ;***** PWM-Time abgelaufen *****
#ifndef test
        btfsc R_FADEIN1,0              ; = fade in LED1
         bcf LED_01                    ; LED_1 off
        btfsc R_FADEIN1,1              ; = fade in LED2
         bcf LED_02                    ; LED_2 off
        btfsc R_FADEIN1,2              ; = fade in LED3
         bcf LED_03                    ; LED_3 off
        btfsc R_FADEIN1,3              ; = fade in LED4
         bcf LED_04                    ; LED_4 off
#endif;  test
        btfsc R_FADEIN1,4              ; = fade in LED5
         bcf LED_05                    ; LED_5 off
        btfsc R_FADEIN1,5              ; = fade in LED6
         bcf LED_06                    ; LED_6 off
        btfsc R_FADEIN1,6              ; = fade in LED7
         bcf LED_07                    ; LED_7 off
        btfsc R_FADEIN1,7              ; = fade in LED8
         bcf LED_08                    ; LED_8 off
        btfsc R_FADEIN2,0              ; = fade in LED9
         bcf LED_09                    ; LED_9 off
        btfsc R_FADEIN2,1              ; = fade in LED10
         bcf LED_10                    ; LED_10 off

#ifndef test
        btfsc R_FADEOUT1,0             ; = fade out LED1
         bsf LED_01                    ; LED_1 on
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bsf LED_02                    ; LED_2 on
        btfsc R_FADEOUT1,2             ; = fade out LED3
         bsf LED_03                    ; LED_3 on
        btfsc R_FADEOUT1,3             ; = fade out LED4
         bsf LED_04                    ; LED_4 on
#endif;  test
        btfsc R_FADEOUT1,4             ; = fade out LED5
         bsf LED_05                    ; LED_5 on
        btfsc R_FADEOUT1,5             ; = fade out LED6
         bsf LED_06                    ; LED_6 on
        btfsc R_FADEOUT1,6             ; = fade out LED7
         bsf LED_07                    ; LED_7 on
        btfsc R_FADEOUT1,7             ; = fade out LED8
         bsf LED_08                    ; LED_8 on
        btfsc R_FADEOUT2,0             ; = fade out LED9
         bsf LED_09                    ; LED_9 on
        btfsc R_FADEOUT2,1             ; = fade out LED10
         bsf LED_10                    ; LED_10 on

        return

;*******************************************
;*******************************************
;*******************************************
;*******************************************
;** Routine: shuffle_register             **
;** shuffles R_SHUFFLEx                   **
;*******************************************
;*******************************************
;*******************************************
;*******************************************
shuffle_register
        ; Register SHUFFLE1..8 neu mischen
        movf R_SHUFFLE1,W     
        movwf R_SHUFFLE                ; R_SHUFFLEx into R_SHUFFLE
         call shuffle                  ; shuffle R_SHUFFLE
        movwf R_SHUFFLE1               ; shuffled R_SHUFFLE into R_SHUFFLEx

        movf R_SHUFFLE2,W     
        movwf R_SHUFFLE                ; R_SHUFFLEx into R_SHUFFLE
         call shuffle                  ; shuffle R_SHUFFLE
        movwf R_SHUFFLE2               ; shuffled R_SHUFFLE into R_SHUFFLEx

        movf R_SHUFFLE3,W     
        movwf R_SHUFFLE                ; R_SHUFFLEx into R_SHUFFLE
        btfss R_SHUFFLE1,0             ; mehr Varianz durch Querverbindung
         call shuffle                  ; shuffle R_SHUFFLE
        movwf R_SHUFFLE3               ; shuffled R_SHUFFLE into R_SHUFFLEx

        movf R_SHUFFLE4,W     
        movwf R_SHUFFLE                ; R_SHUFFLEx into R_SHUFFLE
        btfss R_SHUFFLE1,1             ; mehr Varianz durch Querverbindung
         call shuffle                  ; shuffle R_SHUFFLE
        movwf R_SHUFFLE4               ; shuffled R_SHUFFLE into R_SHUFFLEx

       return

;*******************************************
;*******************************************
;*******************************************
;*******************************************
;** Routine: shuffle                      **
;** (A|B) & !(A&B)                        **
;** uses  R_SHUFF (Register to shuffle)   **
;** uses  R_TEMP  (temporary registere)   **
;** returns shuffled value in W_REG!      **
;*******************************************
;*******************************************
;*******************************************
;*******************************************
shuffle
        clrf R_TEMP
        movf R_SHUFFLE,F   ; check for
        btfsc STATUS,Z     ; .. ZERO
         decf R_SHUFFLE,F  ; if ZERO Reload with FF
        btfsc R_SHUFFLE,0  ; = A
         bsf R_TEMP,0       
        btfsc R_SHUFFLE,5  ; = B
         bsf R_TEMP,0      ; R_TEMP#0 = A|B
        movf R_TEMP,W      ; W#0 = A|B
        clrf R_TEMP
        btfss R_SHUFFLE,0  ; = A
         bsf R_TEMP,0       
        btfss R_SHUFFLE,5  ; = B
         bsf R_TEMP,0      ; R_TEMP#0 = (!A)|(!B) = !(A&B)
        andwf R_TEMP,F     ; R_TEMP#0 = (A|B) & !(A&B)
        RRF R_TEMP,F       ; R_TEMP#0 -> CARRY
        RRF R_SHUFFLE,W    ; CARRY in R_SHUFFLE von links hineinschieben
        return             ; und ins W_REG

;*******************************************
;** Routine: pwm_finish                   **
;** switch outputs to static              **
;*******************************************
pwm_finish
        ; PWM fertig, ab jetzt statische LEDs
        ; wenn FADE-IN gewesen, dann entsprechendes Port EIN
#ifndef test
        btfsc R_FADEIN1,0      ; = fade in LED1
         bsf LED_01            ; ROT   fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,1      ; = fade in LED2
         bsf LED_02            ; GELB  fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,2      ; = fade in LED3
         bsf LED_03            ; GRUEN fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,3      ; = fade in LED4
         bsf LED_04            ; ROT   fuer Nebenstrasse = EIN
#endif;  test
        btfsc R_FADEIN1,4      ; = fade in LED5
         bsf LED_05            ; GELB  fuer Nebenstrasse = EIN
        btfsc R_FADEIN1,5      ; = fade in LED6
         bsf LED_06            ; GRUEN fuer Nebenstrasse = EIN
        btfsc R_FADEIN1,6      ; = fade in LED7
         bsf LED_07            ; FUSSG-ROT   fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,7      ; = fade in LED8
         bsf LED_08            ; FUSSG-GRUEN fuer Hauptstrasse = EIN
        btfsc R_FADEIN2,0      ; = fade in LED9
         bsf LED_09            ; FUSSG-ROT   fuer Nebenstrasse = EIN
        btfsc R_FADEIN2,1      ; = fade in LED10
         bsf LED_10            ; FUSSG-GRUEN fuer Nebenstrasse = EIN
        clrf R_FADEIN1
        clrf R_FADEIN2

        ; wenn FADE-OUT gewesen, dann entsprechendes Port AUS
#ifndef test
        btfsc R_FADEOUT1,0     ; = fade out LED1
         bcf LED_01            ; ROT   fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,1     ; = fade out LED2
         bcf LED_02            ; GELB  fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,2     ; = fade out LED3
         bcf LED_03            ; GRUEN fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,3     ; = fade out LED4
         bcf LED_04            ; ROT   fuer Nebenstrasse = AUS
#endif;  test
        btfsc R_FADEOUT1,4     ; = fade out LED5
         bcf LED_05            ; GELB  fuer Nebenstrasse = AUS
        btfsc R_FADEOUT1,5     ; = fade out LED6
         bcf LED_06            ; GRUEN fuer Nebenstrasse = AUS
        btfsc R_FADEOUT1,6     ; = fade out LED7
         bcf LED_07            ; FUSSG-ROT   fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,7     ; = fade out LED8
         bcf LED_08            ; FUSSG-GRUEN fuer Hauptstrasse = US
        btfsc R_FADEOUT2,0     ; = fade out LED9
         bcf LED_09            ; FUSSG-ROT   fuer Nebenstrasse = AUS
        btfsc R_FADEOUT2,1     ; = fade out LED10
         bcf LED_10            ; FUSSG-GRUEN fuer Nebenstrasse = AUS
        clrf R_FADEOUT1
        clrf R_FADEOUT2

       ; *** save old values fr Restore (Verwerfen der neu berechneten Zustnde) und Vergleich mit aktuellem Zustand (Fade IN/OUT)
        movf R_ONOFF1,W
        movwf R_ONOFF1_OLD
        movf R_ONOFF2,W
        movwf R_ONOFF2_OLD

        ; wenn SHUTDOWN-Mode und keine LEDs mehr, dann SHUTDOWN Flag lschen
        movf R_ONOFF1,F                ; get R_ONOFF1 into W-Register
        btfss STATUS,Z                 ; wenn = ZERO
         return
        movf R_ONOFF2,F                ; get R_ONOFF1 into W-Register
        btfss STATUS,Z                 ; wenn = ZERO
         return
        bcf F_SHUTDOWN                ; Shutdown = beendet
        return