; ********************************************************
; ** Version: V3.49 vom 01/04/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2009-2011 Sven Brandt

; ********************************************************
; History:
; ********************************************************
; V3.49 - Sand-4: Relais sinnvoll angesteuert (d.h. Anpassung der vordefinierten Kurven)
; V3.38 - Sand-4: - Nach Ablauf einer Totzeit wird der erste Impuls 'number_first_imp' mal wiederholt
; V3.35 - Sand-4: - Fehlerbeseitigung
; V3.34 - Sand-4: - Taster-Erkennung hinzu
; V3.32 - Sand-4: - Korreturen beim Aufruf (value preload) von "stop" / "break"
;                 - Totzeit zwischen Einschalten Vcc und erstem Inpuls hinzu ("wait"-Flag)
;                 - IR sperren beim Timer lschen fr Inpulse-Generierung
; ********************************************************

;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: statemachine_sand_4 **
;** Blinksmdus                    **
;** needs <    us                 **
;***********************************
;***********************************
;***********************************
;***********************************
 org 0x03c0  ; ntig, da Call via PCL-Manipulation durchgefhrt wird
statemachine_sand_4
;   -----------------------------------
;   --- check 4ms Raster
;   -----------------------------------
        movlw d'02'                    ; Wert ( 4 ms x 5 States = 20 ms Cycle-Time)
        subwf TMR1H,W                  ; W = TMR1H - Wert
        btfss STATUS,C                 ; if C=0, -> TMR1H < Wert
         return                        ; ... then return

#ifdef test
  ; test-Routinen, ab hier Auslsung durch Watch-List
   clrf R_LOOPCOUNT
   btfsc R_LOOPCOUNT,7
    call gruen_2
   btfsc R_LOOPCOUNT,6
    call gruen_6
   btfsc R_LOOPCOUNT,5
    call gruen_3
   btfsc R_LOOPCOUNT,4
    call gruen_4
  ; ENDE Test-Routine, gesetzte BITs werden sowieso gelscht..
#endif

;   -----------------------------------
;   --- check switches
;   -----------------------------------
        call check_switches            ; Abfrage der Schalter
        movf R_FLAG_SWITCH,F
        btfsc STATUS,Z                 ; nur, wenn ein SW-Flag gesetzt 
         goto state_servo_00a 
        movlw hexline_switchFlags      ; access this line
        movwf R_TABLEREAD              ; .. in the table
        btfsc F_SW2_close              ; Abfrage SW2 close
         call TABLE_READ3
        btfsc F_SW2_open               ; Abfrage SW2 open
         call TABLE_READ4

        btfsc R_MODE,0                 ; wenn R_MODE#0 gesetzt, ist Taster SW1 auf Lern-Funktion gemappt
         goto state_servo_000          ; .. also hier keine Abfrage auf SW2 ->Jump

        btfsc F_SW1_close              ; Abfrage SW1 close  
         call TABLE_READ1
        btfsc F_SW1_open               ; ABfrage SW1 open
         call TABLE_READ2

state_servo_000
        movwf R_ACTIVATOR              ; Tabellenwert zwischenspeichern
        btfsc R_ACTIVATOR,0            ; je nach gesetztem Bit jetzt die
         call gruen_1                  ; .. entsprechende Aktion auslsen.
        btfsc R_ACTIVATOR,1
         call gruen_2
        btfsc R_ACTIVATOR,2
         call gruen_3
        btfsc R_ACTIVATOR,3
         call gruen_4
        btfsc R_ACTIVATOR,4
         call gruen_5
        btfsc R_ACTIVATOR,5
         call gruen_6
        btfsc R_ACTIVATOR,6
         call gruen_7
        btfsc R_ACTIVATOR,7
         call gruen_8

;   -----------------------------------
;   --- UND-Gatter-Status Reset
;   -----------------------------------
state_servo_00a
        bcf AND_S1
        bcf AND_S2
        bcf AND_S3
        bcf AND_S4

;   -----------------------------------
;   --- Verzweigung gem. R_LOOPCOUNT
;   -----------------------------------
        movlw b'00000111'              ; Zahlenbereich R_LOOPCOUNT = [0...7]
        andwf R_LOOPCOUNT,F            ; Zahlenbereich R_LOOPCOUNT = [0...7]
        movlw b'00000011'              ; load PCLATH with High Adress (Tabelle ab $0300)
        movwf PCLATH
        movf R_LOOPCOUNT,W
        addwf PCL,F                    ; Sprung abhngig von R_LOOPCOUNT
        goto state_servo_001        
        goto state_servo_002        
        goto state_servo_003        
        goto state_servo_004        
        goto state_servo_005        

;***********************************
; in jedem State:
; Lesezugriff auf
; - R_COUNT_0x
; - R_COUNT_STRETCH_0x
; - R_KURVE_NUMBER_0x
; - R_KURVE_STRETCH_0x
; - R_FAKTOR_SERVO_0x
; - R_OFFSET_SERVO_0x
; - R_SERVO_MAP_0x
; - SERVO_0x_USE_Cy        (Flag in R_SERVO_MAP_0x)
; - S0x_SERVO_FLAG_WAIT    (Flag in R_SERVO_FLAGS_0x)
; - S0x_SERVO_FLAG_V_OFF   (Flag in R_SERVO_FLAGS_0x)
; - S0x_SERVO_FLAG_I_OFF   (Flag in R_SERVO_FLAGS_0x)
; - SERVOx_ACTIVE          (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
; - SERVOx_LOOP            (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
;
; Schreibzugriff auf
; - R_COUNT_0x
; - R_COUNT_STRETCH_0x
; - S0x_SERVO_FLAG_WAIT    (Flag in R_SERVO_FLAGS_0x)
; - SERVOx_ACTIVE          (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
; - R_4094_VALUES          (nicht Servo-Spezifisch)
; - V1_ON                  (Flag in R_4094_VALUES, kein servo-spez. Reg)
; - R_PWMVALUE             (nicht Servo-Spezifisch)
; - R_FAKTOR               (nicht Servo-Spezifisch)
; - R_OFFSET               (nicht Servo-Spezifisch)
;
;***********************************
state_servo_001
        ; switch AND Gates        
        bsf AND_S1

        ; servo active?
        btfss SERVO1_ACTIVE
         goto state_servo_099

        ; Counter in Transfer-Register umladen
        movf R_COUNT_01,w              ; access this line
        movwf R_TABLEREAD              ; .. in the table
        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
        btfsc SERVO_01_USE_C1
         call TABLE_READ1                   ; get PWM-value from table1 if selected
        btfsc SERVO_01_USE_C2
         call TABLE_READ2                   ; get PWM-value from table2 if selected
        btfsc SERVO_01_USE_C3
         call TABLE_READ3                   ; get PWM-value from table3 if selected
        btfsc SERVO_01_USE_C4
         call TABLE_READ4                   ; get PWM-value from table4 if selected
        movwf R_PWMVALUE               ; .. into this register

        ; check if Relais 1 should be on (= bit #7 of table value)
        ; 4094                                     0  1  0  1
        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
        ;                                          ==========
        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
        comf R_SERVO_MAP_01,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
        movf R_SERVO_MAP_01,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt

        ; load transfer register 
        movf R_FAKTOR_SERVO_01,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_01,W
        movwf R_OFFSET
        ; calculate CCP values and generate impuls via ECCP
        btfss S01_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
         call generate_servo_impuls 

        ; Check, if there ist any stretching ongoing
        decfsz R_COUNT_STRETCH_01,F    ; wenn Stretch-Counter noch nicht auf ZERO
         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
        movf R_KURVE_STRETCH_01,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
        btfsc S01_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
        movwf R_COUNT_STRETCH_01       ; .. Counter laden

        ; read next table value (if there is a 'next' one)
        btfss S01_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
         incf R_COUNT_01,F             ; table pointer weiterzhlen
        bcf S01_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
        movf R_KURVE_NUMBER_01,W       ; Wert
        subwf R_COUNT_01,W             ; W = R_COUNT_01 - Wert
        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
         goto state_servo_099          ; ... then jump

        ; Kurve wurde durchlaufen
        btfsc R_COUNT_01,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
         goto servo1_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
        btfss SERVO1_LOOP
         goto servo1_shutdown 
        ; start again
        clrf R_COUNT_01  
        goto state_servo_099

servo1_shutdown ; stop servo-service
        bcf SERVO1_LOOP                ; ntig, wenn ber STOP hierhergekommen
        btfsc S01_SERVO_FLAG_V_OFF     ; switch off the voltage?
         bcf V1_ON

        btfsc S01_SERVO_FLAG_I_OFF     ; switch off the impuls?
         bcf SERVO1_ACTIVE             ; .. Servo nicht mehr aktiv

        btfss S01_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         decf R_COUNT_01,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        btfss S01_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         bsf R_COUNT_01,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_002
        ; switch AND Gates        
        bsf AND_S2
 
        ; servo active?
        btfss SERVO2_ACTIVE
         goto state_servo_099

        ; Counter in Transfer-Register umladen
        movf R_COUNT_02,w              ; access this line
        movwf R_TABLEREAD              ; .. in the table
        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
        btfsc SERVO_02_USE_C1
         call TABLE_READ1                   ; get PWM-value from table1 if selected
        btfsc SERVO_02_USE_C2
         call TABLE_READ2                   ; get PWM-value from table2 if selected
        btfsc SERVO_02_USE_C3
         call TABLE_READ3                   ; get PWM-value from table3 if selected
        btfsc SERVO_02_USE_C4
         call TABLE_READ4                   ; get PWM-value from table4 if selected
        movwf R_PWMVALUE               ; .. into this register

        ; check if Relais 2 should be on (= bit #7 of table value)
        ; 4094                                     0  1  0  1
        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
        ;                                          ==========
        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
        comf R_SERVO_MAP_02,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
        movf R_SERVO_MAP_02,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt

        ; load transfer register 
        movf R_FAKTOR_SERVO_02,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_02,W
        movwf R_OFFSET
        ; calculate CCP values and generate impuls via ECCP
        btfss S02_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
         call generate_servo_impuls 

        ; Check, if there ist any stretching ongoing
        decfsz R_COUNT_STRETCH_02,F    ; wenn Stretch-Counter noch nicht auf ZERO
         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
        movf R_KURVE_STRETCH_02,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
        btfsc S02_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
        movwf R_COUNT_STRETCH_02       ; .. Counter laden

        ; read next table value (if there is a 'next' one)
        btfss S02_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
         incf R_COUNT_02,F             ; table pointer weiterzhlen
        bcf S02_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
        movf R_KURVE_NUMBER_02,W       ; Wert
        subwf R_COUNT_02,W             ; W = R_COUNT_02 - Wert
        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
         goto state_servo_099          ; ... then jump

        ; Kurve wurde durchlaufen
        btfsc R_COUNT_02,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
         goto servo2_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
        btfss SERVO2_LOOP
         goto servo2_shutdown 
        ; start again
        clrf R_COUNT_02  
        goto state_servo_099

servo2_shutdown ; stop servo-service 
        bcf SERVO2_LOOP                ; ntig, wenn ber STOP hierhergekommen
        btfsc S02_SERVO_FLAG_V_OFF     ; switch off the voltage?
         bcf V2_ON

        btfsc S02_SERVO_FLAG_I_OFF     ; switch off the impuls?
         bcf SERVO2_ACTIVE             ; .. Servo nicht mehr aktiv

        btfss S02_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         decf R_COUNT_02,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        btfss S02_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         bsf R_COUNT_02,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_003
        ; switch AND Gates        
        bsf AND_S3

        ; servo active?
        btfss SERVO3_ACTIVE
         goto state_servo_099

        ; Counter in Transfer-Register umladen
        movf R_COUNT_03,w              ; access this line
        movwf R_TABLEREAD              ; .. in the table
        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
        btfsc SERVO_03_USE_C1
         call TABLE_READ1                   ; get PWM-value from table1 if selected
        btfsc SERVO_03_USE_C2
         call TABLE_READ2                   ; get PWM-value from table2 if selected
        btfsc SERVO_03_USE_C3
         call TABLE_READ3                   ; get PWM-value from table3 if selected
        btfsc SERVO_03_USE_C4
         call TABLE_READ4                   ; get PWM-value from table4 if selected
        movwf R_PWMVALUE               ; .. into this register

        ; check if Relais 3 should be on (= bit #7 of table value)
        ; 4094                                     0  1  0  1
        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
        ;                                          ==========
        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
        comf R_SERVO_MAP_03,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
        movf R_SERVO_MAP_03,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt

        ; load transfer register 
        movf R_FAKTOR_SERVO_03,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_03,W
        movwf R_OFFSET
        ; calculate CCP values and generate impuls via ECCP
        btfss S03_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
         call generate_servo_impuls 

        ; Check, if there ist any stretching ongoing
        decfsz R_COUNT_STRETCH_03,F    ; wenn Stretch-Counter noch nicht auf ZERO
         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
        movf R_KURVE_STRETCH_03,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
        btfsc S03_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
        movwf R_COUNT_STRETCH_03       ; .. Counter laden

        ; read next table value (if there is a 'next' one)
        btfss S03_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
         incf R_COUNT_03,F             ; table pointer weiterzhlen
        bcf S03_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
        movf R_KURVE_NUMBER_03,W       ; Wert
        subwf R_COUNT_03,W             ; W = R_COUNT_03 - Wert
        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
         goto state_servo_099          ; ... then jump

        ; Kurve wurde durchlaufen
        btfsc R_COUNT_03,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
         goto servo3_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
        btfss SERVO3_LOOP
         goto servo3_shutdown 
        ; start again
        clrf R_COUNT_03  
        goto state_servo_099

servo3_shutdown ; stop servo-service 
        bcf SERVO3_LOOP                ; ntig, wenn ber STOP hierhergekommen
        btfsc S03_SERVO_FLAG_V_OFF     ; switch off the voltage?
         bcf V3_ON

        btfsc S03_SERVO_FLAG_I_OFF     ; switch off the impuls?
         bcf SERVO3_ACTIVE             ; .. Servo nicht mehr aktiv

        btfss S03_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         decf R_COUNT_03,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        btfss S03_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         bsf R_COUNT_03,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_004
        ; switch AND Gates        
        bsf AND_S4

        ; servo active?
        btfss SERVO4_ACTIVE
         goto state_servo_099

        ; Counter in Transfer-Register umladen
        movf R_COUNT_04,w              ; access this line
        movwf R_TABLEREAD              ; .. in the table
        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
        btfsc SERVO_04_USE_C1
         call TABLE_READ1                   ; get PWM-value from table1 if selected
        btfsc SERVO_04_USE_C2
         call TABLE_READ2                   ; get PWM-value from table2 if selected
        btfsc SERVO_04_USE_C3
         call TABLE_READ3                   ; get PWM-value from table3 if selected
        btfsc SERVO_04_USE_C4
         call TABLE_READ4                   ; get PWM-value from table4 if selected
        movwf R_PWMVALUE               ; .. into this register
 
        ; check if Relais 4 should be on (= bit #7 of table value)
        ; 4094                                     0  1  0  1
        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
        ;                                          ==========
        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
        comf R_SERVO_MAP_04,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
        movf R_SERVO_MAP_04,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt

        ; load transfer register 
        movf R_FAKTOR_SERVO_04,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_04,W
        movwf R_OFFSET
        ; calculate CCP values and generate impuls via ECCP
        btfss S04_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
         call generate_servo_impuls 

        ; Check, if there ist any stretching ongoing
        decfsz R_COUNT_STRETCH_04,F    ; wenn Stretch-Counter noch nicht auf ZERO
         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
        movf R_KURVE_STRETCH_04,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
        btfsc S04_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
        movwf R_COUNT_STRETCH_04       ; .. Counter laden

        ; read next table value (if there is a 'next' one)
        btfss S04_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
         incf R_COUNT_04,F             ; table pointer weiterzhlen
        bcf S04_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
        movf R_KURVE_NUMBER_04,W       ; Wert
        subwf R_COUNT_04,W             ; W = R_COUNT_04 - Wert
        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
         goto state_servo_099          ; ... then jump

        ; Kurve wurde durchlaufen
        btfsc R_COUNT_04,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
         goto servo4_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
        btfss SERVO4_LOOP
         goto servo4_shutdown 
        ; start again
        clrf R_COUNT_04  
        goto state_servo_099

servo4_shutdown ; stop servo-service 
        bcf SERVO1_LOOP                ; ntig, wenn ber STOP hierhergekommen
        btfsc S04_SERVO_FLAG_V_OFF     ; switch off the voltage?
         bcf V4_ON

        btfsc S04_SERVO_FLAG_I_OFF     ; switch off the impuls?
         bcf SERVO4_ACTIVE             ; .. Servo nicht mehr aktiv

        btfss S04_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         decf R_COUNT_04,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        btfss S04_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         bsf R_COUNT_04,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_005 ; Time-Slot for (fake-)servo 5: now ist ist ime for the 4094 and LED-action...
        call update_4094              ; ... then call ausfhren

        ; Slow Down LED Signalisation
        decfsz R_LED_SLOW,F
         goto state_servo_099          ; .. springen
        movlw d'10'
        movwf R_LED_SLOW

        btfss LED1
         goto state_servo_005_is_off

state_servo_005_is_on
        bcf LED1
        goto state_servo_099           ; .. springen

state_servo_005_is_off
        movf R_LEDSIGNAL,F             ; wenn R_LEDSIGNAL = ZERO...
        btfsc STATUS,Z                 ; .. dann 
         goto state_servo_099          ; .. springen

        bsf LED1
        bcf STATUS,C
        rrf R_LEDSIGNAL,F
        goto state_servo_099

;***********************************
state_servo_099 ; R_LOOPCOUNT hochzhlen und berlauf abfangen
        incf R_LOOPCOUNT,F             ; select next Servo 
        movlw d'5'                     ; check
         subwf R_LOOPCOUNT,W           ; .. if counter > 5
        btfsc STATUS,C                 ; .. then reset  
         clrf R_LOOPCOUNT              ; .. to ZERO

        clrf TMR1H                     ; TIMER1 H-Reg reset

        return





; ********************************************************
; **  Tabellen-Zugriff                                  **
; ********************************************************
TABLE_READ1
        movlw b'00000101'           ; load PCLATH with High Adress (Tabelle ab $0500)
        movwf PCLATH
        bcf R_TABLEREAD,7           ; unterer Adress-Bereich (bis $0580)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL
        
TABLE_READ2
        movlw b'00000101'           ; load PCLATH with High Adress (Tabelle ab $0500)
        movwf PCLATH
        bsf R_TABLEREAD,7           ; Oberer ADress-Bereich (ab $0580)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL

TABLE_READ3
        movlw b'00000110'           ; load PCLATH with High Adress (Tabelle ab $0600)
        movwf PCLATH
        bcf R_TABLEREAD,7           ; unterer Adress-Bereich (bis $0680)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL

TABLE_READ4
        movlw b'00000110'           ; load PCLATH with High Adress (Tabelle ab $0600)
        movwf PCLATH
        bsf R_TABLEREAD,7           ; Oberer ADress-Bereich (ab $0680)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL




; *************************************************************
; **  Tabelle fuer Kurve 1 / Adresse 1 / Adresse 5 / Servo 1 **
; *************************************************************
 org 0x0500
        retlw d'0'   ; #0
        retlw d'1'   ; #1
        retlw d'3'   ; #2
        retlw d'4'   ; #3
        retlw d'5'   ; #4
        retlw d'6'   ; #5
        retlw d'8'   ; #6
        retlw d'9'   ; #7
        retlw d'10'   ; #8
        retlw d'11'   ; #9
        retlw d'13'   ; #10
        retlw d'14'   ; #11
        retlw d'15'   ; #12
        retlw d'17'   ; #13
        retlw d'18'   ; #14
        retlw d'19'   ; #15
        retlw d'20'   ; #16
        retlw d'22'   ; #17
        retlw d'23'   ; #18
        retlw d'24'   ; #19
        retlw d'25'   ; #20
        retlw d'27'   ; #21
        retlw d'28'   ; #22
        retlw d'29'   ; #23
        retlw d'30'   ; #24
        retlw d'32'   ; #25
        retlw d'33'   ; #26
        retlw d'34'   ; #27
        retlw d'36'   ; #28
        retlw d'37'   ; #29
        retlw d'38'   ; #30
        retlw d'39'   ; #31
        retlw d'41'   ; #32
        retlw d'42'   ; #33
        retlw d'43'   ; #34
        retlw d'44'   ; #35
        retlw d'46'   ; #36
        retlw d'47'   ; #37
        retlw d'48'   ; #38
        retlw d'50'   ; #39
        retlw d'51'   ; #40
        retlw d'52'   ; #41
        retlw d'53'   ; #42
        retlw d'55'   ; #43
        retlw d'56'   ; #44
        retlw d'57'   ; #45
        retlw d'58'   ; #46
        retlw d'60'   ; #47
        retlw d'61'   ; #48
        retlw d'62'   ; #49
        retlw d'64'   ; #50
        retlw d'193'   ; #51
        retlw d'194'   ; #52
        retlw d'195'   ; #53
        retlw d'197'   ; #54
        retlw d'198'   ; #55
        retlw d'199'   ; #56
        retlw d'200'   ; #57
        retlw d'202'   ; #58
        retlw d'203'   ; #59
        retlw d'204'   ; #60
        retlw d'205'   ; #61
        retlw d'207'   ; #62
        retlw d'208'   ; #63
        retlw d'209'   ; #64
        retlw d'211'   ; #65
        retlw d'212'   ; #66
        retlw d'213'   ; #67
        retlw d'214'   ; #68
        retlw d'216'   ; #69
        retlw d'217'   ; #70
        retlw d'218'   ; #71
        retlw d'219'   ; #72
        retlw d'221'   ; #73
        retlw d'222'   ; #74
        retlw d'223'   ; #75
        retlw d'225'   ; #76
        retlw d'226'   ; #77
        retlw d'227'   ; #78
        retlw d'228'   ; #79
        retlw d'230'   ; #80
        retlw d'231'   ; #81
        retlw d'232'   ; #82
        retlw d'233'   ; #83
        retlw d'235'   ; #84
        retlw d'236'   ; #85
        retlw d'237'   ; #86
        retlw d'238'   ; #87
        retlw d'240'   ; #88
        retlw d'241'   ; #89
        retlw d'242'   ; #90
        retlw d'244'   ; #91
        retlw d'245'   ; #92
        retlw d'246'   ; #93
        retlw d'247'   ; #94
        retlw d'249'   ; #95
        retlw d'250'   ; #96
        retlw d'251'   ; #97
        retlw d'252'   ; #98
        retlw d'254'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw b'00000001'    ; #103 = Flags 1 Flags for Adress #1 (this is not a curve parameter!!)
            ; #0 Adress 1 use Servo 1
            ; #1 Adress 1 use Servo 2
            ; #2 Adress 1 use Servo 3
            ; #3 Adress 1 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00010001'    ; #104 = Flags 2 Flags for Adress #1 (this is not a curve parameter!!)
            ; #0 Adress 1 use Relais 1
            ; #1 Adress 1 use Relais 2
            ; #2 Adress 1 use Relais 3
            ; #3 Adress 1 use Relais 4
            ; #4 Adress 1 use Kurve 1
            ; #5 Adress 1 use Kurve 2
            ; #6 Adress 1 use Kurve 3
            ; #7 Adress 1 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw b'00000100'     ; #106 = Flags 1 Flags for Adress #5 (this is not a curve parameter!!)
            ; #0 Adress 5 use Servo 1
            ; #1 Adress 5 use Servo 2
            ; #2 Adress 5 use Servo 3
            ; #3 Adress 5 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00010100'     ; #107 = Flags 2 Flags for Adress #5 (this is not a curve parameter!!)
            ; #0 Adress 5 use Relais 1
            ; #1 Adress 5 use Relais 2
            ; #2 Adress 5 use Relais 3
            ; #3 Adress 5 use Relais 4
            ; #4 Adress 5 use Kurve 1
            ; #5 Adress 5 use Kurve 2
            ; #6 Adress 5 use Kurve 3
            ; #7 Adress 5 use Kurve 4
        retlw d'0'            ; #108 = not used
        
        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'01'   ; #110 Info switch 1 close
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 2 / Adresse 2 / Adresse 6 / Servo 2 **
; *************************************************************
 org 0x0580
        retlw d'255'   ; #0
        retlw d'254'   ; #1
        retlw d'252'   ; #2
        retlw d'251'   ; #3
        retlw d'250'   ; #4
        retlw d'249'   ; #5
        retlw d'247'   ; #6
        retlw d'246'   ; #7
        retlw d'245'   ; #8
        retlw d'244'   ; #9
        retlw d'242'   ; #10
        retlw d'241'   ; #11
        retlw d'240'   ; #12
        retlw d'238'   ; #13
        retlw d'237'   ; #14
        retlw d'236'   ; #15
        retlw d'235'   ; #16
        retlw d'233'   ; #17
        retlw d'232'   ; #18
        retlw d'231'   ; #19
        retlw d'230'   ; #20
        retlw d'228'   ; #21
        retlw d'227'   ; #22
        retlw d'226'   ; #23
        retlw d'225'   ; #24
        retlw d'223'   ; #25
        retlw d'222'   ; #26
        retlw d'221'   ; #27
        retlw d'219'   ; #28
        retlw d'218'   ; #29
        retlw d'217'   ; #30
        retlw d'216'   ; #31
        retlw d'214'   ; #32
        retlw d'213'   ; #33
        retlw d'212'   ; #34
        retlw d'211'   ; #35
        retlw d'209'   ; #36
        retlw d'208'   ; #37
        retlw d'207'   ; #38
        retlw d'205'   ; #39
        retlw d'204'   ; #40
        retlw d'203'   ; #41
        retlw d'202'   ; #42
        retlw d'200'   ; #43
        retlw d'199'   ; #44
        retlw d'198'   ; #45
        retlw d'197'   ; #46
        retlw d'195'   ; #47
        retlw d'194'   ; #48
        retlw d'193'   ; #49
        retlw d'63'   ; #50
        retlw d'62'   ; #51
        retlw d'61'   ; #52
        retlw d'60'   ; #53
        retlw d'58'   ; #54
        retlw d'57'   ; #55
        retlw d'56'   ; #56
        retlw d'55'   ; #57
        retlw d'53'   ; #58
        retlw d'52'   ; #59
        retlw d'51'   ; #60
        retlw d'50'   ; #61
        retlw d'48'   ; #62
        retlw d'47'   ; #63
        retlw d'46'   ; #64
        retlw d'44'   ; #65
        retlw d'43'   ; #66
        retlw d'42'   ; #67
        retlw d'41'   ; #68
        retlw d'39'   ; #69
        retlw d'38'   ; #70
        retlw d'37'   ; #71
        retlw d'36'   ; #72
        retlw d'34'   ; #73
        retlw d'33'   ; #74
        retlw d'32'   ; #75
        retlw d'30'   ; #76
        retlw d'29'   ; #77
        retlw d'28'   ; #78
        retlw d'27'   ; #79
        retlw d'25'   ; #80
        retlw d'24'   ; #81
        retlw d'23'   ; #82
        retlw d'22'   ; #83
        retlw d'20'   ; #84
        retlw d'19'   ; #85
        retlw d'18'   ; #86
        retlw d'17'   ; #87
        retlw d'15'   ; #88
        retlw d'14'   ; #89
        retlw d'13'   ; #90
        retlw d'11'   ; #91
        retlw d'10'   ; #92
        retlw d'9'   ; #93
        retlw d'8'   ; #94
        retlw d'6'   ; #95
        retlw d'5'   ; #96
        retlw d'4'   ; #97
        retlw d'3'   ; #98
        retlw d'1'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw b'00000001'    ; #103 = Flags 1 Flags for Adress #2 (this is not a curve parameter!!)
            ; #0 Adress 2 use Servo 1
            ; #1 Adress 2 use Servo 2
            ; #2 Adress 2 use Servo 3
            ; #3 Adress 2 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00100001'     ; #104 = Flags 2 Flags for Adress #2 (this is not a curve parameter!!)
            ; #0 Adress 2 use Relais 1
            ; #1 Adress 2 use Relais 2
            ; #2 Adress 2 use Relais 3
            ; #3 Adress 2 use Relais 4
            ; #4 Adress 2 use Kurve 1
            ; #5 Adress 2 use Kurve 2
            ; #6 Adress 2 use Kurve 3
            ; #7 Adress 2 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw b'00000100'     ; #106 = Flags 1 Flags for Adress #6 (this is not a curve parameter!!)
            ; #0 Adress 6 use Servo 1
            ; #1 Adress 6 use Servo 2
            ; #2 Adress 6 use Servo 3
            ; #3 Adress 6 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00100100'     ; #107 = Flags 2 Flags for Adress #6 (this is not a curve parameter!!)
            ; #0 Adress 6 use Relais 1
            ; #1 Adress 6 use Relais 2
            ; #2 Adress 6 use Relais 3
            ; #3 Adress 6 use Relais 4
            ; #4 Adress 6 use Kurve 1
            ; #5 Adress 6 use Kurve 2
            ; #6 Adress 6 use Kurve 3
            ; #7 Adress 6 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'10'   ; #110 Info switch 1 open
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 3 / Adresse 3 / Adresse 7 / Servo 3 **
; *************************************************************
 org 0x0600
        retlw d'128'   ; #0
        retlw d'19'   ; #1
        retlw d'30'   ; #2
        retlw d'38'   ; #3
        retlw d'44'   ; #4
        retlw d'49'   ; #5
        retlw d'54'   ; #6
        retlw d'57'   ; #7
        retlw d'61'   ; #8
        retlw d'64'   ; #9
        retlw d'66'   ; #10
        retlw d'69'   ; #11
        retlw d'71'   ; #12
        retlw d'73'   ; #13
        retlw d'75'   ; #14
        retlw d'76'   ; #15
        retlw d'78'   ; #16
        retlw d'80'   ; #17
        retlw d'81'   ; #18
        retlw d'83'   ; #19
        retlw d'84'   ; #20
        retlw d'85'   ; #21
        retlw d'86'   ; #22
        retlw d'88'   ; #23
        retlw d'89'   ; #24
        retlw d'90'   ; #25
        retlw d'91'   ; #26
        retlw d'92'   ; #27
        retlw d'93'   ; #28
        retlw d'94'   ; #29
        retlw d'95'   ; #30
        retlw d'96'   ; #31
        retlw d'96'   ; #32
        retlw d'97'   ; #33
        retlw d'98'   ; #34
        retlw d'99'   ; #35
        retlw d'100'   ; #36
        retlw d'100'   ; #37
        retlw d'101'   ; #38
        retlw d'102'   ; #39
        retlw d'102'   ; #40
        retlw d'103'   ; #41
        retlw d'104'   ; #42
        retlw d'104'   ; #43
        retlw d'105'   ; #44
        retlw d'106'   ; #45
        retlw d'106'   ; #46
        retlw d'107'   ; #47
        retlw d'107'   ; #48
        retlw d'108'   ; #49
        retlw d'108'   ; #50
        retlw d'109'   ; #51
        retlw d'109'   ; #52
        retlw d'110'   ; #53
        retlw d'111'   ; #54
        retlw d'111'   ; #55
        retlw d'111'   ; #56
        retlw d'112'   ; #57
        retlw d'112'   ; #58
        retlw d'113'   ; #59
        retlw d'113'   ; #60
        retlw d'114'   ; #61
        retlw d'114'   ; #62
        retlw d'115'   ; #63
        retlw d'115'   ; #64
        retlw d'116'   ; #65
        retlw d'116'   ; #66
        retlw d'116'   ; #67
        retlw d'117'   ; #68
        retlw d'117'   ; #69
        retlw d'118'   ; #70
        retlw d'118'   ; #71
        retlw d'118'   ; #72
        retlw d'119'   ; #73
        retlw d'119'   ; #74
        retlw d'119'   ; #75
        retlw d'120'   ; #76
        retlw d'120'   ; #77
        retlw d'120'   ; #78
        retlw d'121'   ; #79
        retlw d'121'   ; #80
        retlw d'122'   ; #81
        retlw d'122'   ; #82
        retlw d'122'   ; #83
        retlw d'123'   ; #84
        retlw d'123'   ; #85
        retlw d'123'   ; #86
        retlw d'123'   ; #87
        retlw d'124'   ; #88
        retlw d'124'   ; #89
        retlw d'124'   ; #90
        retlw d'125'   ; #91
        retlw d'125'   ; #92
        retlw d'125'   ; #93
        retlw d'126'   ; #94
        retlw d'126'   ; #95
        retlw d'126'   ; #96
        retlw d'126'   ; #97
        retlw d'127'   ; #98
        retlw d'127'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw b'00000010'    ; #103 = Flags 1 Flags for Adress #3 (this is not a curve parameter!!)
            ; #0 Adress 3 use Servo 1
            ; #1 Adress 3 use Servo 2
            ; #2 Adress 3 use Servo 3
            ; #3 Adress 3 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00010010'     ; #104 = Flags 2 Flags for Adress #3 (this is not a curve parameter!!)
            ; #0 Adress 3 use Relais 1
            ; #1 Adress 3 use Relais 2
            ; #2 Adress 3 use Relais 3
            ; #3 Adress 3 use Relais 4
            ; #4 Adress 3 use Kurve 1
            ; #5 Adress 3 use Kurve 2
            ; #6 Adress 3 use Kurve 3
            ; #7 Adress 3 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw b'00001000'     ; #106 = Flags 1 Flags for Adress #7 (this is not a curve parameter!!)
            ; #0 Adress 7 use Servo 1
            ; #1 Adress 7 use Servo 2
            ; #2 Adress 7 use Servo 3
            ; #3 Adress 7 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00011000'     ; #107 = Flags 2 Flags for Adress #7 (this is not a curve parameter!!)
            ; #0 Adress 7 use Relais 1
            ; #1 Adress 7 use Relais 2
            ; #2 Adress 7 use Relais 3
            ; #3 Adress 7 use Relais 4
            ; #4 Adress 7 use Kurve 1
            ; #5 Adress 7 use Kurve 2
            ; #6 Adress 7 use Kurve 3
            ; #7 Adress 7 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'02'   ; #110 Info switch 2 close
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 4 / Adresse 4 / Adresse 8 / Servo 4 **
; *************************************************************
 org 0x0680
        retlw d'128'   ; #0
        retlw d'0'   ; #1
        retlw d'0'   ; #2
        retlw d'1'   ; #3
        retlw d'2'   ; #4
        retlw d'3'   ; #5
        retlw d'4'   ; #6
        retlw d'6'   ; #7
        retlw d'8'   ; #8
        retlw d'10'   ; #9
        retlw d'12'   ; #10
        retlw d'14'   ; #11
        retlw d'17'   ; #12
        retlw d'20'   ; #13
        retlw d'23'   ; #14
        retlw d'26'   ; #15
        retlw d'29'   ; #16
        retlw d'33'   ; #17
        retlw d'36'   ; #18
        retlw d'40'   ; #19
        retlw d'44'   ; #20
        retlw d'47'   ; #21
        retlw d'51'   ; #22
        retlw d'55'   ; #23
        retlw d'59'   ; #24
        retlw d'63'   ; #25
        retlw d'67'   ; #26
        retlw d'71'   ; #27
        retlw d'75'   ; #28
        retlw d'79'   ; #29
        retlw d'82'   ; #30
        retlw d'86'   ; #31
        retlw d'90'   ; #32
        retlw d'93'   ; #33
        retlw d'97'   ; #34
        retlw d'100'   ; #35
        retlw d'103'   ; #36
        retlw d'106'   ; #37
        retlw d'109'   ; #38
        retlw d'112'   ; #39
        retlw d'114'   ; #40
        retlw d'116'   ; #41
        retlw d'118'   ; #42
        retlw d'120'   ; #43
        retlw d'122'   ; #44
        retlw d'123'   ; #45
        retlw d'124'   ; #46
        retlw d'125'   ; #47
        retlw d'126'   ; #48
        retlw d'126'   ; #49
        retlw d'126'   ; #50
        retlw d'126'   ; #51
        retlw d'126'   ; #52
        retlw d'125'   ; #53
        retlw d'124'   ; #54
        retlw d'123'   ; #55
        retlw d'122'   ; #56
        retlw d'120'   ; #57
        retlw d'118'   ; #58
        retlw d'116'   ; #59
        retlw d'114'   ; #60
        retlw d'112'   ; #61
        retlw d'109'   ; #62
        retlw d'106'   ; #63
        retlw d'103'   ; #64
        retlw d'100'   ; #65
        retlw d'97'   ; #66
        retlw d'93'   ; #67
        retlw d'90'   ; #68
        retlw d'86'   ; #69
        retlw d'82'   ; #70
        retlw d'79'   ; #71
        retlw d'75'   ; #72
        retlw d'71'   ; #73
        retlw d'67'   ; #74
        retlw d'63'   ; #75
        retlw d'59'   ; #76
        retlw d'55'   ; #77
        retlw d'51'   ; #78
        retlw d'47'   ; #79
        retlw d'44'   ; #80
        retlw d'40'   ; #81
        retlw d'36'   ; #82
        retlw d'33'   ; #83
        retlw d'29'   ; #84
        retlw d'26'   ; #85
        retlw d'23'   ; #86
        retlw d'20'   ; #87
        retlw d'17'   ; #88
        retlw d'14'   ; #89
        retlw d'12'   ; #90
        retlw d'10'   ; #91
        retlw d'8'   ; #92
        retlw d'6'   ; #93
        retlw d'4'   ; #94
        retlw d'3'   ; #95
        retlw d'2'   ; #96
        retlw d'1'   ; #97
        retlw d'0'   ; #98
        retlw d'0'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw b'00000010'    ; #103 = Flags 1 Flags for Adress #4 (this is not a curve parameter!!)
            ; #0 Adress 4 use Servo 1
            ; #1 Adress 4 use Servo 2
            ; #2 Adress 4 use Servo 3
            ; #3 Adress 4 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00100010'     ; #104 = Flags 2 Flags for Adress #4 (this is not a curve parameter!!)
            ; #0 Adress 4 use Relais 1
            ; #1 Adress 4 use Relais 2
            ; #2 Adress 4 use Relais 3
            ; #3 Adress 4 use Relais 4
            ; #4 Adress 4 use Kurve 1
            ; #5 Adress 4 use Kurve 2
            ; #6 Adress 4 use Kurve 3
            ; #7 Adress 4 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw b'00001000'     ; #106 = Flags 1 Flags for Adress #8 (this is not a curve parameter!!)
            ; #0 Adress 8 use Servo 1
            ; #1 Adress 8 use Servo 2
            ; #2 Adress 8 use Servo 3
            ; #3 Adress 8 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw b'00101000'     ; #107 = Flags 2 Flags for Adress #8 (this is not a curve parameter!!)
            ; #0 Adress 8 use Relais 1
            ; #1 Adress 8 use Relais 2
            ; #2 Adress 8 use Relais 3
            ; #3 Adress 8 use Relais 4
            ; #4 Adress 8 use Kurve 1
            ; #5 Adress 8 use Kurve 2
            ; #6 Adress 8 use Kurve 3
            ; #7 Adress 8 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'20'   ; #110 Info switch 2 open
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 
