; ********************************************************
; ** Version: V3.58 vom 17/03/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2009-2012 Sven Brandt

; ********************************************************
; History:
; ********************************************************
; V3.58 - 16.03.2013 Sand-Ei: Korrektur Taster-Eingnge
; V3.54 - 27.03.2012- SandEi: X7 kann per Schwellwert geschaltet werde
; V3.47 - sand-ei: wh. der letzten Impulse aus der Tabelle, damit Servo wirklich in die Endlage kommt.
; V3.43 - Sand-ei: - Neuerstellung aus state_sand_4.asm V3.38
; V3.38 - Sand-4: - Nach Ablauf einer Totzeit wird der erste Impuls 'number_first_imp' mal wiederholt
; V3.35 - Sand-4: - Fehlerbeseitigung
; V3.34 - Sand-4: - Taster-Erkennung hinzu
; V3.32 - Sand-4: - Korreturen beim Aufruf (value preload) von "stop" / "break"
;                 - Totzeit zwischen Einschalten Vcc und erstem Inpuls hinzu ("wait"-Flag)
;                 - IR sperren beim Timer lschen fr Inpulse-Generierung
; ********************************************************

;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: statemachine_sand_4 **
;** Blinksmdus                    **
;** needs <    us                 **
;***********************************
;***********************************
;***********************************
;***********************************
 org 0x03c0  ; ntig, da Call via PCL-Manipulation durchgefhrt wird
statemachine_sand_ei
;   -----------------------------------
;   --- check 4ms Raster
;   -----------------------------------
        movlw d'02'                    ; Wert ( 4 ms x 5 States = 20 ms Cycle-Time)
        subwf TMR1H,W                  ; W = TMR1H - Wert
        btfss STATUS,C                 ; if C=0, -> TMR1H < Wert
         return                        ; ... then return

;   -----------------------------------
;   --- check switches
;   -----------------------------------
        btfss F_use_sw                 ; Flag -> read switches ??
         goto state_servo_00a          ; .. skip switch check
        call check_switches_extern     ; Abfrage der Schalter 3+4

        ; Switch 3
        movf R_FLAG_SWITCH2,W
        andlw b'00000011'
        btfsc STATUS,Z                 ; nur, wenn ein SW-Flag SW3 gesetzt 
         goto state_servo_00_sw4 

        movlw hexline_switchFlags      ; access this line
        movwf R_TABLEREAD              ; .. in the table
        btfsc F_SW3_close              ; Abfrage SW3 close  
         call TABLE_READ1
        btfsc F_SW3_open               ; ABfrage SW3 open
         call TABLE_READ2

        movwf R_ACTIVATOR              ; Tabellenwert zwischenspeichern
        btfsc R_ACTIVATOR,0            ; je nach gesetztem Bit jetzt die
         call gruen_1                  ; .. entsprechende Aktion auslsen.
        btfsc R_ACTIVATOR,1
         call gruen_2
        btfsc R_ACTIVATOR,2
         call gruen_3
        btfsc R_ACTIVATOR,3
         call gruen_4

state_servo_00_sw4
        ; Switch 4
        movf R_FLAG_SWITCH2,W
        andlw b'00001100'
        btfsc STATUS,Z                 ; nur, wenn ein SW-Flag SW4 gesetzt 
         goto state_servo_00a  

        movlw hexline_switchFlags      ; access this line
        movwf R_TABLEREAD              ; .. in the table
        btfsc F_SW4_close              ; Abfrage SW4 close
         call TABLE_READ3
        btfsc F_SW4_open               ; Abfrage SW4 open
         call TABLE_READ4

        movwf R_ACTIVATOR              ; Tabellenwert zwischenspeichern
        btfsc R_ACTIVATOR,0            ; je nach gesetztem Bit jetzt die
         call gruen_1                  ; .. entsprechende Aktion auslsen.
        btfsc R_ACTIVATOR,1
         call gruen_2
        btfsc R_ACTIVATOR,2
         call gruen_3
        btfsc R_ACTIVATOR,3
         call gruen_4

state_servo_00a
;   -----------------------------------
;   --- Verzweigung gem. R_LOOPCOUNT
;   -----------------------------------
        movlw b'00000111'              ; Zahlenbereich R_LOOPCOUNT = [0...7]
        andwf R_LOOPCOUNT,F            ; Zahlenbereich R_LOOPCOUNT = [0...7]
        movlw b'00000011'              ; load PCLATH with High Adress (Tabelle ab $0400)
        movwf PCLATH
        movf R_LOOPCOUNT,W
        addwf PCL,F                    ; Sprung abhngig von R_LOOPCOUNT
        goto state_servo_001        
        goto state_servo_002        
        goto state_servo_003        
        goto state_servo_004        
        goto state_servo_005        

;***********************************
; in jedem State:
; Lesezugriff auf
; - R_COUNT_0x
; - R_COUNT_STRETCH_0x
; - R_KURVE_NUMBER_0x
; - R_KURVE_STRETCH_0x
; - R_FAKTOR_SERVO_0x
; - R_OFFSET_SERVO_0x
; - R_SERVO_MAP_0x
; - SERVO_0x_USE_Cy        (Flag in R_SERVO_MAP_0x)
; - S0x_SERVO_FLAG_WAIT    (Flag in R_SERVO_FLAGS_0x)
; - S0x_SERVO_FLAG_V_OFF   (Flag in R_SERVO_FLAGS_0x)
; - S0x_SERVO_FLAG_I_OFF   (Flag in R_SERVO_FLAGS_0x)
; - SERVOx_ACTIVE          (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
; - SERVOx_LOOP            (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
;
; Schreibzugriff auf
; - R_COUNT_0x
; - R_COUNT_STRETCH_0x
; - S0x_SERVO_FLAG_WAIT    (Flag in R_SERVO_FLAGS_0x)
; - SERVOx_ACTIVE          (Flag in R_SERVO_ACTIVE, kein servo-spez. Reg)
; - R_4094_VALUES          (nicht Servo-Spezifisch)
; - V1_ON                  (Flag in R_4094_VALUES, kein servo-spez. Reg)
; - R_PWMVALUE             (nicht Servo-Spezifisch)
; - R_FAKTOR               (nicht Servo-Spezifisch)
; - R_OFFSET               (nicht Servo-Spezifisch)
;
;***********************************
state_servo_001
        ; switch AND Gates        
        ;bsf AND_S1

        ; servo active?
        btfss SERVO1_ACTIVE
         goto state_servo_099

        ; Counter in Transfer-Register umladen
        movf R_COUNT_01,w              ; access this line
        movwf R_TABLEREAD              ; .. in the table
        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
        btfsc SERVO_01_USE_C1
         call TABLE_READ1                   ; get PWM-value from table1 if selected
        btfsc SERVO_01_USE_C2
         call TABLE_READ2                   ; get PWM-value from table2 if selected
        btfsc SERVO_01_USE_C3
         call TABLE_READ3                   ; get PWM-value from table3 if selected
        btfsc SERVO_01_USE_C4
         call TABLE_READ4                   ; get PWM-value from table4 if selected
        movwf R_PWMVALUE               ; .. into this register

        ; check if out_x7 should be on
        movf R_X7_SWITCHVALUE,W
        subwf R_PWMVALUE,W
        btfsc STATUS,C
         bsf OUT_X7 
        btfss STATUS,C
         bcf OUT_X7

        ; load transfer register 
        movf R_FAKTOR_SERVO_01,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_01,W
        movwf R_OFFSET
        ; calculate CCP values and generate impuls via ECCP
        btfss S01_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
         call generate_servo_impuls 

        ; Check, if there ist any stretching ongoing
        decfsz R_COUNT_STRETCH_01,F    ; wenn Stretch-Counter noch nicht auf ZERO
         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
        movf R_KURVE_STRETCH_01,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
        btfsc S01_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
        movwf R_COUNT_STRETCH_01       ; .. Counter laden

        ; read next table value (if there is a 'next' one)
        btfss S01_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
         incf R_COUNT_01,F             ; table pointer weiterzhlen
        bcf S01_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
        movf R_KURVE_NUMBER_01,W       ; Wert
        subwf R_COUNT_01,W             ; W = R_COUNT_01 - Wert
        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
         goto state_servo_099          ; ... then jump

        ; Kurve wurde durchlaufen
        btfsc R_COUNT_01,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
         goto servo1_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
        btfss SERVO1_LOOP
         goto servo1_shutdown 
        ; start again
        clrf R_COUNT_01  
        goto state_servo_099

servo1_shutdown
        btfsc F_FINISH_PHASE_1         ; wenn finish phase gestartet wird (wh. des letzten Impulses, damit Servo in Endlage kommt) ...
         goto servo1_shutdown_2        ; .. dann
        bsf F_FINISH_PHASE_1           ; .. flag setzten
        movlw d'9'                     ; .. Counter laden mit 9 (d.h. letzer Wert wird 10x gesendet)
        movwf R_COUNT_STRETCH_01 
        decf R_COUNT_01,F              ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        goto state_servo_099           ; .. warten

servo1_shutdown_2 ; stop servo-service
        bcf SERVO1_LOOP                ; ntig, wenn ber STOP hierhergekommen
        btfsc S01_SERVO_FLAG_V_OFF     ; switch off the voltage?
         bcf V_SERVO

        btfsc S01_SERVO_FLAG_I_OFF     ; switch off the impuls?
         bcf SERVO1_ACTIVE             ; .. Servo nicht mehr aktiv

        btfss S01_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         decf R_COUNT_01,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
        btfss S01_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
         bsf R_COUNT_01,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_002
        ; switch AND Gates        
;        bsf AND_S2
; 
;        ; servo active?
;        btfss SERVO2_ACTIVE
;         goto state_servo_099
;
;        ; Counter in Transfer-Register umladen
;        movf R_COUNT_02,w              ; access this line
;        movwf R_TABLEREAD              ; .. in the table
;        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
;        btfsc SERVO_02_USE_C1
;         call TABLE_READ1                   ; get PWM-value from table1 if selected
;        btfsc SERVO_02_USE_C2
;         call TABLE_READ2                   ; get PWM-value from table2 if selected
;        btfsc SERVO_02_USE_C3
;         call TABLE_READ3                   ; get PWM-value from table3 if selected
;        btfsc SERVO_02_USE_C4
;         call TABLE_READ4                   ; get PWM-value from table4 if selected
;        movwf R_PWMVALUE               ; .. into this register
;
;        ; check if Relais 2 should be on (= bit #7 of table value)
;        ; 4094                                     0  1  0  1
;        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
;        ;                                          ==========
;        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
;        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
;        comf R_SERVO_MAP_02,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
;        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
;        movf R_SERVO_MAP_02,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
;        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
;         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt
;
;        ; load transfer register 
;        movf R_FAKTOR_SERVO_02,W
;        movwf R_FAKTOR
;        movf R_OFFSET_SERVO_02,W
;        movwf R_OFFSET
;        ; calculate CCP values and generate impuls via ECCP
;        btfss S02_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
;         call generate_servo_impuls 
;
;        ; Check, if there ist any stretching ongoing
;        decfsz R_COUNT_STRETCH_02,F    ; wenn Stretch-Counter noch nicht auf ZERO
;         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
;        movf R_KURVE_STRETCH_02,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
;        btfsc S02_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
;         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
;        movwf R_COUNT_STRETCH_02       ; .. Counter laden
;
;        ; read next table value (if there is a 'next' one)
;        btfss S02_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
;         incf R_COUNT_02,F             ; table pointer weiterzhlen
;        bcf S02_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
;        movf R_KURVE_NUMBER_02,W       ; Wert
;        subwf R_COUNT_02,W             ; W = R_COUNT_02 - Wert
;        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
;         goto state_servo_099          ; ... then jump
;
;        ; Kurve wurde durchlaufen
;        btfsc R_COUNT_02,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
;         goto servo2_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
;        btfss SERVO2_LOOP
;         goto servo2_shutdown 
;        ; start again
;        clrf R_COUNT_02  
;        goto state_servo_099
;
;servo2_shutdown ; stop servo-service 
;        bcf SERVO2_LOOP                ; ntig, wenn ber STOP hierhergekommen
;        btfsc S02_SERVO_FLAG_V_OFF     ; switch off the voltage?
;         bcf V2_ON
;
;        btfsc S02_SERVO_FLAG_I_OFF     ; switch off the impuls?
;         bcf SERVO2_ACTIVE             ; .. Servo nicht mehr aktiv
;
;        btfss S02_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         decf R_COUNT_02,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
;        btfss S02_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         bsf R_COUNT_02,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_003
        ; switch AND Gates        
;        bsf AND_S3
;
;        ; servo active?
;        btfss SERVO3_ACTIVE
;         goto state_servo_099
;
;        ; Counter in Transfer-Register umladen
;        movf R_COUNT_03,w              ; access this line
;        movwf R_TABLEREAD              ; .. in the table
;        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
;        btfsc SERVO_03_USE_C1
;         call TABLE_READ1                   ; get PWM-value from table1 if selected
;        btfsc SERVO_03_USE_C2
;         call TABLE_READ2                   ; get PWM-value from table2 if selected
;        btfsc SERVO_03_USE_C3
;         call TABLE_READ3                   ; get PWM-value from table3 if selected
;        btfsc SERVO_03_USE_C4
;         call TABLE_READ4                   ; get PWM-value from table4 if selected
;        movwf R_PWMVALUE               ; .. into this register
;
;        ; check if Relais 3 should be on (= bit #7 of table value)
;        ; 4094                                     0  1  0  1
;        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
;        ;                                          ==========
;        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
;        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
;        comf R_SERVO_MAP_03,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
;        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
;        movf R_SERVO_MAP_03,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
;        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
;         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt
;
;        ; load transfer register 
;        movf R_FAKTOR_SERVO_03,W
;        movwf R_FAKTOR
;        movf R_OFFSET_SERVO_03,W
;        movwf R_OFFSET
;        ; calculate CCP values and generate impuls via ECCP
;        btfss S03_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
;         call generate_servo_impuls 
;
;        ; Check, if there ist any stretching ongoing
;        decfsz R_COUNT_STRETCH_03,F    ; wenn Stretch-Counter noch nicht auf ZERO
;         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
;        movf R_KURVE_STRETCH_03,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
;        btfsc S03_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
;         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
;        movwf R_COUNT_STRETCH_03       ; .. Counter laden
;
;        ; read next table value (if there is a 'next' one)
;        btfss S03_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
;         incf R_COUNT_03,F             ; table pointer weiterzhlen
;        bcf S03_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
;        movf R_KURVE_NUMBER_03,W       ; Wert
;        subwf R_COUNT_03,W             ; W = R_COUNT_03 - Wert
;        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
;         goto state_servo_099          ; ... then jump
;
;        ; Kurve wurde durchlaufen
;        btfsc R_COUNT_03,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
;         goto servo3_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
;        btfss SERVO3_LOOP
;         goto servo3_shutdown 
;        ; start again
;        clrf R_COUNT_03  
;        goto state_servo_099
;
;servo3_shutdown ; stop servo-service 
;        bcf SERVO3_LOOP                ; ntig, wenn ber STOP hierhergekommen
;        btfsc S03_SERVO_FLAG_V_OFF     ; switch off the voltage?
;         bcf V3_ON
;
;        btfsc S03_SERVO_FLAG_I_OFF     ; switch off the impuls?
;         bcf SERVO3_ACTIVE             ; .. Servo nicht mehr aktiv
;
;        btfss S03_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         decf R_COUNT_03,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
;        btfss S03_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         bsf R_COUNT_03,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_004
        ; switch AND Gates        
;        bsf AND_S4
;
;        ; servo active?
;        btfss SERVO4_ACTIVE
;         goto state_servo_099
;
;        ; Counter in Transfer-Register umladen
;        movf R_COUNT_04,w              ; access this line
;        movwf R_TABLEREAD              ; .. in the table
;        ; Tabellenwert laden, abhngig davon, welche Kurve gefahren werden soll
;        btfsc SERVO_04_USE_C1
;         call TABLE_READ1                   ; get PWM-value from table1 if selected
;        btfsc SERVO_04_USE_C2
;         call TABLE_READ2                   ; get PWM-value from table2 if selected
;        btfsc SERVO_04_USE_C3
;         call TABLE_READ3                   ; get PWM-value from table3 if selected
;        btfsc SERVO_04_USE_C4
;         call TABLE_READ4                   ; get PWM-value from table4 if selected
;        movwf R_PWMVALUE               ; .. into this register
; 
;        ; check if Relais 4 should be on (= bit #7 of table value)
;        ; 4094                                     0  1  0  1
;        ; _MAP (0 = no Change for this Relais)     0  0  1  1  
;        ;                                          ==========
;        ; @ #7 (d.h. Relais soll AN  sein) =>4094  0  1  1  1  ==> OR
;        ; @!#7 (d.h. Relais soll AUS sein) =>4094  0  1  0  0  ==> Invertiertes AND 
;        comf R_SERVO_MAP_04,w          ; Map-Register invertiert in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        iorlw b'11110000'              ; #4..#7 unschdlich machen (steht jetzt '1' drinn)
;        andwf R_4094_VALUES,f          ; ..ber AND wird jetzt das Flag des gemappten Relais gelscht.
;        movf R_SERVO_MAP_04,w          ; Map-Register in W laden. hier stehen in den #0..#3 die Relais-Mappings drinn
;        andlw b'00001111'              ; #4..#7 unschdlich machen (steht jetzt '0' drinn)
;        btfsc R_PWMVALUE,7             ; if #7 is set, then.. 
;         iorwf R_4094_VALUES,f         ; .. wird ber IOR das Flag fr gemapptes Relais gesetzt
;
;        ; load transfer register 
;        movf R_FAKTOR_SERVO_04,W
;        movwf R_FAKTOR
;        movf R_OFFSET_SERVO_04,W
;        movwf R_OFFSET
;        ; calculate CCP values and generate impuls via ECCP
;        btfss S04_SERVO_FLAG_WAIT      ; kein Inpuls wenn "WAIT" aktiv ist
;         call generate_servo_impuls 
;
;        ; Check, if there ist any stretching ongoing
;        decfsz R_COUNT_STRETCH_04,F    ; wenn Stretch-Counter noch nicht auf ZERO
;         goto state_servo_099          ; .. dann nochmal mit den alten Werten (kein Tab-Zugriff)
;        movf R_KURVE_STRETCH_04,W      ; neu V 3.38: Stretch-Wert in den dafr ntigen
;        btfsc S04_SERVO_FLAG_WAIT      ; neu V 3.38: wenn erster Impuls nach Totzeit
;         movlw number_first_imp        ; .. dann wird der erste Wert number_first_imp x wiederholt
;        movwf R_COUNT_STRETCH_04       ; .. Counter laden
;
;        ; read next table value (if there is a 'next' one)
;        btfss S04_SERVO_FLAG_WAIT      ; Counter nicht hochzhlen, wenn WAIT aktiv ist
;         incf R_COUNT_04,F             ; table pointer weiterzhlen
;        bcf S04_SERVO_FLAG_WAIT        ; "wait" nur beim Start aktiv (Totzeit)
;        movf R_KURVE_NUMBER_04,W       ; Wert
;        subwf R_COUNT_04,W             ; W = R_COUNT_04 - Wert
;        btfss STATUS,C                 ; if C=0, -> R_COUNT < R_KURVE_NUMBER
;         goto state_servo_099          ; ... then jump
;
;        ; Kurve wurde durchlaufen
;        btfsc R_COUNT_04,7             ; wenn #7 nicht gesetzt ist (ntzlich @Break & Impulse halten)
;         goto servo4_shutdown          ; .. dann weiter wie bei shutdown (Abschalten V & Inpulse, wenn gewnschtr)
;        btfss SERVO4_LOOP
;         goto servo4_shutdown 
;        ; start again
;        clrf R_COUNT_04  
;        goto state_servo_099
;
;servo4_shutdown ; stop servo-service 
;        bcf SERVO1_LOOP                ; ntig, wenn ber STOP hierhergekommen
;        btfsc S04_SERVO_FLAG_V_OFF     ; switch off the voltage?
;         bcf V4_ON
;
;        btfsc S04_SERVO_FLAG_I_OFF     ; switch off the impuls?
;         bcf SERVO4_ACTIVE             ; .. Servo nicht mehr aktiv
;
;        btfss S04_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         decf R_COUNT_04,F             ; .. table pointer -1, dann wird im nchsten Time-Slot der letzte Wert genommen
;        btfss S04_SERVO_FLAG_I_OFF     ; do not switch off the impuls?
;         bsf R_COUNT_04,7              ; .. FREEZE-Flag setzten

        goto state_servo_099

;***********************************
state_servo_005 ; Time-Slot for (fake-)servo 5: now ist is time for the 4094 and LED-action...
      ;  call update_4094              ; ... then call ausfhren

        ; Slow Down LED Signalisation
        decfsz R_LED_SLOW,F
         goto state_servo_099          ; .. springen
        movlw d'10'
        movwf R_LED_SLOW

        btfss LED1
         goto state_servo_005_is_off

state_servo_005_is_on
        bcf LED1
        goto state_servo_099           ; .. springen

state_servo_005_is_off
        movf R_LEDSIGNAL,F             ; wenn R_LEDSIGNAL = ZERO...
        btfsc STATUS,Z                 ; .. dann 
         goto state_servo_099          ; .. springen

        bsf LED1
        bcf STATUS,C
        rrf R_LEDSIGNAL,F
        goto state_servo_099

;***********************************
state_servo_099 ; R_LOOPCOUNT hochzhlen und berlauf abfangen
        incf R_LOOPCOUNT,F             ; select next Servo 
        movlw d'5'                     ; check
         subwf R_LOOPCOUNT,W           ; .. if counter > 5
        btfsc STATUS,C                 ; .. then reset  
         clrf R_LOOPCOUNT              ; .. to ZERO

        clrf TMR1H                     ; TIMER1 H-Reg reset

        return





; ********************************************************
; **  Tabellen-Zugriff                                  **
; ********************************************************
TABLE_READ1
        movlw b'00000101'           ; load PCLATH with High Adress (Tabelle ab $0500)
        movwf PCLATH
        bcf R_TABLEREAD,7           ; unterer Adress-Bereich (bis $0580)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL
        
TABLE_READ2
        movlw b'00000101'           ; load PCLATH with High Adress (Tabelle ab $0500)
        movwf PCLATH
        bsf R_TABLEREAD,7           ; Oberer ADress-Bereich (ab $0580)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL

TABLE_READ3
        movlw b'00000110'           ; load PCLATH with High Adress (Tabelle ab $0600)
        movwf PCLATH
        bcf R_TABLEREAD,7           ; unterer Adress-Bereich (bis $0680)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL

TABLE_READ4
        movlw b'00000110'           ; load PCLATH with High Adress (Tabelle ab $0600)
        movwf PCLATH
        bsf R_TABLEREAD,7           ; Oberer ADress-Bereich (ab $0680)
        movf R_TABLEREAD,W          ; Tabelle Zeile R_'TABLEREAD' 
        movwf PCL




; *************************************************************
; **  Tabelle fuer Kurve 1 / Adresse 1 / Adresse 5 / Servo 1 **
; *************************************************************
 org 0x0500
        retlw d'0'   ; #0
        retlw d'1'   ; #1
        retlw d'3'   ; #2
        retlw d'4'   ; #3
        retlw d'5'   ; #4
        retlw d'6'   ; #5
        retlw d'8'   ; #6
        retlw d'9'   ; #7
        retlw d'10'   ; #8
        retlw d'11'   ; #9
        retlw d'13'   ; #10
        retlw d'14'   ; #11
        retlw d'15'   ; #12
        retlw d'17'   ; #13
        retlw d'18'   ; #14
        retlw d'19'   ; #15
        retlw d'20'   ; #16
        retlw d'22'   ; #17
        retlw d'23'   ; #18
        retlw d'24'   ; #19
        retlw d'25'   ; #20
        retlw d'27'   ; #21
        retlw d'28'   ; #22
        retlw d'29'   ; #23
        retlw d'30'   ; #24
        retlw d'32'   ; #25
        retlw d'33'   ; #26
        retlw d'34'   ; #27
        retlw d'36'   ; #28
        retlw d'37'   ; #29
        retlw d'38'   ; #30
        retlw d'39'   ; #31
        retlw d'41'   ; #32
        retlw d'42'   ; #33
        retlw d'43'   ; #34
        retlw d'44'   ; #35
        retlw d'46'   ; #36
        retlw d'47'   ; #37
        retlw d'48'   ; #38
        retlw d'50'   ; #39
        retlw d'51'   ; #40
        retlw d'52'   ; #41
        retlw d'53'   ; #42
        retlw d'55'   ; #43
        retlw d'56'   ; #44
        retlw d'57'   ; #45
        retlw d'58'   ; #46
        retlw d'60'   ; #47
        retlw d'61'   ; #48
        retlw d'62'   ; #49
        retlw d'64'   ; #50
        retlw d'65'   ; #51
        retlw d'66'   ; #52
        retlw d'67'   ; #53
        retlw d'69'   ; #54
        retlw d'70'   ; #55
        retlw d'71'   ; #56
        retlw d'72'   ; #57
        retlw d'74'   ; #58
        retlw d'75'   ; #59
        retlw d'76'   ; #60
        retlw d'77'   ; #61
        retlw d'79'   ; #62
        retlw d'80'   ; #63
        retlw d'81'   ; #64
        retlw d'83'   ; #65
        retlw d'84'   ; #66
        retlw d'85'   ; #67
        retlw d'86'   ; #68
        retlw d'88'   ; #69
        retlw d'89'   ; #70
        retlw d'90'   ; #71
        retlw d'91'   ; #72
        retlw d'93'   ; #73
        retlw d'94'   ; #74
        retlw d'95'   ; #75
        retlw d'97'   ; #76
        retlw d'98'   ; #77
        retlw d'99'   ; #78
        retlw d'100'   ; #79
        retlw d'102'   ; #80
        retlw d'103'   ; #81
        retlw d'104'   ; #82
        retlw d'105'   ; #83
        retlw d'107'   ; #84
        retlw d'108'   ; #85
        retlw d'109'   ; #86
        retlw d'110'   ; #87
        retlw d'112'   ; #88
        retlw d'113'   ; #89
        retlw d'114'   ; #90
        retlw d'116'   ; #91
        retlw d'117'   ; #92
        retlw d'118'   ; #93
        retlw d'120'   ; #94
        retlw d'122'   ; #95
        retlw d'123'   ; #96
        retlw d'125'   ; #97
        retlw d'127'   ; #98
        retlw d'127'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw h'01'           ; #103 = Flags 1 Flags for Adress #1 (this is not a curve parameter!!)
            ; #0 Adress 1 use Servo 1
            ; #1 Adress 1 use Servo 2
            ; #2 Adress 1 use Servo 3
            ; #3 Adress 1 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'11'           ; #104 = Flags 2 Flags for Adress #1 (this is not a curve parameter!!)
            ; #0 Adress 1 use Relais 1
            ; #1 Adress 1 use Relais 2
            ; #2 Adress 1 use Relais 3
            ; #3 Adress 1 use Relais 4
            ; #4 Adress 1 use Kurve 1
            ; #5 Adress 1 use Kurve 2
            ; #6 Adress 1 use Kurve 3
            ; #7 Adress 1 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw h'01'           ; #106 = Flags 1 Flags for Adress #5 (this is not a curve parameter!!)
            ; #0 Adress 5 use Servo 1
            ; #1 Adress 5 use Servo 2
            ; #2 Adress 5 use Servo 3
            ; #3 Adress 5 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'11'           ; #107 = Flags 2 Flags for Adress #5 (this is not a curve parameter!!)
            ; #0 Adress 5 use Relais 1
            ; #1 Adress 5 use Relais 2
            ; #2 Adress 5 use Relais 3
            ; #3 Adress 5 use Relais 4
            ; #4 Adress 5 use Kurve 1
            ; #5 Adress 5 use Kurve 2
            ; #6 Adress 5 use Kurve 3
            ; #7 Adress 5 use Kurve 4
        retlw d'0'            ; #108 = not used
        
        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'01'   ; #110 Info switch 3 close
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 2 / Adresse 2 / Adresse 6 / Servo 2 **
; *************************************************************
 org 0x0580
        retlw d'127'   ; #0
        retlw d'126'   ; #1
        retlw d'124'   ; #2
        retlw d'123'   ; #3
        retlw d'122'   ; #4
        retlw d'121'   ; #5
        retlw d'119'   ; #6
        retlw d'118'   ; #7
        retlw d'117'   ; #8
        retlw d'116'   ; #9
        retlw d'114'   ; #10
        retlw d'113'   ; #11
        retlw d'112'   ; #12
        retlw d'110'   ; #13
        retlw d'109'   ; #14
        retlw d'108'   ; #15
        retlw d'107'   ; #16
        retlw d'105'   ; #17
        retlw d'104'   ; #18
        retlw d'103'   ; #19
        retlw d'102'   ; #20
        retlw d'100'   ; #21
        retlw d'99'   ; #22
        retlw d'98'   ; #23
        retlw d'97'   ; #24
        retlw d'95'   ; #25
        retlw d'94'   ; #26
        retlw d'93'   ; #27
        retlw d'91'   ; #28
        retlw d'90'   ; #29
        retlw d'89'   ; #30
        retlw d'88'   ; #31
        retlw d'86'   ; #32
        retlw d'85'   ; #33
        retlw d'84'   ; #34
        retlw d'83'   ; #35
        retlw d'81'   ; #36
        retlw d'80'   ; #37
        retlw d'79'   ; #38
        retlw d'77'   ; #39
        retlw d'76'   ; #40
        retlw d'75'   ; #41
        retlw d'74'   ; #42
        retlw d'72'   ; #43
        retlw d'71'   ; #44
        retlw d'70'   ; #45
        retlw d'69'   ; #46
        retlw d'67'   ; #47
        retlw d'66'   ; #48
        retlw d'65'   ; #49
        retlw d'63'   ; #50
        retlw d'62'   ; #51
        retlw d'61'   ; #52
        retlw d'60'   ; #53
        retlw d'58'   ; #54
        retlw d'57'   ; #55
        retlw d'56'   ; #56
        retlw d'55'   ; #57
        retlw d'53'   ; #58
        retlw d'52'   ; #59
        retlw d'51'   ; #60
        retlw d'50'   ; #61
        retlw d'48'   ; #62
        retlw d'47'   ; #63
        retlw d'46'   ; #64
        retlw d'44'   ; #65
        retlw d'43'   ; #66
        retlw d'42'   ; #67
        retlw d'41'   ; #68
        retlw d'39'   ; #69
        retlw d'38'   ; #70
        retlw d'37'   ; #71
        retlw d'36'   ; #72
        retlw d'34'   ; #73
        retlw d'33'   ; #74
        retlw d'32'   ; #75
        retlw d'30'   ; #76
        retlw d'29'   ; #77
        retlw d'28'   ; #78
        retlw d'27'   ; #79
        retlw d'25'   ; #80
        retlw d'24'   ; #81
        retlw d'23'   ; #82
        retlw d'22'   ; #83
        retlw d'20'   ; #84
        retlw d'19'   ; #85
        retlw d'18'   ; #86
        retlw d'17'   ; #87
        retlw d'15'   ; #88
        retlw d'14'   ; #89
        retlw d'13'   ; #90
        retlw d'11'   ; #91
        retlw d'10'   ; #92
        retlw d'9'   ; #93
        retlw d'6'   ; #94
        retlw d'5'   ; #95
        retlw d'4'   ; #96
        retlw d'3'   ; #97
        retlw d'1'   ; #98
        retlw d'0'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw h'01'           ; #103 = Flags 1 Flags for Adress #2 (this is not a curve parameter!!)
            ; #0 Adress 2 use Servo 1
            ; #1 Adress 2 use Servo 2
            ; #2 Adress 2 use Servo 3
            ; #3 Adress 2 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'22'           ; #104 = Flags 2 Flags for Adress #2 (this is not a curve parameter!!)
            ; #0 Adress 2 use Relais 1
            ; #1 Adress 2 use Relais 2
            ; #2 Adress 2 use Relais 3
            ; #3 Adress 2 use Relais 4
            ; #4 Adress 2 use Kurve 1
            ; #5 Adress 2 use Kurve 2
            ; #6 Adress 2 use Kurve 3
            ; #7 Adress 2 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw h'01'           ; #106 = Flags 1 Flags for Adress #6 (this is not a curve parameter!!)
            ; #0 Adress 6 use Servo 1
            ; #1 Adress 6 use Servo 2
            ; #2 Adress 6 use Servo 3
            ; #3 Adress 6 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'22'           ; #107 = Flags 2 Flags for Adress #6 (this is not a curve parameter!!)
            ; #0 Adress 6 use Relais 1
            ; #1 Adress 6 use Relais 2
            ; #2 Adress 6 use Relais 3
            ; #3 Adress 6 use Relais 4
            ; #4 Adress 6 use Kurve 1
            ; #5 Adress 6 use Kurve 2
            ; #6 Adress 6 use Kurve 3
            ; #7 Adress 6 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'00'   ; #110 Info switch 3 open
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 3 / Adresse 3 / Adresse 7 / Servo 3 **
; *************************************************************
 org 0x0600
        retlw d'0'   ; #0
        retlw d'0'   ; #1
        retlw d'0'   ; #2
        retlw d'1'   ; #3
        retlw d'2'   ; #4
        retlw d'3'   ; #5
        retlw d'4'   ; #6
        retlw d'6'   ; #7
        retlw d'8'   ; #8
        retlw d'10'   ; #9
        retlw d'12'   ; #10
        retlw d'14'   ; #11
        retlw d'17'   ; #12
        retlw d'20'   ; #13
        retlw d'23'   ; #14
        retlw d'26'   ; #15
        retlw d'29'   ; #16
        retlw d'33'   ; #17
        retlw d'36'   ; #18
        retlw d'40'   ; #19
        retlw d'44'   ; #20
        retlw d'47'   ; #21
        retlw d'51'   ; #22
        retlw d'55'   ; #23
        retlw d'59'   ; #24
        retlw d'63'   ; #25
        retlw d'67'   ; #26
        retlw d'71'   ; #27
        retlw d'75'   ; #28
        retlw d'79'   ; #29
        retlw d'82'   ; #30
        retlw d'86'   ; #31
        retlw d'90'   ; #32
        retlw d'93'   ; #33
        retlw d'97'   ; #34
        retlw d'100'   ; #35
        retlw d'103'   ; #36
        retlw d'106'   ; #37
        retlw d'109'   ; #38
        retlw d'112'   ; #39
        retlw d'114'   ; #40
        retlw d'116'   ; #41
        retlw d'118'   ; #42
        retlw d'120'   ; #43
        retlw d'122'   ; #44
        retlw d'123'   ; #45
        retlw d'124'   ; #46
        retlw d'125'   ; #47
        retlw d'126'   ; #48
        retlw d'126'   ; #49
        retlw d'126'   ; #50
        retlw d'126'   ; #51
        retlw d'126'   ; #52
        retlw d'125'   ; #53
        retlw d'124'   ; #54
        retlw d'123'   ; #55
        retlw d'122'   ; #56
        retlw d'120'   ; #57
        retlw d'118'   ; #58
        retlw d'116'   ; #59
        retlw d'114'   ; #60
        retlw d'112'   ; #61
        retlw d'109'   ; #62
        retlw d'106'   ; #63
        retlw d'103'   ; #64
        retlw d'100'   ; #65
        retlw d'97'   ; #66
        retlw d'93'   ; #67
        retlw d'90'   ; #68
        retlw d'86'   ; #69
        retlw d'82'   ; #70
        retlw d'79'   ; #71
        retlw d'75'   ; #72
        retlw d'71'   ; #73
        retlw d'67'   ; #74
        retlw d'63'   ; #75
        retlw d'59'   ; #76
        retlw d'55'   ; #77
        retlw d'51'   ; #78
        retlw d'47'   ; #79
        retlw d'44'   ; #80
        retlw d'40'   ; #81
        retlw d'36'   ; #82
        retlw d'33'   ; #83
        retlw d'29'   ; #84
        retlw d'26'   ; #85
        retlw d'23'   ; #86
        retlw d'20'   ; #87
        retlw d'17'   ; #88
        retlw d'14'   ; #89
        retlw d'12'   ; #90
        retlw d'10'   ; #91
        retlw d'8'   ; #92
        retlw d'6'   ; #93
        retlw d'4'   ; #94
        retlw d'3'   ; #95
        retlw d'2'   ; #96
        retlw d'1'   ; #97
        retlw d'0'   ; #98
        retlw d'0'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw h'11'           ; #103 = Flags 1 Flags for Adress #3 (this is not a curve parameter!!)
            ; #0 Adress 3 use Servo 1
            ; #1 Adress 3 use Servo 2
            ; #2 Adress 3 use Servo 3
            ; #3 Adress 3 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'44'           ; #104 = Flags 2 Flags for Adress #3 (this is not a curve parameter!!)
            ; #0 Adress 3 use Relais 1
            ; #1 Adress 3 use Relais 2
            ; #2 Adress 3 use Relais 3
            ; #3 Adress 3 use Relais 4
            ; #4 Adress 3 use Kurve 1
            ; #5 Adress 3 use Kurve 2
            ; #6 Adress 3 use Kurve 3
            ; #7 Adress 3 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw h'01'           ; #106 = Flags 1 Flags for Adress #7 (this is not a curve parameter!!)
            ; #0 Adress 7 use Servo 1
            ; #1 Adress 7 use Servo 2
            ; #2 Adress 7 use Servo 3
            ; #3 Adress 7 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'44'           ; #107 = Flags 2 Flags for Adress #7 (this is not a curve parameter!!)
            ; #0 Adress 7 use Relais 1
            ; #1 Adress 7 use Relais 2
            ; #2 Adress 7 use Relais 3
            ; #3 Adress 7 use Relais 4
            ; #4 Adress 7 use Kurve 1
            ; #5 Adress 7 use Kurve 2
            ; #6 Adress 7 use Kurve 3
            ; #7 Adress 7 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'02'   ; #110 Info switch 4 close
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 

; *************************************************************
; **  Tabelle fuer Kurve 4 / Adresse 4 / Adresse 8 / Servo 4 **
; *************************************************************
 org 0x0680
        retlw d'128'   ; #0
        retlw d'0'   ; #1
        retlw d'0'   ; #2
        retlw d'1'   ; #3
        retlw d'2'   ; #4
        retlw d'3'   ; #5
        retlw d'4'   ; #6
        retlw d'6'   ; #7
        retlw d'8'   ; #8
        retlw d'10'   ; #9
        retlw d'12'   ; #10
        retlw d'14'   ; #11
        retlw d'17'   ; #12
        retlw d'20'   ; #13
        retlw d'23'   ; #14
        retlw d'26'   ; #15
        retlw d'29'   ; #16
        retlw d'33'   ; #17
        retlw d'36'   ; #18
        retlw d'40'   ; #19
        retlw d'44'   ; #20
        retlw d'47'   ; #21
        retlw d'51'   ; #22
        retlw d'55'   ; #23
        retlw d'59'   ; #24
        retlw d'63'   ; #25
        retlw d'67'   ; #26
        retlw d'71'   ; #27
        retlw d'75'   ; #28
        retlw d'79'   ; #29
        retlw d'82'   ; #30
        retlw d'86'   ; #31
        retlw d'90'   ; #32
        retlw d'93'   ; #33
        retlw d'97'   ; #34
        retlw d'100'   ; #35
        retlw d'103'   ; #36
        retlw d'106'   ; #37
        retlw d'109'   ; #38
        retlw d'112'   ; #39
        retlw d'114'   ; #40
        retlw d'116'   ; #41
        retlw d'118'   ; #42
        retlw d'120'   ; #43
        retlw d'122'   ; #44
        retlw d'123'   ; #45
        retlw d'124'   ; #46
        retlw d'125'   ; #47
        retlw d'126'   ; #48
        retlw d'126'   ; #49
        retlw d'126'   ; #50
        retlw d'126'   ; #51
        retlw d'126'   ; #52
        retlw d'125'   ; #53
        retlw d'124'   ; #54
        retlw d'123'   ; #55
        retlw d'122'   ; #56
        retlw d'120'   ; #57
        retlw d'118'   ; #58
        retlw d'116'   ; #59
        retlw d'114'   ; #60
        retlw d'112'   ; #61
        retlw d'109'   ; #62
        retlw d'106'   ; #63
        retlw d'103'   ; #64
        retlw d'100'   ; #65
        retlw d'97'   ; #66
        retlw d'93'   ; #67
        retlw d'90'   ; #68
        retlw d'86'   ; #69
        retlw d'82'   ; #70
        retlw d'79'   ; #71
        retlw d'75'   ; #72
        retlw d'71'   ; #73
        retlw d'67'   ; #74
        retlw d'63'   ; #75
        retlw d'59'   ; #76
        retlw d'55'   ; #77
        retlw d'51'   ; #78
        retlw d'47'   ; #79
        retlw d'44'   ; #80
        retlw d'40'   ; #81
        retlw d'36'   ; #82
        retlw d'33'   ; #83
        retlw d'29'   ; #84
        retlw d'26'   ; #85
        retlw d'23'   ; #86
        retlw d'20'   ; #87
        retlw d'17'   ; #88
        retlw d'14'   ; #89
        retlw d'12'   ; #90
        retlw d'10'   ; #91
        retlw d'8'   ; #92
        retlw d'6'   ; #93
        retlw d'4'   ; #94
        retlw d'3'   ; #95
        retlw d'2'   ; #96
        retlw d'1'   ; #97
        retlw d'0'   ; #98
        retlw d'0'   ; #99
        ; --------------- Kurve Ende ----------------

        ; Servo Parameter $100
        retlw b'11000100'   ; #100 = Flags       
            ; #1  ; switch V off after RUN      
            ; #2  ; switch Impulse off after RUN
            ; #6  ; reject interrupt of running servo.
            ; #7  ; reject repetition of same adr.

        ; Curve Parameter $101
        retlw d'100'         ; #101 = number of steps
        retlw d'1'           ; #102 = Stretch-Faktor

        ; Adress Parameter $103..108
        retlw h'41'           ; #103 = Flags 1 Flags for Adress #4 (this is not a curve parameter!!)
            ; #0 Adress 4 use Servo 1
            ; #1 Adress 4 use Servo 2
            ; #2 Adress 4 use Servo 3
            ; #3 Adress 4 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'88'           ; #104 = Flags 2 Flags for Adress #4 (this is not a curve parameter!!)
            ; #0 Adress 4 use Relais 1
            ; #1 Adress 4 use Relais 2
            ; #2 Adress 4 use Relais 3
            ; #3 Adress 4 use Relais 4
            ; #4 Adress 4 use Kurve 1
            ; #5 Adress 4 use Kurve 2
            ; #6 Adress 4 use Kurve 3
            ; #7 Adress 4 use Kurve 4
        retlw d'0'            ; #105 = not used
        retlw h'01'           ; #106 = Flags 1 Flags for Adress #8 (this is not a curve parameter!!)
            ; #0 Adress 8 use Servo 1
            ; #1 Adress 8 use Servo 2
            ; #2 Adress 8 use Servo 3
            ; #3 Adress 8 use Servo 4
            ; #4 Loop
            ; #5 abbruch (sofort, Kurve wird abgebrochen)
            ; #6 beenden (Stopp nach Ablauf der Kurve)
        retlw h'88'           ; #107 = Flags 2 Flags for Adress #8 (this is not a curve parameter!!)
            ; #0 Adress 8 use Relais 1
            ; #1 Adress 8 use Relais 2
            ; #2 Adress 8 use Relais 3
            ; #3 Adress 8 use Relais 4
            ; #4 Adress 8 use Kurve 1
            ; #5 Adress 8 use Kurve 2
            ; #6 Adress 8 use Kurve 3
            ; #7 Adress 8 use Kurve 4
        retlw d'0'            ; #108 = not used

        retlw d'0'    ; #109 Dummy, damit keine Lcken im HEX-File 
        retlw h'00'   ; #110 Info switch 4 open
            ; #0 like Adress 1
            ; #1 like Adress 2
            ; #2 like Adress 3
            ; #3 like Adress 4
            ; #4 like Adress 5
            ; #5 like Adress 6
            ; #6 like Adress 7
            ; #7 like Adress 8
        retlw d'0'   ; #111 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #112 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #113 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #114 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #115 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #116 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #117 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #118 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #119 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #120 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #121 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #122 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #123 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #124 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #125 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #126 Dummy, damit keine Lcken im HEX-File 
        retlw d'0'   ; #127 Dummy, damit keine Lcken im HEX-File 
