; ********************************************************
; ** Version: V3.50 vom 01/06/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09/10/11 Sven Brandt
; ********************************************************
; History:
; V3.50 - DUNKELTAST_ALL hinzu
; V3.26 - nderung in der State-Machine "state_signal", jetzt sind alle Outputs auch als Blink konfigurierbar
; V3.22 - neue Register-Namen, Kommentar berarbeitet (no changes in HEX)
; V3.20 - erstellt aus state_led_signal2.asm V3.19

;***************************************
;***************************************
;***************************************
;***************************************
;** Routine: statemachine_signal      **
;** state manager signal              **
;** Aufruf alle 2-3 ms                **
;** needs <    us                     **
;***************************************
;***************************************
;***************************************
;***************************************
;   ------------------
;   --- Signal     ---
;   ------------------
;R_STATEx   *              *           
;------------\             :  : /--------  LEDxA (red)
;           : \----------------/ :       
;           :  :        :  :       
;#1         :#2 #3             #4:#1  
;
;Time *: receive order to switch    
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms

statemachine_signal
;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
#ifndef test
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
#endif
        clrf TMR1H                     ; .. then reset to ZERO

        movlw CONST_FADE_01            ;
        movwf TMR1L                    ; fr schnelleres Blinken / Fading

        ; PWM hoch- bzw. runterfahren
        incfsz R_COUNT,F               ; if R_COUNT != 0
         goto signal2_1                ; .. then change R_STATE 
        incf R_COUNT,F                 ; if R_COUNT = 0: R_COUNT starts with '1'

        ;***** Change STATE *****

STATUS_REG_1
        ; *** LED1 (Ausgang 1) ***
#ifdef dunkeltast_all_out01 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED1OFF                  ; .. then LED off
#endif ; dunkeltast_all_out01
#ifdef dunkeltast_out01
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED1OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT1
         goto LED1OFF                  ; .. then jump
        btfss SET_OUTPUT1_FL
         goto LED1ON                   ; .. then jump

LED1FLASH ; *** LED1 FLASH ***      
FL1_1   btfss R_STATE1,1               ; check R_STATE1 #1
         goto FL1_2
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2
        goto STATUS_REG_2              ; continue

FL1_2   btfss R_STATE1,2               ; check R_STATE1 #2
         goto FL1_3
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,3                 ; continue R_STATE1#3
        goto STATUS_REG_2              ; continue

FL1_3   btfss R_STATE1,3               ; check R_STATE1 #3
         goto FL1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

FL1_4   btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,1                 ; continue R_STATE1#1
        goto STATUS_REG_2              ; continue

LED1ON  ; *** LED1 ON ***      
ON1_2   btfss R_STATE1,2               ; check R_STATE1 #2
         goto ON1_3
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_3   btfss R_STATE1,3               ; check R_STATE1 #3
         goto ON1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_4   btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,1                 ; continue R_STATE1#1
        goto STATUS_REG_2              ; continue
        
LED1OFF ; *** LED1 OFF ***      
OFF1_1  btfss R_STATE1,1               ; check R_STATE1 #1
         goto OFF1_2
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2
         goto STATUS_REG_2             ; continue
        
OFF1_2  btfss R_STATE1,2               ; check R_STATE1 #2
         goto OFF1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,3                 ; continue R_STATE1#3
        goto STATUS_REG_2              ; continue

OFF1_4  btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2



STATUS_REG_2
        ; *** LED2 (Ausgang 2) ***
#ifdef dunkeltast_all_out02 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED2OFF                  ; .. then LED off
#endif ; dunkeltast_all_out02
#ifdef dunkeltast_out02
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED2OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT2
         goto LED2OFF                  ; .. then jump
        btfss SET_OUTPUT2_FL
         goto LED2ON                   ; .. then jump

LED2FLASH ; *** LED2 FLASH ***      
FL2_1   btfss R_STATE2,1               ; check R_STATE2 #1
         goto FL2_2
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2
        goto STATUS_REG_3              ; continue

FL2_2   btfss R_STATE2,2               ; check R_STATE2 #2
         goto FL2_3
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,3                 ; continue R_STATE2#3
        goto STATUS_REG_3              ; continue

FL2_3   btfss R_STATE2,3               ; check R_STATE2 #3
         goto FL2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

FL2_4   btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,1                 ; continue R_STATE2#1
        goto STATUS_REG_3              ; continue
           
LED2ON  ; *** LED2 ON *** 
ON2_2   btfss R_STATE2,2               ; check R_STATE2 #2
         goto ON2_3
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_3   btfss R_STATE2,3               ; check R_STATE2 #3
         goto ON2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_4   btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,1                 ; continue R_STATE2#1
        goto STATUS_REG_3              ; continue
        
LED2OFF ; *** LED2 OFF ***      
OFF2_1  btfss R_STATE2,1               ; check R_STATE2 #1
         goto OFF2_2
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2
         goto STATUS_REG_3             ; continue
        
OFF2_2  btfss R_STATE2,2               ; check R_STATE2 #2
         goto OFF2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,3                 ; continue R_STATE2#3
        goto STATUS_REG_3              ; continue

OFF2_4  btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2



STATUS_REG_3
        ; *** LED3 (Ausgang 3) ***
#ifdef dunkeltast_all_out03 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED3OFF                  ; .. then LED off
#endif ; dunkeltast_all_out03
#ifdef dunkeltast_out03
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED3OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT3
         goto LED3OFF                  ; .. then jump
        btfss SET_OUTPUT3_FL
         goto LED3ON                   ; .. then jump
          
LED3FLASH ; *** LED3 FLASH ***      
FL3_1   btfss R_STATE3,1               ; check R_STATE3 #1
         goto FL3_2
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2
        goto STATUS_REG_4              ; continue

FL3_2   btfss R_STATE3,2               ; check R_STATE3 #2
         goto FL3_3
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,3                 ; continue R_STATE3#3
        goto STATUS_REG_4              ; continue

FL3_3   btfss R_STATE3,3               ; check R_STATE3 #3
         goto FL3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

FL3_4   btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,1                 ; continue R_STATE3#1
        goto STATUS_REG_4              ; continue

LED3ON  ; *** LED3 ON ***  
ON3_2   btfss R_STATE3,2               ; check R_STATE3 #2
         goto ON3_3
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_3   btfss R_STATE3,3               ; check R_STATE3 #3
         goto ON3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_4   btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,1                 ; continue R_STATE3#1
        goto STATUS_REG_4              ; continue
        
LED3OFF ; *** LED3 OFF ***  
OFF3_1  btfss R_STATE3,1               ; check R_STATE3 #1
         goto OFF3_2
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2
         goto STATUS_REG_4             ; continue
        
OFF3_2  btfss R_STATE3,2               ; check R_STATE3 #2
         goto OFF3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,3                 ; continue R_STATE3#3
        goto STATUS_REG_4              ; continue

OFF3_4  btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2



STATUS_REG_4
        ; *** LED4 (Ausgang 4) ***
#ifdef dunkeltast_all_out04 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED4OFF                  ; .. then LED off
#endif ; dunkeltast_all_out04
#ifdef dunkeltast_out04
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED4OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT4
         goto LED4OFF                  ; .. then jump
        btfss SET_OUTPUT4_FL
         goto LED4ON                   ; .. then jump

LED4FLASH ; *** LED4 FLASH ***      
FL4_1   btfss R_STATE4,1               ; check R_STATE4 #1
         goto FL4_2
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2
        bcf ALTER_FLASH_S1             ; now other LED may also start flashing...
        goto STATUS_REG_5              ; continue

FL4_2  btfss R_STATE4,2                ; check R_STATE4 #2
         goto FL4_3
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,3                 ; continue R_STATE4#3
        goto STATUS_REG_5              ; continue

FL4_3  btfss R_STATE4,3                ; check R_STATE4 #3
         goto FL4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

FL4_4  btfss R_STATE4,4                ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,1                 ; continue R_STATE4#1
        goto STATUS_REG_5              ; continue

LED4ON  ; *** LED4 ON ***      
ON4_2   btfss R_STATE4,2               ; check R_STATE4 #2
         goto ON4_3
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_3   btfss R_STATE4,3               ; check R_STATE4 #3
         goto ON4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_4   btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,1                 ; continue R_STATE4#1
        goto STATUS_REG_5              ; continue
        
LED4OFF ; *** LED4 OFF ***      
OFF4_1  btfss R_STATE4,1               ; check R_STATE4 #1
         goto OFF4_2
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2
         goto STATUS_REG_5             ; continue
        
OFF4_2  btfss R_STATE4,2               ; check R_STATE4 #2
         goto OFF4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,3                 ; continue R_STATE4#3
        goto STATUS_REG_5              ; continue

OFF4_4  btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2



STATUS_REG_5
        ; *** LED5 (Ausgang 5) ***
#ifdef dunkeltast_all_out05 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED5OFF                  ; .. then LED off
#endif ; dunkeltast_all_out05
#ifdef dunkeltast_out05
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED5OFF                  ; .. then LED off
#endif
        btfsc ALTER_FLASH_S1           ; if this bit is #1, then no action on this output,
         goto LED5OFF                  ; .. because other LED has first fade-in
        btfss SET_OUTPUT5
         goto LED5OFF                  ; .. then jump
        btfss SET_OUTPUT5_FL
         goto LED5ON                   ; .. then jump

LED5FLASH ; *** LED5 FLASH ***      
FL5_1  btfss R_STATE5,1                ; check R_STATE5 #1
         goto FL5_2
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2
        goto STATUS_REG_6              ; continue

FL5_2  btfss R_STATE5,2                ; check R_STATE5 #2
         goto FL5_3
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,3                 ; continue R_STATE5#3
        goto STATUS_REG_6              ; continue

FL5_3  btfss R_STATE5,3                ; check R_STATE5 #3
         goto FL5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

FL5_4  btfss R_STATE5,4                ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,1                 ; continue R_STATE5#1
        goto STATUS_REG_6              ; continue

LED5ON  ; *** LED5 ON ***      
ON5_2   btfss R_STATE5,2               ; check R_STATE5 #2
         goto ON5_3
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

ON5_3   btfss R_STATE5,3               ; check R_STATE5 #3
         goto ON5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

ON5_4   btfss R_STATE5,4               ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,1                 ; continue R_STATE5#1
        goto STATUS_REG_6              ; continue
        
LED5OFF ; *** LED5 OFF ***      
OFF5_1  btfss R_STATE5,1               ; check R_STATE5 #1
         goto OFF5_2
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2
         goto STATUS_REG_6             ; continue
        
OFF5_2  btfss R_STATE5,2               ; check R_STATE5 #2
         goto OFF5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,3                 ; continue R_STATE5#3
        goto STATUS_REG_6              ; continue

OFF5_4  btfss R_STATE5,4               ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2



STATUS_REG_6
        ; *** LED6 (Ausgang 6) ***
#ifdef dunkeltast_all_out06 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED6OFF                  ; .. then LED off
#endif ; dunkeltast_all_out06
#ifdef dunkeltast_out06
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED6OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT6
         goto LED6OFF                  ; .. then jump
        btfss SET_OUTPUT6_FL
         goto LED6ON                   ; .. then jump

LED6FLASH ; *** LED6 FLASH ***      
FL6_1  btfss R_STATE6,1                ; check R_STATE6 #1
         goto FL6_2
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2
        goto STATUS_REG_7              ; continue

FL6_2  btfss R_STATE6,2                ; check R_STATE6 #2
         goto FL6_3
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,3                 ; continue R_STATE6#3
        goto STATUS_REG_7              ; continue

FL6_3  btfss R_STATE6,3                ; check R_STATE6 #3
         goto FL6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

FL6_4  btfss R_STATE6,4                ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,1                 ; continue R_STATE6#1
        goto STATUS_REG_7              ; continue

LED6ON  ; *** LED6 ON ***      
ON6_2   btfss R_STATE6,2               ; check R_STATE6 #2
         goto ON6_3
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

ON6_3   btfss R_STATE6,3               ; check R_STATE6 #3
         goto ON6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

ON6_4   btfss R_STATE6,4               ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,1                 ; continue R_STATE6#1
        goto STATUS_REG_7              ; continue
        
LED6OFF ; *** LED6 OFF ***      
OFF6_1  btfss R_STATE6,1               ; check R_STATE6 #1
         goto OFF6_2
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2
         goto STATUS_REG_7             ; continue
        
OFF6_2  btfss R_STATE6,2               ; check R_STATE6 #2
         goto OFF6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,3                 ; continue R_STATE6#3
        goto STATUS_REG_7              ; continue

OFF6_4  btfss R_STATE6,4               ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2



STATUS_REG_7
        ; *** LED7 (Ausgang 7) ***
#ifdef dunkeltast_all_out07 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED7OFF                  ; .. then LED off
#endif ; dunkeltast_all_out07
#ifdef dunkeltast_out07
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED7OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT7
         goto LED7OFF                  ; .. then jump
        btfss SET_OUTPUT7_FL
         goto LED7ON                   ; .. then jump

LED7FLASH ; *** LED7 FLASH ***      
FL7_1  btfss R_STATE7,1                ; check R_STATE7 #1
         goto FL7_2
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2
        goto STATUS_REG_8              ; continue

FL7_2  btfss R_STATE7,2                ; check R_STATE7 #2
         goto FL7_3
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,3                 ; continue R_STATE7#3
        goto STATUS_REG_8              ; continue

FL7_3  btfss R_STATE7,3                ; check R_STATE7 #3
         goto FL7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

FL7_4  btfss R_STATE7,4                ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,1                 ; continue R_STATE7#1
        goto STATUS_REG_8              ; continue

LED7ON  ; *** LED7 ON ***      
ON7_2   btfss R_STATE7,2               ; check R_STATE7 #2
         goto ON7_3
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

ON7_3   btfss R_STATE7,3               ; check R_STATE7 #3
         goto ON7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

ON7_4   btfss R_STATE7,4               ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,1                 ; continue R_STATE7#1
        goto STATUS_REG_8              ; continue
        
LED7OFF ; *** LED7 OFF ***      
OFF7_1  btfss R_STATE7,1               ; check R_STATE7 #1
         goto OFF7_2
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2
         goto STATUS_REG_8             ; continue
        
OFF7_2  btfss R_STATE7,2               ; check R_STATE7 #2
         goto OFF7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,3                 ; continue R_STATE7#3
        goto STATUS_REG_8              ; continue

OFF7_4  btfss R_STATE7,4               ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2



STATUS_REG_8
        ; *** LED8 (Ausgang 8) ***
#ifdef dunkeltast_all_out08 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED8OFF                  ; .. then LED off
#endif ; dunkeltast_all_out08
#ifdef dunkeltast_out08
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED8OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT8
         goto LED8OFF                  ; .. then jump
        btfss SET_OUTPUT8_FL
         goto LED8ON                   ; .. then jump

LED8FLASH ; *** LED8 FLASH ***      
FL8_1  btfss R_STATE8,1                ; check R_STATE8 #1
         goto FL8_2
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2
        goto STATUS_REG_9              ; continue

FL8_2  btfss R_STATE8,2                ; check R_STATE8 #2
         goto FL8_3
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,3                 ; continue R_STATE8#3
        goto STATUS_REG_9              ; continue

FL8_3  btfss R_STATE8,3                ; check R_STATE8 #3
         goto FL8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

FL8_4  btfss R_STATE8,4                ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,1                 ; continue R_STATE8#1
        goto STATUS_REG_9              ; continue

LED8ON  ; *** LED8 ON ***      
ON8_2   btfss R_STATE8,2               ; check R_STATE8 #2
         goto ON8_3
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

ON8_3   btfss R_STATE8,3               ; check R_STATE8 #3
         goto ON8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

ON8_4   btfss R_STATE8,4               ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,1                 ; continue R_STATE8#1
        goto STATUS_REG_9              ; continue
        
LED8OFF ; *** LED8 OFF ***      
OFF8_1  btfss R_STATE8,1               ; check R_STATE8 #1
         goto OFF8_2
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2
         goto STATUS_REG_9             ; continue
        
OFF8_2  btfss R_STATE8,2               ; check R_STATE8 #2
         goto OFF8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,3                 ; continue R_STATE8#3
        goto STATUS_REG_9              ; continue

OFF8_4  btfss R_STATE8,4               ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2



STATUS_REG_9
        ; *** LED9 (Ausgang 9) ***
#ifdef dunkeltast_all_out09 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED9OFF                  ; .. then LED off
#endif ; dunkeltast_all_out09
#ifdef dunkeltast_out09
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED9OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT9
         goto LED9OFF                  ; .. then jump
        btfss SET_OUTPUT9_FL
         goto LED9ON                   ; .. then jump

LED9FLASH ; *** LED9 FLASH ***      
FL9_1   btfss R_STATE9,1               ; check R_STATE9 #1
         goto FL9_2
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2
        bcf ALTER_FLASH_S2             ; now other LED may also start flashing...
        goto STATUS_REG_10             ; continue

FL9_2   btfss R_STATE9,2               ; check R_STATE9 #2
         goto FL9_3
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,3                 ; continue R_STATE9#3
        goto STATUS_REG_10             ; continue

FL9_3   btfss R_STATE9,3               ; check R_STATE9 #3
         goto FL9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

FL9_4   btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,1                 ; continue R_STATE9#1
        goto STATUS_REG_10             ; continue

LED9ON  ; *** LED9 ON ***      
ON9_2   btfss R_STATE9,2               ; check R_STATE9 #2
         goto ON9_3
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

ON9_3   btfss R_STATE9,3               ; check R_STATE9 #3
         goto ON9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

ON9_4   btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,1                 ; continue R_STATE9#1
        goto STATUS_REG_10             ; continue
        
LED9OFF ; *** LED9 OFF ***      
OFF9_1  btfss R_STATE9,1               ; check R_STATE9 #1
         goto OFF9_2
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2
         goto STATUS_REG_10            ; continue
        
OFF9_2  btfss R_STATE9,2               ; check R_STATE9 #2
         goto OFF9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,3                 ; continue R_STATE9#3
        goto STATUS_REG_10             ; continue

OFF9_4  btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2



STATUS_REG_10
        ; *** LED10 (Ausgang 10 ***
#ifdef dunkeltast_all_out10 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED10OFF                 ; .. then LED off
#endif ; dunkeltast_all_out10
#ifdef dunkeltast_out10
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED10OFF                 ; .. then LED off
#endif
        btfsc ALTER_FLASH_S2           ; if this bit is #1, then no action on this output,
         goto LED10OFF                 ; .. because other LED has first fade-in
        btfss SET_OUTPUT10
         goto LED10OFF                 ; .. then jump
        btfss SET_OUTPUT10_FL
         goto LED10ON                  ; .. then jump

LED10FLASH ; *** LED10 FLASH ***      
FL10_1  btfss R_STATE10,1              ; check R_STATE10 #1
         goto FL10_2
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2
        goto STATUS_REG_11             ; continue

FL10_2  btfss R_STATE10,2              ; check R_STATE10 #2
         goto FL10_3
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,3                ; continue R_STATE10#3
        goto STATUS_REG_11             ; continue

FL10_3  btfss R_STATE10,3              ; check R_STATE10 #3
         goto FL10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

FL10_4  btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                  ; clear R_STATE10  
        bsf R_STATE10,1                ; continue R_STATE10#1
        goto STATUS_REG_11             ; continue

LED10ON ; *** LED10 ON ***      
ON10_2  btfss R_STATE10,2              ; check R_STATE10 #2
         goto ON10_3
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

ON10_3  btfss R_STATE10,3              ; check R_STATE10 #3
         goto ON10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

ON10_4  btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,1                ; continue R_STATE10#1
        goto STATUS_REG_11             ; continue
        
LED10OFF; *** LED10 OFF ***      
OFF10_1 btfss R_STATE10,1              ; check R_STATE10 #1
         goto OFF10_2
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2
         goto STATUS_REG_11            ; continue
        
OFF10_2 btfss R_STATE10,2              ; check R_STATE10 #2
         goto OFF10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,3                ; continue R_STATE10#3
        goto STATUS_REG_11             ; continue

OFF10_4 btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2



STATUS_REG_11
        ; --- Status speichern ? ---
        call save_status               ; .. 

        ; bername von PRE zu AKT (realisiert Dunkelphase)
        movf R_STATUS_ON_S1_pre,W      ; copy R_STATUS_ON_S1_pre into R_STATUS_ON_S1_akt
        movwf R_STATUS_ON_S1_akt       ; (fuer Dunkelphase)
        movf R_STATUS_ON_S2_pre,W      ; copy R_STATUS_ON_S2_pre into R_STATUS_ON_S2_akt
        movwf R_STATUS_ON_S2_akt       ; (fuer Dunkelphase)
        btfsc DUNKELTAST_PRE           ; hierdurch faded nach Dunkeltastung S2
         bsf DUNKELTAST_AKT            ; .. zur selben Zeit wie S1
        btfss DUNKELTAST_PRE           ; .. ein (nach Dunkelphase)
         bcf DUNKELTAST_AKT            ; .. 

#ifdef use_all_dark
        btfsc DUNKELTAST_ALL_PRE       ; hierdurch faded nach Dunkeltastung S2
         bsf DUNKELTAST_ALL            ; .. zur selben Zeit wie S1
        btfss DUNKELTAST_ALL_PRE       ; .. ein (nach Dunkelphase)
         bcf DUNKELTAST_ALL            ; .. 
#endif ; use_all_dark

signal2_1 ;**** LED steuern ****
        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

        ;***** PWM-Time ON *****
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE1,W               ; = off or fade out LED1
        btfss STATUS,Z
         bcf LED_01                    ; LED_1 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE2,W               ; = off or fade out LED2
        btfss STATUS,Z
         bcf LED_02                    ; LED_2 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE3,W               ; = off or fade out LED3
        btfss STATUS,Z
         bcf LED_03                    ; LED_3 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE4,W               ; = off or fade out LED4
        btfss STATUS,Z
         bcf LED_04                    ; LED_4 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE5,W               ; = off or fade out LED5
        btfss STATUS,Z
         bcf LED_05                    ; LED_5 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE6,W               ; = off or fade out LED6
        btfss STATUS,Z
         bcf LED_06                    ; LED_6 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE7,W               ; = off or fade out LED7
        btfss STATUS,Z
         bcf LED_07                    ; LED_7 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE8,W               ; = off or fade out LED8
        btfss STATUS,Z
         bcf LED_08                    ; LED_8 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE9,W               ; = off or fade out LED9
        btfss STATUS,Z
         bcf LED_09                    ; LED_9 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE10,W              ; = off or fade out LED10
        btfss STATUS,Z
         bcf LED_10                    ; LED_10 off

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
        goto signal4_2                 ; ... then JUMP

        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE1,W               ; = fade in/on LED1
        btfss STATUS,Z
         bsf LED_01                    ; LED_1 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE2,W               ; = fade in/on LED2
        btfss STATUS,Z
         bsf LED_02                    ; LED_2 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE3,W               ; = fade in/on LED3
        btfss STATUS,Z
         bsf LED_03                    ; LED_3 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE4,W               ; = fade in/on LED4
        btfss STATUS,Z
         bsf LED_04                    ; LED_4 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE5,W               ; = fade in/on LED5
        btfss STATUS,Z
         bsf LED_05                    ; LED_5 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE6,W               ; = fade in/on LED6
        btfss STATUS,Z
         bsf LED_06                    ; LED_6 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE7,W               ; = fade in/on LED7
        btfss STATUS,Z
         bsf LED_07                    ; LED_7 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE8,W               ; = fade in/on LED8
        btfss STATUS,Z
         bsf LED_08                    ; LED_8 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE9,W               ; = fade in/on LED9
        btfss STATUS,Z
         bsf LED_09                    ; LED_9 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE10,W              ; = fade in/on LED10
        btfss STATUS,Z
         bsf LED_10                    ; LED_9 on

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

signal4_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto signal4_2                ; ... then JUMP_2

        ; ***** PWM-Time ready *****
        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE1,W               ; = off or fade in LED1
        btfss STATUS,Z
         bcf LED_01                    ; LED_1 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE1,W               ; = on or fade out LED1
        btfss STATUS,Z
         bsf LED_01                    ; LED_1 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE2,W               ; = off or fade in LED2
        btfss STATUS,Z
         bcf LED_02                    ; LED_2 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE2,W               ; = on or fade out LED2
        btfss STATUS,Z
         bsf LED_02                    ; LED_2 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE3,W               ; = off or fade in LED3
        btfss STATUS,Z
         bcf LED_03                    ; LED_3 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE3,W               ; = on or fade out LED3
        btfss STATUS,Z
         bsf LED_03                    ; LED_3 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE4,W               ; = off or fade in LED4
        btfss STATUS,Z
         bcf LED_04                    ; LED_4 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE4,W               ; = on or fade out LED4
        btfss STATUS,Z
         bsf LED_04                    ; LED_4 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE5,W               ; = off or fade in LED5
        btfss STATUS,Z
         bcf LED_05                    ; LED_5 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE5,W               ; = on or fade out LED5
        btfss STATUS,Z
         bsf LED_05                    ; LED_5 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE6,W               ; = off or fade in LED6
        btfss STATUS,Z
         bcf LED_06                    ; LED_6 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE6,W               ; = on or fade out LED6
        btfss STATUS,Z
         bsf LED_06                    ; LED_6 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE7,W               ; = off or fade in LED7
        btfss STATUS,Z
         bcf LED_07                    ; LED_7 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE7,W               ; = on or fade out LED7
        btfss STATUS,Z
         bsf LED_07                    ; LED_7 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE8,W               ; = off or fade in LED8
        btfss STATUS,Z
         bcf LED_08                    ; LED_8 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE8,W               ; = on or fade out LED8
        btfss STATUS,Z
         bsf LED_08                    ; LED_8 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE9,W               ; = off or fade in LED9
        btfss STATUS,Z
         bcf LED_09                    ; LED_9 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE9,W               ; = on or fade out LED9
        btfss STATUS,Z
         bsf LED_09                    ; LED_9 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE10,W              ; = off or fade in LED9
        btfss STATUS,Z
         bcf LED_10                    ; LED_10 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE10,W              ; = on or fade out LED9
        btfss STATUS,Z
         bsf LED_10                    ; LED_10 on

        return                         ; .. then return
