; ********************************************************
; ** Version: V3.00 vom 24/07/2006                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05 Sven Brandt

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: ANDREAS            **
;** state manager andreas       **
;** needs <    us               **
;*********************************
;*********************************
;*********************************
;*********************************
;   --------------------
;   --- Andreaskreuz ---
;   --------------------
;    : /----------\          : /----------\          : /----------\     LED1
;-----/ :        : \----------/ :        : \----------/          : \----
;    :  :        :  :        :  :        :  :        :  :        :
;    :  :        : /----------\ :        : /----------\ :        :      LED2
;-----------------/ :        : \----------/ :        : \----------------
;    :  :        :  :        :  :        :  :        :  :        :
;#1  :#2         :#3         :#4         :#3         :#4         :#5  R_STATE
;
;Time *: TMR1H <> ZERO and R_COUNT=0
;#1: ready for start (START IT!)
;#2: LED1 fade in/on    LED2 off
;#3: LED1 fade out/off  LED2 fade in/on
;#4: LED1 fade in/on    LED2 fade out/off
;#5: LED1 fade out/off  LED2 off
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  1        2        3        4        5        6       7       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0, STATE-Wechsel
;@2: LED off ("andreas_2")
;@3: LED on, R_COUNT increment every 2ms ("andreas_1")

statemachine_andreas
;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
        clrf TMR1H                     ; .. then reset to ZERO

        incfsz R_COUNT,F               ; if R_COUNT = 0
         goto andreas_1                ; .. then change R_STATE
        incf R_COUNT,F                 ; R_COUNT starts with '1'

        ;***** Change STATE *****
        bcf STATUS,C                   ; clear Carry and
        rlf R_STATE,F                  ; .. rotate left
        btfsc R_STATE2,0               ; if STATE2#0 (fade-out flag)
         goto andreas_1                ; .. then JUMP (means STOP IT!)
        btfsc R_STATE,5                ; if #5
         bsf R_STATE,3                 ; .. change to #3
        bcf R_STATE,5                  ; .. and delete #5

     
andreas_1
        call TABLE                     ; get PWM-value from table
        movwf R_PWMVALUE
        ;***** PWM-Time ON *****
      
        movlw b'01101000'              ; check STATE #3,#5,#6
        andwf R_STATE,W                ; = fade out LED1
        btfss STATUS,Z
         bcf PORTA,0                    ; LED1 off
        bcf R_STATE,6                  ; tritt nach #5 (fade-out) auf

        movlw b'00110100'              ; check status #2,#4,#5
        andwf R_STATE,W                ; = off or fade out LED 2
        btfss STATUS,Z
         bcf PORTA,4                    ; LED2 off

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto andreas_2                    ; ... then JUMP

        movlw b'00010100'              ; check status #2,#4
        andwf R_STATE,W                ; = fade in LED1
        btfss STATUS,Z
         bsf PORTA,0                    ; LED1 on

        movlw b'00001000'              ; check status #3
        andwf R_STATE,W                ; = fade in LED2
        btfss STATUS,Z
         bsf PORTA,4                    ; LED2 on


andreas_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; return

        clrwdt                         ; clear Watchdog-Timer
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto andreas_2                ; ... then JUMP

        ; ***** PWM-Time ready *****
        movlw b'00010100'              ; check status #2,#4
        andwf R_STATE,W                ; = fade in LED1
        btfss STATUS,Z
         bcf PORTA,0                    ; LED1 off
      
        movlw b'00101000'              ; check status #3,#5
        andwf R_STATE,W                ; = fade out LED1
        btfss STATUS,Z
         bsf PORTA,0                    ; LED1 on

        movlw b'00101100'              ; check status #2,#3,#5
        andwf R_STATE,W                ; = off or fade in LED2
        btfss STATUS,Z
         bcf PORTA,4                    ; LED2 off

        movlw b'00010000'              ; check status #4
        andwf R_STATE,W                ; = fade out LED 2
        btfss STATUS,Z
         bsf PORTA,4                    ; LED2 on

        ; --- Status speichern ? ---
        movlw d'01'                    ; wenn R_COUNT = 1
        subwf R_COUNT,W                ; .. dann ist hier 
        btfss STATUS,Z                 ; .. Zeit zum Speichern im EEPROM,
         return                        ; .. ohne grosse Strung in der PWM
        call save_status               ; .. 

        return

; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  Tabelle for PWM for LEDs                          **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
 org 02fc
TABLE   movlw b'00000011'           ; load PCLATH with High Adress
        movwf PCLATH
        decf R_COUNT,w              ; R_COUNT (1..255) -> (0..254)
        addwf PCL,f
        retlw d'0'   ; #0
        retlw d'4'   ; #1
        retlw d'7'   ; #2
        retlw d'11'   ; #3
        retlw d'15'   ; #4
        retlw d'18'   ; #5
        retlw d'22'   ; #6
        retlw d'25'   ; #7
        retlw d'28'   ; #8
        retlw d'32'   ; #9
        retlw d'35'   ; #10
        retlw d'38'   ; #11
        retlw d'41'   ; #12
        retlw d'44'   ; #13
        retlw d'47'   ; #14
        retlw d'51'   ; #15
        retlw d'54'   ; #16
        retlw d'56'   ; #17
        retlw d'59'   ; #18
        retlw d'62'   ; #19
        retlw d'65'   ; #20
        retlw d'68'   ; #21
        retlw d'71'   ; #22
        retlw d'73'   ; #23
        retlw d'76'   ; #24
        retlw d'79'   ; #25
        retlw d'81'   ; #26
        retlw d'84'   ; #27
        retlw d'86'   ; #28
        retlw d'89'   ; #29
        retlw d'91'   ; #30
        retlw d'94'   ; #31
        retlw d'96'   ; #32
        retlw d'98'   ; #33
        retlw d'101'   ; #34
        retlw d'103'   ; #35
        retlw d'105'   ; #36
        retlw d'108'   ; #37
        retlw d'110'   ; #38
        retlw d'112'   ; #39
        retlw d'114'   ; #40
        retlw d'116'   ; #41
        retlw d'118'   ; #42
        retlw d'120'   ; #43
        retlw d'122'   ; #44
        retlw d'124'   ; #45
        retlw d'126'   ; #46
        retlw d'128'   ; #47
        retlw d'130'   ; #48
        retlw d'132'   ; #49
        retlw d'134'   ; #50
        retlw d'136'   ; #51
        retlw d'137'   ; #52
        retlw d'139'   ; #53
        retlw d'141'   ; #54
        retlw d'143'   ; #55
        retlw d'144'   ; #56
        retlw d'146'   ; #57
        retlw d'148'   ; #58
        retlw d'149'   ; #59
        retlw d'151'   ; #60
        retlw d'152'   ; #61
        retlw d'154'   ; #62
        retlw d'155'   ; #63
        retlw d'157'   ; #64
        retlw d'158'   ; #65
        retlw d'160'   ; #66
        retlw d'161'   ; #67
        retlw d'163'   ; #68
        retlw d'164'   ; #69
        retlw d'166'   ; #70
        retlw d'167'   ; #71
        retlw d'168'   ; #72
        retlw d'170'   ; #73
        retlw d'171'   ; #74
        retlw d'172'   ; #75
        retlw d'174'   ; #76
        retlw d'175'   ; #77
        retlw d'176'   ; #78
        retlw d'177'   ; #79
        retlw d'178'   ; #80
        retlw d'180'   ; #81
        retlw d'181'   ; #82
        retlw d'182'   ; #83
        retlw d'183'   ; #84
        retlw d'184'   ; #85
        retlw d'185'   ; #86
        retlw d'186'   ; #87
        retlw d'187'   ; #88
        retlw d'189'   ; #89
        retlw d'190'   ; #90
        retlw d'191'   ; #91
        retlw d'192'   ; #92
        retlw d'193'   ; #93
        retlw d'194'   ; #94
        retlw d'195'   ; #95
        retlw d'196'   ; #96
        retlw d'196'   ; #97
        retlw d'197'   ; #98
        retlw d'198'   ; #99
        retlw d'199'   ; #100
        retlw d'200'   ; #101
        retlw d'201'   ; #102
        retlw d'202'   ; #103
        retlw d'203'   ; #104
        retlw d'204'   ; #105
        retlw d'204'   ; #106
        retlw d'205'   ; #107
        retlw d'206'   ; #108
        retlw d'207'   ; #109
        retlw d'208'   ; #110
        retlw d'208'   ; #111
        retlw d'209'   ; #112
        retlw d'210'   ; #113
        retlw d'211'   ; #114
        retlw d'211'   ; #115
        retlw d'212'   ; #116
        retlw d'213'   ; #117
        retlw d'213'   ; #118
        retlw d'214'   ; #119
        retlw d'215'   ; #120
        retlw d'215'   ; #121
        retlw d'216'   ; #122
        retlw d'217'   ; #123
        retlw d'217'   ; #124
        retlw d'218'   ; #125
        retlw d'219'   ; #126
        retlw d'219'   ; #127
        retlw d'220'   ; #128
        retlw d'221'   ; #129
        retlw d'221'   ; #130
        retlw d'222'   ; #131
        retlw d'222'   ; #132
        retlw d'223'   ; #133
        retlw d'223'   ; #134
        retlw d'224'   ; #135
        retlw d'224'   ; #136
        retlw d'225'   ; #137
        retlw d'226'   ; #138
        retlw d'226'   ; #139
        retlw d'227'   ; #140
        retlw d'227'   ; #141
        retlw d'228'   ; #142
        retlw d'228'   ; #143
        retlw d'229'   ; #144
        retlw d'229'   ; #145
        retlw d'229'   ; #146
        retlw d'230'   ; #147
        retlw d'230'   ; #148
        retlw d'231'   ; #149
        retlw d'231'   ; #150
        retlw d'232'   ; #151
        retlw d'232'   ; #152
        retlw d'233'   ; #153
        retlw d'233'   ; #154
        retlw d'233'   ; #155
        retlw d'234'   ; #156
        retlw d'234'   ; #157
        retlw d'235'   ; #158
        retlw d'235'   ; #159
        retlw d'235'   ; #160
        retlw d'236'   ; #161
        retlw d'236'   ; #162
        retlw d'236'   ; #163
        retlw d'237'   ; #164
        retlw d'237'   ; #165
        retlw d'238'   ; #166
        retlw d'238'   ; #167
        retlw d'238'   ; #168
        retlw d'239'   ; #169
        retlw d'239'   ; #170
        retlw d'239'   ; #171
        retlw d'240'   ; #172
        retlw d'240'   ; #173
        retlw d'240'   ; #174
        retlw d'240'   ; #175
        retlw d'241'   ; #176
        retlw d'241'   ; #177
        retlw d'241'   ; #178
        retlw d'242'   ; #179
        retlw d'242'   ; #180
        retlw d'242'   ; #181
        retlw d'243'   ; #182
        retlw d'243'   ; #183
        retlw d'243'   ; #184
        retlw d'243'   ; #185
        retlw d'244'   ; #186
        retlw d'244'   ; #187
        retlw d'244'   ; #188
        retlw d'244'   ; #189
        retlw d'245'   ; #190
        retlw d'245'   ; #191
        retlw d'245'   ; #192
        retlw d'245'   ; #193
        retlw d'246'   ; #194
        retlw d'246'   ; #195
        retlw d'246'   ; #196
        retlw d'246'   ; #197
        retlw d'247'   ; #198
        retlw d'247'   ; #199
        retlw d'247'   ; #200
        retlw d'247'   ; #201
        retlw d'247'   ; #202
        retlw d'248'   ; #203
        retlw d'248'   ; #204
        retlw d'248'   ; #205
        retlw d'248'   ; #206
        retlw d'248'   ; #207
        retlw d'249'   ; #208
        retlw d'249'   ; #209
        retlw d'249'   ; #210
        retlw d'249'   ; #211
        retlw d'249'   ; #212
        retlw d'250'   ; #213
        retlw d'250'   ; #214
        retlw d'250'   ; #215
        retlw d'250'   ; #216
        retlw d'250'   ; #217
        retlw d'250'   ; #218
        retlw d'251'   ; #219
        retlw d'251'   ; #220
        retlw d'251'   ; #221
        retlw d'251'   ; #222
        retlw d'251'   ; #223
        retlw d'251'   ; #224
        retlw d'251'   ; #225
        retlw d'252'   ; #226
        retlw d'252'   ; #227
        retlw d'252'   ; #228
        retlw d'252'   ; #229
        retlw d'252'   ; #230
        retlw d'252'   ; #231
        retlw d'252'   ; #232
        retlw d'253'   ; #233
        retlw d'253'   ; #234
        retlw d'253'   ; #235
        retlw d'253'   ; #236
        retlw d'253'   ; #237
        retlw d'253'   ; #238
        retlw d'253'   ; #239
        retlw d'254'   ; #240
        retlw d'254'   ; #241
        retlw d'254'   ; #242
        retlw d'254'   ; #243
        retlw d'254'   ; #244
        retlw d'254'   ; #245
        retlw d'254'   ; #246
        retlw d'254'   ; #247
        retlw d'254'   ; #248
        retlw d'255'   ; #249
        retlw d'255'   ; #250
        retlw d'255'   ; #251
        retlw d'255'   ; #252
        retlw d'255'   ; #253
        retlw d'255'   ; #254
