; ********************************************************
; ** Version: V3.61 vom 06/03/2015                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2015 Sven Brandt

;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: statemachine_weichei **
;** needs <    us                 **
;***********************************
;***********************************
;***********************************
;***********************************
statemachine_weichei

; **********************************************
; **** Fading WeichEi                       ****
; **********************************************

;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
        movf TMR1H,F
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
        clrf TMR1H                     ; .. then reset to ZERO

        movlw CONST_FADE_01            ;
        movwf TMR1L;                   ; fr schnelleres Blinken / Fading

;   -----------------------------
;   --- Generate ON/OFF Flags ---
;   -----------------------------
        ; ONOFF OLD 0 0 1 1    -> 1 1 0 0  (COMF)
        ; ONOFF     0 1 0 1    -> 0 1 0 1  (unverndert)
        ; FADEIN    0 1 0 0    -> 0 1 0 0  (AND)
        comf R_ONOFF1_OLD,W            ; alte ONOFF Bits invertieren
        andwf R_ONOFF1,W               ; .. AND mit aktuellen ONOFF Werten
        movwf R_FADEIN1                ; .. Ergebnis

        ; ONOFF OLD 0 0 1 1    -> 0 0 1 1  (unverndert)
        ; ONOFF     0 1 0 1    -> 1 0 1 0  (COMF)
        ; FADEOUT   0 0 1 0    -> 0 0 1 0  (AND)
        comf R_ONOFF1,W                ; aktuelle ONOFF Bits invertieren
        andwf R_ONOFF1_OLD,W           ; .. AND mit alten ONOFF Werten
        movwf R_FADEOUT1               ; .. Ergebnis

;   -----------------------------
;   --- PWM oder StateMachine? --
;   -----------------------------
        movf R_COUNT,F                 ; wenn R_COUNT = ZERO, dann ist
        btfsc STATUS,Z                 ; .. PWM abgeschlossen,
         goto weichei_state            ; .. jetzt State-Machine

        ;**** PWM nur in Mode 1 ****
        movlw h'FF'
        btfss R_MODE,0                 ; wenn nicht Mode 1:
         movwf R_COUNT                 ; ... keine PWM -> R_COUNT auf ZERO

        incfsz R_COUNT,F               ; PWM fertig ?
         goto pwm                      ; .. wenn nicht, jump

        call pwm_finish                ; PWM wurde gerade beendet, einmaliger Aufruf bei Ende der PWM

weichei_state
;   ---------------------------
;   --- create 500ms-Raster ---
;   ---------------------------
#ifdef test
   call toggle_led_5
#endif ; test
        incf R_VERZ2,F                    ; R_VERZ2 = R_VERZ2 + 1
        btfsc STATUS,Z                    ; wenn R_VERZ2 == ZERO (berlauf)
         incf R_VERZ,F                    ; Z=1 -> R_VERZ = R_VERZ + 1

;
;                     |   .   .   .   .   .   .   .   .
;    R_VERZ2:         |  /|  /|  /|  /|  /|  /|  /|  /|
;    ------           | / | 0 | / | 2 | / | / | / | / |
;  (inc. 2ms)         |/  |/  |/  |/  |/  |/  |/  |/  |
;                     |----------------------------------------
;
;    R_VERZ:          |                    ----
;    -------          |                ----   |
;  increment          |            -1--       |
;                     |        ----           |
;                     |    -0--               |        ----
;                     |----                   |    ----
;                     |----------------------------------------
;
; 0: Time where command occurs ('gruen_1' etc.)
; 1: Value is calculated and stored @ '0' in R1_ABSCHALT_x (R_VERZ1 + R_MAKESLOW)
; 2: Value is stored @ '0' in R2_ABSCHALT_x (R_VERZ2)
;
; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (R_VERZ2 >= R2_ABSCHALT_x) ;
; R_VERZ2 is changing every 8x255 us so there should be no problem with overflow / carry
; (d.h. diese Routine wird in 2 ms bestimmt durchlaufen, sodass R_VERZ2 = R2_ABSCHALT
; .. zu einer Abschaltung fhrt. Dies gilt auch wenn R_VERZ2 = R2_ABSCHALT = h'FF'
; .. und somit 2 ms spter R_VERZ2 = ZERO ist.)

        ; ist ein Abarbeiten des OFF-Befehl & TimeOut zugelassen (neu v3.58)??? Wenn NEIN, dann ABBRUCH
        movf R_OFF_OK,W
        andlw b'00001111'              ; nur Bit 3-0 betrachten
        sublw h'0a'                    ; nur wenn dieses Bitmuster in EE-Zelle steht (nur Bit 3-0 betrachten)
        btfsc STATUS,Z                 ; .. wird abgebrochen
          return

#ifdef test
   call toggle_led_7
#endif ; test

        movf R1_ABSCHALT_1,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         return                        ; ...then RETURN

#ifdef test
   call toggle_led_8
#endif ; test

        movf R2_ABSCHALT_1,W
        subwf R_VERZ2,W                ; W = R_VERZ2 - W = R_VERZ2 - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> R_VERZ2 < R2_ABSCHALT_1
         return                        ; ...then RETURN

#ifdef test
   call toggle_led_10
#endif ; test

        bcf WEICHE_01                  ; Weiche 1
        bcf WEICHE_02                  ; Weiche 1

        return                         ; .. return


pwm     ;**** LED steuern ****
        ; R_PWMVALUE wird bei jedem Step vergrssert.
        ; Bei berlauf wird auf FF gefreezed
        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

pwm_on
        ; ***** PWM-Time Einschaltpunkt *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bsf LED_01                   ; LED_1 on
        btfsc R_FADEIN1,1              ; = fade in LED2
         bsf LED_02                    ; LED_2 on

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
         goto pwm_off                  ; ... then JUMP

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bcf LED_01                    ; LED_1 off
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bcf LED_02                    ; LED_2 off

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

pwm_off ; ** warten auf PWM Off-Time **
        movf TMR1H,F
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto pwm_off                  ; ... then to PWM_OFF

        ;***** PWM-Time abgelaufen *****
        btfsc R_FADEIN1,0              ; = fade in LED1
         bcf LED_01                    ; LED_1 off
        btfsc R_FADEIN1,1              ; = fade in LED2
         bcf LED_02                    ; LED_2 off

        btfsc R_FADEOUT1,0             ; = fade out LED1
         bsf LED_01                    ; LED_1 on
        btfsc R_FADEOUT1,1             ; = fade out LED2
         bsf LED_02                    ; LED_2 on

        return

;*******************************************
;** Routine: pwm_finish                   **
;** switch outputs to static              **
;*******************************************
pwm_finish
        ; PWM fertig, ab jetzt statische LEDs
        ; wenn FADE-IN gewesen, dann entsprechendes Port EIN
        btfsc R_FADEIN1,0      ; = fade in LED1
         bsf LED_01            ; ROT   fuer Hauptstrasse = EIN
        btfsc R_FADEIN1,1      ; = fade in LED2
         bsf LED_02            ; GELB  fuer Hauptstrasse = EIN
        clrf R_FADEIN1

        ; wenn FADE-OUT gewesen, dann entsprechendes Port AUS
        btfsc R_FADEOUT1,0     ; = fade out LED1
         bcf LED_01            ; ROT   fuer Hauptstrasse = AUS
        btfsc R_FADEOUT1,1     ; = fade out LED2
         bcf LED_02            ; GELB  fuer Hauptstrasse = AUS
        clrf R_FADEOUT1

        ; *** save old values fr Restore (Verwerfen der neu berechneten Zustnde) und Vergleich mit aktuellem Zustand (Fade IN/OUT)
        movf R_ONOFF1,W
        movwf R_ONOFF1_OLD

        ; jetzt STATUS Speichern, unschdlich fr PWM
        call save_status

        return