; ********************************************************
; ** Version: V3.11 vom 22/10/2007                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05/06/07 Sven Brandt

; ********************************************************
; History:
; V3.11 - wz_signal: Tristate @ Dunkeltastung fr ZweiPol (Stromaufnahme senken)
; ********************************************************

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: SIGNAL             **
;** state manager signal        **
;** needs <    us               **
;*********************************
;*********************************
;*********************************
;*********************************
;   ---------------
;   --- Signal  ---
;   ---------------
;R_STATEx   *              *           
;------------\             :  : /--------  LEDxA (red)
;           : \----------------/ :       
;           :  :        :  :       
;#1         :#2 #3             #4:#1  
;
;Time *: receive order to switch    
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms

statemachine_signal
;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
        clrf TMR1H                     ; .. then reset to ZERO

        incfsz R_COUNT,F               ; if R_COUNT = 0
         goto signal_1                ; .. then change R_STATE 
        incf R_COUNT,F                 ; R_COUNT starts with '1'

        ;***** Change STATE *****
        ; *** LED1 (Ausgang 1) ***
        movlw b'00000010'              ; check Output#2
        andwf R_STATUS2,W              ; = LED 1 off
        btfss STATUS,Z
         goto LED1OFF                  ; .. then jump

LED1ON  ; *** LED1 ON ***      
ON1_2   btfss R_STATE1,2               ; check R_STATE1 #2
         goto ON1_3
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_3   btfss R_STATE1,3               ; check R_STATE1 #3
         goto ON1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_4   btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,1                 ; continue R_STATE1#1
        goto STATUS_REG_2              ; continue
        
LED1OFF ; *** LED1 OFF ***      
OFF1_1  btfss R_STATE1,1               ; check R_STATE1 #1
         goto OFF1_2
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2
         goto STATUS_REG_2             ; continue
        
OFF1_2  btfss R_STATE1,2               ; check R_STATE1 #2
         goto OFF1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,3                 ; continue R_STATE1#3
        goto STATUS_REG_2              ; continue

OFF1_4  btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2



STATUS_REG_2
        ; *** LED2 (Ausgang 2) ***
        movlw b'00000001'              ; check Output#2
        andwf R_STATUS2,W              ; = LED 2 off
        btfss STATUS,Z
         goto LED2OFF                  ; .. then jump
           
LED2ON  ; *** LED2 ON *** 
ON2_2   btfss R_STATE2,2               ; check R_STATE2 #2
         goto ON2_3
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_3   btfss R_STATE2,3               ; check R_STATE2 #3
         goto ON2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_4   btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,1                 ; continue R_STATE2#1
        goto STATUS_REG_3              ; continue
        
LED2OFF ; *** LED2 OFF ***      
OFF2_1  btfss R_STATE2,1               ; check R_STATE2 #1
         goto OFF2_2
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2
         goto STATUS_REG_3             ; continue
        
OFF2_2  btfss R_STATE2,2               ; check R_STATE2 #2
         goto OFF2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,3                 ; continue R_STATE2#3
        goto STATUS_REG_3              ; continue

OFF2_4  btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2



STATUS_REG_3
        ; *** LED3 (Ausgang 3) ***
        movlw b'00001000'              ; check Output#3
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         movlw b'00001010'             ; auch AUS bei ROT = Ausgang 2
        andwf R_STATUS2,W              ; = LED 3 off
        btfss STATUS,Z
         goto LED3OFF                  ; .. then jump
          
LED3ON  ; *** LED3 ON ***  
ON3_2   btfss R_STATE3,2               ; check R_STATE3 #2
         goto ON3_3
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_3   btfss R_STATE3,3               ; check R_STATE3 #3
         goto ON3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_4   btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,1                 ; continue R_STATE3#1
        goto STATUS_REG_4              ; continue
        
LED3OFF ; *** LED3 OFF ***  
OFF3_1  btfss R_STATE3,1               ; check R_STATE3 #1
         goto OFF3_2
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2
         goto STATUS_REG_4             ; continue
        
OFF3_2  btfss R_STATE3,2               ; check R_STATE3 #2
         goto OFF3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,3                 ; continue R_STATE3#3
        goto STATUS_REG_4              ; continue

OFF3_4  btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2



STATUS_REG_4
        ; *** LED4 (Ausgang 4) ***
        movlw b'00000100'              ; check Output#4
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         movlw b'00000110'             ; auch AUS bei ROT = Ausgang 2
        andwf R_STATUS2,W              ; = LED 4 off
        btfss STATUS,Z
         goto LED4OFF                  ; .. then jump

LED4ON  ; *** LED4 ON ***      
ON4_2   btfss R_STATE4,2               ; check R_STATE4 #2
         goto ON4_3
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_3   btfss R_STATE4,3               ; check R_STATE4 #3
         goto ON4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_4   btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,1                 ; continue R_STATE4#1
        goto STATUS_REG_5              ; continue
        
LED4OFF ; *** LED4 OFF ***      
OFF4_1  btfss R_STATE4,1               ; check R_STATE4 #1
         goto OFF4_2
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2
         goto STATUS_REG_5             ; continue
        
OFF4_2  btfss R_STATE4,2               ; check R_STATE4 #2
         goto OFF4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,3                 ; continue R_STATE4#3
        goto STATUS_REG_5              ; continue

OFF4_4  btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2



STATUS_REG_5
        ; --- TRIS-STATE @ Dunkel-tastung & Signal_1 = ROT ---
        btfss R_MODE,1                 ; wenn nicht Mode mit Dunkeltastung
         goto STATUS_REG_6             ; .. continue
        btfss R_STATUS2,1              ; ROT = Ausgang 2
         goto STATUS_REG_6             ; .. continue
        btfss R_STATE2,1               ; check R_STATE2 #1 (ON)
         goto STATUS_REG_6             ; .. continue
        movlw b'00111100'              ; #0, #1 OUTPUT and #4, #5 TRIS-STATE @ Dunkel-tastung
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0
        goto STATUS_REG_7              ; .. continue

STATUS_REG_6
        ; --- kein TRIS-STATE ---
        movlw b'00001100'              ; #0, #1, #4, #5 OUTPUT
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0

STATUS_REG_7
        ; --- Status speichern ? ---
        call save_status               ; .. 

        movf R_STATUS,W                ; copy R_STATUS into R_STATUS2
        movwf R_STATUS2                ; (fuer Dunkelphase)

signal_1 ;**** LED steuern ****
        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

        ;***** PWM-Time ON *****
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE1,W               ; = off or fade out LED1
        btfss STATUS,Z
         bcf PORTA,1                    ; LED_1 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE2,W               ; = off or fade out LED2
        btfss STATUS,Z
         bcf PORTA,0                    ; LED_2 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE3,W               ; = off or fade out LED3
        btfss STATUS,Z
         bcf PORTA,5                    ; LED_3 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE4,W               ; = off or fade out LED4
        btfss STATUS,Z
         bcf PORTA,4                    ; LED_4 off

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
        goto signal_2                 ; ... then JUMP

        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE1,W               ; = fade in/on LED1
        btfss STATUS,Z
         bsf PORTA,1                    ; LED_1 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE2,W               ; = fade in/on LED2
        btfss STATUS,Z
         bsf PORTA,0                    ; LED_2 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE3,W               ; = fade in/on LED3
        btfss STATUS,Z
         bsf PORTA,5                    ; LED_3 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE4,W               ; = fade in/on LED4
        btfss STATUS,Z
         bsf PORTA,4                    ; LED_4 on

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

signal_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto signal_2                ; ... then JUMP_2

        ; ***** PWM-Time ready *****
        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE1,W               ; = off or fade in LED1
        btfss STATUS,Z
         bcf PORTA,1                    ; LED_1 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE1,W               ; = on or fade out LED1
        btfss STATUS,Z
         bsf PORTA,1                    ; LED_1 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE2,W               ; = off or fade in LED2
        btfss STATUS,Z
         bcf PORTA,0                    ; LED_2 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE2,W               ; = on or fade out LED2
        btfss STATUS,Z
         bsf PORTA,0                    ; LED_2 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE3,W               ; = off or fade in LED3
        btfss STATUS,Z
         bcf PORTA,5                    ; LED_3 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE3,W               ; = on or fade out LED3
        btfss STATUS,Z
         bsf PORTA,5                    ; LED_3 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE4,W               ; = off or fade in LED4
        btfss STATUS,Z
         bcf PORTA,4                    ; LED_4 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE4,W               ; = on or fade out LED4
        btfss STATUS,Z
         bsf PORTA,4                    ; LED_4 on

        return                        ; .. then return
