; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05/06/07/08/09/10 Sven Brandt

; ********************************************************
; History:
; V3.60 - 11.03.2014: in NODECODER: Korrektur zu 'change_status_on' und 'change_status_off'
; V3.48 - LED_ampel: in NODECODER 'change_status_on' und 'change_status_off' hinzu
; V3.47 - Wert fr Phasenlnge aus EEprom
; V3.41 - sterreichversion angepasst Roland Zagler in MODE2
; V3.40 - Fading einstellbar 
; ********************************************************

  ; ****************************
  ; ** nur fuer led_ampel     **
  ; ****************************
R_MAKESLOW    equ  41h          ; Verzgerungs-Counter 1
R_ADD_1       equ  42h          ; PWM Addition 1
R_ADD_2       equ  43h          ; PWM Addition 2

R_COUNT       equ  50h          ; Counter fuer PWM
R_PWMVALUE    equ  51h          ; PWM-Value, bernahme von R_COUNT
R_PHASE       equ  53h          ; Ampelphase (State Machine)
R_DELAY       equ  54h          ; Counter fuer Phasenlaenge
R_DELAY2      equ  55h          ; Counter fuer Phasenlaenge
R_FADEIN1     equ  56h          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEIN2     equ  57h          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEOUT1    equ  58h          ; wenn Bit gesetzt, entsprechende LED ausblenden
R_FADEOUT2    equ  59h          ; wenn Bit gesetzt, entsprechende LED ausblenden
R_BL_CT       equ  04h          ; Anzahl Blinkzyklen grn

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE     h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1       h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_MS       h'20'           ; EEprom Adresse: R_MAKESLOW: Wert fr Fade Geschwindigkeit
#define EE_ADR_T_G1     h'60'           ; EEprom Adresse: Lnge Grn Strae 1
#define EE_ADR_T_G2     h'70'           ; EEprom Adresse: Lnge Grn Strae 2
#define EE_ADR_T_GE     h'71'           ; EEprom Adresse: Lnge GELB Phase
#define EE_ADR_T_RG     h'72'           ; EEprom Adresse: Lnge GELB/ROT Phase
#define EE_ADR_T_RR     h'73'           ; EEprom Adresse: Lnge ROT Phase
#define EE_ADR_T_VG     h'74'           ; EEprom Adresse: Lnge Vorlauf fr Fussgnger vor GRN
#define EE_ADR_T_VR     h'75'           ; EEprom Adresse: Lnge Vorlauf fr Fussgnger vor ROT

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition


; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movf R_MAKESLOW,W              ; ..
        andlw b'00001111'              ; nur untere 4 Bit nehmen
        movwf R_ADD_1                  ; .. und ins Register 
 
        swapf R_MAKESLOW,W             ; ..
        andlw b'00001111'              ; nur obere 4 Bit nehmen
        movwf R_ADD_2                  ; .. und ins Register

        movlw d'1'                     ; Default-Wert, wenn R_ADD_1 = ZERO
        movf R_ADD_1,F
        btfsc STATUS,Z                 ; .. d'1'
         movwf R_ADD_1                 ; .. into R_ADD_1, wenn == 0

        movlw d'1'                     ; Default-Wert, wenn R_ADD_2 = ZERO
        movf R_ADD_2,F
        btfsc STATUS,Z                 ; .. d'1'
         movwf R_ADD_2                 ; .. into R_ADD_2, wenn == 0

        return
        
; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00000010'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'00000100'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'00001000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder ; als NODECODER muss #0 gesetzt sein (RUN-Modus)
        clrf R_STATUS
        bsf R_STATUS,0
#endif ;zu #ifdef nodecoder

        movlw b'00000001'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         goto start_ampel_0            ; .. then OK -> jump
        movlw b'00000010'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         goto start_ampel_1            ; .. then OK -> jump
        movlw b'00000100'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         goto start_ampel_2            ; .. then OK -> jump
        movlw b'00001000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         goto start_ampel_3            ; .. then OK -> jump

        movlw b'00000001'              ;
        movwf R_STATUS                 ; ..into R_STATUS
        bsf R_SAVE_F,0                 ; set Flag for save

start_ampel_0  ;  Startkondition bei "RUN": GRUEN/GELB
        movlw d'88'
        movwf R_PHASE         ; naechster STATE
        bcf PORTA,4           ; ROT   fuer Hauptstrasse = AUS
        bcf PORTA,5           ; GELB  fuer Hauptstrasse = AUS
        bsf PORTA,0           ; GRUEN fuer Hauptstrasse = EIN
        bcf PORTA,1           ; ROT   fuer Nebenstrasse = AUS
        bsf PORTC,1           ; GELB  fuer Nebenstrasse = EIN 
        bcf PORTC,2           ; GRUEN fuer Nebenstrasse = AUS
        bsf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = EIN
        bcf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = AUS
        bsf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = EIN
        bcf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = AUS
        movlw T_S0                     ; Verzoegerungszeit
        movwf R_DELAY
        return
start_ampel_1  ;  Startkondition bei "NACHT": AUS/AUS
        movlw d'66'
        movwf R_PHASE         ; naechster STATE
        bcf PORTA,4           ; ROT   fuer Hauptstrasse = AUS
        bcf PORTA,5           ; GELB  fuer Hauptstrasse = AUS
        bcf PORTA,0           ; GRUEN fuer Hauptstrasse = AUS
        bcf PORTA,1           ; ROT   fuer Nebenstrasse = AUS
        bcf PORTC,1           ; GELB  fuer Nebenstrasse = AUS 
        bcf PORTC,2           ; GRUEN fuer Nebenstrasse = AUS
        bcf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = AUS
        bcf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = AUS
        bcf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = AUS
        bcf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = AUS
        movlw T_S1                     ; Verzoegerungszeit
        movwf R_DELAY
        return
start_ampel_2  ;  Startkondition bei "GRN": GRUEN/ROT
        movlw d'05'
        movwf R_PHASE         ; naechster STATE
        bcf PORTA,4           ; ROT   fuer Hauptstrasse = AUS
        bcf PORTA,5           ; GELB  fuer Hauptstrasse = AUS
        bsf PORTA,0           ; GRUEN fuer Hauptstrasse = EIN
        bsf PORTA,1           ; ROT   fuer Nebenstrasse = EIN
        bcf PORTC,1           ; GELB  fuer Nebenstrasse = AUS
        bcf PORTC,2           ; GRUEN fuer Nebenstrasse = AUS
        bcf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = AUS
        bsf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = EIN
        bsf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = EIN
        bcf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = AUS
        movlw T_S2                     ; Verzoegerungszeit
        movwf R_DELAY
        return
start_ampel_3  ;  Startkondition bei "ROT": ROT/GRUEN
        movlw d'15'
        movwf R_PHASE         ; naechster STATE
        bsf PORTA,4           ; ROT   fuer Hauptstrasse = EIN
        bcf PORTA,5           ; GELB  fuer Hauptstrasse = AUS
        bcf PORTA,0           ; GRUEN fuer Hauptstrasse = AUS
        bcf PORTA,1           ; ROT   fuer Nebenstrasse = AUS
        bcf PORTC,1           ; GELB  fuer Nebenstrasse = AUS
        bsf PORTC,2           ; GRUEN fuer Nebenstrasse = EIN
        bsf PORTC,0           ; FUSSG-ROT   fuer Hauptstrasse = EIN
        bcf PORTC,5           ; FUSSG-GRUEN fuer Hauptstrasse = AUS
        bcf PORTC,4           ; FUSSG-ROT   fuer Nebenstrasse = AUS
        bsf PORTC,3           ; FUSSG-GRUEN fuer Nebenstrasse = EIN
        movlw T_S3                     ; Verzoegerungszeit
        movwf R_DELAY
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Tastendruck (=_off) wechselt Blinkmode, keine Aktion fr _on
change_status_on ; = Taste release
        return

change_status_off; = Taste PUSH
        bsf R_SAVE_F,0                 ; set Flag for save

        bcf STATUS,C                   ; change mode
        rlf R_STATUS,F                 ; ..

        btfsc R_STATUS,4               ; check if it is mode "#4"
         bsf R_STATUS,0                ; .. then change to mode "0"
         bcf R_STATUS,4                ; .. then change to mode "0"               
 
        call status_to_output          ; check if R_STATUS is valid
        return
#endif

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
; Tastendruck wechselt Mode (nur 16f684)
change_mode

#ifdef use16f684
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL
       
;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'00110001'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002 

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: h'00'
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
#endif ; use16f684

        return
