; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006/2007/2008/2009/2011 Sven Brandt

; ********************************************************
; History:
; V3.48 - LED_andreas: in NODECODER 'change_status_on' und 'change_status_off' hinzu
; V3.26 - Dumm-Mode Hinzu
;       - noch "Nachblinken" nach Schrankenffnung
; V3.10 - 'save_01' hinzu
; ********************************************************

; ********************************************************
  ; ****************************
  ; ** nur fuer led_andreas   **
  ; ****************************
R_COUNT       equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE    equ  51h          ; PWM-Value, bernahme von R_COUNT
R_PHASE       equ  53h          ; Ampelphase (State Machine)
R_DELAY       equ  54h          ; Counter fuer Phasenlaenge
R_DELAY2      equ  55h          ; Counter fuer Phasenlaenge
R_FADEIN1     equ  56h          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEOUT1    equ  58h          ; wenn Bit gesetzt, entsprechende LED ausblenden

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

        btfss R_MODE,0                 ; restoring of relays
         return

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        clrf PORTC                     ; alle Schranken-Ausgnge aus
        movlw T_08
        movwf R_DELAY                  ; Verzoegerungszeit, bis Blinklicht/Ampel aus geht

        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         movf PORTA,W
         xorlw b'00110011'              ; kein ON fr Schranken-Anschluesse
         movwf PORTA
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder   ; als NODECODER muss #0 gesetzt sein (RUN-Modus)
        clrf R_STATUS                  ; RUN-Mode,
        bsf R_STATUS,0                 ; .. aber mit allen Start-Funktionen
        return                         ; .. deshalb hier RETURN
#endif ;zu #ifdef nodecoder

        movlw b'00000001'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         goto start_andreas_0          ; .. then -> jump
        return

start_andreas_0  ;  
        movlw d'1'                     ; Verzoegerungszeit minimal
        movwf R_DELAY

        bsf R_FADEIN1,2                ; LED 3 (Ampel ROT) einschalten

        movlw d'07'                    ; STATE "eingeschaltet & eingeschwungen"
        movwf R_PHASE                  ; (Start ohne die Start-Funktionen)
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Bettigung des Tasters wirkt wie ein Weichen-Befehl
change_status_on
        bsf R_STATUS,0                 ; STATUS Bit#0 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call gruen_1
        return

change_status_off
        bcf R_STATUS,0                 ; STATUS Bit#1 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call rot_1 
        return
#endif

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
; Tastendruck wechselt Mode
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL
        
;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'00110011'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002 

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
