; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006/07/11 Sven Brandt

; ********************************************************
; History:
; V3.60 - 11.03.2014: in NODECODER: Korrektur zu 'change_status_on' und 'change_status_off'
; V3.48 - LED_blink:  in NODECODER 'change_status_on' und 'change_status_off' hinzu
; V3.08 - LED-Dekoder_blink: bei NODECODER Fehler beim Weiterschalten behoben
; ********************************************************

  ; ****************************
  ; ** nur fuer led_blink     **
  ; ****************************
R_COUNT       equ  50h          ; Counter fuer PWM - LED1
R_FLAG_F1     equ  51h          ; flags double-flash 1-8
R_FLAG_F2     equ  52h          ; flags double-flash 9-10
R_STATE1      equ  54h          ; Status-Flags 1 - LED1
R_STATE2      equ  55h          ; Status-Flags 2 - LED2
R_STATE3      equ  56h          ; Status-Flags 3 - LED3
R_STATE4      equ  57h          ; Status-Flags 4 - LED4
R_STATE5      equ  58h          ; Status-Flags 5 - LED5
R_STATE6      equ  59h          ; Status-Flags 6 - LED6
R_STATE7      equ  5ah          ; Status-Flags 7 - LED7
R_STATE8      equ  5bh          ; Status-Flags 8 - LED8
R_STATE9      equ  5ch          ; Status-Flags 9 - LED9
R_STATE10     equ  5dh          ; Status-Flags 10 - LED10
R_DELAY       equ  5eh          ; register for "fade out" @ foto-mode

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00000010'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'00000100'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'00001000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#5 ***
; ********************
gruen_5
        movlw b'00010000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#6 ***
; ********************
gruen_6
        movlw b'00100000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#7 ***
; ********************
gruen_7
        movlw b'01000000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#8 ***
; ********************
gruen_8
        movlw b'10000000'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder  ; auch als NODECODER sind verschiedene Blinkmodi mglich
        movlw b'00000001'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00000010'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00000100'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00001000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00010000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00100000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'01000000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'10000000'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump

        movlw b'00000001'              ;
        movwf R_STATUS                 ; ..into R_STATUS
        bsf R_SAVE_F,0                 ; set Flag for save
#endif
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Tastendruck (=_off) wechselt Blinkmode, keine Aktion fr _on
change_status_on ; = Taste release
        return

change_status_off; = Taste PUSH
        bsf R_SAVE_F,0                 ; set Flag for save

        bcf STATUS,C                   ; change mode
        rlf R_STATUS,F                 ; ..

        btfsc R_STATUS,7               ; check if it is mode "#7"
         bsf R_STATUS,0                ; .. then change to mode "0"
         bcf R_STATUS,7                ; .. then change to mode "0"               
 
        call status_to_output          ; check if R_STATUS is valid
        return
#endif
