; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2011 Sven Brandt

; ********************************************************
; History:
; V3.48 - LED_gas: in NODECODER Starten und Stoppen per Taster, fr DCC/MM durch skip_g2 ff. Platzbedarf reduziert
; V3.07 - LED-Dekoder_gas: jetzt mit Gasdruckschwankungen (nderungen nur in _SUB und _STATS)
; ********************************************************

  ; ****************************
  ; ** nur fuer led_gas       **
  ; ****************************
R_STATE1      equ  40h          ; State Output #1
R_STATE2      equ  41h          ; State Output #2
R_STATE3      equ  42h          ; State Output #3
R_STATE4      equ  43h          ; State Output #4
R_STATE5      equ  44h          ; State Output #5
R_STATE6      equ  45h          ; State Output #6
R_STATE7      equ  46h          ; State Output #7
R_STATE8      equ  47h          ; State Output #8
R_STATE9      equ  48h          ; State Output #9
R_STATE10     equ  49h          ; State Output #10
R_VERZ1       equ  4ah          ; Verzgerungs-Counter #1
R_VERZ2       equ  4bh          ; Verzgerungs-Counter #2
R_VERZ3       equ  4ch          ; Verzgerungs-Counter #3
R_VERZ4       equ  4dh          ; Verzgerungs-Counter #4
R_VERZ5       equ  4eh          ; Verzgerungs-Counter #5
R_VERZ6       equ  4fh          ; Verzgerungs-Counter #6
R_VERZ7       equ  50h          ; Verzgerungs-Counter #7
R_VERZ8       equ  51h          ; Verzgerungs-Counter #8
R_VERZ9       equ  52h          ; Verzgerungs-Counter #9
R_VERZ10      equ  53h          ; Verzgerungs-Counter #10
R_PWM1        equ  54h          ; LED-PWM #1
R_PWM2        equ  55h          ; LED-PWM #2
R_PWM3        equ  56h          ; LED-PWM #3
R_PWM4        equ  57h          ; LED-PWM #4
R_PWM5        equ  58h          ; LED-PWM #5
R_PWM6        equ  59h          ; LED-PWM #6
R_PWM7        equ  5ah          ; LED-PWM #7
R_PWM8        equ  5bh          ; LED-PWM #8
R_PWM9        equ  5ch          ; LED-PWM #9
R_PWM10       equ  5dh          ; LED-PWM #10
R_VERZ_A      equ  5eh          ; 2. Verzgerungs-Zhler (Gasdruckschwankung)
R_VERZ_B      equ  5fh          ; 2. Verzgerungs-Zhler (Gasdruckschwankung)

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

        movlw d'20'                    ; laenge des FLASH
        movwf R_VERZ1
        movwf R_VERZ2
        movwf R_VERZ3
        movwf R_VERZ4
        movwf R_VERZ5
        movwf R_VERZ6
        movwf R_VERZ7
        movwf R_VERZ8
        movwf R_VERZ9
        movwf R_VERZ10

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzen
        btfsc R_MODE,1                 ; Skip EINSCHALTBLITZ in Mode 2
         movlw b'00000010'
        btfsc R_MODE,3                 ; Skip EINSCHALTBLITZ in Mode 4
         movlw b'00000010'
        movwf R_STATE1                 ; change R_STATEx
        movwf R_STATE2                 ; change R_STATEx
        movwf R_STATE3                 ; change R_STATEx
        movwf R_STATE4                 ; change R_STATEx
        movwf R_STATE5                 ; change R_STATEx
        movwf R_STATE6                 ; change R_STATEx
        movwf R_STATE7                 ; change R_STATEx
        movwf R_STATE8                 ; change R_STATEx
        movwf R_STATE9                 ; change R_STATEx
        movwf R_STATE10                ; change R_STATEx
        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        movlw b'00001000' 
        movwf R_STATE1                 ; change R_STATEx
        movwf R_STATE2                 ; change R_STATEx
        movwf R_STATE3                 ; change R_STATEx
        movwf R_STATE4                 ; change R_STATEx
        movwf R_STATE5                 ; change R_STATEx
        movwf R_STATE6                 ; change R_STATEx
        movwf R_STATE7                 ; change R_STATEx
        movwf R_STATE8                 ; change R_STATEx
        movwf R_STATE9                 ; change R_STATEx
        movwf R_STATE10                ; change R_STATEx
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        comf R_TGLA,W
        andlw b'00110011'
        movwf PORTA
        movwf R_TGLA
        comf R_TGLC,W
        andlw b'00111111'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        movlw b'00010001'
        movwf PORTA
        movwf R_TGLA
        movlw b'00010011'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        clrf R_TGLA
        clrf R_TGLC
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder ; starten mit Tater-Routine
        return
#endif ;zu #ifdef nodecoder

        btfsc R_STATUS,0               ; wenn R_STATUS#0 gesetzt, dann
         call gruen_1
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Bettigung des Tasters wirkt wie ein Weichen-Befehl
change_status_on
        bsf R_STATUS,0                 ; STATUS Bit#0 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call gruen_1
        return

change_status_off
        bcf R_STATUS,0                 ; STATUS Bit#1 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call rot_1 
        return
#endif

;*********************************
;** Routine: change_mode        **
;** invert R_MODE and save      **
;** into EEPROM $00             **
;*********************************
change_mode
#ifndef dummy
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        btfss R_MODE,1                 ; change from #1 -> #2
         goto $+4
         clrf R_MODE
         bsf R_MODE,2
         goto change_mode_001

        btfss R_MODE,2                 ; change from #1 -> #2
         goto $+4
         clrf R_MODE
         bsf R_MODE,3
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        btfsc R_CNT2_TGL,7             ; if CNT2 > 124
         bsf PORTA,4                   ; .. -> LED 1 on
        btfss R_CNT2_TGL,7             ; if CNT2 < 124
         bcf PORTA,4                   ; .. -> LED 1 off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
#endif; dummy
