; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2011 Sven Brandt

; ********************************************************
; History:
; ********************************************************
; V3.60 - 11.03.2014 - beim Digital-Befehl EIN wird auch Ausgang 1 mit eingeschaltet
; V3.48 - LED_haus: in NODECODER Starten und Stoppen per Taster, fr DCC/MM durch skip_g2 ff. Platzbedarf reduziert
; V3.64 - 12.04.2017 - Shuffle-Funktion auch im IDLE, Laden und schreiben eines Shuffle Wertes aus dem EE 0x70
; ********************************************************

  ; ****************************
  ; ** nur fuer led_haus      **
  ; ****************************
R_TEMP        equ  40h          ;

R_VERZ        equ  41h          ; Verzgerungs-Counter 1
R_VERZ2       equ  42h          ; Verzgerungs-Counter 2
R_MAKESLOW    equ  43h          ; Verzoegerung aus dem EEPROM;

R_MINOUT      equ  44h          ; Mindest-Anzahl der eingeschalteten Ausgnge
R_ANZOUT      equ  45h          ; Anzahl der eingeschalteten Ausgnge (SOLL)
R_ANZOUT_OLD  equ  46h          ; Anzahl der eingeschalteten Ausgnge (IST)
R_ANZCNG      equ  47h          ; Zhler: wieviele Ausgnge haben einen neuen Status

R_COUNT       equ  48h          ; Counter fuer PWM
R_PWMVALUE    equ  49h          ; PWM-Value, bernahme von R_COUNT
R_FADEIN1     equ  4ah          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEIN2     equ  4bh          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEOUT1    equ  4ch          ; wenn Bit gesetzt, entsprechende LED ausblenden
R_FADEOUT2    equ  4dh          ; wenn Bit gesetzt, entsprechende LED ausblenden

R_CNT_SHUTDOWN equ  4eh         ; Counter nach Shutdown: alle Ausgnge abschalten nach Couter-Ablauf

;R_CNT4        equ  4fh          ; Verzgerungs-Counter
;R_CNT5        equ  50h          ; Verzgerungs-Counter
;R_CNT6        equ  51h          ; Verzgerungs-Counter
;R_CNT7        equ  52h          ; Verzgerungs-Counter

R_FLAGS        equ  54h          ; verschiedene Flags
        #define F_STARTUP   R_FLAGS,0
        #define F_RUNNING   R_FLAGS,1
        #define F_SHUTDOWN  R_FLAGS,2

R_SHUFFLE1    equ  55h          ; Zufalls-Wert
R_SHUFFLE2    equ  56h          ; Zufalls-Wert
R_SHUFFLE3    equ  57h          ; Zufalls-Wert
R_SHUFFLE4    equ  58h          ; Zufalls-Wert
;R_SHUFFLE5    equ  59h          ; Zufalls-Wert
;R_SHUFFLE6    equ  5ah          ; Zufalls-Wert
R_ONOFF1      equ  5bh          ; Status ON/OFF (Flags fr Ausgang 1..8 )
R_ONOFF2      equ  5ch          ; Status ON/OFF (Flags fr Ausgang 9..10)
R_ONOFF1_OLD  equ  5dh          ; Status ON/OFF (Flags fr Ausgang 1..8 ) fr Restore
R_ONOFF2_OLD  equ  5eh          ; Status ON/OFF (Flags fr Ausgang 9..10) fr Restore
R_SHUFFLE     equ  5fh          ; bergabe-Register fr Shuffle-Funktion


;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE    h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1      h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_MS      h'20'           ; EEprom Adresse: R_MAKESLOW: Wert fr Fade Geschwindigkeit
#define EE_ADR_MO      h'30'           ; EEprom Adresse: R_MINOUT: Wert fr Mindest-Anzahl aktiver Outputs
#define EE_ADR_SCH1    h'70'           ; EEprom Adresse: Schuffle Start Wert

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition

;   ----------------------------------------------------
;   --- Parameter
;   ----------------------------------------------------
CONST_FADE_01          equ d'120'        ; Fade Geschwindigkeit @ Signalbild-Wechsel
CONST_BREMS            equ d'100'        ; zustzliche "Bremse"
CONST_SHUTDOWN         equ d'20'         ; TimeOut fr Abschalten alles Ausgnge nach OFF

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

;   --------------------------------------
;   --- read R_MINOUT from EEPROM $2F  ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MO
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MINOUT                 ; ..into R_MINOUT

;   --------------------------------------
;   --- read R_SHUFFLE
;   --------------------------------------
        movlw b'01000100'
        movwf R_SHUFFLE1

        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_SCH1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_SHUFFLE2               ; ..into R_FLAGS

        addwf TMR1L,W
        movwf R_SHUFFLE3               ; R_SHUFFLE3 = R_SHUFFLE2 + TME1L (variiert ggf. durch ISR)

        addwf R_SHUFFLE3,W
        addwf R_SHUFFLE1,W
        movwf R_SHUFFLE4

;   --------------------------------------
;   --- write R_SHUFFLE
;   --------------------------------------
        movlw EE_ADR_SCH1
        movwf R_EEADR                  ; Address: h'10'
        movf R_SHUFFLE3,W
        movwf R_EEDAT                  ; Data: R_SHUFFLE3
        call saveroutine

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw d'2'
        movwf R_VERZ
        movwf R_VERZ2

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten

        movlw d'1'
        movwf R_VERZ                   ; Wert 1 -> schnelle Reaktion
        movwf R_VERZ2                  ; Wert 1 -> schnelle Reaktion
        movwf R_COUNT

        bsf F_STARTUP
        bcf F_RUNNING
        bcf F_SHUTDOWN

        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        ;bcf R_ONOFF2,1                  ; .. Ausgang 10 AUS
        movlw d'1'
        movwf R_VERZ                   ; Wert 1 -> schnelle Reaktion
        movwf R_VERZ2                  ; Wert 1 -> schnelle Reaktion
;        movwf R_COUNT

        bcf F_STARTUP
        bcf F_RUNNING
        bsf F_SHUTDOWN
#ifdef test
    call toggle_led_1
#endif
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        comf R_TGLA,W
        andlw b'00110011'
        movwf PORTA
        movwf R_TGLA
        comf R_TGLC,W
        andlw b'00111111'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        movlw b'00010001'
        movwf PORTA
        movwf R_TGLA
        movlw b'00010011'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        clrf R_TGLA
        clrf R_TGLC
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder ; starten mit Tater-Routine
        return
#endif ;zu #ifdef nodecoder

        btfsc R_STATUS,0               ; wenn R_STATUS#0 gesetzt, dann
         call gruen_1
        return


;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Bettigung des Tasters wirkt wie ein Weichen-Befehl
change_status_on
        bsf R_STATUS,0                 ; STATUS Bit#0 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call gruen_1
        return

change_status_off
        bcf R_STATUS,0                 ; STATUS Bit#1 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call rot_1 
        return
#endif

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
#ifndef dummy
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'11111111'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTC                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTC                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
#endif; dummy

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: toggle_xxx         **
;*********************************
;*********************************
;*********************************
;*********************************

#ifdef test
toggle_led_1  ; test routine
     movf PORTA,W ;test
     xorlw b'00010000' ;test        ; toggle PORTA,4
     movwf PORTA ;test
     return

toggle_led_2  ; test routine
     movf PORTA,W ;test
     xorlw b'00100000' ;test        ; toggle PORTA,5
     movwf PORTA ;test
     return

toggle_led_3  ; test routine
     movf PORTA,W ;test
     xorlw b'00000001' ;test        ; toggle PORTA,0
     movwf PORTA ;test
     return

toggle_led_4  ; test routine
     movf PORTA,W ;test
     xorlw b'00000010' ;test        ; toggle PORTA,1
     movwf PORTA ;test
     return

#endif ; test

