; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006/07/11 Sven Brandt

; ********************************************************
; History:
; V3.60 - 11.03.2014: in NODECODER: Korrektur zu 'change_status_on' und 'change_status_off'
; V3.48 - LED_kirmes: in NODECODER 'change_status_on' und 'change_status_off' hinzu, fr DCC/MM durch skip_g5 ff. Platzbedarf reduziert
; V3.08 - LED-Dekoder_blink: bei NODECODER Fehler beim Weiterschalten behoben
; ********************************************************

  ; ****************************
  ; ** nur fuer led_kirmes    **
  ; ****************************
R_VERZ        equ  41h          ; Verzgerungs-Counter 1
R_MAKESLOW    equ  42h          ; Verzoegerung aus dem EEPROM;

R_POINTER1    equ  50h          ; Pointer fuer Tabellenwert (unterer State)
R_POINTER2    equ  51h          ; Counter fuer Tabellenwert (oberer State)
R_EFFECT      equ  52h          ; Flags fr LED Effect
; #7-5 : Speed of effect (x111 xxxx = slowest, x000 xxx = fastest)
; #4: 0= Counting Down, 1= Counting Up (Effect running "left" / "right")
; #3-0 : Number of Runnings with this effect (xxxx 1111 = 16 runs)
R_EFFECT_NR   equ  54h          ; EFFECT-Number
R_TIME        equ  55h          ; Register Timing Increment
R_TIME_L      equ  56h          ; Register Timing Load

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt
        movlw b'00000010'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt
        movlw b'00000100'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzten
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt

        clrf R_STATUS                  ; alles ausschalten
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        return

; ************************
; *** save_toggle_next ***
; ************************
save_toggle_next
        clrf R_TGLA
        clrf R_TGLC
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder  ; auch als NODECODER sind verschiedene Blinkmodi mglich
        movlw b'00000001'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00000010'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump
        movlw b'00000100'              ;
        subwf R_STATUS,W
        btfsc STATUS,Z                 ; if the same
         return                        ; .. then OK -> jump

        movlw b'00000001'              ;
        movwf R_STATUS                 ; ..into R_STATUS
        bsf R_SAVE_F,0                 ; set Flag for save
#endif
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Tastendruck (=_off) wechselt Blinkmode, keine Aktion fr _on
change_status_on ; = Taste release
        return

change_status_off; = Taste PUSH
        bsf R_SAVE_F,0                 ; set Flag for save

        bcf STATUS,C                   ; change mode
        rlf R_STATUS,F                 ; ..

        btfsc R_STATUS,3               ; check if it is mode "#3"
         bsf R_STATUS,0                ; .. then change to mode "0"
         bcf R_STATUS,3                ; .. then change to mode "0"

        movlw d'01'                    ; init mit '01'
        movwf R_TIME                   ; -> sofort ausfhren
        movwf R_EFFECT                 ; -> letzter Durchlauf
        movlw h'ee'                    ; init mit 'ee'
        movwf R_POINTER1               ; -> berlauf -> neue Effekt-Stufe
        movwf R_POINTER2               ; -> berlauf -> neuer Effekt

        return
#endif
