; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2005-2011 Sven Brandt

; ********************************************************
; History:
; ********************************************************
; V3.48 - LED_Neon: Ausgnge konfigurierbar, in NODECODER Starten und SToppen per Taster
; V3.38 - LED_neon: - Ausgang 10 "Schweisslicht" und 9 "defekt" auch als normale Neon-Ausgnge nutzbar (Flags in EE $20)
; V3.64 - 12.04.2017 - Laden und schreiben eines Shuffle Wertes aus dem EE 0x70

  ; ****************************
  ; ** nur fuer led_neon      **
  ; ****************************
R_TEMP        equ  40h          ;
R_START01     equ  41h          ; Startverzgerung
R_START02     equ  42h          ; Startverzgerung
R_START03     equ  43h          ; Startverzgerung
R_START04     equ  44h          ; Startverzgerung
R_START05     equ  45h          ; Startverzgerung
R_START06     equ  46h          ; Startverzgerung
R_START07     equ  47h          ; Startverzgerung
R_START08     equ  48h          ; Startverzgerung
R_START09     equ  49h          ; Startverzgerung
R_START10     equ  4ah          ; Startverzgerung         , kann selbe location wie R_STAT10 haben

R_CNT1        equ  4bh          ; Verzgerungs-Counter
R_CNT2        equ  4ch          ; Verzgerungs-Counter
R_CNT3        equ  4dh          ; Verzgerungs-Counter
R_CNT4        equ  4eh          ; Verzgerungs-Counter
R_CNT5        equ  4fh          ; Verzgerungs-Counter
R_CNT6        equ  50h          ; Verzgerungs-Counter
R_CNT7        equ  51h          ; Verzgerungs-Counter
R_CNT8        equ  52h          ; Verzgerungs-Counter
R_CNT9        equ  53h          ; Verzgerungs-Counter
R_CNT10       equ  54h          ; Verzgerungs-Counter
R_SHUFFLE1    equ  55h          ; Zufalls-Wert
R_SHUFFLE2    equ  56h          ; Zufalls-Wert
R_SHUFFLE3    equ  57h          ; Zufalls-Wert
R_SHUFFLE4    equ  58h          ; Zufalls-Wert
R_SHUFFLE5    equ  59h          ; Zufalls-Wert
R_SHUFFLE6    equ  5ah          ; Zufalls-Wert
R_SHUFFLE7    equ  5bh          ; Zufalls-Wert

R_ONOFF1      equ  5ch          ; Status ON/OF
R_ONOFF2      equ  5dh          ; Status ON/OF

R_SHUFFLE     equ  5eh          ;
R_FLAGS       equ  5fh          ; Flags, read from EEProm
 #define      F_out09_defekt       R_FLAGS,0
 #define      F_out10_schweiss     R_FLAGS,1
 #define      F_out01_schweiss     R_FLAGS,2
 #define      F_out02_schweiss     R_FLAGS,2
 #define      F_out03_schweiss     R_FLAGS,2
 #define      F_out04_schweiss     R_FLAGS,3
 #define      F_out05_schweiss     R_FLAGS,4
 #define      F_out06_schweiss     R_FLAGS,4
 #define      F_out07_schweiss     R_FLAGS,4
 #define      F_out08_schweiss     R_FLAGS,4
 #define      F_out01_defekt       R_FLAGS,5
 #define      F_out09_schweiss     R_FLAGS,6
 #define      F_out04_defekt       R_FLAGS,7

;----------------------------------------------------
; Mapping LED-Nummer zu Port
;----------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition


;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_RF    h'20'           ; EEprom Adresse: R_FLAGS

#define EE_ADR_SCH1    h'70'           ; EEprom Adresse: Schuffle Start Wert
#define EE_ADR_SCH2    h'71'           ; EEprom Adresse: Schuffle Start Wert

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- read R_FLAGS from EEPROM $20  ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_RF
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_FLAGS                  ; ..into R_FLAGS

;   --------------------------------------
;   --- read R_SHUFFLE
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_SCH1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_SHUFFLE2               ; ..into R_FLAGS

        addwf TMR1L,W
        movwf R_SHUFFLE3               ; R_SHUFFLE3 = R_SHUFFLE2 + TME1L (variiert ggf. durch ISR)

        addwf R_SHUFFLE3,W
        movwf R_SHUFFLE4
        addwf R_SHUFFLE2,W
        movwf R_SHUFFLE7
        addwf R_SHUFFLE4,W
        movwf R_SHUFFLE6
        addwf R_SHUFFLE2,W
        movwf R_SHUFFLE5
        addwf R_SHUFFLE4,W
        movwf R_SHUFFLE1

;   --------------------------------------
;   --- write R_SHUFFLE
;   --------------------------------------
        movlw EE_ADR_SCH1
        movwf R_EEADR                  ; Address: h'10'
        movf R_SHUFFLE3,W
        movwf R_EEDAT                  ; Data: R_SHUFFLE3
        call saveroutine

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'
        movwf R_STATUS                 ; nur 1 STATUS-Flag setzen

        movlw d'4'
        movwf R_START02                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START05                ; Wert in R_STARTx = Lnge der Starter-Flackerns

        movlw d'5'
        movwf R_START04                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START06                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START10                ; shuffled R_SHUFFLE into R_STARTx

        movlw d'7'
        movwf R_START01                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START03                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START07                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START08                ; Wert in R_STARTx = Lnge der Starter-Flackerns
        movwf R_START09                ; Wert in R_STARTx = Lnge der Starter-Flackerns

        btfsc F_out01_schweiss         ; nur bei Schweisslicht
         bsf R_START01,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out02_schweiss         ; nur bei Schweisslicht
         bsf R_START02,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out03_schweiss         ; nur bei Schweisslicht
         bsf R_START03,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out04_schweiss         ; nur bei Schweisslicht
         bsf R_START04,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out05_schweiss         ; nur bei Schweisslicht
         bsf R_START05,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out06_schweiss         ; nur bei Schweisslicht
         bsf R_START06,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out07_schweiss         ; nur bei Schweisslicht
         bsf R_START07,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out08_schweiss         ; nur bei Schweisslicht
         bsf R_START08,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out09_schweiss         ; nur bei Schweisslicht
         bsf R_START09,7               ; .. #7 setzen (Zeitverzgerung)
        btfsc F_out10_schweiss         ; nur bei Schweisslicht
         bsf R_START10,7               ; .. #7 setzen (Zeitverzgerung)

        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        clrf PORTA
        clrf PORTC                     ; clear Ports
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        comf R_TGLA,W
        andlw b'00110011'
        movwf PORTA
        movwf R_TGLA
        comf R_TGLC,W
        andlw b'00111111'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        movlw b'00010001'
        movwf PORTA
        movwf R_TGLA
        movlw b'00010011'
        movwf PORTC
        movwf R_TGLC
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        clrf R_TGLA
        clrf R_TGLC
        return


;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
#ifdef nodecoder ; 
        ; Reaktion auf Taste: drcken (Port=L) -> AUS, Loslassen (Port=H) -> AN
        btfss PORTA,3                  ; check Button (released = H)
         decf R_BUTTON,F               ; .. bei L (Button=pressed) do this line (R_BUTTON auf FF setzen)
                                       ;  -> Erkennung des Tasten-Status in "Read Input BUTTON" in jedem Fall
#endif ;zu #ifdef nodecoder

    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Durch Aufruf der Routine "gruen_1" etc. werden diese in die
    ; Signalbilder umgewandelt.
#ifndef skip_g1
        btfsc R_STATUS,0
         call gruen_1
#endif

        return                         ; .. then OK -> jump

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_status      **
;** change R_STATUS on switch   **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef nodecoder
; Bettigung des Tasters wirkt wie ein Weichen-Befehl
change_status_on
        bsf R_STATUS,0                 ; STATUS Bit#0 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call gruen_1
        return

change_status_off
        bcf R_STATUS,0                 ; STATUS Bit#1 on
        bsf R_SAVE_F,0                 ; set Flag for save
        call rot_1 
        return
#endif
