; ********************************************************
; ** Version: V3.19 vom 07/12/2008                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008 Sven Brandt

; ********************************************************
; History:
; V3.09 - vor allen Signalbildern jetzt Dunkelphase
; ********************************************************
  ; ****************************
  ; ** nur fuer led_signal4   **
  ; ****************************
; Adresse -{CHECK}-> R_STATUS->R_STATUS_a/b -{STATEMACHINE}-> R_STATUS_a/b2 -{STATEMACHINE}-> R_STATEx -{LOOP}-> LEDs  

; in R_STATUS sind die Signalstellungen abgelegt.
; in R_STATUS_a bzw. R_STATUS_b sind die LED-Stellungen abgelegt (war ntig wg. Dunkelphase).

; In R_STATUS_a/b und R_STATUS_a/b2 sind die Ausgnge als Flags vergeben:
; _a #0: 1= Ausgang 1
; _a #1: 1= Ausgang 2
; _a #2: 1= Ausgang 3
; _a #3: 1= Ausgang 4
; _a #4: 1= Ausgang 5
; _a #5: 1= Ausgang 6
; _a #6: 1= Ausgang 7
; _a #7: 1= Ausgang 8
; _b #0: 1= Ausgang 9
; _b #1: 1= Ausgang 10

; Im Gegensatz zu "led_signal4" oder "wz_signal" oder "wz_signal4" arbeiten wir hier mit
; ROTen und GRNen Adressen. Dadurch lassen sich 5 x 2er Signale in einem Status-Register
; abspeichern und es mssen nur 5 Adressen (hier = Paare) gelernt, abgespeichert und
; berprft werden. Um dies zu realisieren, mussten jedoch R_STATUS_a und R_STATUS_b
; eingefhrt werden. Ein direktes Arbeiten mit den R_STATUS Flags
; ist nicht mehr mglich, da hier jetzt die Zurodnung zu den LEDs nicht 
; mehr stimmt.
; Beispiel:
; ALT: (R_STATUS-Flags sind hier zum Teil exklusiv)
; R_STATUS#0=1 (gesetzt ber G1) -> Signalbild 'HP00'
; R_STATUS#1=1 (gesetzt ber G2) -> Signalbild 'HP1'

; NEU: (R_STATUS-Flags sind NICHT exclusiv)
; R_STATUS#0=1 (s/r ber G1/R1) -> Signal 1 auf ROT, d.h. LED0 = ein, LED1 = aus
; R_STATUS#1=1 (s/r ber G2/R2) -> Signal 2 auf ROT, d.h. LED2 = ein, LED3 = aus usw.

; gearbeitet werden kann. 
; Bit 0 von R_STATUS beeinflusst die LEDs 0 und 1, d.h. R_STATUS_a#0 und#1.
; Beispiel: 
; R_STATUS = xxxx xxx1 -> R_STATUS_a = xxxx xx01
; R_STATUS = xxxx xxx0 -> R_STATUS_a = xxxx xx10 usw.

;
R_STATUS_a    equ  4eh
R_STATUS_b    equ  4fh
R_COUNT       equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE    equ  51h          ; PWM-Value, bernahme von R_COUNT
R_STATUS_a2   equ  52h
R_STATUS_b2   equ  53h
R_STATE1      equ  54h          ; Status-Flags 1 - LED1
R_STATE2      equ  55h          ; Status-Flags 2 - LED2
R_STATE3      equ  56h          ; Status-Flags 3 - LED3
R_STATE4      equ  57h          ; Status-Flags 4 - LED4
R_STATE5      equ  58h          ; Status-Flags 5 - LED5
R_STATE6      equ  59h          ; Status-Flags 6 - LED6
R_STATE7      equ  5ah          ; Status-Flags 7 - LED7
R_STATE8      equ  5bh          ; Status-Flags 8 - LED8
R_STATE9      equ  5ch          ; Status-Flags 9 - LED9
R_STATE10     equ  5dh          ; Status-Flags 10 - LED10

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3
        bsf R_STATE2,3
        bsf R_STATE3,3
        bsf R_STATE4,3
        bsf R_STATE5,3
        bsf R_STATE6,3
        bsf R_STATE7,3
        bsf R_STATE8,3
        bsf R_STATE9,3
        bsf R_STATE10,3
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bcf R_STATUS_a2,1
        bsf R_STATUS_a,0
        bcf R_STATUS_a,1
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bcf R_STATUS_a2,3
        bsf R_STATUS_a,2
        bcf R_STATUS_a,3
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        bcf R_STATUS_a2,5
        bsf R_STATUS_a,4
        bcf R_STATUS_a,5
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        bcf R_STATUS_a2,7
        bsf R_STATUS_a,6
        bcf R_STATUS_a,7
        return

; ********************
; *** ADR Gruen-#5 ***
; ********************
gruen_5
        bcf R_STATUS_b2,1
        bsf R_STATUS_b,0
        bcf R_STATUS_b,1
        return

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bcf R_STATUS_a2,0
        bsf R_STATUS_a,1
        bcf R_STATUS_a,0
        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        bcf R_STATUS_a2,2
        bsf R_STATUS_a,3
        bcf R_STATUS_a,2
        return
 
; ******************
; *** ADR Rot-#3 ***
; ******************
rot_3
        bcf R_STATUS_a2,4
        bsf R_STATUS_a,5
        bcf R_STATUS_a,4
        return

; ******************
; *** ADR Rot-#4 ***
; ******************
rot_4
        bcf R_STATUS_a2,6
        bsf R_STATUS_a,7
        bcf R_STATUS_a,6
        return

; ******************
; *** ADR Rot-#5 ***
; ******************
rot_5
        bcf R_STATUS_b2,0
        bsf R_STATUS_b,1
        bcf R_STATUS_b,0
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        movlw b'00110000'             ; Ausgang 1-2
        xorwf PORTA,F
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
        movlw b'00000011'             ; Ausgang 3-4
        xorwf PORTA,F
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
        movlw b'00000110'             ; Ausgang 5-6
        xorwf PORTC,F
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
        movlw b'00100001'             ; Ausgang 7-8
        xorwf PORTC,F
        return

; ********************
; *** Adresse #5   ***
; ********************
save_05
        movlw b'00011000'             ; Ausgang 9-10
        xorwf PORTC,F
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         call save_01          ; Signalbild 1 einschalten
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Beispiel: 
    ; R_STATUS = xxxx xxx1 -> R_STATUS_a = xxxx xx01
    ; R_STATUS = xxxx xxx0 -> R_STATUS_a = xxxx xx10 usw.

        btfsc R_STATUS,0
         bsf R_STATUS_a,0
        btfss R_STATUS,0
         bsf R_STATUS_a,1

        btfsc R_STATUS,1
         bsf R_STATUS_a,2
        btfss R_STATUS,1
         bsf R_STATUS_a,3

        btfsc R_STATUS,2
         bsf R_STATUS_a,4
        btfss R_STATUS,2
         bsf R_STATUS_a,5

        btfsc R_STATUS,3
         bsf R_STATUS_a,6
        btfss R_STATUS,3
         bsf R_STATUS_a,7

        btfsc R_STATUS,4
         bsf R_STATUS_b,0
        btfss R_STATUS,4
         bsf R_STATUS_b,1

        movf R_STATUS_a,W
        movwf R_STATUS_a2
        movf R_STATUS_b,W
        movwf R_STATUS_b2

        return
