; ********************************************************
; ** Version: V3.22 vom 02/02/2009                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09 Sven Brandt
; ********************************************************
; History:
; V3.22 - first Version

;Mit diesem Include-File knnen Signale konfiguriert werden:
; - Fr LED-Dekoder (idR. mit 16F684)
; - 2 unabhngige Signale mglich
; - jedes Signalbild reagiert auf einen Digital-Befehl (keine Paar-Bildung)
; - 2. Signal kann vom 2. dunkelgetastet werden
; - Blinken einiger Ausgnge (1/2/4/5/6/9/10) mglich
; - Wechselblinken Ausgang 4/5 und 9/10 mglich
; - Signal 1 kann auf Ausgnge 1-8  zugreifen 
; - Signal 2 kann auf Ausgnge 3-10 zugreifen 
; - konfigurierbar, ob das erste Signal auf 4/5/6/7 oder 8 Befehle reagiert 

; ================> Adresse 
; -{CHECK}--------> R_STATUS ->R_STATUS_ON_S1/2_pre 
; -{STATEMACHINE}-> R_STATUS_ON_S1/2_akt 
; -{STATEMACHINE}-> R_STATEx 
; -{LOOP}---------> LEDs  

; in R_STATUS sind die Signalstellungen abgelegt.
; in R_STATUS_ON_S1_xxx bzw. R_STATUS_ON_S2_xxx sind die LED-Stellungen abgelegt.

; In R_STATUS_ON_S1/2_pre/akt und R_STATUS_ON_S1/2_pre/akt sind die 
; Ausgnge als Flags vergeben:

; In dieser SW wird (wie bei der "LED_SIGNAL2") mit den Zwischen-Registern
; R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gearbeitet. In dieses zeigen die Flags an, ob
; ein Ausgang gesetzt / blinkend / aus sein soll. 
; Im Gegensatz zu LED_SIGNAL2 wird hier jedoch nicht mit ROTen und GRNen Adressen
; gearbeitet, sondern es werden 8 GRNe Adressen verwendet.
; Die Signal-Bilder werden in R_STATUS abgelegt (8 Adressen = 8 Bits). In R_STATUS
; liegt demnach die letzte bettigte Adresse und damit das anzuzeigene Signal-Bild.

; In den Routinen "gruen_1" usw. (oder "status_to_output" beim Prog-Start) werden dann
; die Flags in R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gesetzt. Hier findet also die 
; Umsetzung von Signal-Bild zu den LED-Ausgngen statt.
; Die STATE-Machine schaut sich dann nur die Flags in R_STATUS_ON_S1_akt an und entscheidet
; dann, ob die LED ein-/ausgeblendet oder auch blinken. 
;

;----------------------------------------------------------------------
; Definition: welche Befehle (nicht OUTPUTs!) gruen1..10 gehren zu welchem Signal
;----------------------------------------------------------------------
;#define aufteilung4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
#define aufteilung6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
;#define aufteilung8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2

;----------------------------------------------------------------------
; Definition: welche Outputs (nicht Befehle!) Out1..10 gehren zu welchem Signal
;----------------------------------------------------------------------
;#define aufteilung_output4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung_output5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
#define aufteilung_output6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung_output7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
;#define aufteilung_output8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2

;#define  gruen5_flash_alternating_4_5  ; alternate flashing out 4/5  on green5
;#define  gruen6_flash_alternating_4_5  ; alternate flashing out 4/5  on green6
;#define  gruen7_flash_alternating_4_5  ; alternate flashing out 4/5  on green7
;#define  gruen8_flash_alternating_9_10 ; alternate flashing out 9/10 on green8
;#define  gruen9_flash_alternating_9_10 ; alternate flashing out 9/10 on green9
;#define gruen10_flash_alternating_9_10 ; alternate flashing out 9/10 on green10
;#define gruen11_flash_alternating_9_10 ; alternate flashing out 9/10 on green11
;#define gruen12_flash_alternating_9_10 ; alternate flashing out 9/10 on green12
;#define gruen13_flash_alternating_9_10 ; alternate flashing out 9/10 on green13
;#define gruen14_flash_alternating_9_10 ; alternate flashing out 9/10 on green14
;----------------------------------------------------
; Definition der Signal-Bilder 
;----------------------------------------------------
; diese Bits geben an, welches Flag in R_STATUS_ON_S1_pre/akt bzw. in R_STATUS_ON_S2_pre/akt
; gesetzt wird. 

; *** Signalbilder S1 ein
#define SIG_BILD_S1_01  b'00000001'  ; @ Gruen #1 
#define SIG_BILD_S1_02  b'00000010'  ; @ Gruen #2
#define SIG_BILD_S1_03  b'00000110'  ; @ Gruen #3
#define SIG_BILD_S1_04  b'00011000'  ; @ Gruen #4
#define SIG_BILD_S1_05  b'00100010'  ; @ Gruen #5  ; if Green 5 is defined to S1 
#define SIG_BILD_S1_06  b'00100110'  ; @ Gruen #6  ; if Green 6 is defined to S1 
#define SIG_BILD_S1_07  b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1 
#define SIG_BILD_S1_08  b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1 
;                         +--------- #7: Set Output #8
;                          +-------- #6: Set Output #7
;                           +------- #5: Set Output #6
;                            +------ #4: Set Output #5
;                             +----- #3: Set Output #4
;                              +---- #2: Set Output #3
;                               +--- #1: Set Output #2
;                                +-- #0: Set Output #1

; *** Signalbilder S2
#define SIG_BILD_S2_05  b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2  
#define SIG_BILD_S2_06  b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2 
#define SIG_BILD_S2_07  b'11000000'  ; @ Gruen #7  ; if Green 7 is defined to S2 
#define SIG_BILD_S2_08  b'00110000'  ; @ Gruen #8  ; if Green 8 is defined to S2 
#define SIG_BILD_S2_09  b'10010000'  ; @ Gruen #9
#define SIG_BILD_S2_10  b'11000000'  ; @ Gruen #10
#define SIG_BILD_S2_11  b'00110000'  ; @ Gruen #11
#define SIG_BILD_S2_12  b'10010000'  ; @ Gruen #12
#define SIG_BILD_S2_13  b'00000000'  ; @ Gruen #13
#define SIG_BILD_S2_14  b'00000000'  ; @ Gruen #14
;                         +--------- #7: Set Output #10
;                          +-------- #6: Set Output #9
;                           +------- #5: Set Output #8
;                            +------ #4: Set Output #7
;                             +----- #3: Set Output #6
;                              +---- #2: Set Output #5
;                               +--- #1: 
;                                +-- #0: 

; *** Signalbilder S1 flash 
; - entsprechendes Flag muss auch fr "ein" gesetzt werden 

#define SIG_BILD_S1_01F b'00000000'  ; @ Gruen #1  
#define SIG_BILD_S1_02F b'00000000'  ; @ Gruen #2 
#define SIG_BILD_S1_03F b'00000000'  ; @ Gruen #3 
#define SIG_BILD_S1_04F b'00000000'  ; @ Gruen #4 
#define SIG_BILD_S1_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S1 
#define SIG_BILD_S1_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S1 
#define SIG_BILD_S1_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1 
#define SIG_BILD_S1_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1 
;                         +--------- #7: Flash Output #8
;                          +-------- #6: Flash Output #7
;                           +------- #5: Flash Output #6
;                            +------ #4: Flash Output #5
;                             +----- #3: Flash Output #4
;                              +---- #2: Flash Output #3
;                               +--- #1: Flash Output #2
;                                +-- #0: Flash Output #1
                                          
#define SIG_BILD_S2_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2
#define SIG_BILD_S2_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2
#define SIG_BILD_S2_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S2 
#define SIG_BILD_S2_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S2
#define SIG_BILD_S2_09F b'00000000'  ; @ Gruen #9 
#define SIG_BILD_S2_10F b'00000000'  ; @ Gruen #10 
#define SIG_BILD_S2_11F b'00000000'  ; @ Gruen #11 
#define SIG_BILD_S2_12F b'00000000'  ; @ Gruen #12 
#define SIG_BILD_S2_13F b'00000000'  ; @ Gruen #13 
#define SIG_BILD_S2_14F b'00000000'  ; @ Gruen #14 
;                         +--------- #7: Flash Output #10
;                          +-------- #6: Flash Output #9
;                           +------- #5: Flash Output #8
;                            +------ #4: Flash Output #7
;                             +----- #3: Flash Output #6
;                              +---- #2: Flash Output #5
;                               +--- #1: 
;                                +-- #0: 

;----------------------------------------------------------------------
; Definition Dunkeltastung -> Durch welche Befehle S2 Dunkel wird
;----------------------------------------------------------------------
#define s2_dark_by_1 ; Dunkeltastung von S2 durch Grn 1
;#define s2_dark_by_2 ; Dunkeltastung von S2 durch Grn 2
;#define s2_dark_by_3 ; Dunkeltastung von S2 durch Grn 3
#define s2_dark_by_4 ; Dunkeltastung von S2 durch Grn 4
;#define s2_dark_by_5 ; Dunkeltastung von S2 durch Grn 5
;#define s2_dark_by_6 ; Dunkeltastung von S2 durch Grn 6
;#define s2_dark_by_7 ; Dunkeltastung von S2 durch Grn 7
;#define s2_dark_by_8 ; Dunkeltastung von S2 durch Grn 8

;----------------------------------------------------------------------
; Definition Dunkeltastung -> Welche Outputs dunkel werden
;----------------------------------------------------------------------
;#define dunkeltast_out01      ; hier wird definiert,
;#define dunkeltast_out02      ; .. welche 
;#define dunkeltast_out03      ; .. Ausgnge 
;#define dunkeltast_out04      ; .. auf die
;#define dunkeltast_out05      ; .. Dunkeltastung 
;#define dunkeltast_out06      ; .. reagieren und bei
#define dunkeltast_out07      ; .. Dunkeltastung 
#define dunkeltast_out08      ; .. dann auch 
#define dunkeltast_out09      ; .. abschalten
#define dunkeltast_out10      ; .. 

;----------------------------------------------------
; Parameter
;----------------------------------------------------
CONST_FADE_01          equ d'120'        ; Fade Geschwindigkeit @ Signalbild-Wechsel

;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;-- Ab hier muss man idR. nichts mehr anpassen !! (ausser SAVE Blinkmuster)
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------

;----------------------------------------------------------------------
; alternative Zuordnungen, hier nichts ndern!
;----------------------------------------------------------------------
; select, if Output 5/6/7/8 belongs to S1/S2
#ifdef aufteilung_output4_x
 #define out5_to_s2  ; Out 5 belongs to S2
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output5_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output6_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output7_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output8_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s1  ; Out 8 belongs to S2
#endif

; select, if Grn5/6/7/8 belongs to S1/S2 (for filter R_STATUS / -_2)
#ifdef aufteilung4_x
 #define gruen5_to_s2  ; Green 5 belongs to S2
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11110001' 
status_g02_a equ b'11110010' 
status_g03_a equ b'11110100' 
status_g04_a equ b'11111000' 
status_g05_a equ b'00011111' 
status_g06_a equ b'00101111' 
status_g07_a equ b'01001111' 
status_g08_a equ b'10001111' 
status_g09_a equ b'00001111' 
status_g10_a equ b'00001111' 
status_g11_a equ b'00001111' 
status_g12_a equ b'00001111' 
status_g13_a equ b'00001111' 
status_g14_a equ b'00001111' 
status_g01_b equ b'11111111' 
status_g02_b equ b'11111111' 
status_g03_b equ b'11111111' 
status_g04_b equ b'11111111' 
status_g05_b equ b'11000000' 
status_g06_b equ b'11000000' 
status_g07_b equ b'11000000' 
status_g08_b equ b'11000000' 
status_g09_b equ b'11000001' 
status_g10_b equ b'11000010' 
status_g11_b equ b'11000100' 
status_g12_b equ b'11001000' 
status_g13_b equ b'11010000' 
status_g14_b equ b'11100000' 
#endif
#ifdef aufteilung5_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11100001' 
status_g02_a equ b'11100010' 
status_g03_a equ b'11100100' 
status_g04_a equ b'11101000' 
status_g05_a equ b'11110000' 
status_g06_a equ b'00111111' 
status_g07_a equ b'01011111' 
status_g08_a equ b'10011111' 
status_g09_a equ b'00011111' 
status_g10_a equ b'00011111' 
status_g11_a equ b'00011111' 
status_g12_a equ b'00011111' 
status_g13_a equ b'00011111' 
status_g14_a equ b'00011111' 
status_g01_b equ b'11111111' 
status_g02_b equ b'11111111' 
status_g03_b equ b'11111111' 
status_g04_b equ b'11111111' 
status_g05_b equ b'11111111' 
status_g06_b equ b'11000000' 
status_g07_b equ b'11000000' 
status_g08_b equ b'11000000' 
status_g09_b equ b'11000001' 
status_g10_b equ b'11000010' 
status_g11_b equ b'11000100' 
status_g12_b equ b'11001000' 
status_g13_b equ b'11010000' 
status_g14_b equ b'11100000' 
#endif
#ifdef aufteilung6_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11000001' 
status_g02_a equ b'11000010' 
status_g03_a equ b'11000100' 
status_g04_a equ b'11001000' 
status_g05_a equ b'11010000' 
status_g06_a equ b'11100000' 
status_g07_a equ b'01111111' 
status_g08_a equ b'10111111' 
status_g09_a equ b'00111111' 
status_g10_a equ b'00111111' 
status_g11_a equ b'00111111' 
status_g12_a equ b'00111111' 
status_g13_a equ b'00111111' 
status_g14_a equ b'00111111' 
status_g01_b equ b'11111111' 
status_g02_b equ b'11111111' 
status_g03_b equ b'11111111' 
status_g04_b equ b'11111111' 
status_g05_b equ b'11111111' 
status_g06_b equ b'11111111' 
status_g07_b equ b'11000000' 
status_g08_b equ b'11000000' 
status_g09_b equ b'11000001' 
status_g10_b equ b'11000010' 
status_g11_b equ b'11000100' 
status_g12_b equ b'11001000' 
status_g13_b equ b'11010000' 
status_g14_b equ b'11100000' 
#endif
#ifdef aufteilung7_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'10000001' 
status_g02_a equ b'10000010' 
status_g03_a equ b'10000100' 
status_g04_a equ b'10001000' 
status_g05_a equ b'10010000' 
status_g06_a equ b'10100000' 
status_g07_a equ b'11000000' 
status_g08_a equ b'11111111' 
status_g09_a equ b'01111111' 
status_g10_a equ b'01111111' 
status_g11_a equ b'01111111' 
status_g12_a equ b'01111111' 
status_g13_a equ b'01111111' 
status_g14_a equ b'01111111' 
status_g01_b equ b'11111111' 
status_g02_b equ b'11111111' 
status_g03_b equ b'11111111' 
status_g04_b equ b'11111111' 
status_g05_b equ b'11111111' 
status_g06_b equ b'11111111' 
status_g07_b equ b'11111111' 
status_g08_b equ b'11000000' 
status_g09_b equ b'11000001' 
status_g10_b equ b'11000010' 
status_g11_b equ b'11000100' 
status_g12_b equ b'11001000' 
status_g13_b equ b'11010000' 
status_g14_b equ b'11100000' 
#endif
#ifdef aufteilung8_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s1  ; Green 8 belongs to S2
status_g01_a equ b'00000001' 
status_g02_a equ b'00000010' 
status_g03_a equ b'00000100' 
status_g04_a equ b'00001000' 
status_g05_a equ b'00010000' 
status_g06_a equ b'00100000' 
status_g07_a equ b'01000000' 
status_g08_a equ b'10000000' 
status_g09_a equ b'11111111' 
status_g10_a equ b'11111111' 
status_g11_a equ b'11111111' 
status_g12_a equ b'11111111' 
status_g13_a equ b'11111111' 
status_g14_a equ b'11111111' 
status_g01_b equ b'11111111' 
status_g02_b equ b'11111111' 
status_g03_b equ b'11111111' 
status_g04_b equ b'11111111' 
status_g05_b equ b'11111111' 
status_g06_b equ b'11111111' 
status_g07_b equ b'11111111' 
status_g08_b equ b'11111111' 
status_g09_b equ b'11000001' 
status_g10_b equ b'11000010' 
status_g11_b equ b'11000100' 
status_g12_b equ b'11001000' 
status_g13_b equ b'11010000' 
status_g14_b equ b'11100000' 
#endif

;----------------------------------------------------
; Mapping: Flag -> Output, hier nichts ndern!
;----------------------------------------------------
; hier wird definiert, bei welchem Flag von R_STATUS_ON_S1_akt bzw, R_STATUS_ON_S2_akt
; der entsprechende Output reagiert. Die Zuordnung S1/S2 wird durch die #defines geregelt,
; daher sind in diesem Abschnitt keine nderungen zu Anpassung ntig.

; *** S1
#define SET_OUTPUT1          R_STATUS_ON_S1_akt,0   
#define SET_OUTPUT1_FL       R_STATUS_FL_S1_akt,0    ;
#define SET_OUTPUT2          R_STATUS_ON_S1_akt,1   
#define SET_OUTPUT2_FL       R_STATUS_FL_S1_akt,1    ;
#define SET_OUTPUT3          R_STATUS_ON_S1_akt,2   
#define SET_OUTPUT3_FL       R_STATUS_FL_S1_akt,2    ;
#define SET_OUTPUT4          R_STATUS_ON_S1_akt,3   
#define SET_OUTPUT4_FL       R_STATUS_FL_S1_akt,3    ;
#ifdef out5_to_s1 
  #define SET_OUTPUT5        R_STATUS_ON_S1_akt,4  
  #define SET_OUTPUT5_FL     R_STATUS_FL_S1_akt,4    ;
#endif
#ifdef out6_to_s1 
  #define SET_OUTPUT6        R_STATUS_ON_S1_akt,5  
  #define SET_OUTPUT6_FL     R_STATUS_FL_S1_akt,5    ;
#endif
#ifdef out7_to_s1 
  #define SET_OUTPUT7        R_STATUS_ON_S1_akt,6  
  #define SET_OUTPUT7_FL     R_STATUS_FL_S1_akt,6    ;
#endif
#ifdef out8_to_s1 
  #define SET_OUTPUT8        R_STATUS_ON_S1_akt,7  
  #define SET_OUTPUT8_FL     R_STATUS_FL_S1_akt,7    ;
#endif   

; *** S2                    
#ifdef out5_to_s2 
  #define SET_OUTPUT5        R_STATUS_ON_S2_akt,2 
  #define SET_OUTPUT5_FL     R_STATUS_FL_S2_akt,2    ;
#endif                      
#ifdef out6_to_s2         
  #define SET_OUTPUT6        R_STATUS_ON_S2_akt,3 
  #define SET_OUTPUT6_FL     R_STATUS_FL_S2_akt,3    ;
#endif                      
#ifdef out7_to_s2         
  #define SET_OUTPUT7        R_STATUS_ON_S2_akt,4 
  #define SET_OUTPUT7_FL     R_STATUS_FL_S2_akt,4    ;
#endif                      
#ifdef out8_to_s2         
  #define SET_OUTPUT8        R_STATUS_ON_S2_akt,5 
  #define SET_OUTPUT8_FL     R_STATUS_FL_S2_akt,5    ;
#endif                      
#define SET_OUTPUT9          R_STATUS_ON_S2_akt,6    
#define SET_OUTPUT9_FL       R_STATUS_FL_S2_akt,6    ;
#define SET_OUTPUT10         R_STATUS_ON_S2_akt,7    
#define SET_OUTPUT10_FL      R_STATUS_FL_S2_akt,7    ;
                            

;----------------------------------------------------
; Register, hier nichts ndern!
;----------------------------------------------------
R_STATUS_ON_S1_pre    equ  40h
R_STATUS_ON_S2_pre    equ  41h
R_STATUS_FL_S1_pre    equ  42h
R_STATUS_FL_S2_pre    equ  43h
R_STATUS_ON_S1_akt    equ  44h
R_STATUS_ON_S2_akt    equ  45h
R_STATUS_FL_S1_akt    equ  46h
R_STATUS_FL_S2_akt    equ  47h

R_COUNT               equ  48h          ; Counter fuer PWM - LED1
R_PWMVALUE            equ  49h          ; PWM-Value, bernahme von R_COUNT
R_STATE1              equ  4ah          ; Status-Flags 1 - LED1
R_STATE2              equ  4bh          ; Status-Flags 2 - LED2
R_STATE3              equ  4ch          ; Status-Flags 3 - LED3
R_STATE4              equ  4dh          ; Status-Flags 4 - LED4
R_STATE5              equ  4eh          ; Status-Flags 5 - LED5
R_STATE6              equ  4fh          ; Status-Flags 6 - LED6
R_STATE7              equ  50h          ; Status-Flags 7 - LED7
R_STATE8              equ  51h          ; Status-Flags 8 - LED8
R_STATE9              equ  52h          ; Status-Flags 9 - LED9
R_STATE10             equ  53h          ; Status-Flags 10 - LED10

R_TESTTEMP            equ  5ch
;R_FADEVALUE           equ  5dh          ; contains valid fade constant
R_STATUS_2            equ  5eh          ; for >8 Adresses
R_FLAG_001            equ  5fh          ; various Flags
    #define DUNKELTAST_PRE R_FLAG_001,0
    #define DUNKELTAST_AKT R_FLAG_001,1
    #define ALTER_FLASH_S1 R_FLAG_001,6 ; if set alternate Flashing S1 out 4/5
    #define ALTER_FLASH_S2 R_FLAG_001,7 ; if set alternate Flashing S2 out 9/10


;----------------------------------------------------
; Mapping LED-Nummer zu Port
;----------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition




;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_S2    h'30'           ; EEprom Adresse: R_STATUS_2: Status #2

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- read R_STATUS_2 from EEPROM $30 ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S2
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_2               ; ..into R_STATUS_2

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3                 ; Grundzustand "off"
        bsf R_STATE2,3                 ; Grundzustand "off"
        bsf R_STATE3,3                 ; Grundzustand "off"
        bsf R_STATE4,3                 ; Grundzustand "off"
        bsf R_STATE5,3                 ; Grundzustand "off"
        bsf R_STATE6,3                 ; Grundzustand "off"
        bsf R_STATE7,3                 ; Grundzustand "off"
        bsf R_STATE8,3                 ; Grundzustand "off"
        bsf R_STATE9,3                 ; Grundzustand "off"
        bsf R_STATE10,3                ; Grundzustand "off"
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
#ifndef skip_g1
gruen_1
        ; Signal-Status
        movlw status_g01_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g01_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild statisch: 
        movlw SIG_BILD_S1_01        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_01F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; Dunkeltastung?
#ifdef s2_dark_by_1
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_1
        bcf DUNKELTAST_PRE
#endif

        return
#endif ; skip_g1

; ********************
; *** ADR Gruen-#2 ***
; ********************
#ifndef skip_g2
gruen_2
        ; Signal-Status
        movlw status_g02_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g02_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild statisch: 
        movlw SIG_BILD_S1_02        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_02F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; Dunkeltastung?
#ifdef s2_dark_by_2
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_2
        bcf DUNKELTAST_PRE
#endif

        return
#endif ; skip_g2
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
#ifndef skip_g3
gruen_3
        ; Signal-Status
        movlw status_g03_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g03_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S1_03        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_03F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; Dunkeltastung?
#ifdef s2_dark_by_3
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_3
        bcf DUNKELTAST_PRE
#endif

        return
#endif ; skip_g3

; ********************
; *** ADR Gruen-#4 ***
; ********************
#ifndef skip_g4
gruen_4
        ; Signal-Status
        movlw status_g04_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g04_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S1_04        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_04F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; Dunkeltastung?
#ifdef s2_dark_by_4
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_4
        bcf DUNKELTAST_PRE
#endif
        return
#endif ; skip_g4

; ********************
; *** ADR Gruen-#5 ***
; ********************
#ifndef skip_g5
gruen_5
        ; Signal-Status
        movlw status_g05_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g05_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
#ifdef gruen5_to_s1
        movlw SIG_BILD_S1_05        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_05F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif
#ifdef gruen5_to_s2
        movlw SIG_BILD_S2_05        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_05F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif
        
#ifdef gruen5_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  #ifdef s2_dark_by_5
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_5
          bcf DUNKELTAST_PRE
  #endif
#endif

#ifdef gruen5_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g5

; ********************
; *** ADR Gruen-#6 ***
; ********************
#ifndef skip_g6
gruen_6
        ; Signal-Status
        movlw status_g06_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g06_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
#ifdef gruen6_to_s1
        movlw SIG_BILD_S1_06        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_06F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif
#ifdef gruen6_to_s2
        movlw SIG_BILD_S2_06        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_06F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen6_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  #ifdef s2_dark_by_6
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_6
          bcf DUNKELTAST_PRE
  #endif
#endif

#ifdef gruen6_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g6

; ********************
; *** ADR Gruen-#7 ***
; ********************
#ifndef skip_g7
gruen_7
        ; Signal-Status
        movlw status_g07_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g07_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
#ifdef gruen7_to_s1
        movlw SIG_BILD_S1_07        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_07F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif
#ifdef gruen7_to_s2
        movlw SIG_BILD_S2_07        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_07F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen7_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  #ifdef s2_dark_by_7
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_7
          bcf DUNKELTAST_PRE
  #endif
#endif

#ifdef gruen7_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g7

; ********************
; *** ADR Gruen-#8 ***
; ********************
#ifndef skip_g8
gruen_8
        ; Signal-Status
        movlw status_g08_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g08_b   ; hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
#ifdef gruen8_to_s1
        movlw SIG_BILD_S1_08        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S1_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S1_08F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif
#ifdef gruen8_to_s2
        movlw SIG_BILD_S2_08        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_08F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen8_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  #ifdef s2_dark_by_8
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
          btfsc R_MODE,1                 ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_8
          bcf DUNKELTAST_PRE
  #endif
#endif

#ifdef gruen8_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g8

; ********************
; *** ADR Gruen-#9 ***
; ********************
#ifndef skip_g9
gruen_9
        ; Signal-Status
        movlw status_g09_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g09_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_09        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_09F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen9_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g9

; ********************
; *** ADR Gruen-#10***
; ********************
#ifndef skip_g10
gruen_10
        ; Signal-Status
        movlw status_g10_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g10_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_10        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_10F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen10_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g10

; ********************
; *** ADR Gruen-#11***
; ********************
#ifndef skip_g11
gruen_11
        ; Signal-Status
        movlw status_g11_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g11_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_11        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_11F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen11_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g11

; ********************
; *** ADR Gruen-#12***
; ********************
#ifndef skip_g12
gruen_12
        ; Signal-Status
        movlw status_g12_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g12_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_12        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_12F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen12_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g12

; ********************
; *** ADR Gruen-#13***
; ********************
#ifndef skip_g13
gruen_13
        ; Signal-Status
        movlw status_g13_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g13_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_13        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_13F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen13_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g13

; ********************
; *** ADR Gruen-#14***
; ********************
#ifndef skip_g14
gruen_14
        ; Signal-Status
        movlw status_g14_a   ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS) 
        andwf R_STATUS,F     ; gehren, mssen genullt werden. 
        movlw status_g14_b   ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F   ; eine '1' fr alle unchanged Status-Bits

        ; Signalbild: 
        movlw SIG_BILD_S2_14        ; hier steht drinne, welche Outputs eingeschaltet werden 
        andwf R_STATUS_ON_S2_akt,F  ; LEDs, die im aktive und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre    ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing: 
        movlw SIG_BILD_S2_14F       ; hier steht drinne, welche Outputs blinken sollen 
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt

#ifdef gruen14_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9       ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10       ; Blink-Outputs erst einmal "aus"
#endif

        return
#endif ; skip_g14

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
#ifndef skip_g1
save_01
        movlw b'00010000'             ; 
        xorwf PORTA,F

        movlw b'00000000'             ; 
        xorwf PORTC,F
        return
#endif ; skip_g1

; ********************
; *** Adresse #2   ***
; ********************
#ifndef skip_g2
save_02
        movlw b'00100000'             ; 
        xorwf PORTA,F

        movlw b'00000000'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #3   ***
; ********************
#ifndef skip_g3
save_03
        movlw b'00100001'             ; 
        xorwf PORTA,F

        movlw b'00000000'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #4   ***
; ********************
#ifndef skip_g4
save_04
        movlw b'00000010'             ; 
        xorwf PORTA,F

        movlw b'00000010'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #5   ***
; ********************
#ifndef skip_g5
save_05
        movlw b'00100000'             ; 
        xorwf PORTA,F

        movlw b'00000100'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #6   ***
; ********************
#ifndef skip_g6
save_06
        movlw b'00100001'             ; 
        xorwf PORTA,F

        movlw b'00000100'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #7   ***
; ********************
#ifndef skip_g7
save_07
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00011000'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #8   ***
; ********************
#ifndef skip_g8
save_08
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00100001'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #9   ***
; ********************
#ifndef skip_g9
save_09
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00001001'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #10  ***
; ********************
#ifndef skip_g10
save_10
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00011000'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #11  ***
; ********************
#ifndef skip_g11
save_11
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00100001'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #12  ***
; ********************
#ifndef skip_g12
save_12
        movlw b'00000000'             ; 
        xorwf PORTA,F

        movlw b'00001001'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #13  ***
; ********************
#ifndef skip_g13
save_13
        movlw b'11111111'             ; 
        xorwf PORTA,F

        movlw b'11111111'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #14  ***
; ********************
#ifndef skip_g14
save_14
        movlw b'11111111'             ; 
        xorwf PORTA,F

        movlw b'11111111'             ; 
        xorwf PORTC,F
        return
#endif ; skip_gx

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Durch Aufruf der Routine "gruen_1" etc. werden diese in die
    ; Signalbilder umgewandelt.
#ifndef skip_g1
        btfsc R_STATUS,0
         call gruen_1
#endif

#ifndef skip_g3
        btfsc R_STATUS,1
         call gruen_2
#endif

#ifndef skip_g3
        btfsc R_STATUS,2
         call gruen_3
#endif

#ifndef skip_g4
        btfsc R_STATUS,3
         call gruen_4
#endif

#ifndef skip_g5
        btfsc R_STATUS,4
         call gruen_5
#endif

#ifndef skip_g6
        btfsc R_STATUS,5
         call gruen_6
#endif

#ifndef skip_g7
        btfsc R_STATUS,6
         call gruen_7
#endif

#ifndef skip_g8
        btfsc R_STATUS,7
         call gruen_8
#endif

#ifndef skip_g9
        btfsc R_STATUS_2,0
         call gruen_9
#endif

#ifndef skip_g10
        btfsc R_STATUS_2,1
         call gruen_10
#endif

#ifndef skip_g11
        btfsc R_STATUS_2,2
         call gruen_11
#endif

#ifndef skip_g12
        btfsc R_STATUS_2,3
         call gruen_12
#endif

#ifndef skip_g13
        btfsc R_STATUS_2,4
         call gruen_13
#endif

#ifndef skip_g14
        btfsc R_STATUS_2,5
         call gruen_14
#endif

        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL
        
;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'11111111'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002 

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw h'00'
        movwf R_EEADR                  ; Address: h'00'
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return

;*******************************************
; +++ for debug +++
;*******************************************
;      movf R_ADR_3,W ; test debug
;      movwf R_TESTTEMP
;      call show_testtemp

show_testtemp
; for DEBUG  btfss R_TESTTEMP
         bcf LED_01 
        btfsc R_TESTTEMP,0
         bsf LED_01 

        btfss R_TESTTEMP,1
         bcf LED_02 
        btfsc R_TESTTEMP,1
         bsf LED_02 

        btfss R_TESTTEMP,2
         bcf LED_03 
        btfsc R_TESTTEMP,2
         bsf LED_03 

        btfss R_TESTTEMP,3
         bcf LED_04 
        btfsc R_TESTTEMP,3
         bsf LED_04 

        btfss R_TESTTEMP,4
         bcf LED_05 
        btfsc R_TESTTEMP,4
         bsf LED_05 

        btfss R_TESTTEMP,5
         bcf LED_06 
        btfsc R_TESTTEMP,5
         bsf LED_06 

        btfss R_TESTTEMP,6
         bcf LED_07 
        btfsc R_TESTTEMP,6
         bsf LED_07 

        btfss R_TESTTEMP,7
         bcf LED_08 
        btfsc R_TESTTEMP,7
         bsf LED_08 
         
        return
