; ********************************************************
; ** Version: V3.10 vom 06/10/2007                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006 Sven Brandt

; ********************************************************
; History:
; V3.10 - erstellt aus sub_led_signal4.asm V3.09
; ********************************************************

  ; *****************************
  ; ** nur fuer led_signal_sbb **
  ; *****************************
; Adresse -{CHECK}-> R_STATUS -{STATEMACHINE}-> R_STATUS2 -{STATEMACHINE}-> R_STATEx -{LOOP}-> LEDs
; In R_STATUS und R_STATUS2 sind die Signalbilder (HP0, VR1 usw.) als Flags vergeben:
; #0: 1= Halt
; #1: 1= FB1
; #2: 1= FB2
; #3: 1= FB3
; #4: 1= Warn Vorsignal
; #5: 1= FB1 Vorsignal
; #6: 1= FB2 Vorsignal
; #7: 1= FB3 Vorsignal
;
R_STATUS_2     equ  53h          ; only needed for >8 Adresses
R_COUNT        equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE     equ  51h          ; PWM-Value, bernahme von R_COUNT
R_STATUS2      equ  52h
R_STATE1       equ  54h          ; Status-Flags 1 - LED1
R_STATE2       equ  55h          ; Status-Flags 2 - LED2
R_STATE3       equ  56h          ; Status-Flags 3 - LED3
R_STATE4       equ  57h          ; Status-Flags 4 - LED4
R_STATE6       equ  59h          ; Status-Flags 6 - LED6
R_STATE7       equ  5ah          ; Status-Flags 7 - LED7
R_STATE8       equ  5bh          ; Status-Flags 8 - LED8
R_STATE9       equ  5ch          ; Status-Flags 9 - LED9

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_S2    h'30'           ; EEprom Adresse: R_STATUS_2: Status #2

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- read R_STATUS_2 from EEPROM $30 ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S2
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_2               ; ..into R_STATUS_2

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3
        bsf R_STATE2,3
        bsf R_STATE3,3
        bsf R_STATE4,3
        bsf R_STATE6,3
        bsf R_STATE7,3
        bsf R_STATE8,3
        bsf R_STATE9,3
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'11110001'              ; STATUS #1 #2 #3 lschen
        goto clear_hs

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'11110010'              ; STATUS #0 #2 #3 lschen
        goto clear_hs
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'11110100'              ; STATUS #0 #1 #3 lschen
        goto clear_hs

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'11111000'              ; STATUS #0 #1 #2 lschen
        goto clear_hs

; ********************
; *** ADR Gruen-#5 ***
; ********************
gruen_5
        movlw b'00011111'              ; STATUS #7 #6 #5 lschen
        goto clear_vs

; ********************
; *** ADR Gruen-#6 ***
; ********************
gruen_6
        movlw b'00101111'              ; STATUS #7 #6 #4 lschen
        goto clear_vs

; ********************
; *** ADR Gruen-#7 ***
; ********************
gruen_7
        movlw b'01001111'              ; STATUS #7 #5 #4 lschen
        goto clear_vs

; ********************
; *** ADR Gruen-#8 ***
; ********************
gruen_8
        movlw b'10001111'              ; STATUS #6 #5 #4 lschen
        goto clear_vs
         return

; ********************
; *** ADR Gruen-#9 ***
; ********************
gruen_9
        bsf PORTC,1                    ; Output5
         return

; *********************
; *** ADR Gruen-#10 ***
; *********************
gruen_10
        bsf PORTC,2                    ; Output6
         return

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        return
 
; ******************
; *** ADR Rot-#3 ***
; ******************
rot_3
        return

; ******************
; *** ADR Rot-#4 ***
; ******************
rot_4
        return

; ******************
; *** ADR Rot-#5 ***
; ******************
rot_5
        return

; ******************
; *** ADR Rot-#6 ***
; ******************
rot_6
        return

; ******************
; *** ADR Rot-#7 ***
; ******************
rot_7
        return

; ******************
; *** ADR Rot-#8 ***
; ******************
rot_8
        return

; ******************
; *** ADR Rot-#9 ***
; ******************
rot_9
        bcf PORTC,1                    ; Output5
        return

; ******************
; *** ADR Rot-10 ***
; ******************
rot_10
        bcf PORTC,2                    ; Output6
        return

; ****************
; *** clear_HS ***
; ****************
clear_hs
        andwf R_STATUS,F
        movlw b'00001111'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase HS
        return
; ****************
; *** clear_VS ***
; ****************
clear_vs
        andwf R_STATUS,F
        movlw b'11110000'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase VS
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        movlw b'00010000'             ; Halt
        xorwf PORTA,F
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
        movlw b'00100000'             ; FB1
        xorwf PORTA,F
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
        movlw b'00100001'             ; FB2
        xorwf PORTA,F
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
        movlw b'00100010'             ; FB3
        xorwf PORTA,F
        return

; ********************
; *** Adresse #5   ***
; ********************
save_05
        movlw b'00011000'             ; Warn
        xorwf PORTC,F
        return

; ********************
; *** Adresse #6   ***
; ********************
save_06
        movlw b'00100001'             ; FB1
        xorwf PORTC,F
        return

; ********************
; *** Adresse #7   ***
; ********************
save_07
        movlw b'00001001'             ; FB2
        xorwf PORTC,F
        return

; ********************
; *** Adresse #8   ***
; ********************
save_08
        movlw b'00101001'             ; FB3
        xorwf PORTC,F
        return

; ********************
; *** Adresse #8   ***
; ********************
save_09
        movlw b'00000010'             ; Out 5
        xorwf PORTC,F
        return

; ********************
; *** Adresse #8   ***
; ********************
save_10
        movlw b'00000100'             ; Out 6
        xorwf PORTC,F
        return

; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         call save_01          ; Signalbild 1 einschalten
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -----------------------------------------
;   --- Restore Out 5/6                   ---
;   -----------------------------------------
        btfsc R_STATUS_2,0
         bsf PORTC,1
        btfsc R_STATUS_2,1
         bsf PORTC,2

;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
        movf R_STATUS,W
        movwf R_STATUS2
        btfss STATUS,Z                 ; wenn R_STATUS = ZERO
         return
        movlw b'00010001'
        movwf R_STATUS
        movwf R_STATUS2

        return

;*********************************
;** Routine: change_mode        **
;** invert R_MODE and save      **
;** into EEPROM $00             **
;*********************************
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from # -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         bsf PORTA,4                   ; .. -> Output on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf PORTA,4                   ; .. -> Output off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw h'00'
        movwf R_EEADR                  ; Address: h'00'
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
