; ********************************************************
; ** Version: V3.59 vom 26/08/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09/10/11/12/13 Sven Brandt
; ********************************************************
; History:
; V3.59 - 27.08.2013 Syncron-Blinken hinzu. Ab jetzt sind die Unterprogramme in "state_signal_v2" und nicht mehr in "sub_led_signal_xxx".
; V3.57 - 04.03.2013 signal_030 bis 034 hinzu (Skandinavien)
; V3.56 - 07.11.2012 signal_022 hinzu (Belgien)
; V3.53 - 25.01.2012 signal_021: bei Dunkeltastung werden jetzt alle Ausgnge (auch Kennlicht!) abgeschaltet
; V3.50 - 'all_dark_by...' hinzu, ermglicht Dunkeltastung des gesamten Signals durch Befehle
;       - Save-Blinkmuster ber #defines
;       - Fahrstrassenabhngige Adressen mglich
;       - Fahrstrassenabhngige Signalbilder mglich
; V3.47 - LED_Signal_011 hinzu, alle Outputs knnen zu einem Signal gehren (jetzt ber #defines), 16 Adressen mglich
; V3.45 - LED_Signal_010 hinzu, alle Outputs knnen zu einem Signal gehren
; V3.41 - corrections Save-Outputs
; V3.22 - first Version

;Mit diesem Include-File knnen Signale konfiguriert werden:
; - Fr LED-Dekoder (idR. mit 16F684)
; - 2 unabhngige Signale mglich
; - jedes Signalbild reagiert auf einen Digital-Befehl (keine Paar-Bildung)
; - 2. Signal kann vom 1. dunkelgetastet werden
; - Blinken aller Ausgnge mglich
; - Wechselblinken Ausgang 4/5 und 9/10 mglich
; - Signal 1 kann auf Ausgnge 1-8  zugreifen
; - Signal 2 kann auf Ausgnge 3-10 zugreifen
; - alle Ausgnge knnen zu einem Signal zugeordnet werden
; - max. 16 Adressen
; - konfigurierbar, ob das erste Signal auf 4/5/6/7 oder 8 Befehle reagiert
; - Dunkeltastung gesamtes Signal mglich, Ein- und Ausschalten ber eigene Befehle

; ================> Adresse
; -{CHECK}--------> R_STATUS ->R_STATUS_ON_S1/2_pre
; -{STATEMACHINE}-> R_STATUS_ON_S1/2_akt
; -{STATEMACHINE}-> R_STATEx
; -{LOOP}---------> LEDs

; in R_STATUS sind die Signalstellungen abgelegt.
; in R_STATUS_ON_S1_xxx bzw. R_STATUS_ON_S2_xxx sind die LED-Stellungen abgelegt.

; In R_STATUS_ON_S1/2_pre/akt und R_STATUS_ON_S1/2_pre/akt sind die
; Ausgnge als Flags vergeben:

; In dieser SW wird (wie bei der "LED_SIGNAL2") mit den Zwischen-Registern
; R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gearbeitet. In dieses zeigen die Flags an, ob
; ein Ausgang gesetzt / blinkend / aus sein soll.
; Im Gegensatz zu LED_SIGNAL2 wird hier jedoch nicht mit ROTen und GRNen Adressen
; gearbeitet, sondern es werden bis zu 14 GRNe Adressen verwendet.
; Die Signal-Bilder werden in R_STATUS und R_STATUS_2 abgelegt (8 Adressen = 8 Bits). In R_STATUS / R_STATUS_2
; liegt demnach die letzte bettigte Adresse und damit das anzuzeigene Signal-Bild.

; In den Routinen "gruen_1" usw. (oder "status_to_output" beim Prog-Start) werden dann
; die Flags in R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gesetzt. Hier findet also die
; Umsetzung von Signal-Bild zu den LED-Ausgngen statt.
; Die STATE-Machine schaut sich dann nur die Flags in R_STATUS_ON_S1_akt an und entscheidet
; dann, ob die LED ein-/ausgeblendet oder auch blinken.
;

;----------------------------------------------------------------------
; Definition: welche Befehle (nicht OUTPUTs!) gruen_1..10 gehren zu welchem Signal
; - bei Signal-Dunkeltastung (use_all_dark) mssen die entsprechenden Befehle zu S2 zugeordnet werden (ansonsten ist R_STATUS nach Dunkel-Befehl nicht korrekt)
; - bei fahrwegabhngigen Adressen (gruenx_pagex_aktiv) sollten die Page-Befehle zu S2 gehren, damit Out9/10 ber die Page-Befehle verwendet werden knnen
; - bei fahrwegabhngigen Signalbildern (gruenx_depends_on_page) Adressen mssen die Page-Befehle zu S2 gehren (ansonsten ist R_STATUS nach Page-Wechsel nicht korrekt)
;----------------------------------------------------------------------
;#define aufteilung4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
;#define aufteilung6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
;#define aufteilung8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2
#define aufteilung_all ; keine Trennung in S1/S2. Alle Befehle gehren zum selben Signal

;----------------------------------------------------------------------
; Definition: welche Outputs (nicht Befehle!) Out_1..10 gehren zu welchem Signal
; - bei 'all_in_one' wird nichts aktiviert!
; - bei fahrwegabhngigen Adressen (gruenx_pagex_aktiv) mssen diese Signalbilder zu S1 gehren (sonst geht die Buffer-Lsung nicht)!
;----------------------------------------------------------------------
;#define aufteilung_output4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung_output5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
;#define aufteilung_output6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung_output7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
;#define aufteilung_output8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2
#define aufteilung_output_all ; alle Outputs gehren zu S1, die S2-Befehle machen Dunkeltastung, Page-Umschaltung etc.

;----------------------------------------------------------------------
; Definition: soll Blinken syncron sein? 
; - nur ntig, wenn >=1 Ausgang blinkt und >=1 Ausgang noch dazu kommt
;----------------------------------------------------------------------
#define use_flash_syncro

;----------------------------------------------------------------------
; Definition: welche Outputs bei welcher Adresse im Wechsel blinken sollen
;----------------------------------------------------------------------
;#define  gruen5_flash_alternating_4_5  ; alternate flashing out 4/5  on green5
;#define  gruen6_flash_alternating_4_5  ; alternate flashing out 4/5  on green6
;#define  gruen7_flash_alternating_4_5  ; alternate flashing out 4/5  on green7
;#define  gruen8_flash_alternating_9_10 ; alternate flashing out 9/10 on green8
;#define  gruen9_flash_alternating_9_10 ; alternate flashing out 9/10 on green9
;#define gruen10_flash_alternating_9_10 ; alternate flashing out 9/10 on green10
;#define gruen11_flash_alternating_9_10 ; alternate flashing out 9/10 on green11
;#define gruen12_flash_alternating_9_10 ; alternate flashing out 9/10 on green12
;#define gruen13_flash_alternating_9_10 ; alternate flashing out 9/10 on green13
;#define gruen14_flash_alternating_9_10 ; alternate flashing out 9/10 on green14
;#define gruen15_flash_alternating_9_10 ; alternate flashing out 9/10 on green15
;#define gruen16_flash_alternating_9_10 ; alternate flashing out 9/10 on green16


;----------------------------------------------------
; Definition der Signal-Bilder
; diese Bits geben an, welches Flag in R_STATUS_ON_S1_pre/akt bzw. in R_STATUS_ON_S2_pre/akt
; gesetzt wird.
;----------------------------------------------------

#ifndef aufteilung_output_all
;----------------------------------------------------
; wenn Aufteilung der Outputs auf 2 Signale
;----------------------------------------------------
; *** Signalbilder S1 ein (1. Signal oder Hauptsignal)
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_S1_01  b'00011100'  ; @ Gruen #1
#define SIG_BILD_S1_02  b'00110011'  ; @ Gruen #2
#define SIG_BILD_S1_03  b'00111001'  ; @ Gruen #3
#define SIG_BILD_S1_04  b'01001100'  ; @ Gruen #4
#define SIG_BILD_S1_05  b'11000011'  ; @ Gruen #5  ; if Green 5 is defined to S1
#define SIG_BILD_S1_06  b'11001001'  ; @ Gruen #6  ; if Green 6 is defined to S1
#define SIG_BILD_S1_07  b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1
#define SIG_BILD_S1_08  b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1
;                         +--------- #7: Set Output #8
;                          +-------- #6: Set Output #7
;                           +------- #5: Set Output #6
;                            +------ #4: Set Output #5
;                             +----- #3: Set Output #4
;                              +---- #2: Set Output #3
;                               +--- #1: Set Output #2
;                                +-- #0: Set Output #1

; *** Signalbilder S2 (2. Signal oder Vorsignal)
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_S2_05  b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2
#define SIG_BILD_S2_06  b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2
#define SIG_BILD_S2_07  b'01000000'  ; @ Gruen #7  ; if Green 7 is defined to S2
#define SIG_BILD_S2_08  b'10000000'  ; @ Gruen #8  ; if Green 8 is defined to S2
#define SIG_BILD_S2_09  b'00000000'  ; @ Gruen #9
#define SIG_BILD_S2_10  b'00000000'  ; @ Gruen #10
#define SIG_BILD_S2_11  b'00000000'  ; @ Gruen #11
#define SIG_BILD_S2_12  b'00000000'  ; @ Gruen #12
#define SIG_BILD_S2_13  b'00000000'  ; @ Gruen #13
#define SIG_BILD_S2_14  b'00000000'  ; @ Gruen #14
#define SIG_BILD_S2_15  b'00000000'  ; @ Gruen #15
#define SIG_BILD_S2_16  b'00000000'  ; @ Gruen #16
;                         +--------- #7: Set Output #10
;                          +-------- #6: Set Output #9
;                           +------- #5: Set Output #8
;                            +------ #4: Set Output #7
;                             +----- #3: Set Output #6
;                              +---- #2: Set Output #5
;                               +--- #1:
;                                +-- #0:

; *** Signalbilder S1 flash (1. Signal oder Hauptsignal)
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_S1_01F b'00000000'  ; @ Gruen #1
#define SIG_BILD_S1_02F b'00000000'  ; @ Gruen #2
#define SIG_BILD_S1_03F b'00000000'  ; @ Gruen #3
#define SIG_BILD_S1_04F b'00000000'  ; @ Gruen #4
#define SIG_BILD_S1_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S1
#define SIG_BILD_S1_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S1
#define SIG_BILD_S1_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1
#define SIG_BILD_S1_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1
;                         +--------- #7: Flash Output #8
;                          +-------- #6: Flash Output #7
;                           +------- #5: Flash Output #6
;                            +------ #4: Flash Output #5
;                             +----- #3: Flash Output #4
;                              +---- #2: Flash Output #3
;                               +--- #1: Flash Output #2
;                                +-- #0: Flash Output #1

; *** Signalbilder S2 flash (2. Signal oder Vorsignal)
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_S2_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2
#define SIG_BILD_S2_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2
#define SIG_BILD_S2_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S2
#define SIG_BILD_S2_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S2
#define SIG_BILD_S2_09F b'00000000'  ; @ Gruen #9
#define SIG_BILD_S2_10F b'00000000'  ; @ Gruen #10
#define SIG_BILD_S2_11F b'00000000'  ; @ Gruen #11
#define SIG_BILD_S2_12F b'00000000'  ; @ Gruen #12
#define SIG_BILD_S2_13F b'00000000'  ; @ Gruen #13
#define SIG_BILD_S2_14F b'00000000'  ; @ Gruen #14
#define SIG_BILD_S2_15F b'00000000'  ; @ Gruen #15
#define SIG_BILD_S2_16F b'00000000'  ; @ Gruen #16
;                         +--------- #7: Flash Output #10
;                          +-------- #6: Flash Output #9
;                           +------- #5: Flash Output #8
;                            +------ #4: Flash Output #7
;                             +----- #3: Flash Output #6
;                              +---- #2: Flash Output #5
;                               +--- #1:
;                                +-- #0:
#endif  ;#ifndef aufteilung_output_all

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
;----------------------------------------------------
; wenn KEINE Aufteilung der Ausgnge auf 2 Signale
;----------------------------------------------------
; *** Signalbilder ein Out 1..8
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
; werden Adress-Page spezifische Signalbilder verwendet (d.h. es erscheinen Fahrweg-Abhngige Signal-Bilder)
; dann stellen die hier definierten Signalbilder die Page1 dar.
#define SIG_BILD_01_a      b'00000101'  ; @ Gruen #1 
#define SIG_BILD_02_a      b'00110101'  ; @ Gruen #2
#define SIG_BILD_03_a      b'00000101'  ; @ Gruen #3
#define SIG_BILD_04_a      b'01001000'  ; @ Gruen #4
#define SIG_BILD_05_a      b'11001010'  ; @ Gruen #5 
#define SIG_BILD_06_a      b'00101001'  ; @ Gruen #6 
#define SIG_BILD_07_a      b'00011001'  ; @ Gruen #7 
#define SIG_BILD_08_a      b'10101010'  ; @ Gruen #8 
#define SIG_BILD_09_a      b'10011010'  ; @ Gruen #9
#define SIG_BILD_10_a      b'00111001'  ; @ Gruen #10
#define SIG_BILD_11_a      b'10110010'  ; @ Gruen #11
#define SIG_BILD_12_a      b'00100010'  ; @ Gruen #12
#define SIG_BILD_13_a      b'00110010'  ; @ Gruen #13
#define SIG_BILD_14_a      b'00001000'  ; @ Gruen #14
#define SIG_BILD_15_a      b'00000000'  ; @ Gruen #15
#define SIG_BILD_16_a      b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02_a_p2   b'00000000'  ; @ Gruen #2
#define SIG_BILD_03_a_p2   b'00000000'  ; @ Gruen #3
;                            +--------- #7: Set   Output #8
;                             +-------- #6: Set   Output #7
;                              +------- #5: Set   Output #6
;                               +------ #4: Set   Output #5
;                                +----- #3: Set   Output #4
;                                 +---- #2: Set   Output #3
;                                  +--- #1: Set   Output #2
;                                   +-- #0: Set   Output #1

; *** Signalbilder ein Out 9..10
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_01_b      b'01000000'  ; @ Gruen #1
#define SIG_BILD_02_b      b'01000000'  ; @ Gruen #2
#define SIG_BILD_03_b      b'01000000'  ; @ Gruen #3
#define SIG_BILD_04_b      b'10000000'  ; @ Gruen #4
#define SIG_BILD_05_b      b'10000000'  ; @ Gruen #5
#define SIG_BILD_06_b      b'01000000'  ; @ Gruen #6
#define SIG_BILD_07_b      b'01000000'  ; @ Gruen #7
#define SIG_BILD_08_b      b'10000000'  ; @ Gruen #8
#define SIG_BILD_09_b      b'10000000'  ; @ Gruen #9
#define SIG_BILD_10_b      b'01000000'  ; @ Gruen #10
#define SIG_BILD_11_b      b'10000000'  ; @ Gruen #11
#define SIG_BILD_12_b      b'01000000'  ; @ Gruen #12
#define SIG_BILD_13_b      b'01000000'  ; @ Gruen #13
#define SIG_BILD_14_b      b'11000000'  ; @ Gruen #14
#define SIG_BILD_15_b      b'00000000'  ; @ Gruen #15
#define SIG_BILD_16_b      b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02_b_p2   b'00000000'  ; @ Gruen #2
#define SIG_BILD_03_b_p2   b'00000000'  ; @ Gruen #3
;                            +--------- #7: Set   Output #10
;                             +-------- #6: Set   Output #9



; *** Signalbilder S1 flash
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag muss auch fr "ein" gesetzt werden
; *** Signalbilder Flash Out 1..8
#define SIG_BILD_01F_a     b'00000000'  ; @ Gruen #1
#define SIG_BILD_02F_a     b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_a     b'00000100'  ; @ Gruen #3
#define SIG_BILD_04F_a     b'00000000'  ; @ Gruen #4
#define SIG_BILD_05F_a     b'10000000'  ; @ Gruen #5
#define SIG_BILD_06F_a     b'00000000'  ; @ Gruen #6
#define SIG_BILD_07F_a     b'00000000'  ; @ Gruen #7
#define SIG_BILD_08F_a     b'10000000'  ; @ Gruen #8
#define SIG_BILD_09F_a     b'10000000'  ; @ Gruen #9
#define SIG_BILD_10F_a     b'00000000'  ; @ Gruen #10
#define SIG_BILD_11F_a     b'10000000'  ; @ Gruen #11
#define SIG_BILD_12F_a     b'00000010'  ; @ Gruen #12
#define SIG_BILD_13F_a     b'00010010'  ; @ Gruen #13
#define SIG_BILD_14F_a     b'00000000'  ; @ Gruen #14
#define SIG_BILD_15F_a     b'00000000'  ; @ Gruen #15
#define SIG_BILD_16F_a     b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02F_a_p2  b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_a_p2  b'00000000'  ; @ Gruen #3
;                            +--------- #7: Flash Output #8
;                             +-------- #6: Flash Output #7
;                              +------- #5: Flash Output #6
;                               +------ #4: Flash Output #5
;                                +----- #3: Flash Output #4
;                                 +---- #2: Flash Output #3
;                                  +--- #1: Flash Output #2
;                                   +-- #0: Flash Output #1

; *** Signalbilder Flash Out 9..10
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_01F_b     b'01000000'  ; @ Gruen #1
#define SIG_BILD_02F_b     b'01000000'  ; @ Gruen #2
#define SIG_BILD_03F_b     b'01000000'  ; @ Gruen #3
#define SIG_BILD_04F_b     b'10000000'  ; @ Gruen #4
#define SIG_BILD_05F_b     b'10000000'  ; @ Gruen #5
#define SIG_BILD_06F_b     b'01000000'  ; @ Gruen #6
#define SIG_BILD_07F_b     b'01000000'  ; @ Gruen #7
#define SIG_BILD_08F_b     b'10000000'  ; @ Gruen #8
#define SIG_BILD_09F_b     b'10000000'  ; @ Gruen #9
#define SIG_BILD_10F_b     b'01000000'  ; @ Gruen #10
#define SIG_BILD_11F_b     b'10000000'  ; @ Gruen #11
#define SIG_BILD_12F_b     b'01000000'  ; @ Gruen #12
#define SIG_BILD_13F_b     b'01000000'  ; @ Gruen #13
#define SIG_BILD_14F_b     b'11000000'  ; @ Gruen #14
#define SIG_BILD_15F_b     b'00000000'  ; @ Gruen #15
#define SIG_BILD_16F_b     b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02F_b_p2  b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_b_p2  b'00000000'  ; @ Gruen #3
;                            +--------- #7: Flash Output #10
;                             +-------- #6: Flash Output #9
#endif  ;#ifdef aufteilung_output_all

;----------------------------------------------------------------------
; Definition, ob bei bestimmten Adressen keine nderung des Signal-Bildes stattfinden soll
;----------------------------------------------------------------------
; - insbesondere, wenn Adresse Dunkeltastung aktivieren / deaktivieren soll
; - auch, wenn Adresse den Wechsel der Adress-Page hervorruft
;#define adr1_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr2_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr3_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr4_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr5_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr6_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr7_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr8_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr9_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr10_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr11_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr12_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr13_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr14_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr15_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr16_no_change  ; diese Adresse beeinflusst Signalbild nicht

;---------------------------------------------------------------------
; Definition Dunkeltastung S2 -> Durch welche Befehle S2 (Vorsignal) Dunkel wird
;----------------------------------------------------------------------
;#define s2_dark_by_1 ; Dunkeltastung von S2 durch Grn 1
;#define s2_dark_by_2 ; Dunkeltastung von S2 durch Grn 2
;#define s2_dark_by_3 ; Dunkeltastung von S2 durch Grn 3
;#define s2_dark_by_4 ; Dunkeltastung von S2 durch Grn 4
;#define s2_dark_by_5 ; Dunkeltastung von S2 durch Grn 5
;#define s2_dark_by_6 ; Dunkeltastung von S2 durch Grn 6
;#define s2_dark_by_7 ; Dunkeltastung von S2 durch Grn 7
;#define s2_dark_by_8 ; Dunkeltastung von S2 durch Grn 8

;----------------------------------------------------------------------
; Definition Dunkeltastung S2 -> Welche Outputs dunkel werden
;----------------------------------------------------------------------
; Definition, weclhe Ausgnge durch 's2_dark_by_..' dunkel werden
;#define dunkeltast_out01      ; hier wird definiert,
;#define dunkeltast_out02      ; .. welche
;#define dunkeltast_out03      ; .. Ausgnge
;#define dunkeltast_out04      ; .. auf die
;#define dunkeltast_out05      ; .. Dunkeltastung (des Signal2)
;#define dunkeltast_out06      ; .. reagieren und bei
;#define dunkeltast_out07      ; .. Dunkeltastung
;#define dunkeltast_out08      ; .. dann auch
;#define dunkeltast_out09      ; .. abschalten
;#define dunkeltast_out10      ; ..

;----------------------------------------------------------------------
; Definition Dunkeltastung ALLE -> Durch welche Befehle ALLES Dunkel wird
;----------------------------------------------------------------------
; - wird hier fr ein Befehl aktiviert, ndert die entsprechende Adresse das Signalbild nicht
; - die Befehle sollten zu S2 gehren

;#define use_all_dark    ; Dunkeltastung ALL verwenden

;#define all_dark_by_1  ; Dunkeltastung des gesamten Signal durch Grn 1
;#define all_dark_by_2  ; Dunkeltastung des gesamten Signal durch Grn 2
;#define all_dark_by_3  ; Dunkeltastung des gesamten Signal durch Grn 3
;#define all_dark_by_4  ; Dunkeltastung des gesamten Signal durch Grn 4
;#define all_dark_by_5  ; Dunkeltastung des gesamten Signal durch Grn 5
;#define all_dark_by_6  ; Dunkeltastung des gesamten Signal durch Grn 6
;#define all_dark_by_7  ; Dunkeltastung des gesamten Signal durch Grn 7
;#define all_dark_by_8  ; Dunkeltastung des gesamten Signal durch Grn 8
;#define all_dark_by_9  ; Dunkeltastung des gesamten Signal durch Grn 9
;#define all_dark_by_10 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_11 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_12 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_13 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_14 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_15 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_16 ; Dunkeltastung des gesamten Signal durch Grn 10

; hier wird definiert, welche Befehle die Dukeltastung ALLE wieder aufgehoben wird.
; - wird hier fr ein Befehl aktiviert, ndert die entsprechende Adresse das Signalbild nicht
; - die Befehle knnen zu S1 oder S2 gehren
;#define quit_dark_by_1  ; Dunkeltastung des gesamten Signals durch Grn 1  beenden
;#define quit_dark_by_2  ; Dunkeltastung des gesamten Signals durch Grn 2  beenden
;#define quit_dark_by_3  ; Dunkeltastung des gesamten Signals durch Grn 3  beenden
;#define quit_dark_by_4  ; Dunkeltastung des gesamten Signals durch Grn 4  beenden
;#define quit_dark_by_5  ; Dunkeltastung des gesamten Signals durch Grn 5  beenden
;#define quit_dark_by_6  ; Dunkeltastung des gesamten Signals durch Grn 6  beenden
;#define quit_dark_by_7  ; Dunkeltastung des gesamten Signals durch Grn 7  beenden
;#define quit_dark_by_8  ; Dunkeltastung des gesamten Signals durch Grn 8  beenden
;#define quit_dark_by_9  ; Dunkeltastung des gesamten Signals durch Grn 9  beenden
;#define quit_dark_by_10 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_11 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_12 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_13 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_14 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_15 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_16 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden

; Definition, welche Ausgnge durch 'all_dark_by_..' dunkel werden
#define dunkeltast_all_out01      ; hier wird definiert,
#define dunkeltast_all_out02      ; .. welche
#define dunkeltast_all_out03      ; .. Ausgnge
#define dunkeltast_all_out04      ; .. auf die
#define dunkeltast_all_out05      ; .. Dunkeltastung (des Signal2)
#define dunkeltast_all_out06      ; .. reagieren und bei
#define dunkeltast_all_out07      ; .. Dunkeltastung
#define dunkeltast_all_out08      ; .. dann auch
#define dunkeltast_all_out09      ; .. abschalten
#define dunkeltast_all_out10      ; ..

;----------------------------------------------------------------------
; Definition Adress-Page: gibt es eine Umschaltung zwischen 2 Adress-Pages
;----------------------------------------------------------------------
; ist dies aktiviert, wird mittels eines Befehles zwischen 2 Adress-Pages umgeschaltet.
; Es kann damit das Signal auf verschiedene Adressen reagieren und auch unterschiedliche Signalbilder anzeigen
; z.B. in Abhngigkeit der folgenden Weiche.
; - Wenn die Adresse nicht zu einer Adress-Page zugeordnet werden soll, dann wird hier weder gruenx_page1_aktiv noch gruenx_page2_aktiv aktiviert.
; - hier wird also nur aktiviert, wenn nur auf eine Adress-Page reagiert werden soll.
; - es werden die EE-Zellen 0x40 0x50 0x60 0x70 genutzt, um die Buffer zu speichern, sodass diese nach einem RESET wieder zur Verfgung stehen

;#define use_paged_adressing ;  = Page_Adressen verwenden
;#define use_buffer_register ;  muss verwendet werden, wenn fahrwegabhngige Adressen, nicht bei fahrwegabhngigen Signalbildern!

; hier wird definiert, durch welche Adresse zwischen den Adress-Pages gewechselt wird
;#define gruen1_page1_aktiv
;#define gruen2_page1_aktiv
;#define gruen3_page1_aktiv
;#define gruen4_page1_aktiv
;#define gruen5_page1_aktiv
;#define gruen6_page1_aktiv
;#define gruen7_page1_aktiv
;#define gruen8_page1_aktiv
;#define gruen9_page1_aktiv
;#define gruen10_page1_aktiv
;#define gruen11_page1_aktiv
;#define gruen12_page1_aktiv
;#define gruen13_page1_aktiv
;#define gruen14_page1_aktiv
;#define gruen15_page1_aktiv
;#define gruen16_page1_aktiv

;#define gruen1_page2_aktiv
;#define gruen2_page2_aktiv
;#define gruen3_page2_aktiv
;#define gruen4_page2_aktiv
;#define gruen5_page2_aktiv
;#define gruen6_page2_aktiv
;#define gruen7_page2_aktiv
;#define gruen8_page2_aktiv
;#define gruen9_page2_aktiv
;#define gruen10_page2_aktiv
;#define gruen11_page2_aktiv
;#define gruen12_page2_aktiv
;#define gruen13_page2_aktiv
;#define gruen14_page2_aktiv
;#define gruen15_page2_aktiv
;#define gruen16_page2_aktiv

; hier wird definiert, ob auf eine Adresse nur bei einer bestimmten Adress-Page reagiert werden darf.
; (d.h. fahrwegabhngige Adressen - im Gegensatz zu fahrwegabhngigen Signalbildern!)
;#define  gruen1_if_page1
;#define  gruen2_if_page1
;#define  gruen3_if_page1
;#define  gruen4_if_page1
;#define  gruen5_if_page1
;#define  gruen6_if_page1
;#define  gruen7_if_page1
;#define  gruen8_if_page1
;#define  gruen9_if_page1
;#define gruen10_if_page1
;#define gruen11_if_page1
;#define gruen12_if_page1
;#define gruen13_if_page1
;#define gruen14_if_page1
;#define gruen15_if_page1
;#define gruen16_if_page1

;#define  gruen1_if_page2
;#define  gruen2_if_page2
;#define  gruen3_if_page2
;#define  gruen4_if_page2
;#define  gruen5_if_page2
;#define  gruen6_if_page2
;#define  gruen7_if_page2
;#define  gruen8_if_page2
;#define  gruen9_if_page2
;#define gruen10_if_page2
;#define gruen11_if_page2
;#define gruen12_if_page2
;#define gruen13_if_page2
;#define gruen14_if_page2
;#define gruen15_if_page2
;#define gruen16_if_page2

; hier wird definiert, ob ein Adress-Page spezifisches Signalbild angezeigt werden soll.
; (d.h. fahrwegabhngige Signalbilder - im Gegensatz zu fahrwegabhngigen Adressen)
;#define  gruen1_depends_on_page
;#define  gruen2_depends_on_page
;#define  gruen3_depends_on_page
;#define  gruen4_depends_on_page
;#define  gruen5_depends_on_page
;#define  gruen6_depends_on_page
;#define  gruen7_depends_on_page
;#define  gruen8_depends_on_page
;#define  gruen9_depends_on_page
;#define gruen10_depends_on_page
;#define gruen11_depends_on_page
;#define gruen12_depends_on_page
;#define gruen13_depends_on_page
;#define gruen14_depends_on_page
;#define gruen15_depends_on_page
;#define gruen16_depends_on_page


;----------------------------------------------------------------------
; Definition Blinkmuster f. SAVE-Blinken
;----------------------------------------------------------------------
; eine '1' bedeutet, dass der Ausgang beim Lernen der entsprechenden Adresse blinkt
#define save_01_port_a  b'00010001'    ; save Adresse 01 
#define save_02_port_a  b'00010001'    ; save Adresse 02 
#define save_03_port_a  b'00010001'    ; save Adresse 03 
#define save_04_port_a  b'00000010'    ; save Adresse 04 
#define save_05_port_a  b'00100010'    ; save Adresse 05 
#define save_06_port_a  b'00010010'    ; save Adresse 06 
#define save_07_port_a  b'00010010'    ; save Adresse 07 
#define save_08_port_a  b'00100010'    ; save Adresse 08 
#define save_09_port_a  b'00100010'    ; save Adresse 09 
#define save_10_port_a  b'00010010'    ; save Adresse 10 
#define save_11_port_a  b'00100000'    ; save Adresse 11 
#define save_12_port_a  b'00100000'    ; save Adresse 12 
#define save_13_port_a  b'00100000'    ; save Adresse 13 
#define save_14_port_a  b'00000010'    ; save Adresse 14 
#define save_15_port_a  b'00000000'    ; save Adresse 15 
#define save_16_port_a  b'00000000'    ; save Adresse 16 
;                           +------- #5: Flash Output #2
;                            +------ #4: Flash Output #1
;                               +--- #1: Flash Output #4
;                                +-- #0: Flash Output #3

#define save_01_port_c  b'00010000'  ; save Adresse 01
#define save_02_port_c  b'00010110'  ; save Adresse 02
#define save_03_port_c  b'00010000'  ; save Adresse 03
#define save_04_port_c  b'00001001'  ; save Adresse 04
#define save_05_port_c  b'00101001'  ; save Adresse 05
#define save_06_port_c  b'00010100'  ; save Adresse 06
#define save_07_port_c  b'00010010'  ; save Adresse 07
#define save_08_port_c  b'00101100'  ; save Adresse 08
#define save_09_port_c  b'00101010'  ; save Adresse 09
#define save_10_port_c  b'00010110'  ; save Adresse 10
#define save_11_port_c  b'00101110'  ; save Adresse 11
#define save_12_port_c  b'00010100'  ; save Adresse 12
#define save_13_port_c  b'00010110'  ; save Adresse 13
#define save_14_port_c  b'00011000'  ; save Adresse 14
#define save_15_port_c  b'00000000'  ; save Adresse 15
#define save_16_port_c  b'00000000'  ; save Adresse 16
;                           +------- #5: Flash Output #8
;                            +------ #4: Flash Output #9
;                             +----- #3: Flash Output #10
;                              +---- #2: Flash Output #6
;                               +--- #1: Flash Output #5
;                                +-- #0: Flash Output #7

;----------------------------------------------------
; Parameter
;----------------------------------------------------
CONST_FADE_01          equ d'120'        ; Fade Geschwindigkeit @ Signalbild-Wechsel

;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;-- Ab hier muss man idR. nichts mehr anpassen !!
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
; wenn Adresse Dunkeltastung EIN oder AUS schaltet-> kein Signalbild verndern
;----------------------------------------------------------------------
#ifdef all_dark_by_1
 #define adr1_no_change
#endif
#ifdef all_dark_by_2
 #define adr2_no_change
#endif
#ifdef all_dark_by_3
 #define adr3_no_change
#endif
#ifdef all_dark_by_4
 #define adr4_no_change
#endif
#ifdef all_dark_by_5
 #define adr5_no_change
#endif
#ifdef all_dark_by_6
 #define adr6_no_change
#endif
#ifdef all_dark_by_7
 #define adr7_no_change
#endif
#ifdef all_dark_by_8
 #define adr8_no_change
#endif
#ifdef all_dark_by_9
 #define adr9_no_change
#endif
#ifdef all_dark_by_10
 #define adr10_no_change
#endif
#ifdef all_dark_by_11
 #define adr11_no_change
#endif
#ifdef all_dark_by_12
 #define adr12_no_change
#endif
#ifdef all_dark_by_13
 #define adr13_no_change
#endif
#ifdef all_dark_by_14
 #define adr14_no_change
#endif
#ifdef all_dark_by_15
 #define adr15_no_change
#endif
#ifdef all_dark_by_16
 #define adr16_no_change
#endif

#ifdef quit_dark_by_1
 #define adr1_no_change
#endif
#ifdef quit_dark_by_2
 #define adr2_no_change
#endif
#ifdef quit_dark_by_3
 #define adr3_no_change
#endif
#ifdef quit_dark_by_4
 #define adr4_no_change
#endif
#ifdef quit_dark_by_5
 #define adr5_no_change
#endif
#ifdef quit_dark_by_6
 #define adr6_no_change
#endif
#ifdef quit_dark_by_7
 #define adr7_no_change
#endif
#ifdef quit_dark_by_8
 #define adr8_no_change
#endif
#ifdef quit_dark_by_9
 #define adr9_no_change
#endif
#ifdef quit_dark_by_10
 #define adr10_no_change
#endif
#ifdef quit_dark_by_11
 #define adr11_no_change
#endif
#ifdef quit_dark_by_12
 #define adr12_no_change
#endif
#ifdef quit_dark_by_13
 #define adr13_no_change
#endif
#ifdef quit_dark_by_14
 #define adr14_no_change
#endif
#ifdef quit_dark_by_15
 #define adr14_no_change
#endif
#ifdef quit_dark_by_16
 #define adr14_no_change
#endif

;----------------------------------------------------------------------
; alternative Zuordnungen, hier nichts ndern!
;----------------------------------------------------------------------
; select, if Output 5/6/7/8 belongs to S1/S2
#ifdef aufteilung_output4_x
 #define out5_to_s2  ; Out 5 belongs to S2
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output5_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output6_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output7_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output8_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s1  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output_all
 #define out5_to_s1  ; Out 5 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out6_to_s1  ; Out 6 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out7_to_s1  ; Out 7 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out8_to_s1  ; Out 8 belongs to S2 (S1 ist hier kein eigenstndiges Signal!)
#endif

; select, if Grn5/6/7/8 belongs to S1/S2 (for filter R_STATUS / -_2)
#ifdef aufteilung4_x
 #define gruen5_to_s2  ; Green 5 belongs to S2
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11110001'
status_g02_a equ b'11110010'
status_g03_a equ b'11110100'
status_g04_a equ b'11111000'
status_g05_a equ b'00011111'
status_g06_a equ b'00101111'
status_g07_a equ b'01001111'
status_g08_a equ b'10001111'
status_g09_a equ b'00001111'
status_g10_a equ b'00001111'
status_g11_a equ b'00001111'
status_g12_a equ b'00001111'
status_g13_a equ b'00001111'
status_g14_a equ b'00001111'
status_g15_a equ b'00001111'
status_g16_a equ b'00001111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'00000000'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung5_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11100001'
status_g02_a equ b'11100010'
status_g03_a equ b'11100100'
status_g04_a equ b'11101000'
status_g05_a equ b'11110000'
status_g06_a equ b'00111111'
status_g07_a equ b'01011111'
status_g08_a equ b'10011111'
status_g09_a equ b'00011111'
status_g10_a equ b'00011111'
status_g11_a equ b'00011111'
status_g12_a equ b'00011111'
status_g13_a equ b'00011111'
status_g14_a equ b'00011111'
status_g15_a equ b'00011111'
status_g16_a equ b'00011111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung6_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11000001'
status_g02_a equ b'11000010'
status_g03_a equ b'11000100'
status_g04_a equ b'11001000'
status_g05_a equ b'11010000'
status_g06_a equ b'11100000'
status_g07_a equ b'01111111'
status_g08_a equ b'10111111'
status_g09_a equ b'00111111'
status_g10_a equ b'00111111'
status_g11_a equ b'00111111'
status_g12_a equ b'00111111'
status_g13_a equ b'00111111'
status_g14_a equ b'00111111'
status_g15_a equ b'00111111'
status_g16_a equ b'00111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung7_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'10000001'
status_g02_a equ b'10000010'
status_g03_a equ b'10000100'
status_g04_a equ b'10001000'
status_g05_a equ b'10010000'
status_g06_a equ b'10100000'
status_g07_a equ b'11000000'
status_g08_a equ b'11111111'
status_g09_a equ b'01111111'
status_g10_a equ b'01111111'
status_g11_a equ b'01111111'
status_g12_a equ b'01111111'
status_g13_a equ b'01111111'
status_g14_a equ b'01111111'
status_g15_a equ b'01111111'
status_g16_a equ b'01111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'11111111'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung8_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s1  ; Green 8 belongs to S2
status_g01_a equ b'00000001'
status_g02_a equ b'00000010'
status_g03_a equ b'00000100'
status_g04_a equ b'00001000'
status_g05_a equ b'00010000'
status_g06_a equ b'00100000'
status_g07_a equ b'01000000'
status_g08_a equ b'10000000'
status_g09_a equ b'11111111'
status_g10_a equ b'11111111'
status_g11_a equ b'11111111'
status_g12_a equ b'11111111'
status_g13_a equ b'11111111'
status_g14_a equ b'11111111'
status_g15_a equ b'11111111'
status_g16_a equ b'11111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'11111111'
status_g08_b equ b'11111111'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung_all
status_g01_a equ b'00000001'
status_g02_a equ b'00000010'
status_g03_a equ b'00000100'
status_g04_a equ b'00001000'
status_g05_a equ b'00010000'
status_g06_a equ b'00100000'
status_g07_a equ b'01000000'
status_g08_a equ b'10000000'
status_g09_a equ b'00000000'
status_g10_a equ b'00000000'
status_g11_a equ b'00000000'
status_g12_a equ b'00000000'
status_g13_a equ b'00000000'
status_g14_a equ b'00000000'
status_g15_a equ b'00000000'
status_g16_a equ b'00000000'
status_g01_b equ b'00000000'
status_g02_b equ b'00000000'
status_g03_b equ b'00000000'
status_g04_b equ b'00000000'
status_g05_b equ b'00000000'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif

;----------------------------------------------------
; Mapping: Flag -> Output, hier nichts ndern!
;----------------------------------------------------
; hier wird definiert, bei welchem Flag von R_STATUS_ON_S1_akt bzw, R_STATUS_ON_S2_akt
; der entsprechende Output reagiert. Die Zuordnung S1/S2 wird durch die #defines geregelt,
; daher sind in diesem Abschnitt keine nderungen zur Anpassung ntig.

; *** S1
#define SET_OUTPUT1          R_STATUS_ON_S1_akt,0
#define SET_OUTPUT1_FL       R_STATUS_FL_S1_akt,0    ;
#define SET_OUTPUT1_FL_PRE   R_STATUS_FL_S1_pre,0    ;
#define SET_OUTPUT2          R_STATUS_ON_S1_akt,1
#define SET_OUTPUT2_FL       R_STATUS_FL_S1_akt,1    ;
#define SET_OUTPUT2_FL_PRE   R_STATUS_FL_S1_pre,1    ;
#define SET_OUTPUT3          R_STATUS_ON_S1_akt,2
#define SET_OUTPUT3_FL       R_STATUS_FL_S1_akt,2    ;
#define SET_OUTPUT3_FL_PRE   R_STATUS_FL_S1_pre,2    ;
#define SET_OUTPUT4          R_STATUS_ON_S1_akt,3
#define SET_OUTPUT4_FL       R_STATUS_FL_S1_akt,3    ;
#define SET_OUTPUT4_FL_PRE   R_STATUS_FL_S1_pre,3    ;
#ifdef out5_to_s1
  #define SET_OUTPUT5        R_STATUS_ON_S1_akt,4
  #define SET_OUTPUT5_FL     R_STATUS_FL_S1_akt,4    ;
  #define SET_OUTPUT5_FL_PRE R_STATUS_FL_S1_pre,4    ;
#endif
#ifdef out6_to_s1
  #define SET_OUTPUT6        R_STATUS_ON_S1_akt,5
  #define SET_OUTPUT6_FL     R_STATUS_FL_S1_akt,5    ;
  #define SET_OUTPUT6_FL_PRE R_STATUS_FL_S1_pre,5    ;
#endif
#ifdef out7_to_s1
  #define SET_OUTPUT7        R_STATUS_ON_S1_akt,6
  #define SET_OUTPUT7_FL     R_STATUS_FL_S1_akt,6    ;
  #define SET_OUTPUT7_FL_PRE R_STATUS_FL_S1_pre,6    ;
#endif
#ifdef out8_to_s1
  #define SET_OUTPUT8        R_STATUS_ON_S1_akt,7
  #define SET_OUTPUT8_FL     R_STATUS_FL_S1_akt,7    ;
  #define SET_OUTPUT8_FL_PRE R_STATUS_FL_S1_pre,7    ;
#endif

; *** S2
#ifdef out5_to_s2
  #define SET_OUTPUT5        R_STATUS_ON_S2_akt,2
  #define SET_OUTPUT5_FL     R_STATUS_FL_S2_akt,2    ;
  #define SET_OUTPUT5_FL_PRE R_STATUS_FL_S2_pre,2    ;
#endif
#ifdef out6_to_s2
  #define SET_OUTPUT6        R_STATUS_ON_S2_akt,3
  #define SET_OUTPUT6_FL     R_STATUS_FL_S2_akt,3    ;
  #define SET_OUTPUT6_FL_PRE R_STATUS_FL_S2_pre,3    ;
#endif
#ifdef out7_to_s2
  #define SET_OUTPUT7        R_STATUS_ON_S2_akt,4
  #define SET_OUTPUT7_FL     R_STATUS_FL_S2_akt,4    ;
  #define SET_OUTPUT7_FL_PRE R_STATUS_FL_S2_pre,4    ;
#endif
#ifdef out8_to_s2
  #define SET_OUTPUT8        R_STATUS_ON_S2_akt,5
  #define SET_OUTPUT8_FL     R_STATUS_FL_S2_akt,5    ;
  #define SET_OUTPUT8_FL_PRE R_STATUS_FL_S2_pre,5    ;
#endif
#define SET_OUTPUT9          R_STATUS_ON_S2_akt,6
#define SET_OUTPUT9_FL       R_STATUS_FL_S2_akt,6    ;
#define SET_OUTPUT9_FL_PRE   R_STATUS_FL_S2_pre,6    ;
#define SET_OUTPUT10         R_STATUS_ON_S2_akt,7
#define SET_OUTPUT10_FL      R_STATUS_FL_S2_akt,7    ;
#define SET_OUTPUT10_FL_PRE  R_STATUS_FL_S2_pre,7    ;


