; ********************************************************
; ** Version: V3.38 vom 22/02/2010                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2009-2010 Sven Brandt

; ********************************************************
; History:
; ********************************************************
; V3.38 - Sand-4: - lngere Totzeit beim Restore (delay_v_i war '40' jetzt '50')
; V3.35 - Sand-4: - Fehlerbeseitigung
; V3.34 - Sand-4: - Taster-Erkennung hinzu
; V3.33 - Sand-4: - ROM-Einsparung
;                 - Vorbereitungen zu Taster-Erkennung
;                 - Verzgerung Voltage-Impulse uber Konstante definierbar
; V3.32 - Sand-4: - Korreturen beim Aufruf (value preload) von "stop" / "break"
;                 - Totzeit zwischen Einschalten Vcc und erstem Inpuls hinzu ("wait"-Flag)
;                 - IR sperren beim Timer lschen fr Inpulse-Generierung & EEPROM-Lesen
; V3.30 - Preload R_COUNT_STRETCH_0x korr., PWM-Berechnung korr. (Bit #7)
; V3.29 - Korrektur PWM-Berechnung, Justage (PWM-Vorgabe fixiert)
; V3.28 - Korrektur u. Erweiterung Justage-Modus, Sperre Wiederholer hinzu, Sperre Unterbrechung hinzu, LED-Signalisierung


  ; ****************************
  ; ** Register & Flags       **
  ; ****************************
R_COUNT_01            equ  40h          ; Table counter
R_COUNT_02            equ  41h          ; Table counter
R_COUNT_03            equ  42h          ; Table counter
R_COUNT_04            equ  43h          ; Table counter
R_KURVE_NUMBER_01     equ  44h          ; Kurven-Parameter Anzahl Kurvenpunkte
R_KURVE_NUMBER_02     equ  45h          ; Kurven-Parameter Anzahl Kurvenpunkte
R_KURVE_NUMBER_03     equ  46h          ; Kurven-Parameter Anzahl Kurvenpunkte
R_KURVE_NUMBER_04     equ  47h          ; Kurven-Parameter Anzahl Kurvenpunkte
R_COUNT_STRETCH_01    equ  48h          ; Counter for Stretching
R_COUNT_STRETCH_02    equ  49h          ; Counter for Stretching
R_COUNT_STRETCH_03    equ  4ah          ; Counter for Stretching
R_COUNT_STRETCH_04    equ  4bh          ; Counter for Stretching
R_KURVE_STRETCH_01    equ  4ch          ; Kurven-Parameter Anzahl der Impulse der selben Lnge (1..255)
R_KURVE_STRETCH_02    equ  4dh          ; Kurven-Parameter Anzahl der Impulse der selben Lnge (1..255)
R_KURVE_STRETCH_03    equ  4eh          ; Kurven-Parameter Anzahl der Impulse der selben Lnge (1..255)
R_KURVE_STRETCH_04    equ  4fh          ; Kurven-Parameter Anzahl der Impulse der selben Lnge (1..255)

R_SERVO_ACTIVE        equ  50h          ; Servo Selection Register
    #define SERVO1_ACTIVE             R_SERVO_ACTIVE,0; Servo 1 active
    #define SERVO2_ACTIVE             R_SERVO_ACTIVE,1; Servo 2 active
    #define SERVO3_ACTIVE             R_SERVO_ACTIVE,2; Servo 3 active
    #define SERVO4_ACTIVE             R_SERVO_ACTIVE,3; Servo 4 active
    #define SERVO1_LOOP               R_SERVO_ACTIVE,4; Servo 1 Loop
    #define SERVO2_LOOP               R_SERVO_ACTIVE,5; Servo 2 Loop
    #define SERVO3_LOOP               R_SERVO_ACTIVE,6; Servo 3 Loop
    #define SERVO4_LOOP               R_SERVO_ACTIVE,7; Servo 4 Loop
R_ADR_FLAGS_SERVO     equ  51h          ; Flags for Adress Info
    #define USE_SERVO1                R_ADR_FLAGS_SERVO,0; Servo 1 used with this adress
    #define USE_SERVO2                R_ADR_FLAGS_SERVO,1; Servo 2 used with this adress
    #define USE_SERVO3                R_ADR_FLAGS_SERVO,2; Servo 3 used with this adress
    #define USE_SERVO4                R_ADR_FLAGS_SERVO,3; Servo 4 used with this adress
    #define USE_SERVOx_LOOP           R_ADR_FLAGS_SERVO,4; loop 
    #define USE_SERVOx_BREAK          R_ADR_FLAGS_SERVO,5; abbruch (sofort, Kurve wird abgebrochen)
    #define USE_SERVOx_STOP           R_ADR_FLAGS_SERVO,6; beenden (Stopp nach Ablauf der Kurve)
R_ADR_FLAGS_MAP       equ  52h          ; Flags for Adress Info
    #define USE_RELAIS1               R_ADR_FLAGS_MAP,0; Relais 1 used with this adress
    #define USE_RELAIS2               R_ADR_FLAGS_MAP,1; Relais 2 used with this adress
    #define USE_RELAIS3               R_ADR_FLAGS_MAP,2; Relais 3 used with this adress
    #define USE_RELAIS4               R_ADR_FLAGS_MAP,3; Relais 4 used with this adress
    #define USE_CURVE1                R_ADR_FLAGS_MAP,4; Curve 1 used with this adress
    #define USE_CURVE2                R_ADR_FLAGS_MAP,5; Curve 2 used with this adress
    #define USE_CURVE3                R_ADR_FLAGS_MAP,6; Curve 3 used with this adress
    #define USE_CURVE4                R_ADR_FLAGS_MAP,7; Curve 4 used with this adress
R_4094_VALUES         equ  54h          ; Flags for 4014 contens
    #define R1_ON                     R_4094_VALUES,0; switch Relais 1 on
    #define R2_ON                     R_4094_VALUES,1; switch Relais 2 on
    #define R3_ON                     R_4094_VALUES,2; switch Relais 3 on
    #define R4_ON                     R_4094_VALUES,3; switch Relais 4 on
    #define V4_ON                     R_4094_VALUES,4; switch Vcc for Servo 1 on
    #define V3_ON                     R_4094_VALUES,5; switch Vcc for Servo 2 on
    #define V2_ON                     R_4094_VALUES,7; switch Vcc for Servo 3 on
    #define V1_ON                     R_4094_VALUES,6; switch Vcc for Servo 4 on

R_TABLEREAD           equ  55h          ; request this line for Table-Read
R_PWMVALUE            equ  56h          ; Pulse Width 124..255(dez)

R_SERVO_MAP_01        equ  57h          ; Mapping fr Servo 1
    #define SERVO_01_USE_R1            R_SERVO_MAP_01,0
    #define SERVO_01_USE_R2            R_SERVO_MAP_01,1
    #define SERVO_01_USE_R3            R_SERVO_MAP_01,2
    #define SERVO_01_USE_R4            R_SERVO_MAP_01,3
    #define SERVO_01_USE_C1            R_SERVO_MAP_01,4
    #define SERVO_01_USE_C2            R_SERVO_MAP_01,5
    #define SERVO_01_USE_C3            R_SERVO_MAP_01,6
    #define SERVO_01_USE_C4            R_SERVO_MAP_01,7
R_SERVO_MAP_02        equ  58h          ; Mapping fr Servo 2
    #define SERVO_02_USE_R1            R_SERVO_MAP_02,0
    #define SERVO_02_USE_R2            R_SERVO_MAP_02,1
    #define SERVO_02_USE_R3            R_SERVO_MAP_02,2
    #define SERVO_02_USE_R4            R_SERVO_MAP_02,3
    #define SERVO_02_USE_C1            R_SERVO_MAP_02,4
    #define SERVO_02_USE_C2            R_SERVO_MAP_02,5
    #define SERVO_02_USE_C3            R_SERVO_MAP_02,6
    #define SERVO_02_USE_C4            R_SERVO_MAP_02,7
R_SERVO_MAP_03        equ  59h          ; Mapping fr Servo 3
    #define SERVO_03_USE_R1            R_SERVO_MAP_03,0
    #define SERVO_03_USE_R2            R_SERVO_MAP_03,1
    #define SERVO_03_USE_R3            R_SERVO_MAP_03,2
    #define SERVO_03_USE_R4            R_SERVO_MAP_03,3
    #define SERVO_03_USE_C1            R_SERVO_MAP_03,4
    #define SERVO_03_USE_C2            R_SERVO_MAP_03,5
    #define SERVO_03_USE_C3            R_SERVO_MAP_03,6
    #define SERVO_03_USE_C4            R_SERVO_MAP_03,7
R_SERVO_MAP_04        equ  5ah          ; Mapping fr Servo 4
    #define SERVO_04_USE_R1            R_SERVO_MAP_04,0
    #define SERVO_04_USE_R2            R_SERVO_MAP_04,1
    #define SERVO_04_USE_R3            R_SERVO_MAP_04,2
    #define SERVO_04_USE_R4            R_SERVO_MAP_04,3
    #define SERVO_04_USE_C1            R_SERVO_MAP_04,4
    #define SERVO_04_USE_C2            R_SERVO_MAP_04,5
    #define SERVO_04_USE_C3            R_SERVO_MAP_04,6
    #define SERVO_04_USE_C4            R_SERVO_MAP_04,7

R_COUNT_TEMP          equ  5bh          ; R_COUNT Transferregister
R_SERVO_FLAGS_01      equ  5ch          ; Kurven-Parameter Flags
    #define S01_SERVO_FLAG_WAIT        R_SERVO_FLAGS_01,0  ; Wait after Start (Totzeit)
    #define S01_SERVO_FLAG_V_OFF       R_SERVO_FLAGS_01,1  ; switch V off after RUN
    #define S01_SERVO_FLAG_I_OFF       R_SERVO_FLAGS_01,2  ; switch Impulse off after RUN
    #define S01_SERVO_FLAG_NOINT       R_SERVO_FLAGS_01,6  ; reject interrupt of running servo.
    #define S01_SERVO_FLAG_ONLY1       R_SERVO_FLAGS_01,7  ; reject repetition of same adr.
R_SERVO_FLAGS_02      equ  5dh          ; Kurven-Parameter Flags
    #define S02_SERVO_FLAG_WAIT        R_SERVO_FLAGS_02,0  ; Wait after Start (Totzeit)
    #define S02_SERVO_FLAG_V_OFF       R_SERVO_FLAGS_02,1  ; switch V off after RUN
    #define S02_SERVO_FLAG_I_OFF       R_SERVO_FLAGS_02,2  ; switch Impulse off after RUN
    #define S02_SERVO_FLAG_NOINT       R_SERVO_FLAGS_02,6  ; reject interrupt of running servo.
    #define S02_SERVO_FLAG_ONLY1       R_SERVO_FLAGS_02,7  ; reject repetition of same adr.
R_SERVO_FLAGS_03      equ  5eh          ; Kurven-Parameter Flags
    #define S03_SERVO_FLAG_WAIT        R_SERVO_FLAGS_03,0  ; Wait after Start (Totzeit)
    #define S03_SERVO_FLAG_V_OFF       R_SERVO_FLAGS_03,1  ; switch V off after RUN
    #define S03_SERVO_FLAG_I_OFF       R_SERVO_FLAGS_03,2  ; switch Impulse off after RUN
    #define S03_SERVO_FLAG_NOINT       R_SERVO_FLAGS_03,6  ; reject interrupt of running servo.
    #define S03_SERVO_FLAG_ONLY1       R_SERVO_FLAGS_03,7  ; reject repetition of same adr.
R_SERVO_FLAGS_04      equ  5fh          ; Kurven-Parameter Flags
    #define S04_SERVO_FLAG_WAIT        R_SERVO_FLAGS_04,0  ; Wait after Start (Totzeit)
    #define S04_SERVO_FLAG_V_OFF       R_SERVO_FLAGS_04,1  ; switch V off after RUN
    #define S04_SERVO_FLAG_I_OFF       R_SERVO_FLAGS_04,2  ; switch Impulse off after RUN
    #define S04_SERVO_FLAG_NOINT       R_SERVO_FLAGS_04,6  ; reject interrupt of running servo.
    #define S04_SERVO_FLAG_ONLY1       R_SERVO_FLAGS_04,7  ; reject repetition of same adr.


R_OFFSET_SERVO_01     equ  60h          ; !! don't change !! Offset Value for impulse width Servo 1
R_FAKTOR_SERVO_01     equ  61h          ; !! don't change !! Multiplie factor for Servo 1
R_OFFSET_SERVO_02     equ  62h          ; !! don't change !! Offset Value for impulse width Servo 2
R_FAKTOR_SERVO_02     equ  63h          ; !! don't change !! Multiplie factor for Servo 2
R_OFFSET_SERVO_03     equ  64h          ; !! don't change !! Offset Value for impulse width Servo 3
R_FAKTOR_SERVO_03     equ  65h          ; !! don't change !! Multiplie factor for Servo 3
R_OFFSET_SERVO_04     equ  66h          ; !! don't change !! Offset Value for impulse width Servo 4
R_FAKTOR_SERVO_04     equ  67h          ; !! don't change !! Multiplie factor for Servo 4

R_ACTIVATOR           equ  68h          ; needed for restore of servo status (servo acivated by address #..)
R_LEDSIGNAL           equ  69h          ; LED signaling of command

R_FLAG_01             equ  6ah          ; Working register for Adjust left / Right Positions 
    #define F_V_was_off                R_FLAG_01,0         ; set to 1 if V for servo was switched off
    #define F_ONLY1                    R_FLAG_01,5         ; set to 1 if reject repetition of same adr.
    #define F_LEDON                    R_FLAG_01,6         ; set to 1 if LED is ON (save LED-Status during 4094)
    #define F_RESTORE                  R_FLAG_01,7         ; set to 1 during RESTORE
R_LOOPCOUNT           equ  6bh          ; Working register for Adjust left / Right Positions
R_SWITCH01            equ  6ch          ; Counter for debounce switch 01 (Adjust left / Right Positions)
R_SWITCH02            equ  6dh          ; Counter for debounce switch 02 (Adjust left / Right Positions)


R_FLAG_SWITCH         equ  70h          ; Working register switch result
    #define F_SW1_close                R_FLAG_SWITCH,0         ; Switch 1 closes
    #define F_SW1_open                 R_FLAG_SWITCH,1         ; Switch 1 opens
    #define F_SW2_close                R_FLAG_SWITCH,2         ; Switch 2 closes
    #define F_SW2_open                 R_FLAG_SWITCH,3         ; Switch 2 opens
    #define F_SW1_SW2_close            R_FLAG_SWITCH,4         ; Switch 1+2 closes

R_LED_SLOW            equ  73h          ; Slow Down LED Signalisation

R_FAKTOR              equ  75h          ; transfer register
R_OFFSET              equ  76h          ; transfer register

AARGB0                equ  7bh          ; used for 
AARGB1                equ  7dh          ; multiplie-routine
BARGB0                equ  7eh          ; UMUL0707L
LOOPCOUNT             equ  7fh          ; .. (see Microchip AN 00617)

;   --------------------------------------------------------------------------
;   --- Port-Definitionen
;   --------------------------------------------------------------------------
#define AND_S1      PORTC,2        ;Layout Definition AND for impulse to servo 1
#define AND_S2      PORTC,1        ;Layout Definition AND for impulse to servo 2
#define AND_S3      PORTC,3        ;Layout Definition AND for impulse to servo 3
#define AND_S4      PORTC,4        ;Layout Definition AND for impulse to servo 4

#define STROBE_4094 PORTA,4        ;Layout Definition
#define CLK_4094    PORTC,0        ;Layout Definition
#define DATA_4094   PORTA,5        ;Layout Definition (= LED1)
#define OE_4094     PORTA,1        ;Layout Definition Output Enable 4094
                                      
#define LED1        PORTA,5        ;Layout Definition (= DATA_4094)

#define SWITCH01    PORTA,3        ;Layout Definition Switch 01 (adjust left / right positions)
#define SWITCH02    PORTA,0        ;Layout Definition Switch 02 (adjust left / right positions)

;   --------------------------------------------------------------------------
;   --- Adressen im HEX
;   --------------------------------------------------------------------------
#define hexline_servoFlags       d'100'
#define hexline_curveNumber      d'101'
#define hexline_curveStretch     d'102'
;--
#define hexline_adressServo1     d'103'
#define hexline_adressMapping1   d'104'

#define hexline_adressServo2     d'103'
#define hexline_adressMapping2   d'104'

#define hexline_adressServo3     d'103'
#define hexline_adressMapping3   d'104'

#define hexline_adressServo4     d'103'
#define hexline_adressMapping4   d'104'
;--
#define hexline_adressServo5     d'106'
#define hexline_adressMapping5   d'107'

#define hexline_adressServo6     d'106'
#define hexline_adressMapping6   d'107'

#define hexline_adressServo7     d'106'
#define hexline_adressMapping7   d'107'

#define hexline_adressServo8     d'106'
#define hexline_adressMapping8   d'107'

#define hexline_switchFlags      d'110'

#define offset_min               d'80'
#define offset_max               d'200'
#define faktor_min               d'135' ; Einstellbereich Faktor: 128..(-)127 (Sprung bei 127/128)
;#define faktor_min               d'130' ;->  24 us ist der minimale Verstellbereich (faktor_min-127)*8us
;#define faktor_min               d'140' ;-> 104 us ist der minimale Verstellbereich
;#define faktor_min               d'150' ;-> 184 us ist der minimale Verstellbereich
;#define faktor_min               d'160' ;-> 264 us ist der minimale Verstellbereich
#define faktor_max               d'100'

;   --------------------------------------------------------------------------
;   --- Timing
;   --------------------------------------------------------------------------
#define delay_v_i                d'50'  ; Verzgerung V-EIN zu erstem Impuls x 20ms (Vcc Servo war aus)
#define delay_v_i_short          d'4'   ; Verzgerung V-EIN zu erstem Impuls x 20ms (Vcc Servo war an)
#define number_first_imp         d'5'   ; Anzahl der Impulse mit dem ersten Tabellen-Wert nach der Tot-Zeit

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

  ; ****************************************************************************
  ; **  Konfiguration CCP1CON: ENHANCED CCP1 CONTROL REGISTER  (ADDRESS: 15h) ** 
  ; ****************************************************************************
        movlw   B'00000000'
;                 ++-------- #7-6: P1M<1:0>: PWM Output Configuration bits
;                   ||||||       If CCP1M<3:2> = 00, 01, 10:
;                   ||||||       xx = P1A assigned as Capture/Compare input; P1B, P1C, P1D assigned as port pins
;                   ||||||       If CCP1M<3:2> = 11:
;                   ||||||       00 = Single output; P1A modulated; P1B, P1C, P1D assigned as port pins
;                   ||||||       01 = Full-bridge output forward; P1D modulated; P1A active; P1B, P1C inactive
;                   ||||||       10 = Half-bridge output; P1A, P1B modulated with dead-band control; P1C, P1D assigned as port pins
;                   ||||||       11 = Full-bridge output reverse; P1B modulated; P1C active; P1A, P1D inactive
;                   ++------ #5-4: DC1B<1:0>: PWM Duty Cycle Least Significant bits
;                     ||||       Capture mode:
;                     ||||       Unused.
;                     ||||       Compare mode:
;                     ||||       Unused.
;                     ||||       PWM mode:
;                     ||||       These bits are the two LSbs of the PWM duty cycle. The eight MSbs are found in CCPR1L.
;                     ++++--- #3-0:  CCP1M<3:0>: ECCP Mode Select bits
;                                0000 = Capture/Compare/PWM off (resets ECCP module)
;                                0001 = Unused (reserved)
;                                0010 = Compare mode, toggle output on match (CCP1IF bit is set)
;                                0011 = Unused (reserved)
;                                0100 = Capture mode, every falling edge
;                                0101 = Capture mode, every rising edge
;                                0110 = Capture mode, every 4th rising edge
;                                0111 = Capture mode, every 16th rising edge
;                                1000 = Compare mode, set output on match (CCP1IF bit is set)
;                               >1001 = Compare mode, clear output on match (CCP1IF bit is set)
;                                1010 = Compare mode, generate software interrupt on match (CCP1IF bit is set, CCP1 pin is
;                                unaffected)
;                                1011 = Compare mode, trigger special event (CCP1IF bit is set; CCP1 resets TMR1 or TMR2, and starts
;                                an A/D conversion, if the ADC module is enabled)
;                                1100 = PWM mode; P1A, P1C active-high; P1B, P1D active-high
;                                1101 = PWM mode; P1A, P1C active-high; P1B, P1D active-low
;                                1110 = PWM mode; P1A, P1C active-low; P1B, P1D active-high
;                                1111 = PWM mode; P1A, P1C active-low; P1B, P1D active-low





        movwf CCP1CON                    ; CCP1CON:-Reg. konfigurieren

;        bsf SERVO1_SELECTED              ; Statemachine fr die 4 Servo-Time-Slots starten

        movlw d'1'
        movwf R_LED_SLOW

;   -----------------------------------
;   --- read Offset&Faktor $60..$67 ---
;   -----------------------------------
        ; die EEprom-Werte $60..$67 werden in die RAM-Zellen $60..$67 bertragen 
        movlw h'60'                    ; memory start adress
        movwf FSR
next_read
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0                 ; -> BANK 1
           movf FSR,W
           movwf EEADR                    ; Adress to read
           bsf EECON1,RD                  ; initiate reading
           movf EEDATA,W                  ; EEDATA into W
          bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf INDF                     ; ..into the register (indirect)

        incf FSR,F
        movlw h'68'                    ; diese Adresse nicht mehr lesen
        subwf FSR,W                    ; W = FSR - W = FSR - h'68'
        btfss STATUS,C                 ; if C=0, -> FSR < h'68'
         goto next_read                ; next byte
        
;   -----------------------------------
;   --- read Servo-Flags (ROM)      ---
;   -----------------------------------
        ; an der ROM-Stelle "hexline_servoFlags" liegen die Infos 
        ; - switch V off after RUN 
        ; - switch Impulse off after RUN
        ; diese Infos sind Servo-spezifisch.
        ; die Flags mssen in das Register 
        ; -> R_SERVO_FLAGS_0x
        ; bertragen werden 
        movlw hexline_servoFlags       ; access this line
        movwf R_TABLEREAD              ; .. in the table
        call TABLE_READ1
        movwf R_SERVO_FLAGS_01

        movlw hexline_servoFlags       ; access this line
        movwf R_TABLEREAD              ; .. in the table
        call TABLE_READ2
        movwf R_SERVO_FLAGS_02

        movlw hexline_servoFlags       ; access this line
        movwf R_TABLEREAD              ; .. in the table
        call TABLE_READ3
        movwf R_SERVO_FLAGS_03

        movlw hexline_servoFlags       ; access this line
        movwf R_TABLEREAD              ; .. in the table
        call TABLE_READ4
        movwf R_SERVO_FLAGS_04

        movlw d'60'                    ; 
        movwf R_SWITCH01
        movwf R_SWITCH02
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000001'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00000010'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'00000100'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'00001000'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#5 ***
; ********************
gruen_5
        movlw b'00010000'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#6 ***
; ********************
gruen_6
        movlw b'00100000'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#7 ***
; ********************
gruen_7
        movlw b'01000000'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return

; ********************
; *** ADR Gruen-#8 ***
; ********************
gruen_8
        movlw b'10000000'              ; indicates activator for this servo
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss              **
;** EEPROM $70..73                        **
;*******************************************
status_to_output
        bsf F_RESTORE                 ; Restore Mode
;   ------------------------------------------------------
;   --- read last activator for servo1 from EEPROM $70 ---
;   ------------------------------------------------------
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0                 ; -> BANK 1
            movlw h'70'
            movwf EEADR                    ; Adress to read
            bsf EECON1,RD                  ; initiate reading
            movf EEDATA,W                  ; EEDATA into W
          bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register
 
;   ------------------------------------------------------
;   --- read last activator for servo2 from EEPROM $71 ---
;   ------------------------------------------------------
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0                 ; -> BANK 1
            movlw h'71'
            movwf EEADR                    ; Adress to read
            bsf EECON1,RD                  ; initiate reading
            movf EEDATA,W                  ; EEDATA into W
          bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register

;   ------------------------------------------------------
;   --- read last activator for servo3 from EEPROM $72 ---
;   ------------------------------------------------------
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0                 ; -> BANK 1
            movlw h'72'
            movwf EEADR                    ; Adress to read
            bsf EECON1,RD                  ; initiate reading
            movf EEDATA,W                  ; EEDATA into W
          bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register

;   ------------------------------------------------------
;   --- read last activator for servo4 from EEPROM $73 ---
;   ------------------------------------------------------
        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0                 ; -> BANK 1
            movlw h'73'
            movwf EEADR                    ; Adress to read
            bsf EECON1,RD                  ; initiate reading
            movf EEDATA,W                  ; EEDATA into W
          bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        movwf R_ACTIVATOR              ; ..into R_ACTIVATOR

        call restore_servo             ; load servo register

        movlw d'4'                     ; preload R_LOOPCOUNT with this value resuls...
        movwf R_LOOPCOUNT              ; ..in 4094-action (=> switching voltages ON)

        bcf F_RESTORE                  ; Restore Mode finito

        return


;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: restore_servo        **
;** loads values from EEPROM      **
;** depending on flags in         **
;** R_ADR_FLAGS_SERVO
;** R_ADR_FLAGS_MAP
;***********************************
;***********************************
;***********************************
;***********************************
        ; diese Routine wird zum Starten eines
        ; Servos aufgerufen (entweder durch die RESTORE-Funktion oder
        ; durch eine Adresse)
        ;
        ; zunchst laden der Register
        ; R_ADR_FLAGS_SERVO (Servo-Nr.)
        ; R_ADR_FLAGS_MAP   (Mapping curve / relais)
        ; in Abhngigkeit der [letzten] Adresse fr diesen Servo. Dies wird in R_ACTIVATOR
        ; Bit-spezifisch bergeben

restore_servo 
        movf R_ACTIVATOR,F
        btfsc STATUS,Z                    ; return, wenn R_ACTIVATOR = ZERO
         return

        btfss R_ACTIVATOR,0               ; latest Trigger for Servo4 by address G1 
         goto restore_02
           movlw hexline_adressServo1     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ1
           movwf R_ADR_FLAGS_SERVO
   
           movlw hexline_adressMapping1   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ1
           movwf R_ADR_FLAGS_MAP

restore_02
        btfss R_ACTIVATOR,1            ; latest Trigger for Servo4 by address G2 
         goto restore_03
           movlw hexline_adressServo2     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ2
           movwf R_ADR_FLAGS_SERVO
   
           movlw hexline_adressMapping2   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ2
           movwf R_ADR_FLAGS_MAP

restore_03
        btfss R_ACTIVATOR,2            ; latest Trigger for Servo4 by address G3 
         goto restore_04
           movlw hexline_adressServo3     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ3
           movwf R_ADR_FLAGS_SERVO
   
           movlw hexline_adressMapping3   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ3
           movwf R_ADR_FLAGS_MAP

restore_04
        btfss R_ACTIVATOR,3            ; latest Trigger for Servo4 by address G4 
         goto restore_05
           movlw hexline_adressServo4     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ4
           movwf R_ADR_FLAGS_SERVO
       
           movlw hexline_adressMapping4   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ4
           movwf R_ADR_FLAGS_MAP

restore_05
        btfss R_ACTIVATOR,4            ; latest Trigger for Servo4 by address G5 
         goto restore_06
           movlw hexline_adressServo5     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ1
           movwf R_ADR_FLAGS_SERVO
       
           movlw hexline_adressMapping5   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ1
           movwf R_ADR_FLAGS_MAP

restore_06
        btfss R_ACTIVATOR,5            ; latest Trigger for Servo4 by address G6 
         goto restore_07
           movlw hexline_adressServo6     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ2
           movwf R_ADR_FLAGS_SERVO
         
           movlw hexline_adressMapping6   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ2
           movwf R_ADR_FLAGS_MAP

restore_07
        btfss R_ACTIVATOR,6            ; latest Trigger for Servo4 by address G7 
         goto restore_08
           movlw hexline_adressServo7     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ3
           movwf R_ADR_FLAGS_SERVO
      
           movlw hexline_adressMapping7   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ3
           movwf R_ADR_FLAGS_MAP

restore_08
        btfss R_ACTIVATOR,7            ; latest Trigger for Servo4 by address G8 
         goto restore_09
           movlw hexline_adressServo8     ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ4
           movwf R_ADR_FLAGS_SERVO
   
           movlw hexline_adressMapping8   ; access this line
           movwf R_TABLEREAD              ; .. in the table
           call TABLE_READ4
           movwf R_ADR_FLAGS_MAP

restore_09

        ; diese Routine dient dazu, alle ntigen Parameter fr den Start eines
        ; Servos in die entsprechenden Register zu laden.
        ;
        ; es werden die geladenen Register
        ; R_ADR_FLAGS_SERVO (Servo-Nr.)
        ; R_ADR_FLAGS_MAP   (Mapping)
        ; verwendet. 
        ; Diese Register beinhalten die Info, welcher Servo mit welcher Curven-Form
        ; anzusteuern ist. Aus dieser Definition ergibt sich auch die Zuordnung der
        ; Kurven-Flags ('Loop Kurve'/'Switch V off after RUN'/'Switch Impulse off after RUN')
        ; sowie die Kurven-Punkte Anzahl (number of steps) und der Stretch-Faktor 
        ; zum Servo-Ausgang, die in die Servo-Spezifischen Register
        ; R_KURVE_NUMBER_0x und R_KURVE_STRETCH_0x kopiert werden.
        ; Desweiteren konditioniert diese Routine die ntigen Register
        ; (R_COUNT_STRETCH_0x / R_COUNT_0x) und Flags (SERVOx_ACTIVE / V1_ON) fr eine
        ; korrekte Abwicklung in der State-Machine
         
;   --------------------------------------------------------------------------
;   --- Prfen, ob der gewhlte Servo noch fhrt und nicht unterbrochen werden soll
;   --------------------------------------------------------------------------
        bsf F_ONLY1                    ; set Flag to ZERO
        btfsc USE_SERVOx_BREAK         ; wenn dies ein BREAK ist, dann
         goto s_b_005                  ; .. Aufhrung OK, weitermachen...
        btfsc USE_SERVOx_STOP          ; wenn dies ein STOP ist, dann
         goto s_b_005                  ; .. Aufhrung OK, weitermachen...

s_b_001
        btfss USE_SERVO1 
         goto s_b_002
          btfss S01_SERVO_FLAG_ONLY1   ; if reject repetition of same address is not allowed
           bcf F_ONLY1                 ; ... then clear this flag
          btfss S01_SERVO_FLAG_NOINT   ; if interrupt allowed
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfss SERVO1_ACTIVE          ; wenn Servo nicht Aktiv ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfsc R_COUNT_01,7           ; wenn Servo FREEZE ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          return                       ; sonst: Ausfhrung sbbrechen
s_b_002
        btfss USE_SERVO2 
         goto s_b_003
          btfss S02_SERVO_FLAG_ONLY1   ; if reject repetition of same address is not allowed
           bcf F_ONLY1                 ; ... then clear this flag
          btfss S02_SERVO_FLAG_NOINT   ; if interrupt allowed
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfss SERVO2_ACTIVE          ; wenn Servo nicht Aktiv ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfsc R_COUNT_02,7           ; wenn Servo FREEZE ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          return                       ; sonst: Ausfhrung sbbrechen
s_b_003
        btfss USE_SERVO3 
         goto s_b_004
          btfss S03_SERVO_FLAG_ONLY1   ; if reject repetition of same address is not allowed
           bcf F_ONLY1                 ; ... then clear this flag
          btfss S03_SERVO_FLAG_NOINT   ; if interrupt allowed
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfss SERVO3_ACTIVE          ; wenn Servo nicht Aktiv ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfsc R_COUNT_03,7           ; wenn Servo FREEZE ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          return                       ; sonst: Ausfhrung sbbrechen
s_b_004
        btfss USE_SERVO4 
         goto s_b_005
          btfss S04_SERVO_FLAG_ONLY1   ; if reject repetition of same address is not allowed
           bcf F_ONLY1                 ; ... then clear this flag
          btfss S04_SERVO_FLAG_NOINT   ; if interrupt allowed
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfss SERVO4_ACTIVE          ; wenn Servo nicht Aktiv ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          btfsc R_COUNT_04,7           ; wenn Servo FREEZE ist, dann
           goto s_b_005                ; .. Aufhrung OK, weitermachen...
          return                       ; sonst: Ausfhrung sbbrechen
s_b_005 
;   --------------------------------------------------------------------------
;   --- Prfen, ob der gewhlte Servo nicht als letztes von der selben Adresse gefahren wurde (skip last activator)
;   --------------------------------------------------------------------------
        btfss F_ONLY1                  ; if this Flag ist ZERO, same adress may start servo again
         goto s_b_000                  ; .. -> skip this check
        btfsc F_RESTORE                ; Restore Mode
         goto s_b_000                  ; .. dann diesen Check nicht machen

        btfsc USE_SERVO1 
         movlw h'70'
        btfsc USE_SERVO2 
         movlw h'71'
        btfsc USE_SERVO3 
         movlw h'72'
        btfsc USE_SERVO4 
         movlw h'73'

        bcf INTCON,GIE                 ; disable IRs
          bsf STATUS,RP0               ; -> BANK 1
            movwf EEADR                ; Adress to read
            bsf EECON1,RD              ; initiate reading
            movf EEDATA,W              ; EEDATA into W
          bcf STATUS,RP0               ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        subwf R_ACTIVATOR,W            ; W = R_ACTIVATOR - EEPROM-Wert
        btfsc STATUS,Z                 ; Aufsfhrung verhindern wenn R_ACTIVATOR = EEPROM-Wert ist
         return
s_b_000

;   --------------------------------------------------------------------------
;   --- ab hier: Servo wird beeinflusst....
;   --------------------------------------------------------------------------

;   --------------------------------------------------------------------------
;   --- Flag setzen, dass gewhlter Servo jetzt ACTIVE ist
;   --------------------------------------------------------------------------
;   --------------------------------------------------------------------------
;   --- Flag setzen, um Versorgungs-Spannung fuer Servo einzuschalten
;   --------------------------------------------------------------------------
        btfsc USE_SERVOx_BREAK         ; wenn dies ein BREAK ist, dann
         goto s_a_001                  ; .. nichts einschalten...
        btfsc USE_SERVOx_STOP          ; wenn dies ein STOP ist, dann
         goto s_a_001                  ; .. nichts einschalten...

        bcf F_V_was_off             ; Flag lschen
        btfss USE_SERVO1 
        goto $+5
         bsf SERVO1_ACTIVE
         btfss V1_ON
          bsf F_V_was_off
         bsf V1_ON

        btfss USE_SERVO2 
        goto $+5
         bsf SERVO2_ACTIVE
         btfss V2_ON
          bsf F_V_was_off
         bsf V2_ON

        btfss USE_SERVO3 
        goto $+5
         bsf SERVO3_ACTIVE
         btfss V3_ON
          bsf F_V_was_off
         bsf V3_ON

        btfss USE_SERVO4 
        goto $+5
         bsf SERVO4_ACTIVE
         btfss V4_ON
          bsf F_V_was_off
         bsf V4_ON

;   --------------------------------------------------------------------------
;   --- Servo Flags (loop/beak/stop) verarbeiten fr den angeforderten Servo-Ausgang
;   --------------------------------------------------------------------------
s_a_001 btfss USE_SERVO1
         goto s_a_002 
            btfsc USE_SERVOx_LOOP
             bsf SERVO1_LOOP
            btfsc USE_SERVOx_BREAK
             bsf R_COUNT_01,7     ; Flag, um Tabellen-Zeiger zu freezen
            btfsc USE_SERVOx_STOP
             bcf SERVO1_LOOP

s_a_002 btfss USE_SERVO2
         goto s_a_003 
            btfsc USE_SERVOx_LOOP
             bsf SERVO2_LOOP
            btfsc USE_SERVOx_BREAK
             bsf R_COUNT_02,7     ; Flag, um Tabellen-Zeiger zu freezen
            btfsc USE_SERVOx_STOP
             bcf SERVO2_LOOP

s_a_003 btfss USE_SERVO3
         goto s_a_004 
            btfsc USE_SERVOx_LOOP
             bsf SERVO3_LOOP
            btfsc USE_SERVOx_BREAK
             bsf R_COUNT_03,7     ; Flag, um Tabellen-Zeiger zu freezen
            btfsc USE_SERVOx_STOP
             bcf SERVO3_LOOP

s_a_004 btfss USE_SERVO4
         goto s_a_005 
            btfsc USE_SERVOx_LOOP
             bsf SERVO4_LOOP
            btfsc USE_SERVOx_BREAK
             bsf R_COUNT_04,7     ; Flag, um Tabellen-Zeiger zu freezen
            btfsc USE_SERVOx_STOP
             bcf SERVO4_LOOP

s_a_005
;   --------------------------------------------------------------------------
;   --- wenn "break" oder "stop", dann Return (neu ab V3.32)
;   --------------------------------------------------------------------------
        ; bei "break" oder "stop" darf 
        ; - Kurven-Punkte-Anzahl
        ; - Stretch-Faktor
        ; - Kurven-Nummer / Relais-Nummer ("Mapping")
        ; - Counter R_COUNT_0x (Kurven-Punkte)
        ; - Counter R_COUNT_STRETCH_0x (Strech-Situation)
        ; nicht verndert werden, also nur noch "Servostatus speichern"
        btfss F_RESTORE                ; Restore Mode
         goto s_a_005a
        ; if "Restore Mode"
        btfsc USE_SERVOx_BREAK
          return
        btfsc USE_SERVOx_STOP
          return
        goto s_a_005b

s_a_005a ; if not "Restore Mode" -> save servo status
        btfsc USE_SERVOx_BREAK
          goto s_a_007
        btfsc USE_SERVOx_STOP
          goto s_a_007
s_a_005b
;   --------------------------------------------------------------------------
;   --- Kurve Number (=Anzahl Punkte) laden fr die angeforderte Kurve
;   --------------------------------------------------------------------------
        ; an der ROM-Stelle "hexline_curveNumber" liegen die Infos 
        ; - number of steps 
        ; diese Infos sind Kurven-spezifisch und mssen dem Servo zufeordnet werden.
        ; die Flags mssen in das Register 
        ; -> R_KURVE_NUMBER_0x
        ; bertragen werden 

        movlw hexline_curveNumber      ; access this line
        movwf R_TABLEREAD              ; .. in the table
        btfsc USE_CURVE1 
         call TABLE_READ1
        btfsc USE_CURVE2 
         call TABLE_READ2
        btfsc USE_CURVE3 
         call TABLE_READ3
        btfsc USE_CURVE4 
         call TABLE_READ4
        ; Kurve NUMBER speichern fr den angeforderten Servo-Ausgang
        btfsc USE_SERVO1 
         movwf R_KURVE_NUMBER_01
        btfsc USE_SERVO2 
         movwf R_KURVE_NUMBER_02
        btfsc USE_SERVO3 
         movwf R_KURVE_NUMBER_03
        btfsc USE_SERVO4 
         movwf R_KURVE_NUMBER_04

;   --------------------------------------------------------------------------
;   --- Kurve Stretch Faktor laden fr die angeforderte Kurve
;   --------------------------------------------------------------------------
        ; an der ROM-Stelle "hexline_curveNumber" liegen die Infos 
        ; - Stretch-Faktor
        ; diese Infos sind Kurven-spezifisch und mssen dem Servo zufeordnet werden.
        ; die Flags mssen in das Register 
        ; -> R_KURVE_STRETCH_0x
        ; bertragen werden 

        movlw hexline_curveStretch     ; access this line
        movwf R_TABLEREAD              ; .. in the table
        btfsc USE_CURVE1 
         call TABLE_READ1
        btfsc USE_CURVE2 
         call TABLE_READ2
        btfsc USE_CURVE3 
         call TABLE_READ3
        btfsc USE_CURVE4 
         call TABLE_READ4
        ; Kurve Stretch Faktor speichern fr den angeforderten Servo-Ausgang
        btfsc USE_SERVO1 
         movwf R_KURVE_STRETCH_01
        btfsc USE_SERVO2 
         movwf R_KURVE_STRETCH_02
        btfsc USE_SERVO3 
         movwf R_KURVE_STRETCH_03
        btfsc USE_SERVO4 
         movwf R_KURVE_STRETCH_04

;   --------------------------------------------------------------------------
;   --- Mapping fr die Kurven-Nummer / Relais-Nummer
;   --------------------------------------------------------------------------
        ; an der ROM-Stelle "hexline_adressMapping1" liegen die Infos 
        ; - Adress x use Kurve y
        ; diese Infos sind Adress-spezifisch und mssen dem Servo zugeordnet werden.
        ; die Flags mssen in das Register 
        ; -> R_SERVO_MAP_0x
        ; bertragen werden 

        movf R_ADR_FLAGS_MAP,w     ; take this value
        ; Kurve Stretch Faktor speichern fr den angeforderten Servo-Ausgang
        btfsc USE_SERVO1 
         movwf R_SERVO_MAP_01
        btfsc USE_SERVO2 
         movwf R_SERVO_MAP_02
        btfsc USE_SERVO3 
         movwf R_SERVO_MAP_03
        btfsc USE_SERVO4 
         movwf R_SERVO_MAP_04


        ;   --------------------------------------------------------------------------
        ;   --- STRETCH Counter preload
        ;   --------------------------------------------------------------------------
        ; Der Stretch-Counter kann beim Servo-Start wird mit unterschiedlichen Werte geladen werden.
        ; Dies hngt davon ab, ob die Versorgungs-Spannung des Servos abgeschaltet war 
        ; (Laden mit  = delay_v_i) oder nicht (Laden mit  = delay_v_i_short)  
        ; Dadurch ergibt sich eine Tot-Zeit zwischen Einschalten der Versorgungs-Spannung
        ; der Servos und des ersten "echten" Impulses von ('delay_v_i' x 20 ms)
        movlw delay_v_i_short 
        btfsc F_V_was_off              ; lange Verzgerung @ Vcc war abgeschaltet
          movlw delay_v_i 
        btfsc F_RESTORE                ; lange Verzgerung @ Restore (auch wenn Vcc nicht abgeschaltet wird)
          movlw delay_v_i 

        btfsc USE_SERVO1 
         movwf R_COUNT_STRETCH_01      ; .. Counter laden f. Einschalt-Verzgerung
        btfsc USE_SERVO2 
         movwf R_COUNT_STRETCH_02      ; .. Counter laden f. Einschalt-Verzgerung
        btfsc USE_SERVO3 
         movwf R_COUNT_STRETCH_03      ; .. Counter laden f. Einschalt-Verzgerung
        btfsc USE_SERVO4 
         movwf R_COUNT_STRETCH_04      ; .. Counter laden f. Einschalt-Verzgerung

        ;   --------------------------------------------------------------------------
        ;   --- configure "WAIT"-Flag
        ;   --------------------------------------------------------------------------
        ; servo-inpulse has to wait untill first stretch is done
        btfsc USE_SERVO1 
          bsf S01_SERVO_FLAG_WAIT      ; setzte das Flag -> Totzeit
        btfsc USE_SERVO2 
          bsf S02_SERVO_FLAG_WAIT      ; setzte das Flag -> Totzeit
        btfsc USE_SERVO3 
          bsf S03_SERVO_FLAG_WAIT      ; setzte das Flag -> Totzeit
        btfsc USE_SERVO4 
          bsf S04_SERVO_FLAG_WAIT      ; setzte das Flag -> Totzeit

;   --------------------------------------------------------------------------
;   --- Check, if RESTORE or ADRESS-ACTION
;   --------------------------------------------------------------------------
        btfss F_RESTORE                ; Restore Mode
         goto s_a_006

        ;   --------------------------------------------------------------------------
        ;   --- @RESTORE: R_COUNT Counter preload (fr Restore: Zeiger auf letzten Tabellen-Wert)
        ;   --------------------------------------------------------------------------
        ; die Counter R_COUNT (Tabellen-Zhler) des vernderten Servos werden auf (R_KURVE_NUMBER_xx) gesetzt,
        ; sodass in der State-Machine den letzte Tabellen-Wert eingelesen wird.
        btfss USE_SERVO1 
        goto $+3
         decf R_KURVE_NUMBER_01,W      ; Anzahl der Kurven-Punkte in W-Register -> letzten Tabellen-Wert ausfhren
         movwf R_COUNT_01              ; .. Counter laden, damit erst einmal Tab-Werte geladen werden

        btfss USE_SERVO2 
        goto $+3
         decf R_KURVE_NUMBER_02,W      ; Anzahl der Kurven-Punkte in W-Register -> letzten Tabellen-Wert ausfhren
         movwf R_COUNT_02              ; .. Counter laden, damit erst einmal Tab-Werte geladen werden

        btfss USE_SERVO3 
        goto $+3
         decf R_KURVE_NUMBER_03,W      ; Anzahl der Kurven-Punkte in W-Register -> letzten Tabellen-Wert ausfhren
         movwf R_COUNT_03              ; .. Counter laden, damit erst einmal Tab-Werte geladen werden

        btfss USE_SERVO4 
        goto $+3
         decf R_KURVE_NUMBER_04,W      ; Anzahl der Kurven-Punkte in W-Register -> letzten Tabellen-Wert ausfhren
         movwf R_COUNT_04              ; .. Counter laden, damit erst einmal Tab-Werte geladen werden

       return

s_a_006
       ;   --------------------------------------------------------------------------
       ;   --- @ADRESS-ACTION: die Counter R_COUNT_0x -> hier laden mit '00'
       ;   --------------------------------------------------------------------------
        btfsc USE_SERVO1 
         clrf R_COUNT_01               ; .. Counter lschen -> erster Tabellen-Wert wird geladen 
        btfsc USE_SERVO2 
         clrf R_COUNT_02               ; .. Counter lschen -> erster Tabellen-Wert wird geladen 
        btfsc USE_SERVO3 
         clrf R_COUNT_03               ; .. Counter lschen -> erster Tabellen-Wert wird geladen 
        btfsc USE_SERVO4 
         clrf R_COUNT_04               ; .. Counter lschen -> erster Tabellen-Wert wird geladen 

s_a_007 ; Einsprung hier, wenn "BREAK" oder "STOP" & nicht "Restore Mode"
       ;   --------------------------------------------------------------------------
       ;   --- @ADRESS-ACTION: EEPROM-Adress for saving Servo status (for Restore @ Decoder-Start)
       ;   --------------------------------------------------------------------------
       btfsc USE_SERVO1               ; EEPROM Adress depends on servo#
         movlw h'70'
       btfsc USE_SERVO2 
         movlw h'71'
       btfsc USE_SERVO3 
         movlw h'72'
       btfsc USE_SERVO4 
         movlw h'73'

       movwf R_EEADR                  ; Address $70..$73 depending on choosen servo#
       movf R_ACTIVATOR,w             ; R_ACTIVATOR indicates the (latest) activator for this servo
       movwf R_LEDSIGNAL              ; used for LED signaling
       movwf R_EEDAT                  ; Data: R_ACTIVATOR
       call saveroutine
        

       clrf R_STATUS                   ; ermglicht, dass Adresse wieder Action auslsen kann

       return


; ******************************************
; **  update_4094                         **
; ******************************************
update_4094
       bcf F_LEDON                     ; LED-Status retten
       btfsc LED1
        bsf F_LEDON

       bcf DATA_4094
       btfsc R_4094_VALUES,0
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,1
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,2
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,3
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,4
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,5
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,6
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       bcf DATA_4094
       btfsc R_4094_VALUES,7
        bsf DATA_4094
       ;CLK Impuls
       bsf CLK_4094
       bcf CLK_4094

       ;Strobe Impuls
       bsf STROBE_4094
       bcf STROBE_4094

       ; Output Enable setzen
       bsf OE_4094

       bcf LED1                      ; lschen, da evtl. durch 4094 gesetzt wurde
       btfsc F_LEDON                 ; LED-Statur restore
        bsf LED1

       return

;***************************************************************
; 7x7 Bit Unsigned Fixed Point Divide 7x7 -> 14
; Input: 7 bit unsigned fixed point multiplicand in AARGB0
; 7 bit unsigned fixed point multiplier in BARGB0
; Use: CALL FXM0707U
; Output: 14 bit unsigned fixed point product in AARGB0..AARGB1
; Result: AARG <-- AARG x BARG
; Max Timing: 1+64+2 = 67 clks
; Min Timing: 1+47 = 48 clks
;***************************************************************
UMUL0707L    

;       Max Timing:     3+12+5*8+7+2 = 64 clks
;       Min Timing:     3+6*6+5+3 = 47 clks
;       PM: 21            DM: 4
                                
        CLRF    AARGB1 ; clear partial product

        MOVLW   0x07
        MOVWF   LOOPCOUNT
        MOVF    AARGB0,W

LOOPUM0707A
        RRF     BARGB0, F
        BTFSC   STATUS,C
        GOTO    LUM0707NAP
        DECFSZ  LOOPCOUNT, F
        GOTO    LOOPUM0707A

        CLRF    AARGB0
        RETLW   0x00

LUM0707NAP
        BCF     STATUS,C
        GOTO    LUM0707NA

LOOPUM0707
        RRF     BARGB0, F
        BTFSC   STATUS,C
        ADDWF   AARGB0, F
LUM0707NA 
        RRF     AARGB0, F
        RRF     AARGB1, F
        DECFSZ  LOOPCOUNT, F
        GOTO    LOOPUM0707

        RRF     AARGB0, F
        RRF     AARGB1, F

        RETLW   0x00

;***************************************************************
; generates Servo impulse
;***************************************************************
; input register: 
; - R_PWMVALUE
; - R_FAKTOR
; - R_OFFSET
;***************************************************************
generate_servo_impuls
        ; configure capture compare (CCP) Register
        ; Null-Stellung: Timer = 0, erste CCP-Flanke (switch-on edge)
        clrf CCPR1H                    ; compare Value
        clrf CCPR1L                    ; = "0000"
        movlw b'00001000'              ; CCP1 bei Match EIN
        movwf CCP1CON                  ; CCP1CON:-Reg. konfigurieren
        bcf INTCON,GIE                 ; disable IRs
          clrf TMR1L
          clrf TMR1H                     ; clear Timer1 -> CCP1 toggelt ON
        bsf INTCON,GIE                 ; enable IRs

        ; Multiplikation und Addition von (Produkt / 128)
        bcf R_PWMVALUE,7               ; Relais-Bit lschen
        movf R_PWMVALUE,W              ; PWM-Wert laden
        movwf AARGB0                   ; .. ist ein Faktor fr die Multiplikation (multiplicand)
        movf R_FAKTOR,W                ; R_FAKTOR in BARGB0 umladen 
        btfsc R_FAKTOR,7               ; wenn R_FAKTOR_SERVO > 127
         sublw d'255'                  ; .. dann Faktor = 255 - Faktor
        movwf BARGB0                   ; .. und wird dann der andere Faktor fr die Multiplikation
        bcf BARGB0,7                   ; delete #7, multi needs 7bit numbers
        call UMUL0707L                 ; Ergebnis: AARGB0[MSB] und AARGB1[LSB]
        RLF AARGB1,F                   ; #7 rausschieben und in AARGB0->W nachschieben.
        RLF AARGB0,W                   ; damit ist W = AARGB0..AARGB1 / 128
        btfss R_FAKTOR,7               ; Wenn #7 von R_FAKTOR nicht gesetzt ist, dann wird der Korrektur-Wert
         addwf R_PWMVALUE,F            ; .. addiert
        btfsc R_FAKTOR,7
         subwf R_PWMVALUE,F            ; .. ansonsten subtrahiert
        movf R_PWMVALUE,W              ; ...and into W-Reg.
        addwf R_OFFSET,W               ; add offset impulse width
        movwf CCPR1L                   ; sum will be the compare value (switch off edge)
        btfsc STATUS,C                 ; if carry did occur, the transfer to
         bsf CCPR1H,0                  ; CCPR1H-Register

        ; configure CCP fr switch-off edge
        movlw b'00001001'              ; CCP1 bei Match AUS 
        movwf CCP1CON                  ; CCP1CON:-Reg. konfigurieren

        return

;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: change_mode          **
;** Adjust left / Right Positions **
;** into EEPROM $60..$67          **
;***********************************
;***********************************
;***********************************
;***********************************
 org 0x700  ; ntig, da Call via PCL-Manipulation durchgefhrt wird
change_mode
        btfss R_MODE,0                 ; wenn R_MODE#0 nicht gesetzt
         return                        ; .. keine Justage-Funktion fr SW1 (Port A,3)

        bsf R_SWITCH01,0               ; open switch necessary
        bsf R_SWITCH02,0               ; open switch necessary

        clrf R_LOOPCOUNT

        movlw d'20'                    ; 20 x 20 ms = 400 ms
        movwf R_CNT_TGL                ; .. Wert in den Counter laden
        bcf LED1

change_mode_loop_big
;   -----------------------------------
;   --- loop for NEXT, includes new AND settings and new 4094 settings
;   -----------------------------------
        movlw b'00000111'              ; Zahlenbereich R_LOOPCOUNT = [0...7]
        andwf R_LOOPCOUNT,F            ; Zahlenbereich R_LOOPCOUNT = [0...7]

        movlw b'00000111'              ; load PCLATH with High Adress (Tabelle ab $0700)
        movwf PCLATH
        movf R_LOOPCOUNT,W
        addwf PCL,F                    ; Sprung abhngig von R_LOOPCOUNT

        goto change_mode_001_b        
        goto change_mode_002_b        
        goto change_mode_003_b        
        goto change_mode_004_b        
        goto change_mode_005_b        
        goto change_mode_006_b        
        goto change_mode_007_b        
        goto change_mode_008_b        

change_mode_001_b
        ; load working register
        movf R_FAKTOR_SERVO_01,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_01,W
        movwf R_OFFSET
        ; switch AND Gates        
        bsf AND_S1
        bcf AND_S2
        bcf AND_S3
        bcf AND_S4
        ; switch Servo voltage and Relais on
        clrf R_4094_VALUES        
        bsf R1_ON                      ; Relais on
        bsf V1_ON                      ; Voltage on
        call update_4094
        ; PWM-Wert
        clrf R_PWMVALUE                ; MIN-Wert, verwendet fr Offset - Justage   
        goto change_mode_loop_small
change_mode_002_b
        ; PWM-Wert
        movlw d'127'  ; MAX-Wert, verwendet fr Faktor - Justage       
        movwf R_PWMVALUE 
        goto change_mode_loop_small

change_mode_003_b
        ; save prev. working register
        movf R_FAKTOR,W
        movwf R_FAKTOR_SERVO_01
        movf R_OFFSET,W
        movwf R_OFFSET_SERVO_01
        ; load working register
        movf R_FAKTOR_SERVO_02,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_02,W
        movwf R_OFFSET
        ; switch AND Gates        
        bcf AND_S1
        bsf AND_S2
        ; switch Servo voltage and Relais on
        clrf R_4094_VALUES        
        bsf R2_ON                      ; Relais on
        bsf V2_ON                      ; Voltage on
        call update_4094
        ; PWM-Wert
        clrf R_PWMVALUE                ; MIN-Wert, verwendet fr Offset - Justage   
        goto change_mode_loop_small
change_mode_004_b
        ; PWM-Wert
        movlw d'127'  ; MAX-Wert, verwendet fr Faktor - Justage       
        movwf R_PWMVALUE 
        goto change_mode_loop_small

change_mode_005_b
        ; save prev. working register
        movf R_FAKTOR,W
        movwf R_FAKTOR_SERVO_02
        movf R_OFFSET,W
        movwf R_OFFSET_SERVO_02
        ; load working register
        movf R_FAKTOR_SERVO_03,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_03,W
        movwf R_OFFSET
        ; switch AND Gates        
        bcf AND_S2
        bsf AND_S3
        ; switch Servo voltage and Relais on
        clrf R_4094_VALUES        
        bsf R3_ON                      ; Relais on
        bsf V3_ON                      ; Voltage on
        call update_4094
        ; PWM-Wert
        clrf R_PWMVALUE                ; MIN-Wert, verwendet fr Offset - Justage   
        goto change_mode_loop_small
change_mode_006_b
        ; PWM-Wert
        movlw d'127'  ; MAX-Wert, verwendet fr Faktor - Justage       
        movwf R_PWMVALUE 
        goto change_mode_loop_small

change_mode_007_b
        ; save prev. working register
        movf R_FAKTOR,W
        movwf R_FAKTOR_SERVO_03
        movf R_OFFSET,W
        movwf R_OFFSET_SERVO_03
        ; load working register
        movf R_FAKTOR_SERVO_04,W
        movwf R_FAKTOR
        movf R_OFFSET_SERVO_04,W
        movwf R_OFFSET
        ; switch AND Gates        
        bcf AND_S3
        bsf AND_S4
        ; switch Servo voltage and Relais on
        clrf R_4094_VALUES        
        bsf R4_ON                      ; Relais on
        bsf V4_ON                      ; Voltage on
        call update_4094
        ; PWM-Wert
        clrf R_PWMVALUE                ; MIN-Wert, verwendet fr Offset - Justage   
        goto change_mode_loop_small
change_mode_008_b
        ; PWM-Wert
        movlw d'127'  ; MAX-Wert, verwendet fr Faktor - Justage       
        movwf R_PWMVALUE 
        goto change_mode_loop_small



change_mode_loop_small
;   -----------------------------------
;   --- loop without going to the NEXT
;   -----------------------------------
        ; Faktor verstellen?
        btfsc F_SW1_SW2_close
         goto change_values_002
change_offset
        btfsc R_LOOPCOUNT,0
         goto change_faktor
        btfsc F_SW2_close
         incf R_OFFSET,F
        btfsc F_SW1_close
         decf R_OFFSET,F
        goto change_values_done

        ; Offset verstellen?
change_faktor
        btfsc F_SW2_close
         incf R_FAKTOR,F
        btfsc F_SW1_close
         decf R_FAKTOR,F

change_values_done
;   -----------------------------------
;   --- nachladen der Switch-Register, dass repetition mglichist
;   -----------------------------------
        movlw d'80'                    ; gerade Zahl
        btfsc F_SW1_close
         movwf R_SWITCH01
        btfsc F_SW2_close
         movwf R_SWITCH02
;   -----------------------------------
;   --- check over- or underrun of OFFSET & FAKTOR
;   -----------------------------------
       movlw offset_max
       subwf R_OFFSET,W                ; wenn R_OFFSET >= offset_max
       movlw offset_max
       btfsc STATUS,C
        movwf R_OFFSET                 ; .. dann R_OFFSET = offset_max

       movlw offset_min
       subwf R_OFFSET,W                ; wenn R_OFFSET < offset_min
       movlw offset_min
       btfss STATUS,C
        movwf R_OFFSET                 ; .. dann R_OFFSET = offset_min

       ; R_FAKTOR hat den "Nullpunkt" @ 128. Der Min-Wert muss daher
       ; @ R_FAKTOR > 128, der Max-Wert @ R_FAKTOR < 127 betrachtet werden
       btfss R_FAKTOR,7
        goto change_values_001

       ; check faktor_min, wenn #7 von R_FAKTOR gesetzt ist
       movlw faktor_min
       subwf R_FAKTOR,W                ; wenn R_FAKTOR > faktor_min
       movlw faktor_min
       btfss STATUS,C
        movwf R_FAKTOR                 ; .. dann R_FAKTOR = faktor_min
       goto change_values_002

change_values_001
       ; check faktor_max, wenn #7 von R_FAKTOR nicht gesetzt ist
       movlw faktor_max
       subwf R_FAKTOR,W                ; wenn R_FAKTOR >= faktor_max
       movlw faktor_max
       btfsc STATUS,C
        movwf R_FAKTOR                 ; .. dann R_FAKTOR = faktor_max

change_values_002
;   -----------------------------------
;   --- calculate CCP values and generate impuls via ECCP
;   -----------------------------------
        ; PWM-Werte bleiben konstand bei 0 bzw. 127
        movlw d'127'  ; MAX-Wert, verwendet fr Faktor - Justage       
        movwf R_PWMVALUE 

        btfss R_LOOPCOUNT,0            ; Nur wenn Endpunkt-Nummer 0/2/4/6
         clrf R_PWMVALUE               ; MIN-Wert, verwendet fr Offset - Justage   

        call generate_servo_impuls 

        clrf R_FLAG_01               ; reset all flags vor INC / DEC ...

;   -----------------------------------
;   --- check input switches
;   -----------------------------------
        call check_switches
        btfss F_SW1_close
        goto $+3
          btfsc R_SWITCH02,6          ; if R_SWITCH02 > 64
             bsf F_SW1_SW2_close      ; both switches pressed
        btfss F_SW2_close
        goto $+3
          btfsc R_SWITCH01,6          ; if R_SWITCH01 > 64
             bsf F_SW1_SW2_close      ; both switches pressed
 
;   -----------------------------------
;   --- beide Tasten aktiv -> next value
;   -----------------------------------
        btfss F_SW1_SW2_close
          goto change_mode_sw99
        incf R_LOOPCOUNT,F
        btfsc R_LOOPCOUNT,3            ; ende, wenn R_LOOPCOUNT = 8
         goto change_mode_finish       ; .. finished

change_mode_sw99
;   -----------------------------------
;   --- LED Service
;   -----------------------------------
        bcf LED1
        decfsz R_CNT_TGL,F
         goto change_mode_sw100
        bsf LED1
        movlw d'60'                    ; 60 x 20 ms = 1200 ms
        btfsc R_LOOPCOUNT,0
         movlw d'10'                   ; 10 x 20 ms = 200 ms
        movwf R_CNT_TGL                ; .. Wert in den Counter laden

change_mode_sw100
;   -----------------------------------
;   --- create 20ms Raster
;   -----------------------------------
        clrwdt                         ; clear Watchdog-Timer
        movlw d'10'                    ; Wert (20 ms Cycle-Time)
        subwf TMR1H,W                  ; W = TMR1H - Wert
        btfss STATUS,C                 ; if C=0, -> TMR1H < Wert
         goto change_mode_sw100        ; ... then wait here
        clrf TMR1H                     ; TIMER1 H-Reg reset

        btfsc F_SW1_SW2_close          ; was it both switches?
         goto change_mode_loop_big     ; .. next step
        goto change_mode_loop_small    ; next cycle

change_mode_finish
;   -----------------------------------
;   ---  Ende der Justage, letzter Wert ist fertig.
;   -----------------------------------
        ; save last working register
        movf R_FAKTOR,W
        movwf R_FAKTOR_SERVO_04
        movf R_OFFSET,W
        movwf R_OFFSET_SERVO_04

;   -----------------------------------
;   --- write Offset&Faktor $60..$67 ---
;   -----------------------------------
        ; die RAM-Zellen $60..$67 werden in die EEprom-Zellen $60..$67 bertragen 
        movlw h'60'                    ; memory start adress
        movwf FSR
next_write
        movf FSR,W
        movwf R_EEADR                  ; Address: $60..$67
        movf INDF,W
        movwf R_EEDAT                  ; Data from indirect register
        call saveroutine

        incf FSR,F
        movlw h'68'                    ; diese Adresse nicht mehr lesen
        subwf FSR,W                    ; W = FSR - W = FSR - h'68'
        btfss STATUS,C                 ; if C=0, -> FSR < h'68'
         goto next_write               ; next byte

        return

;***********************************
;***********************************
;***********************************
;***********************************
;** Routine: check_switches       **
;** check for close/open of       **
;** switches                      **
;***********************************
;***********************************
;***********************************
;***********************************
check_switches
        ; Wird ein CLOSE erkannt, wird das Flag F_SWx_close gesetzt (einmalig)
        ; Wird ein OPEN erkannt,  wird das Flag F_SWx_open  gesetzt (einmalig)

        ; der Counter R_SWITCH0x luft hier bei gedrcktem Taster nach Oben, dabei werden nur
        ; die geraden Zahlen verwendet. Oben (d.h. bei 120) angekommen wird dann die 
        ; Taster-Aktion ausgelst und der Counter auf 121 gesetzt. 
        ; In diesem Fall muss der Counter erst wieder runter (auf < 21), dort wird der Wert 
        ; dann wieder auf die gerade Zahl '20'
        ; gesetzt. Dadurch wird erreicht, dass nach der Aktion fr Tastendruck die Taster erst
        ; losgelassen werden muss, ehe eine erneute Aktion durch Counter = 120 ausgelst werden kann.

        clrf R_FLAG_SWITCH            ; reset Flags

;   -----------------------------------
;   --- check input switch01
;   -----------------------------------
        movlw d'10'                   ; 
        btfss SWITCH01               
         addwf R_SWITCH01,F           ; pressed (LOW)
        btfsc SWITCH01
         subwf R_SWITCH01,F

        ; check if closed
        movlw d'120'                  ; if R_SWITCH01
        subwf R_SWITCH01,W            ; ..
        btfsc STATUS,Z                ; ..  = 120 then
         bsf F_SW1_close              ; .. set Flag CLOSE
        movlw d'121'
        btfsc STATUS,C                ; ..  >= 120 then
         movwf R_SWITCH01             ; .. freeze @ 121

        ; check if open
        movlw d'21'                   ; if R_SWITCH01
        subwf R_SWITCH01,W            ; ..
        btfsc STATUS,Z                ; ..  = 21 then
         bsf F_SW1_open               ; .. set Flag open
        movlw d'20'
        btfss STATUS,C                ; ..  < 21 then
          movwf R_SWITCH01            ; .. freeze @ 20 

;   -----------------------------------
;   --- check input switch02
;   -----------------------------------
        movlw d'10'                   ; 
        btfss SWITCH02                ; pressed (LOW)
         addwf R_SWITCH02,F
        btfsc SWITCH02
         subwf R_SWITCH02,F

        ; check if closed
        movlw d'120'                  ; if R_SWITCH02
        subwf R_SWITCH02,W            ; ..
        btfsc STATUS,Z                ; ..  = 120 then
         bsf F_SW2_close              ; .. set Flag CLOSE
        movlw d'121'
        btfsc STATUS,C                ; ..  >= 120 then
         movwf R_SWITCH02             ; .. freeze @ 121

        ; check if open
        movlw d'21'                   ; if R_SWITCH02
        subwf R_SWITCH02,W            ; ..
        btfsc STATUS,Z                ; ..  = 21 then
         bsf F_SW2_open               ; .. set Flag open
        movlw d'20'
        btfss STATUS,C                ; ..  < 21 then
          movwf R_SWITCH02            ; .. freeze @ 20 

        return