; ********************************************************
; ** Version: V3.48 vom 23/01/2011                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2011 Sven Brandt

; ********************************************************
; History:
; V3.48 - andreas: fr DCC/MM durch skip_g3 ff. Platzbedarf reduziert
; ********************************************************

  ; ****************************
  ; ** nur fuer Andreas-Kreuz **
  ; ****************************
R_STATE       equ  40h
; #1: ready for start (START IT!)
; #2: LED1 fade in/on    LED2 off
; #3: LED1 fade out/off  LED2 fade in/on
; #4: LED1 fade in/on    LED2 fade out/off
; #5: LED1 fade out/off  LED2 off
R_STATE2      equ  41h
; #0: This Flag is set when after #4 follows #5 (means STOP IT!)
R_PWMVALUE    equ  42h        ; PWM-Value from Table-Read
R_COUNT       equ  43h        ; Counter fuer Zeitverlaengerung (PWM)

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw 0xff                     ; preload with FF for
        movwf R_COUNT                  ; fast start
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bcf R_STATE2,0                 ; clear fade-out-bit
        movlw b'00011110'              ; check STATE#1-#4
        andwf R_STATE,W
        btfss STATUS,Z
         return
        bsf R_STATE,1                  ; ready for START
        btfsc R_STATE,5                ; if STATE#5 (no counter preload..)
         return
        movlw b'11111111'
        movwf R_COUNT                  ; preload counter
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bcf PORTA,5                     ; Switch light off
        bsf PORTA,1                     ; Switch light2 on
        return
 

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bsf R_STATE2,0                 ; ready for fade out
        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        bsf PORTA,5                     ; Switch light on
        bcf PORTA,1                     ; Switch light2 off
        return
 
;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         movf PORTA,W
         xorlw b'00010001'
         movwf PORTA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         movf PORTA,W
         xorlw b'00100010'
         movwf PORTA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        clrf R_STATE
        clrf R_STATE2
        btfsc R_STATUS,0
         bsf R_STATE,1                 ; ready for START
        btfsc R_STATUS,1
         bsf PORTA,1                    ; Switch light2 on
        btfss R_STATUS,1
         bsf PORTA,5                    ; Switch light on
        return
        