; ********************************************************
; ** Version: V3.58 vom 14/05/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2014 Sven Brandt

; ********************************************************
; History:
; V3.72 - 28.11.2019 WeichEi: Restore nach Learning durhc lschen von R_ONOFF1_OLD in STATUS_TO_OUTPUT 
; V3.60 - 14.03.2014 Weichei / Spur1 / 3-wege / weich88: Korrektur in Routine "Abschalten": "return" statt "goto loop"
; V3.58 - 14.05.2013 WeichEi / WeichZwei / Spur1 / Weich88: Abschalten durch OFF und TIMEOUT kann abgewhlt werden 
;                    (Wert "*a" in $00 --> kein Abschalten durch TimeOut),(Wert "a*" in $00 --> kein Abschalten durch OFF-Befehl)
; ********************************************************

 ; ****************************
  ; ** nur fuer weichei       **
  ; ****************************
R_VERZ        equ  40h          ; Verzgerungs-Counter 1
R_VERZ2       equ  41h          ; Verzgerungs-Counter 2
R_MAKESLOW    equ  42h          ; Verzgerungs-Counter 1

R_COUNT       equ  48h          ; Counter fuer PWM
R_PWMVALUE    equ  49h          ; PWM-Value, bernahme von R_COUNT
R_FADEIN1     equ  4ah          ; wenn Bit gesetzt, entsprechende LED einblenden
R_FADEOUT1    equ  4bh          ; wenn Bit gesetzt, entsprechende LED ausblenden
R_ONOFF1      equ  4ch          ; Status ON/OFF (Flags fr Ausgang 1..8 )
R_ONOFF1_OLD  equ  4dh          ; Status ON/OFF (Flags fr Ausgang 1..8 ) fr Restore

R1_ABSCHALT_1 equ  50h          ; Latch R_VERZ fuer Abschalten
R2_ABSCHALT_1 equ  51h          ; Latch TMR1H  fuer Abschalten

R_OFF_OK      equ  5fh          ; EEprom Wert, ob OFF-Befehl abgearbeitet werden darf (neu V3.51)

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out
#define EE_ADR_OFF   h'30'           ; EEprom Adresse: R_OFF_OK: darf OFF-Befehl abgearbeitet werden?

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define WEICHE_01  PORTA,0        ; WeichEi Layout Definition
#define WEICHE_02  PORTA,4        ; WeichEi Layout Definition
#define LED_01     PORTA,1        ; WeichEi Layout Definition
#define LED_02     PORTA,5        ; WeichEi Layout Definition

;   ----------------------------------------------------
;   --- Parameter
;   ----------------------------------------------------
CONST_FADE_01          equ d'60'        ; optimiert fr 500 ms Anschaltzeit @ MAKESLOW == 1

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0
   
;   ---------------------------------------
;   --- read R_OFF_OK from EEPROM $30 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_OFF
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_OFF_OK                 ; ..into R_OFF_OK

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bcf WEICHE_02                  ; Switch 2 off
        bsf WEICHE_01                  ; Switch 1 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_1             ; .. in R1_ABSCHALT_x
        movf R_VERZ2,W                  ; latch vom R_VERZ2 in R2_ABSCHALT_x
        movwf R2_ABSCHALT_1             ; abschalten @ [R_VERZ1 = R1_ABSCHALT_x] & (R_VERZ2 > R2_ABSCHALT_x) ;

        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00000001'
        movwf R_ONOFF1

        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'
        return
 

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bcf WEICHE_01                  ; Switch 1 off
        bsf WEICHE_02                  ; Switch 2 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_1             ; .. in R1_ABSCHALT_x
        movf R_VERZ2,W                  ; latch vom R_VERZ2 in R2_ABSCHALT_x
        movwf R2_ABSCHALT_1             ; abschalten @ [R_VERZ1 = R1_ABSCHALT_x] & (R_VERZ2 > R2_ABSCHALT_x) ;

        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        movlw b'00000010'
        movwf R_ONOFF1

        movlw d'01'                    ; PWM wieder starten
        movwf R_COUNT                  ; .. mit '1'
        return
 

; *******************
; *** Off-Command ***
; *******************
offcommand
        bcf WEICHE_01                  ; Switch 1 off
        bcf WEICHE_02                  ; Switch 2 off
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         comf R_TGLA,W
         andlw b'00010001'
         movwf PORTA
         movwf R_TGLA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         comf R_TGLA,W
         andlw b'00100010'
         movwf PORTA
         movwf R_TGLA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        clrf R_TGLA
        bsf R_TGLA,0     
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        clrf R_TGLA
        bsf R_TGLA,1     
        bsf R_TGLA,5     
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        clrf R_ONOFF1_OLD

        btfsc R_STATUS,1
         call gruen_2

        btfss R_STATUS,1
         call rot_2
 
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
#ifndef dummy
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         bsf LED_01                    ; .. -> Relais on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf LED_01                    ; .. -> Relais off
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         bsf LED_02                    ; .. -> Relais on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf LED_02                    ; .. -> Relais off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
#endif; dummy


;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: toggle_xxx         **
;*********************************
;*********************************
;*********************************
;*********************************

#ifdef test
toggle_led_5  ; test routine
     movf PORTC,W ;test
     xorlw b'00000010' ;test        ; toggle PORTC,1
     movwf PORTC ;test
     return

toggle_led_6  ; test routine
     movf PORTC,W ;test
     xorlw b'00000100' ;test        ; toggle PORTC,2
     movwf PORTC ;test
     return

toggle_led_7  ; test routine
     movf PORTC,W ;test
     xorlw b'00000001' ;test        ; toggle PORTC,0
     movwf PORTC ;test
     return

toggle_led_8  ; test routine
     movf PORTC,W ;test
     xorlw b'00100000' ;test        ; toggle PORTC,5
     movwf PORTC ;test
     return

toggle_led_9  ; test routine
     movf PORTC,W ;test
     xorlw b'00010000' ;test        ; toggle PORTC,4
     movwf PORTC ;test
     return

toggle_led_10  ; test routine
     movf PORTC,W ;test
     xorlw b'00001000' ;test        ; toggle PORTC,3
     movwf PORTC ;test
     return

#endif ; test
