; ********************************************************
; ** Version: V3.38 vom 23/01/2010                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2014 Sven Brandt

; ********************************************************
; History:
; V3.60 - 14.03.2014 Weichei / Spur1 / 3-wege / weich88: Korrektur in Routine "Abschalten": "return" statt "goto loop"

  ; ****************************
  ; ** nur fuer dreiwege      **
  ; ****************************
R_STATE       equ  40h  ; gleiche Adresse wie @ Andreas!
R_VERZ        equ  41h          ; Verzgerungs-Counter 1
R_MAKESLOW    equ  42h          ; Verzgerungs-Counter 1

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bsf PORTA,5                     ; Switch 2 off (Schaltung in Mitte)
        bcf PORTA,4                     ; Switch 2 off (Schaltung in Mitte)
        bcf PORTA,1                     ; Switch 1 off (Schaltung in Mitte)
        bcf PORTA,0                     ; Switch 1 off (Schaltung in Mitte)
        clrf R_STATE
        bsf R_STATE,1                  ; fr 2. Schaltung (Mitte->Links)
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bsf PORTA,5                     ; Switch 2 off (Schaltung in Mitte)
        bcf PORTA,4                     ; Switch 2 off (Schaltung in Mitte)
        bcf PORTA,1                     ; Switch 1 off (Schaltung in Mitte)
        bcf PORTA,0                     ; Switch 1 off (Schaltung in Mitte)
        clrf R_STATE
        bsf R_STATE,2                  ; fr 2. Schaltung (Mitte->Links)
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        bcf PORTA,5                     ; Switch 2 off (Schaltung in Mitte)
        bcf PORTA,4                     ; Switch 2 off (Schaltung in Mitte)
        bsf PORTA,1                     ; Switch 1 off (Schaltung in Mitte)
        bcf PORTA,0                     ; Switch 1 off (Schaltung in Mitte)
        clrf R_STATE
        bsf R_STATE,3                  ; fr 2. Schaltung (Mitte->Rechts)
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        return
 
; ******************
; *** ADR Rot-#3 ***
; ******************
rot_3
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         comf R_TGLA,W
         andlw b'00000011'
         movwf PORTA
         movwf R_TGLA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         incf R_STATE,F
         comf R_TGLA,W
         andlw b'00110011'
         movwf R_TGLA

         btfss R_STATE,1
          andlw b'00000011'
         btfsc R_STATE,1
          andlw b'00110000'
         movwf PORTA
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
         comf R_TGLA,W
         andlw b'00110000'
         movwf PORTA
         movwf R_TGLA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        clrf R_TGLA
        bsf R_TGLA,1     
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        clrf R_TGLA

        movlw h'01'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto $+3                      ; .. then check next
        bsf R_TGLA,1     
        bsf R_TGLA,4     

        movlw h'02'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto $+3                      ; .. then check next
        nop
        bsf R_TGLA,5     
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        return

;*******************************************
;** Routine: abschalten_mode3            **
;*******************************************
abschalten
;   -----------------------------------------
;   --- check for Time Out (0.5 s Raster) ---
;   -----------------------------------------
        btfss PIR1,TMR1IF              ; if Timer1 overflow
         return
        bcf PIR1,TMR1IF                ; clear Flag

        ; *** Verzgerung @ Weichendekoder (abschalten nach x s) ***     
        decfsz R_VERZ,F
         return
        movf R_MAKESLOW,W
        movwf R_VERZ

        clrf PORTA                     ; alles nach 0.5s aus
        btfsc R_STATE,1
         bsf PORTA,0                    ; Schaltung nach Links
        btfsc R_STATE,2
         bsf PORTA,1                    ; Schaltung in Mitte
        btfsc R_STATE,3
         bsf PORTA,4                    ; Schaltung nach Rechts
        clrf R_STATE
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ

        return
