; ********************************************************
; ** Version: V3.39 vom 12/03/2010                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2010 Sven Brandt

; ********************************************************
; History:
; V3.39 - Signalisierung MODE-Umschaltung nur noch mit PortA,1 wg. Problem mit EKGs (Sicherung springt)

  ; ****************************
  ; ** nur fuer schalten      **
  ; ****************************
R_VERZ        equ  40h          ; Verzgerungs-Counter 1
R_MAKESLOW    equ  41h          ; Verzgerungs-Counter 1
R1_ABSCHALT_1 equ  50h          ; fuer Mode3: Latch R_VERZ fuer Anschaltzeit 
R2_ABSCHALT_1 equ  51h          ; fuer Mode3: Latch TMR1H  fuer Anschaltzeit 
R1_ABSCHALT_2 equ  52h          ; fuer Mode3: Latch R_VERZ fuer Anschaltzeit 
R2_ABSCHALT_2 equ  53h          ; fuer Mode3: Latch TMR1H  fuer Anschaltzeit 
R1_ABSCHALT_3 equ  54h          ; fuer Mode3: Latch R_VERZ fuer Anschaltzeit 
R2_ABSCHALT_3 equ  55h          ; fuer Mode3: Latch TMR1H  fuer Anschaltzeit 
R1_ABSCHALT_4 equ  56h          ; fuer Mode3: Latch R_VERZ fuer Anschaltzeit 
R2_ABSCHALT_4 equ  57h          ; fuer Mode3: Latch TMR1H  fuer Anschaltzeit 

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,1        ; WeichEi Layout Definition
#define LED_02     PORTA,0        ; WeichEi Layout Definition
#define LED_03     PORTA,5        ; WeichEi Layout Definition
#define LED_04     PORTA,4        ; WeichEi Layout Definition

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        btfsc R_MODE,0                 ; restoring of output -> skip if R_MODE#0 = FALSE
         movwf R_STATUS                ; ..into R_STATUS (if R_MODE#0 = TRUE)

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bsf LED_01                      ; Ausgang 1
        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_1             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_1             ; abschalten @ [R_VERZ > (R1_ABSCHALT_x + R_MAKESLOW)] & (TMR1H > R2_ABSCHALT_x) 
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bsf LED_02                     ; Ausgang 2
        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_2             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_2             ; abschalten @ [R_VERZ > (R1_ABSCHALT_x + R_MAKESLOW)] & (TMR1H > R2_ABSCHALT_x) 
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        bsf LED_03                     ; Ausgang 3
        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_3             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_3             ; abschalten @ [R_VERZ > (R1_ABSCHALT_x + R_MAKESLOW)] & (TMR1H > R2_ABSCHALT_x) 
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        bsf LED_04                     ; Ausgang 4
        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_4             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_4             ; abschalten @ [R_VERZ > (R1_ABSCHALT_x + R_MAKESLOW)] & (TMR1H > R2_ABSCHALT_x) 
        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bcf LED_01                     ; Ausgang 1
        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        bcf LED_02                     ; Ausgang 2
        return
 
; ******************
; *** ADR Rot-#3 ***
; ******************
rot_3
        bcf LED_03                     ; Ausgang 3
        return

; ******************
; *** ADR Rot-#4 ***
; ******************
rot_4
        bcf LED_04                     ; Ausgang 4
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         movf PORTA,W
         xorlw b'00000010'             ; Ausgang 1
         movwf PORTA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         movf PORTA,W
         xorlw b'00000001'             ; Ausgang 2
         movwf PORTA
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
         movf PORTA,W
         xorlw b'00100000'             ; Ausgang 3
         movwf PORTA
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
         movf PORTA,W
         xorlw b'00010000'             ; Ausgang 4
         movwf PORTA
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         bsf LED_01         ; Ausgang 1
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
#ifndef dummy
        btfsc R_STATUS,0
         bsf LED_01         ; Ausgang 1
        btfsc R_STATUS,1
         bsf LED_02         ; Ausgang 2
        btfsc R_STATUS,2
         bsf LED_03         ; Ausgang 3
        btfsc R_STATUS,3
         bsf LED_04         ; Ausgang 4
        return
#endif; dummy

;*******************************************
;** Routine: abschalten_mode3            **
;*******************************************
abschalten_mode3
#ifndef dummy
        btfss R_MODE,2
         return

abschalt_1
        movf R1_ABSCHALT_1,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         goto abschalt_2               ; ...then jump

        movf R2_ABSCHALT_1,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> TMR1H >= R2_ABSCHALT_1
         goto abschalt_2               ; ...then jump

        bcf LED_01                     ; Ausgang 1

abschalt_2
        movf R1_ABSCHALT_2,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         goto abschalt_3               ; ...then jump

        movf R2_ABSCHALT_2,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> TMR1H >= R2_ABSCHALT_1
         goto abschalt_3               ; ...then jump

        bcf LED_02                     ; Ausgang 2

abschalt_3
        movf R1_ABSCHALT_3,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         goto abschalt_4               ; ...then jump

        movf R2_ABSCHALT_3,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> TMR1H >= R2_ABSCHALT_1
         goto abschalt_4               ; ...then jump

        bcf LED_03                     ; Ausgang 3

abschalt_4
        movf R1_ABSCHALT_4,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         goto abschalt_ende            ; ...then jump

        movf R2_ABSCHALT_4,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> TMR1H >= R2_ABSCHALT_1
         goto abschalt_ende               ; ...then jump

        bcf LED_04                     ; Ausgang 4

abschalt_ende
;   -----------------------------------------
;   --- check for Time Out (0.5 s Raster) ---
;   -----------------------------------------
        btfss PIR1,TMR1IF              ; if Timer1 overflow
         return
        bcf PIR1,TMR1IF                ; clear Flag

        incf R_VERZ,F                  ; R_VERZ = R_VERZ + 1

        return
#endif; dummy

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
#ifndef dummy
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL
        
;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        btfss R_MODE,1                 ; change from #1 -> #2
         goto $+4
         clrf R_MODE
         bsf R_MODE,2
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

    ;   btfsc R_CNT2_TGL,7             ; if CNT2 > 128
    ;    bsf LED_02                    ; .. -> Relais on
    ;   btfss R_CNT2_TGL,7             ; if CNT2 < 128
    ;    bcf LED_02                    ; .. -> Relais off

          ; V3.39: nur mit einem Port signalisieren, da sonst Sicherung kaputt wenn EKGs angeschlossen sind
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         bsf LED_01                    ; .. -> Relais on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf LED_01                    ; .. -> Relais off

    ;    btfsc R_CNT2_TGL,7            ; if CNT2 > 128
    ;     bsf LED_04                   ; .. -> Relais on
    ;    btfss R_CNT2_TGL,7            ; if CNT2 < 128
    ;     bcf LED_04                   ; .. -> Relais off
    ;
    ;    btfsc R_CNT2_TGL,7            ; if CNT2 > 128
    ;     bsf LED_03                   ; .. -> Relais on
    ;    btfss R_CNT2_TGL,7            ; if CNT2 < 128
    ;     bcf LED_03                   ; .. -> Relais off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002 

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
#endif; dummy
