; ********************************************************
; ** Version: V3.11 vom 22/10/2007                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2003/04/05/06/07 Sven Brandt

; ********************************************************
; History:
; V3.11 - wz_signal: Tristate @ Dunkeltastung fr ZweiPol (Stromaufnahme senken)
; ********************************************************

  ; ****************************
  ; ** nur fuer signal        **
  ; ****************************
; Adresse -{CHECK}-> R_STATUS -{STATEMACHINE}-> R_STATUS2 -{STATEMACHINE}-> R_STATEx -{LOOP}-> LEDs  
; In R_STATUS und R_STATUS2 sind die Ausgnge als Flags vergeben:
; #0: 1= Ausgang 1
; #1: 1= Ausgang 2
; #2: 1= Ausgang 3
; #3: 1= Ausgang 4
;
R_COUNT       equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE    equ  51h          ; PWM-Value
R_STATUS2     equ  52h
R_STATE1      equ  54h          ; Status-Flags 1 - LED1
R_STATE2      equ  55h          ; Status-Flags 2 - LED2
R_STATE3      equ  56h          ; Status-Flags 3 - LED3
R_STATE4      equ  57h          ; Status-Flags 4 - LED4

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,1        ; WeichEi Layout Definition
#define LED_02     PORTA,0        ; WeichEi Layout Definition
#define LED_03     PORTA,5        ; WeichEi Layout Definition
#define LED_04     PORTA,4        ; WeichEi Layout Definition

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -----------------------------------
;   --- read R_MODE from EEPROM $00 ---
;   -----------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MODE                   ; ..into R_MODE

;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal immediately
        movwf R_COUNT

        bsf R_STATE1,3                 ; OFF-State LED1
        bsf R_STATE2,3                 ; OFF-State LED2
        bsf R_STATE3,3                 ; OFF-State LED3
        bsf R_STATE4,3                 ; OFF-State LED4
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00000011'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,1                 ; STATUS #1 lschen
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00000011'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,0                 ; STATUS #0 lschen
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'00001100'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,3                 ; STATUS #3 lschen
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'00001100'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,2                 ; STATUS #2 lschen
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         movf PORTA,W
         xorlw b'00000010'             ; Ausgang 1
         movwf PORTA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         movf PORTA,W
         xorlw b'00000001'             ; Ausgang 2
         movwf PORTA
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
        ; --- kein TRIS-STATE ---
        movlw b'00001100'              ; #0, #1, #4, #5 OUTPUT
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0

         movf PORTA,W
         xorlw b'00100000'             ; Ausgang 3
         movwf PORTA
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
         movf PORTA,W
         xorlw b'00010000'             ; Ausgang 4
         movwf PORTA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         bsf LED_01         ; Ausgang 1
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        movf R_STATUS,W
        movwf R_STATUS2
        btfss STATUS,Z                 ; wenn R_STATUS = ZERO
         return
        movlw b'00000101'
        movwf R_STATUS
        movwf R_STATUS2
        bsf F_SAVE_STATUS              ; set Flag for save
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL
        
;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         bsf PORTA,1                    ; .. -> Relais on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf PORTA,1                    ; .. -> Relais off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002 

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw EE_ADR_MODE
        movwf R_EEADR                  ; Address: EE_ADR_MODE
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return
