; ********************************************************
; ** Version: V3.00 vom 24/07/2006                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006 Sven Brandt

  ; ****************************
  ; ** nur fuer signal4       **
  ; ****************************
; Adresse -{CHECK}-> R_STATUS -{STATEMACHINE}-> R_STATUS2 -{STATEMACHINE}-> R_STATEx -{LOOP}-> LEDs  
; In R_STATUS und R_STATUS2 sind die Ausgnge als Flags vergeben:
; #0: 1= Ausgang 1
; #1: 1= Ausgang 2
; #2: 1= Ausgang 3
; #3: 1= Ausgang 4
;
R_COUNT       equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE    equ  51h          ; PWM-Value
R_STATUS2     equ  52h
R_STATE1      equ  54h          ; Status-Flags 1 - LED1
R_STATE2      equ  55h          ; Status-Flags 2 - LED2
R_STATE3      equ  56h          ; Status-Flags 3 - LED3
R_STATE4      equ  57h          ; Status-Flags 4 - LED4

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,1        ; WeichEi Layout Definition
#define LED_02     PORTA,0        ; WeichEi Layout Definition
#define LED_03     PORTA,5        ; WeichEi Layout Definition
#define LED_04     PORTA,4        ; WeichEi Layout Definition

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3                 ; OFF-State LED1
        bsf R_STATE2,3                 ; OFF-State LED2
        bsf R_STATE3,3                 ; OFF-State LED3
        bsf R_STATE4,3                 ; OFF-State LED4
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        movlw b'00001111'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,1                 ; STATUS #1 lschen
        bcf R_STATUS,2                 ; STATUS #2 lschen
        bcf R_STATUS,3                 ; STATUS #3 lschen
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        movlw b'00001111'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,0                 ; STATUS #0 lschen
        bcf R_STATUS,2                 ; STATUS #2 lschen
        bcf R_STATUS,3                 ; STATUS #3 lschen
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        movlw b'00001111'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,0                 ; STATUS #0 lschen
        bcf R_STATUS,1                 ; STATUS #1 lschen
        bcf R_STATUS,3                 ; STATUS #3 lschen
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        movlw b'00001111'
        iorwf R_STATUS2,f              ; Lschen fr Dunkelphase
        bcf R_STATUS,0                 ; STATUS #0 lschen
        bcf R_STATUS,1                 ; STATUS #1 lschen
        bcf R_STATUS,2                 ; STATUS #2 lschen
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         movf PORTA,W
         xorlw b'00000010'             ; Ausgang 1
         movwf PORTA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         movf PORTA,W
         xorlw b'00000001'             ; Ausgang 2
         movwf PORTA
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
         movf PORTA,W
         xorlw b'00100000'             ; Ausgang 3
         movwf PORTA
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
         movf PORTA,W
         xorlw b'00010000'             ; Ausgang 4
         movwf PORTA
        return


; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         bsf LED_01         ; Ausgang 1
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -------------------------------------
;   --- Wartezeit                     ---
;   -------------------------------------
        clrwdt                         ; clear Watchdog-Timer
        decfsz R_CNT1_TGL,F
          goto $-2
        decfsz R_CNT2_TGL,F
          goto $-4

;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
        movf R_STATUS,W
        movwf R_STATUS2
        btfss STATUS,Z                 ; wenn R_STATUS = ZERO
         return
        movlw b'00010001'
        movwf R_STATUS
        movwf R_STATUS2
        bsf F_SAVE_STATUS              ; set Flag for save
        return
