; ********************************************************
; ** Version: V3.22 vom 02/02/2009                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09 Sven Brandt
; ********************************************************
; History:
; V3.22 - neue Register-Namen, Kommentar berarbeitet (no changes in HEX)
; V3.20 - erstellt aus state_led_signal2.asm V3.19

  ; ****************************
  ; ** nur fuer led_signal_hl **
  ; ****************************
; ================> Adresse 
; -{CHECK}--------> R_STATUS ->R_STATUS_ON_S1/2_pre 
; -{STATEMACHINE}-> R_STATUS_ON_S1/2_akt 
; -{STATEMACHINE}-> R_STATEx 
; -{LOOP}---------> LEDs  

; in R_STATUS sind die Signalstellungen abgelegt.
; in R_STATUS_ON_S1_xxx bzw. R_STATUS_ON_S2_xxx sind die LED-Stellungen abgelegt (war ntig wg. Dunkelphase).

; In R_STATUS_ON_S1/2_pre/akt und R_STATUS_ON_S1/2_pre/akt sind die Ausgnge als Flags vergeben:
; R_STATUS_ON_S1_pre/akt #0: 1= Ausgang 1 (Gelb Oben)
; R_STATUS_ON_S1_pre/akt #1: 1= Ausgang 2 (Grn oben)
; R_STATUS_ON_S1_pre/akt #2: 1= not used
; R_STATUS_ON_S1_pre/akt #3: 1= not used
; R_STATUS_ON_S1_pre/akt #4: 1= not used
; R_STATUS_ON_S1_pre/akt #5: 1= not used
; R_STATUS_ON_S1_pre/akt #6: 1= Ausgang 1 blinken (R_STATUS_ON_S1/2_pre/akt #0 muss auch gesetzt sein)
; R_STATUS_ON_S1_pre/akt #7: 1= Ausgang 2 blinken (R_STATUS_ON_S1/2_pre/akt #1 muss auch gesetzt sein)

; R_STATUS_ON_S2_pre/akt #0: 1= Ausgang 3 on
; R_STATUS_ON_S2_pre/akt #1: 1= Ausgang 4 on
; R_STATUS_ON_S2_pre/akt #2: 1= not used
; R_STATUS_ON_S2_pre/akt #3: 1= not used
; R_STATUS_ON_S2_pre/akt #4: 1= not used
; R_STATUS_ON_S2_pre/akt #5: 1= not used
; R_STATUS_ON_S2_pre/akt #6: 1= not used
; R_STATUS_ON_S2_pre/akt #7: 1= not used

; In dieser SW wird (wie bei der "LED_SIGNAL2") mit den Zwischen-Registern
; R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gearbeitet. In dieses zeigen die Flags an, ob
; ein Ausgang gesetzt / blinkend / aus sein soll. 
; Im Gegensatz zu LED_SIGNAL2 wird hier jedoch nicht mit ROTen und GRNen Adressen
; gearbeitet, sondern es werden 8 GRNe Adressen verwendet.
; Die Signal-Bilder werden in R_STATUS abgelegt (8 Adressen = 8 Bits). In R_STATUS
; liegt demnach die letzte bettigte Adresse und damit das anzuzeigene Signal-Bild:

; R_STATUS #0 = Vorsignal Vmax        = LED 1 (ge) off   / LED 2 (gn) on    
; R_STATUS #1 = Vorsignal Halt        = LED 1 (ge) on    / LED 2 (gn) off   
; R_STATUS #2 = Vorsignal 100         = LED 1 (ge) off   / LED 2 (gn) flash 
; R_STATUS #3 = Vorsignal 40/60       = LED 1 (ge) flash / LED 2 (gn) off   
; R_STATUS #4 = Hauptsignal Vmax      = LED 3/4 (ge) off
; R_STATUS #5 = Hauptsignal Halt      = LED 4 (rt) on
; R_STATUS #6 = Hauptsignal 40/60/100 = LED 3 (ge) on
; R_STATUS #7 = not used

; In den Routinen "gruen_1" usw. (oder "status_to_output" beim Prog-Start) werden dann
; die Flags in R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gesetzt. Hier findet also die Umsetzung#
; von Signal-Bild zu den LED-Ausgngen statt.
; Die STATE-Machine schaut sich dann nur die Flags in R_STATUS_ON_S1_pre / b an und entscheidet
; dann, ob die LED ein-/ausgeblendet, oder statisch ein/aus sein soll. 
;

R_STATUS_ON_S1_pre    equ  4eh
R_STATUS_ON_S2_pre    equ  4fh
R_COUNT               equ  50h          ; Counter fuer PWM - LED1
R_PWMVALUE            equ  51h          ; PWM-Value, bernahme von R_COUNT
R_STATUS_ON_S1_akt    equ  52h
R_STATUS_ON_S2_akt    equ  53h
R_STATE1              equ  54h          ; Status-Flags 1 - LED1
R_STATE2              equ  55h          ; Status-Flags 2 - LED2
R_STATE3              equ  56h          ; Status-Flags 3 - LED3
R_STATE4              equ  57h          ; Status-Flags 4 - LED4
;R_STATE5      equ  58h          ; Status-Flags 5 - LED5
;R_STATE6      equ  59h          ; Status-Flags 6 - LED6
;R_STATE7      equ  5ah          ; Status-Flags 7 - LED7
;R_STATE8      equ  5bh          ; Status-Flags 8 - LED8
;R_STATE9      equ  5ch          ; Status-Flags 9 - LED9
;R_STATE10     equ  5dh          ; Status-Flags 10 - LED10

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1

;   --------------------------------------------------------------------------
;   --- Port Definition
;   --------------------------------------------------------------------------
#define LED_01     PORTA,1        ; WeichEi Layout Definition
#define LED_02     PORTA,0        ; WeichEi Layout Definition
#define LED_03     PORTA,5        ; WeichEi Layout Definition
#define LED_04     PORTA,4        ; WeichEi Layout Definition

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3
        bsf R_STATE2,3
        bsf R_STATE3,3
        bsf R_STATE4,3
;        bsf R_STATE5,3
;        bsf R_STATE6,3
;        bsf R_STATE7,3
;        bsf R_STATE8,3
;        bsf R_STATE9,3
;        bsf R_STATE10,3
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bcf R_STATUS,1 ; alle anderen Vorsignale-States
        bcf R_STATUS,2 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,3 ; die Vorsignale aktiv ist
        movlw b'00000010' ; LED2 on 
        movwf R_STATUS_ON_S1_pre
        movwf R_STATUS_ON_S1_akt
        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bcf R_STATUS,0 ; alle anderen Vorsignale-States
        bcf R_STATUS,2 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,3 ; die Vorsignale aktiv ist
        movlw b'00000001' ; LED1 on 
        movwf R_STATUS_ON_S1_pre
        movwf R_STATUS_ON_S1_akt
        return
 
; ********************
; *** ADR Gruen-#3 ***
; ********************
gruen_3
        bcf R_STATUS,0 ; alle anderen Vorsignale-States
        bcf R_STATUS,1 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,3 ; die Vorsignale aktiv ist
        movlw b'10000010' ; LED2 on - flashing
        movwf R_STATUS_ON_S1_pre
        movwf R_STATUS_ON_S1_akt
        return

; ********************
; *** ADR Gruen-#4 ***
; ********************
gruen_4
        bcf R_STATUS,0 ; alle anderen Vorsignale-States
        bcf R_STATUS,1 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,2 ; die Vorsignale aktiv ist
        movlw b'01000001' ; LED1 on - flashing 
        movwf R_STATUS_ON_S1_pre
        movwf R_STATUS_ON_S1_akt
        return

; ********************
; *** ADR Gruen-#5 ***
; ********************
gruen_5
        bcf R_STATUS,5 ; alle anderen Hauptsignal-States
        bcf R_STATUS,6 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,7 ; die Hauptsignale aktiv ist
        movlw b'00000000' ; LED3/4 off 
        movwf R_STATUS_ON_S2_pre
        movwf R_STATUS_ON_S2_akt
        return

; ********************
; *** ADR Gruen-#6 ***
; ********************
gruen_6
        bcf R_STATUS,4 ; alle anderen Hauptsignal-States
        bcf R_STATUS,6 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,7 ; die Hauptsignale aktiv ist
        movlw b'00000010' ; LED4 on 
        movwf R_STATUS_ON_S2_pre
        movwf R_STATUS_ON_S2_akt
        return

; ********************
; *** ADR Gruen-#7 ***
; ********************
gruen_7
        bcf R_STATUS,4 ; alle anderen Hauptsignal-States
        bcf R_STATUS,5 ; .. lschen, damit nur 1 Bit fr
        bcf R_STATUS,7 ; die Hauptsignale aktiv ist
        movlw b'00000001' ; LED3 on 
        movwf R_STATUS_ON_S2_pre
        movwf R_STATUS_ON_S2_akt
        return

; ********************
; *** ADR Gruen-#8 ***
; ********************
gruen_8
        ;clrf R_STATUS
        ;bsf R_STATUS,7 ; nur 1 Bit in R_STATUS darf aktiv sein
        ;movlw b'00000101' ; LED1/3 on 
        ;movwf R_STATUS_ON_S1_pre
        ;movwf R_STATUS_ON_S1_akt
        ;movlw b'00000001' ; LED1 flashing
        ;movwf R_STATUS_ON_S2_pre
        return

;; ******************
;; *** ADR Rot-#1 ***
;; ******************
;rot_1
;        bcf R_STATUS_ON_S1_akt,0
;        bsf R_STATUS_ON_S1_pre,1
;        bcf R_STATUS_ON_S1_pre,0
;        return
;
;; ******************
;; *** ADR Rot-#2 ***
;; ******************
;rot_2
;        bcf R_STATUS_ON_S1_akt,2
;        bsf R_STATUS_ON_S1_pre,3
;        bcf R_STATUS_ON_S1_pre,2
;        return
; 
;; ******************
;; *** ADR Rot-#3 ***
;; ******************
;rot_3
;        bcf R_STATUS_ON_S1_akt,4
;        bsf R_STATUS_ON_S1_pre,5
;        bcf R_STATUS_ON_S1_pre,4
;        return
;
;; ******************
;; *** ADR Rot-#4 ***
;; ******************
;rot_4
;        bcf R_STATUS_ON_S1_akt,6
;        bsf R_STATUS_ON_S1_pre,7
;        bcf R_STATUS_ON_S1_pre,6
;        return
;
;; ******************
;; *** ADR Rot-#5 ***
;; ******************
;rot_5
;        bcf R_STATUS_ON_S2_akt,0
;        bsf R_STATUS_ON_S2_pre,1
;        bcf R_STATUS_ON_S2_pre,0
;        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
        movlw b'00000001'             ; Ausgang 2
        xorwf PORTA,F
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
        movlw b'00000010'             ; Ausgang 1
        xorwf PORTA,F
        return

; ********************
; *** Adresse #3   ***
; ********************
save_03
        movlw b'00000001'             ; Ausgang 2
        xorwf PORTA,F
        return

; ********************
; *** Adresse #4   ***
; ********************
save_04
        movlw b'00000010'             ; Ausgang 1
        xorwf PORTA,F
        return

; ********************
; *** Adresse #5   ***
; ********************
save_05
        movlw b'00110000'             ; Ausgang 3/4
        xorwf PORTA,F
        return

; ********************
; *** Adresse #6   ***
; ********************
save_06
        movlw b'00010000'             ; Ausgang 4
        xorwf PORTA,F
        return

; ********************
; *** Adresse #7   ***
; ********************
save_07
        movlw b'00100000'             ; Ausgang 3
        xorwf PORTA,F
        return

; ********************
; *** Adresse #8   ***
; ********************
save_08
        movlw b'00100010'             ; Ausgang 1 / 3
        xorwf PORTA,F
        return

; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         bsf LED_02         ; Ausgang 2
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
;   -------------------------------------
;   --- Wartezeit                     ---
;   -------------------------------------
        clrwdt                         ; clear Watchdog-Timer
        decfsz R_CNT1_TGL,F
          goto $-2
        decfsz R_CNT2_TGL,F
          goto $-4

;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Beispiel: 
    ; R_STATUS = 0000 0001 -> R_STATUS_ON_S1_pre = xxxx xx01
    ; R_STATUS = 0000 0010 -> R_STATUS_ON_S1_pre = xxxx xx10 usw.

        btfsc R_STATUS,0
         call gruen_1
        btfsc R_STATUS,1
         call gruen_2
        btfsc R_STATUS,2
         call gruen_3
        btfsc R_STATUS,3
         call gruen_4
        btfsc R_STATUS,4
         call gruen_5
        btfsc R_STATUS,5
         call gruen_6
        btfsc R_STATUS,6
         call gruen_7
        btfsc R_STATUS,7
         call gruen_8

;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
        movf R_STATUS,W
        btfss STATUS,Z                 ; wenn R_STATUS = ZERO
         return

        movlw b'00000001'
        movwf R_STATUS
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_1
        return
