; ********************************************************
; ** Version: V3.58 vom 14/05/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2014 Sven Brandt

; ********************************************************
; History:
; V3.60 - 14.03.2014 Weichei / Spur1 / 3-wege / weich88: Korrektur in Routine "Abschalten": "return" statt "goto loop"
; V3.58 - 14.05.2013 WeichEi / WeichZwei / Spur1 / Weich88: Abschalten durch OFF und TIMEOUT kann abgewhlt werden 
;                    (Wert "*a" in $00 --> kein Abschalten durch TimeOut),(Wert "a*" in $00 --> kein Abschalten durch OFF-Befehl)
; ********************************************************

  ; ****************************
  ; ** nur fuer spur1         **
  ; ****************************
R_VERZ        equ  40h          ; Verzgerungs-Counter 1
R_MAKESLOW    equ  41h          ; Verzgerungs-Counter 1

R_OFF_OK      equ  5fh          ; EEprom Wert, ob OFF-Befehl abgearbeitet werden dard (neu V3.51)

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_OFF_OK: darf OFF-Befehl abgearbeitet werden?
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   ---------------------------------------
;   --- read R_OFF_OK from EEPROM $00 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_OFF_OK                 ; ..into R_OFF_OK

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bcf PORTA,5                     ; Switch 2 off
        bcf PORTA,4                     ; Switch 2 off
        bsf PORTA,1                     ; Switch 1 on
        bsf PORTA,0                     ; Switch 1 on
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return


; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bcf PORTA,1                     ; Switch 1 off
        bcf PORTA,0                     ; Switch 1 off
        bsf PORTA,5                     ; Switch 2 on
        bsf PORTA,4                     ; Switch 2 on
        clrf TMR1L
        clrf TMR1H                     ; clear Timer1 for Time-Out
        bcf PIR1,TMR1IF                ; clear Flag
        movf R_MAKESLOW,W              ; Verzgerungswert aus EEPROM
        movwf R_VERZ
        return


; *******************
; *** Off-Command ***
; *******************
offcommand
        bcf PORTA,1                     ; Switch 1 off
        bcf PORTA,0                     ; Switch 1 off
        bcf PORTA,5                     ; Switch 2 off
        bcf PORTA,4                     ; Switch 2 off
        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         comf R_TGLA,W
         andlw b'00110011'
         movwf PORTA
         movwf R_TGLA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        clrf R_TGLA
        bsf R_TGLA,1     
        bsf R_TGLA,0     
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        return

;*******************************************
;** Routine: abschalten_mode3            **
;*******************************************
abschalten
        ; ist ein Abarbeiten des OFF-Befehl & TimeOut zugelassen (neu v3.58)??? Wenn NEIN, dann ABBRUCH
        movf R_OFF_OK,W
        andlw b'00001111'              ; nur Bit 3-0 betrachten
        sublw h'0a'                    ; nur wenn dieses Bitmuster in EE-Zelle steht (nur Bit 3-0 betrachten)
        btfsc STATUS,Z                 ; .. wird abgebrochen
         goto abschalt_ende            ; .. (d.h. abschalt_ende)

;   -----------------------------------------
;   --- check for Time Out (0.5 s Raster) ---
;   -----------------------------------------
        btfss PIR1,TMR1IF              ; if Timer1 overflow
         return
        bcf PIR1,TMR1IF                ; clear Flag

        ; *** Verzgerung @ Weichendekoder (abschalten nach x s) ***     
        decfsz R_VERZ,F
         return
        movf R_MAKESLOW,W
        movwf R_VERZ

        clrf PORTA                     ; alles nach x s aus

abschalt_ende
        return

