; ********************************************************
; ** Version: V3.58 vom 14/05/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2006-2013 Sven Brandt

; ********************************************************
; History:
; V3.58 - 14.05.2013 WeichEi / WeichZwei / Spur1 / Weich88: Abschalten durch OFF und TIMEOUT kann abgewhlt werden 
;                    (Wert "*a" in $00 --> kein Abschalten durch TimeOut),(Wert "a*" in $00 --> kein Abschalten durch OFF-Befehl)
; V3.11 - WeichZwei: Tristate fr ZweiPol (Stromaufnahme senken)
; ********************************************************

  ; ****************************
  ; ** nur fuer weichzwei     **
  ; ****************************
R_VERZ        equ  40h          ; Verzgerungs-Counter 1
R_MAKESLOW    equ  41h          ; Verzgerungs-Counter 1

R1_ABSCHALT_1 equ  50h          ; Latch R_VERZ fuer Abschaltzeit 
R2_ABSCHALT_1 equ  51h          ; Latch TMR1H  fuer Abschaltzeit 
R1_ABSCHALT_2 equ  52h          ; Latch R_VERZ fuer Abschaltzeit 
R2_ABSCHALT_2 equ  53h          ; Latch TMR1H  fuer Abschaltzeit 

R_OFF_OK      equ  5fh          ; EEprom Wert, ob OFF-Befehl abgearbeitet werden darf (neu V3.51)

;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_OFF_OK: darf OFF-Befehl abgearbeitet werden?
#define EE_ADR_MS    h'20'           ; EEprom Adresse: R_MAKESLOW: Wert Time-Out

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   ---------------------------------------
;   --- read R_OFF_OK from EEPROM $00 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MODE
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_OFF_OK                 ; ..into R_OFF_OK

;   ---------------------------------------
;   --- read R_MAKESLOW from EEPROM $20 ---
;   ---------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_MS
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_MAKESLOW               ; ..into R_MAKESLOW

        movlw d'01'                    ; Default-Wert, wenn R_MAKESLOW zu gross oder NULL ist
        btfsc R_MAKESLOW,7             ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn >127 (nur fr Weichen-Varianten)

        movf R_MAKESLOW,F              ; wenn R_MAKESLOW = 0, dann
        btfsc STATUS,Z                 ; .. d'01'
         movwf R_MAKESLOW              ; .. into R_MAKESLOW, wenn == 0

        movf R_MAKESLOW,W              ; Neu 3.23: Preload R_VERZ, damit TRI-State
        movwf R_VERZ                   ; .. gleich beim Start ausgefhrt wird.

        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
gruen_1
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,0                   ; #0, #1 OUTPUT
         bcf TRISA,1                   ; #0, #1 OUTPUT
        bcf STATUS,RP0                 ; -> BANK 0

        bcf PORTA,0                     ; Switch A2 off
        bsf PORTA,1                     ; Switch A1 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_1             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_1             ; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (TMR1H > R2_ABSCHALT_x) ;

        return

; ********************
; *** ADR Gruen-#2 ***
; ********************
gruen_2
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,4                   ; #4, #5 OUTPUT
         bcf TRISA,5                   ; #4, #5 OUTPUT
        bcf STATUS,RP0                 ; -> BANK 0

        bcf PORTA,4                     ; Switch B2 off
        bsf PORTA,5                     ; Switch B1 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_2             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_2             ; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (TMR1H > R2_ABSCHALT_x) ;

        return
 

; ******************
; *** ADR Rot-#1 ***
; ******************
rot_1
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,0                   ; #0, #1 OUTPUT
         bcf TRISA,1                   ; #0, #1 OUTPUT
        bcf STATUS,RP0                 ; -> BANK 0

        bcf PORTA,1                     ; Switch A1 off
        bsf PORTA,0                     ; Switch A2 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_1             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_1             ; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (TMR1H > R2_ABSCHALT_x) ;

        return

; ******************
; *** ADR Rot-#2 ***
; ******************
rot_2
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,4                   ; #4, #5 OUTPUT
         bcf TRISA,5                   ; #4, #5 OUTPUT
        bcf STATUS,RP0                 ; -> BANK 0

        bcf PORTA,5                     ; Switch B1 off
        bsf PORTA,4                     ; Switch B2 on

        movf R_VERZ,W
        addwf R_MAKESLOW,W              ; latch vom (R_VERZ + R_MAKESLOW)
        movwf R1_ABSCHALT_2             ; .. in R1_ABSCHALT_x
        movf TMR1H,W                    ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_2             ; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (TMR1H > R2_ABSCHALT_x) ;

        return
 

; *******************
; *** Off-Command ***
; *******************
offcommand
        clrf PORTA                     ; alle Ouputs gleichzeitig aus
;        bcf PORTA,1                     ; Switch 1 off
;        bcf PORTA,0                     ; Switch 1 off
;        bcf PORTA,5                     ; Switch 2 off
;        bcf PORTA,4                     ; Switch 2 off

        movlw b'00111111'              ; #0, #1, #4, #5 TRIS-STATE
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0

        return

;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
save_01
         comf R_TGLA,W
         andlw b'00000011'
         movwf PORTA
         movwf R_TGLA
        return

; ********************
; *** Adresse #2   ***
; ********************
save_02
         comf R_TGLA,W
         andlw b'00110000'
         movwf PORTA
         movwf R_TGLA
        return


; ************************
; *** save_toggle_start **
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier ein Ausgang fr Adresse #1 eingeschaltet werden.

save_toggle_start
        movlw b'00111100'              ; #0, #1 OUTPUT
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0

        clrf R_TGLA
        bsf R_TGLA,1     
        return

; ************************
; *** save_toggle_next ***
; ************************
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden.

save_toggle_next
        movlw b'00001111'              ; #4, #5 OUTPUT
        bsf STATUS,RP0                 ; -> BANK 1
        movwf TRISA                    ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0

        clrf R_TGLA
        bsf R_TGLA,5     
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output
        return

;*******************************************
;** Routine: abschalten_mode3            **
;*******************************************
abschalten
;
;                     |   .   .   .   .   .   .   .   .
;    TMR1H:           |  /|  /|  /|  /|  /|  /|  /|  /|
;    ------           | / | 0 | / | 2 | / | / | / | / |
;  (inc. 2ms)         |/  |/  |/  |/  |/  |/  |/  |/  |
;                     |----------------------------------------
;                    
;    R_VERZ:          |                    ----
;    -------          |                ----   |
;  increment          |            -1--       |
;  every              |        ----           |
;  TMR1IF (0.5s)      |    -0--               |        ----
; ('abschalt_ende')   |----                   |    ----
;                     |----------------------------------------
;
; 0: Time where command occurs ('gruen_1' etc.)
; 1: Value is calculated and stored @ '0' in R1_ABSCHALT_x (R_VERZ + R_MAKESLOW)
; 2: Value is stored @ '0' in R2_ABSCHALT_x
;
; abschalten @ [R_VERZ = R1_ABSCHALT_x] & (TMR1H >= R2_ABSCHALT_x) ;
; TMR1H is changing every 8x255 us so there should be no problem with overflow / carry
; (d.h. diese Routine wird in 2 ms bestimmt durchlaufen, sodass TMR1H = R2_ABSCHALT
; .. zu einer Abschaltung fhrt. Dies gilt auch wenn TMR1H = R2_ABSCHALT = h'FF'
; .. und somit 2 ms spter TMR1H = ZERO ist.) 
 
        ; ist ein Abarbeiten des OFF-Befehl & TimeOut zugelassen (neu v3.58)??? Wenn NEIN, dann ABBRUCH
        movf R_OFF_OK,W
        andlw b'00001111'              ; nur Bit 3-0 betrachten
        sublw h'0a'                    ; nur wenn dieses Bitmuster in EE-Zelle steht (nur Bit 3-0 betrachten)
        btfsc STATUS,Z                 ; .. wird abgebrochen
         goto abschalt_ende            ; .. (d.h. abschalt_ende)

abschalt_1
        movf R1_ABSCHALT_1,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_1
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_1
         goto abschalt_2               ; ...then jump

        movf R2_ABSCHALT_1,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_1
        btfss STATUS,C                 ; if C=0, -> TMR1H < R2_ABSCHALT_1
         goto abschalt_2               ; ...then jump

        bcf PORTA,0                    ; Weiche 1
        bcf PORTA,1                    ; Weiche 1

        bsf STATUS,RP0                 ; -> BANK 1
          bsf TRISA,0                  ; #0 = input = Tris-State
          bsf TRISA,1                  ; #1 = input = Tris-State
        bcf STATUS,RP0                 ; -> BANK 0

abschalt_2
        movf R1_ABSCHALT_2,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R1_ABSCHALT_2
        btfss STATUS,Z                 ; if Z=0, -> R_VERZ <> R1_ABSCHALT_2
         goto abschalt_3               ; ...then jump

        movf R2_ABSCHALT_2,W
        subwf TMR1H,W                  ; W = TMR1H - W = TMR1H - R2_ABSCHALT_2
        btfss STATUS,C                 ; if C=0, -> TMR1H < R2_ABSCHALT_2
         goto abschalt_3               ; ...then jump

        bcf PORTA,4                    ; Weiche 2
        bcf PORTA,5                    ; Weiche 2

        bsf STATUS,RP0                 ; -> BANK 1
          bsf TRISA,4                  ; #4 = input = Tris-State
          bsf TRISA,5                  ; #5 = input = Tris-State
        bcf STATUS,RP0                 ; -> BANK 0
abschalt_3
abschalt_4

abschalt_ende
;   -----------------------------------------
;   --- check for Time Out (0.5 s Raster) ---
;   -----------------------------------------
        btfss PIR1,TMR1IF              ; if Timer1 overflow
         return
        bcf PIR1,TMR1IF                ; clear Flag

        incf R_VERZ,F                  ; R_VERZ = R_VERZ + 1

        return
