; ********************************************************
; ** Version: V3.59 vom 26/08/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09/10/11/13 Sven Brandt
; ********************************************************
; History:
; V3.59 - Syncron-Blinken hinzu
; V3.50 - DUNKELTAST_ALL hinzu
; V3.26 - nderung in der State-Machine "state_signal", jetzt sind alle Outputs auch als Blink konfigurierbar
; V3.22 - neue Register-Namen, Kommentar berarbeitet (no changes in HEX)
; V3.20 - erstellt aus state_led_signal2.asm V3.19

;----------------------------------------------------
; Register, hier nichts ndern!
;----------------------------------------------------
R_STATUS_ON_S1_pre    equ  40h
R_STATUS_ON_S2_pre    equ  41h
R_STATUS_FL_S1_pre    equ  42h
R_STATUS_FL_S2_pre    equ  43h
R_STATUS_ON_S1_akt    equ  44h
R_STATUS_ON_S2_akt    equ  45h
R_STATUS_FL_S1_akt    equ  46h
R_STATUS_FL_S2_akt    equ  47h

R_COUNT               equ  48h          ; Counter fuer PWM - LED1
R_PWMVALUE            equ  49h          ; PWM-Value, bernahme von R_COUNT
R_STATE1              equ  4ah          ; Status-Flags 1 - LED1
R_STATE2              equ  4bh          ; Status-Flags 2 - LED2
R_STATE3              equ  4ch          ; Status-Flags 3 - LED3
R_STATE4              equ  4dh          ; Status-Flags 4 - LED4
R_STATE5              equ  4eh          ; Status-Flags 5 - LED5
R_STATE6              equ  4fh          ; Status-Flags 6 - LED6
R_STATE7              equ  50h          ; Status-Flags 7 - LED7
R_STATE8              equ  51h          ; Status-Flags 8 - LED8
R_STATE9              equ  52h          ; Status-Flags 9 - LED9
R_STATE10             equ  53h          ; Status-Flags 10 - LED10

R_STATUS_ON_P1_buffer equ  54h          ; Buffer Register Page 1 ON
R_STATUS_FL_P1_buffer equ  55h          ; Buffer Register Page 1 FL
R_STATUS_ON_P2_buffer equ  56h          ; Buffer Register Page 2 ON
R_STATUS_FL_P2_buffer equ  57h          ; Buffer Register Page 2 FL

R_TESTTEMP            equ  5ch
;R_FADEVALUE           equ  5dh          ; contains valid fade constant
R_STATUS_2            equ  5eh          ; for >8 Adresses
R_FLAG_001            equ  5fh          ; various Flags
    #define DUNKELTAST_PRE        R_FLAG_001,0
    #define DUNKELTAST_AKT        R_FLAG_001,1
    #define DUNKELTAST_ALL_PRE    R_FLAG_001,2
    #define DUNKELTAST_ALL        R_FLAG_001,3
    #define F_PAGE1_AKTIV         R_FLAG_001,4 ; set if Adress-Page 1 is activ
    #define F_PAGE2_AKTIV         R_FLAG_001,5 ; set if Adress-Page 2 is activ
    #define ALTER_FLASH_S1        R_FLAG_001,6 ; if set alternate Flashing S1 out 4/5
    #define ALTER_FLASH_S2        R_FLAG_001,7 ; if set alternate Flashing S2 out 9/10

R_STATE_FLASHSYNCRO    equ  60h          ; Status-Flags fr Flash-Syncro
R_FLAG_002            equ  61h           ; various Flags
    #define F_NEW_FLASH_SITUATION R_FLAG_002,0

;----------------------------------------------------
; Mapping LED-Nummer zu Port
;----------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition




;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_S2    h'30'           ; EEprom Adresse: R_STATUS_2: Status #2

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- read R_STATUS_2 from EEPROM $30 ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S2
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_2               ; ..into R_STATUS_2

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3                 ; Grundzustand "off"
        bsf R_STATE2,3                 ; Grundzustand "off"
        bsf R_STATE3,3                 ; Grundzustand "off"
        bsf R_STATE4,3                 ; Grundzustand "off"
        bsf R_STATE5,3                 ; Grundzustand "off"
        bsf R_STATE6,3                 ; Grundzustand "off"
        bsf R_STATE7,3                 ; Grundzustand "off"
        bsf R_STATE8,3                 ; Grundzustand "off"
        bsf R_STATE9,3                 ; Grundzustand "off"
        bsf R_STATE10,3                ; Grundzustand "off"
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
#ifndef skip_g1
gruen_1
        ; Signal-Status
        movlw status_g01_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g01_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen1_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen1_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen1_if_page2
  #ifdef gruen1_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen1_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen1_if_page2
#endif ; use_paged_adressing'

#ifdef adr1_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen1_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_01_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_01F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_01_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_01F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_1
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif ; s2_dark_by_1
#ifndef s2_dark_by_1
        bcf DUNKELTAST_PRE
#endif ; s2_dark_by_1

gruen1_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_1
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif ; all_dark_by_1
#ifdef quit_dark_by_1
        bcf DUNKELTAST_ALL_PRE
#endif ; quit_dark_by_1

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen1_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen1_page1_aktiv

  #ifdef gruen1_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen1_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen1_if_page1
gruen1_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen1_if_page1

 #ifdef gruen1_if_page2
gruen1_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen1_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g1

; ********************
; *** ADR Gruen-#2 ***
; ********************
#ifndef skip_g2
gruen_2
        ; Signal-Status
        movlw status_g02_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g02_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen2_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen2_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen2_if_page2
  #ifdef gruen2_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen2_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen2_if_page2
#endif ; use_paged_adressing'

#ifdef adr2_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen2_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_02_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_02F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_02_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_02F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_2
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_2
        bcf DUNKELTAST_PRE
#endif

gruen2_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_2
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_2
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen2_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen2_page1_aktiv

  #ifdef gruen2_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen2_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen2_if_page1
gruen2_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen2_if_page1

 #ifdef gruen2_if_page2
gruen2_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen2_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g2

; ********************
; *** ADR Gruen-#3 ***
; ********************
#ifndef skip_g3
gruen_3
        ; Signal-Status
        movlw status_g03_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g03_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen3_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen3_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen3_if_page2
  #ifdef gruen3_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen3_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen3_if_page2
#endif ; use_paged_adressing'

#ifdef adr3_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen3_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_03_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_03F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_03_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_03F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_3
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_3
        bcf DUNKELTAST_PRE
#endif

gruen3_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_3
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_3
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen3_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen3_page1_aktiv

  #ifdef gruen3_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen3_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen3_if_page1
gruen3_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen3_if_page1

 #ifdef gruen3_if_page2
gruen3_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen3_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g3

; ********************
; *** ADR Gruen-#4 ***
; ********************
#ifndef skip_g4
gruen_4
        ; Signal-Status
        movlw status_g04_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g04_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen4_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen4_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen4_if_page2
  #ifdef gruen4_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen4_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen4_if_page2
#endif ; use_paged_adressing'

#ifdef adr4_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen4_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_04_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_04F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_04_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_04F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_4
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif ; s2_dark_by_4
#ifndef s2_dark_by_4
        bcf DUNKELTAST_PRE
#endif ; s2_dark_by_4

gruen4_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_4
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif ; all_dark_by_4
#ifdef quit_dark_by_4
        bcf DUNKELTAST_ALL_PRE
#endif ; quit_dark_by_4

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen4_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen4_page1_aktiv

  #ifdef gruen4_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen4_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen4_if_page1
gruen4_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen4_if_page1

 #ifdef gruen4_if_page2
gruen4_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen4_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g4

; ********************
; *** ADR Gruen-#5 ***
; ********************
#ifndef skip_g5
gruen_5
        ; Signal-Status
        movlw status_g05_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g05_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen5_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen5_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen5_if_page2
  #ifdef gruen5_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen5_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen5_if_page2
#endif ; use_paged_adressing

#ifdef adr5_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen5_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen5_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen5_to_s1

 #ifdef gruen5_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen5_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_05_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_05F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_05_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_05F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

#ifdef gruen5_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_5
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_5
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen5_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_5
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_5
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen5_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen5_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen5_page1_aktiv

  #ifdef gruen5_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen5_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen5_if_page1
gruen5_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen5_if_page1

 #ifdef gruen5_if_page2
gruen5_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen5_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g5

; ********************
; *** ADR Gruen-#6 ***
; ********************
#ifndef skip_g6
gruen_6
        ; Signal-Status
        movlw status_g06_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g06_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen6_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen6_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen6_if_page2
  #ifdef gruen6_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen6_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen6_if_page2
#endif ; use_paged_adressing

#ifdef adr6_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen6_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen6_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen6_to_s1

 #ifdef gruen6_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen6_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_06_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_06F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_06_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_06F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

#ifdef gruen6_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_6
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_6
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen6_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_6
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_6
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen6_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen6_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen6_page1_aktiv

  #ifdef gruen6_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen6_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen6_if_page1
gruen6_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen6_if_page1

 #ifdef gruen6_if_page2
gruen6_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen6_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g6

; ********************
; *** ADR Gruen-#7 ***
; ********************
#ifndef skip_g7
gruen_7
        ; Signal-Status
        movlw status_g07_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g07_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen7_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen7_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen7_if_page2
  #ifdef gruen7_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen7_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen7_if_page2
#endif ; use_paged_adressing

#ifdef adr7_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen7_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen7_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen7_to_s1

 #ifdef gruen7_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen7_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_07_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_07F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_07_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_07F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

#ifdef gruen7_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_7
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_7
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen7_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_7
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_7
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen7_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen7_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen7_page1_aktiv

  #ifdef gruen7_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen7_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen7_if_page1
gruen7_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen7_if_page1

 #ifdef gruen7_if_page2
gruen7_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen7_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g7

; ********************
; *** ADR Gruen-#8 ***
; ********************
#ifndef skip_g8
gruen_8
        ; Signal-Status
        movlw status_g08_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g08_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen8_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen8_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
  #ifdef gruen8_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen8_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
#endif ; use_paged_adressing

#ifdef adr8_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen8_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen8_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen8_to_s1

 #ifdef gruen8_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
 #endif ; gruen8_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_08_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_08F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_08_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_08F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

#ifdef gruen8_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_8
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_8
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen8_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_8
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_8
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen8_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen8_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen8_page1_aktiv

  #ifdef gruen8_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen8_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen8_if_page1
gruen8_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen8_if_page1

 #ifdef gruen8_if_page2
gruen8_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen8_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g8

; ********************
; *** ADR Gruen-#9 ***
; ********************
#ifndef skip_g9
gruen_9
        ; Signal-Status
        movlw status_g09_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g09_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen8_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen8_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
  #ifdef gruen8_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen8_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
#endif ; use_paged_adressing'

#ifdef adr9_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen9_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_09_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_ON_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_09F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_FL_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_09_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_09F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen9_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_9
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_9
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen9_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen9_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen9_page1_aktiv

  #ifdef gruen9_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen9_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen9_if_page1
gruen9_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen9_if_page1

 #ifdef gruen9_if_page2
gruen9_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen9_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g9

; ********************
; *** ADR Gruen-#10***
; ********************
#ifndef skip_g10
gruen_10
        ; Signal-Status
        movlw status_g10_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g10_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen10_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen10_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen10_if_page2
  #ifdef gruen10_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen10_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen10_if_page2
#endif ; use_paged_adressing'

#ifdef adr10_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen10_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_10_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_ON_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_10F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_FL_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_10_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_10F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen10_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_10
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_10
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen10_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen10_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen10_page1_aktiv

  #ifdef gruen10_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen10_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen10_if_page1
gruen10_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen10_if_page1

 #ifdef gruen10_if_page2
gruen10_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen10_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g10

; ********************
; *** ADR Gruen-#11***
; ********************
#ifndef skip_g11
gruen_11
        ; Signal-Status
        movlw status_g11_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g11_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen11_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen11_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen11_if_page2
  #ifdef gruen11_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen11_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen11_if_page2
#endif ; use_paged_adressing'

#ifdef adr11_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen11_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_11_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_11F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_11_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_11F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen11_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_11
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_11
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen11_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen11_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen11_page1_aktiv

  #ifdef gruen11_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen11_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen11_if_page1
gruen11_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen11_if_page1

 #ifdef gruen11_if_page2
gruen11_buffer_p2                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen11_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g11

; ********************
; *** ADR Gruen-#12***
; ********************
#ifndef skip_g12
gruen_12
        ; Signal-Status
        movlw status_g12_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g12_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen12_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen12_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen12_if_page2
  #ifdef gruen12_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen12_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen12_if_page2
#endif ; use_paged_adressing'

#ifdef adr12_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen12_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_12_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_12F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_12_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_12F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen12_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_12
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_12
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen12_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen12_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen12_page1_aktiv

  #ifdef gruen12_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen12_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen12_if_page1
gruen12_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen12_if_page1

 #ifdef gruen12_if_page2
gruen12_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen12_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g12

; ********************
; *** ADR Gruen-#13***
; ********************
#ifndef skip_g13
gruen_13
        ; Signal-Status
        movlw status_g13_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g13_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen13_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen13_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen13_if_page2
  #ifdef gruen13_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen13_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen13_if_page2
#endif ; use_paged_adressing'

#ifdef adr13_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen13_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_13_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_13F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_13_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_13F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen13_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_13
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_13
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen13_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen13_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen13_page1_aktiv

  #ifdef gruen13_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen13_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen13_if_page1
gruen13_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen13_if_page1

 #ifdef gruen13_if_page2
gruen13_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen13_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g13

; ********************
; *** ADR Gruen-#14***
; ********************
#ifndef skip_g14
gruen_14
        ; Signal-Status
        movlw status_g14_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g14_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen14_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen14_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen14_if_page2
  #ifdef gruen14_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen14_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen14_if_page2
#endif ; use_paged_adressing'

#ifdef adr14_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen14_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_14_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_14F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_14_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_14F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen14_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_14
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_14
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen14_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen14_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen14_page1_aktiv

  #ifdef gruen14_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen14_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen14_if_page1
gruen14_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen14_if_page1

 #ifdef gruen14_if_page2
gruen14_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen14_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g14

; ********************
; *** ADR Gruen-#15***
; ********************
#ifndef skip_g15
gruen_15
        ; Signal-Status
        movlw status_g15_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g15_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen15_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen15_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen15_if_page2
  #ifdef gruen15_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen15_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen15_if_page2
#endif ; use_paged_adressing'

#ifdef adr15_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen15_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_15_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_15F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_15_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_15F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen15_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_15
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_15
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen15_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen15_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen15_page1_aktiv

  #ifdef gruen15_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen15_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen15_if_page1
gruen15_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen15_if_page1

 #ifdef gruen15_if_page2
gruen15_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen15_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g15

; ********************
; *** ADR Gruen-#16***
; ********************
#ifndef skip_g16
gruen_16
        ; Signal-Status
        movlw status_g16_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g16_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        bsf F_NEW_FLASH_SITUATION      ; wird gesetzt, wenn sich am Status was ndert

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen16_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen16_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen16_if_page2
  #ifdef gruen16_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen16_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen16_if_page2
#endif ; use_paged_adressing'

#ifdef adr16_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen16_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_16_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_16F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen

        ; OUT 9..10 statisch:
        movlw SIG_BILD_16_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_16F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        andwf R_STATUS_FL_S2_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif

gruen16_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_16
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_16
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen16_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen16_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen16_page1_aktiv

  #ifdef gruen16_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen16_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen16_if_page1
gruen16_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen16_if_page1

 #ifdef gruen16_if_page2
gruen16_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen16_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g16

;*********************************
;** Routine: switch_to_page_1   **
;** goto Page1,                 **
;** restore Signalbild          **
;*********************************
#ifdef use_paged_adressing
switch_to_page_1
        ; check, ob schon page 1 aktiv ist
        btfsc F_PAGE1_AKTIV            ; wenn Page1 aktiv,
         return                        ; .. dann nichts machen

        ; Flags umschalten
        bsf F_PAGE1_AKTIV
        bcf F_PAGE2_AKTIV

#ifdef use_buffer_register
        ; wenn fahrwegabhngige Adress-Reaktion, dann muss jetzt das gespeicherte Bild des anderen Fahrweges eingeblendet werden

        ; Signalbild statisch, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 1):
        movf R_STATUS_ON_P1_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 1):
        movf R_STATUS_FL_P1_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; use_buffer_register

#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige Signalbilder
        goto restore_after_pagechange
#endif  ; NOT use_buffer_register

        return
#endif ; use_paged_adressing

;*********************************
;** Routine: switch_to_page_2   **
;** goto Page2,                 **
;** restore Signalbild          **
;*********************************
#ifdef use_paged_adressing
switch_to_page_2
        ; check, ob schon page 2 aktiv ist
        btfsc F_PAGE2_AKTIV            ; wenn Page1 aktiv,
         return                        ; .. dann nichts machen

        ; Flags umschalten
        bsf F_PAGE2_AKTIV
        bcf F_PAGE1_AKTIV

#ifdef use_buffer_register
        ; wenn fahrwegabhngige Adress-Reaktion, dann muss jetzt das gespeicherte Bild des anderen Fahrweges eingeblendet werden

        ; Signalbild statisch, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 2):
        movf R_STATUS_ON_P2_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 2):
        movf R_STATUS_FL_P2_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_FL_S1_pre
        ; andwf R_STATUS_FL_S1_akt,f     ; in AKT die Bits lschen, die nicht mehr blinken sollen
#endif ; use_buffer_register

#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige Signalbilder
        goto restore_after_pagechange
#endif  ; NOT use_buffer_register

        return
#endif ; use_paged_adressing

;*****************************************
;** Routine: restore_after_pagechange   **
;*****************************************
#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige  Signalbilder
restore_after_pagechange
 #ifdef gruen1_depends_on_page
        btfsc R_STATUS,0
         call gruen_1
 #endif
 #ifdef gruen2_depends_on_page
        btfsc R_STATUS,1
         call gruen_2
 #endif
 #ifdef gruen3_depends_on_page
        btfsc R_STATUS,2
         call gruen_3
 #endif
 #ifdef gruen4_depends_on_page
        btfsc R_STATUS,3
         call gruen_4
 #endif
 #ifdef gruen5_depends_on_page
        btfsc R_STATUS,4
         call gruen_5
 #endif
 #ifdef gruen6_depends_on_page
        btfsc R_STATUS,5
         call gruen_6
 #endif
 #ifdef gruen7_depends_on_page
        btfsc R_STATUS,6
         call gruen_7
 #endif
 #ifdef gruen8_depends_on_page
        btfsc R_STATUS,7
         call gruen_8
 #endif
 #ifdef gruen9_depends_on_page
        btfsc R_STATUS_2,0
         call gruen_9
 #endif
 #ifdef gruen10_depends_on_page
        btfsc R_STATUS_2,1
         call gruen_10
 #endif
 #ifdef gruen11_depends_on_page
        btfsc R_STATUS_2,2
         call gruen_11
 #endif
 #ifdef gruen12_depends_on_page
        btfsc R_STATUS_2,3
         call gruen_12
 #endif
 #ifdef gruen13_depends_on_page
        btfsc R_STATUS_2,4
         call gruen_13
 #endif
 #ifdef gruen14_depends_on_page
        btfsc R_STATUS_2,5
         call gruen_14
 #endif
 #ifdef gruen15_depends_on_page
        btfsc R_STATUS_2,6
         call gruen_15
 #endif
 #ifdef gruen16_depends_on_page
        btfsc R_STATUS_2,7
         call gruen_16
 #endif
#endif ; NOT use_buffer_register
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
#ifndef skip_g1
save_01
        movlw save_01_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_01_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_g1

; ********************
; *** Adresse #2   ***
; ********************
#ifndef skip_g2
save_02
        movlw save_02_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_02_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #3   ***
; ********************
#ifndef skip_g3
save_03
        movlw save_03_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_03_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #4   ***
; ********************
#ifndef skip_g4
save_04
        movlw save_04_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_04_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #5   ***
; ********************
#ifndef skip_g5
save_05
        movlw save_05_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_05_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #6   ***
; ********************
#ifndef skip_g6
save_06
        movlw save_06_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_06_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #7   ***
; ********************
#ifndef skip_g7
save_07
        movlw save_07_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_07_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #8   ***
; ********************
#ifndef skip_g8
save_08
        movlw save_08_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_08_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #9   ***
; ********************
#ifndef skip_g9
save_09
        movlw save_09_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_09_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #10  ***
; ********************
#ifndef skip_g10
save_10
        movlw save_10_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_10_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #11  ***
; ********************
#ifndef skip_g11
save_11
        movlw save_11_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_11_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #12  ***
; ********************
#ifndef skip_g12
save_12
        movlw save_12_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_12_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #13  ***
; ********************
#ifndef skip_g13
save_13
        movlw save_13_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_13_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #14  ***
; ********************
#ifndef skip_g14
save_14
        movlw save_14_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_14_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #15  ***
; ********************
#ifndef skip_g15
save_15
        movlw save_15_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_15_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #16  ***
; ********************
#ifndef skip_g16
save_16
        movlw save_16_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_16_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output

#ifdef use_paged_adressing
#ifdef use_buffer_register
        ; Bffer laden
        call load_buffer
#endif ; use_buffer_register

; wenn Adress-Pages verwendet werden, muss das letzte gltige Flag wieder hergestellt werden
        bcf F_PAGE1_AKTIV
        bcf F_PAGE2_AKTIV

 #ifdef gruen1_page2_aktiv
        btfsc R_STATUS,0
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen2_page2_aktiv
        btfsc R_STATUS,1
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen3_page2_aktiv
        btfsc R_STATUS,2
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen4_page2_aktiv
        btfsc R_STATUS,3
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen5_page2_aktiv
        btfsc R_STATUS,4
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen6_page2_aktiv
        btfsc R_STATUS,5
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen7_page2_aktiv
        btfsc R_STATUS,6
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen8_page2_aktiv
        btfsc R_STATUS,7
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen9_page2_aktiv
        btfsc R_STATUS_2,0
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen10_page2_aktiv
        btfsc R_STATUS_2,1
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen11_page2_aktiv
        btfsc R_STATUS_2,2
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen12_page2_aktiv
        btfsc R_STATUS_2,3
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen13_page2_aktiv
        btfsc R_STATUS_2,4
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen14_page2_aktiv
        btfsc R_STATUS_2,5
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen15_page2_aktiv
        btfsc R_STATUS_2,6
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen16_page2_aktiv
        btfsc R_STATUS_2,7
         bsf F_PAGE2_AKTIV
 #endif
        ; wenn nicht Page2, dann Page1 (eine Page muss aktiv sein, im Zweifel: Page1!)
        btfss F_PAGE2_AKTIV
         bsf F_PAGE1_AKTIV
#endif ; use_paged_adressing


;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Durch Aufruf der Routine "gruen_1" etc. werden diese in die
    ; Signalbilder umgewandelt.
#ifndef skip_g1
        btfsc R_STATUS,0
         call gruen_1
#endif

#ifndef skip_g2
        btfsc R_STATUS,1
         call gruen_2
#endif

#ifndef skip_g3
        btfsc R_STATUS,2
         call gruen_3
#endif

#ifndef skip_g4
        btfsc R_STATUS,3
         call gruen_4
#endif

#ifndef skip_g5
        btfsc R_STATUS,4
         call gruen_5
#endif

#ifndef skip_g6
        btfsc R_STATUS,5
         call gruen_6
#endif

#ifndef skip_g7
        btfsc R_STATUS,6
         call gruen_7
#endif

#ifndef skip_g8
        btfsc R_STATUS,7
         call gruen_8
#endif

#ifndef skip_g9
        btfsc R_STATUS_2,0
         call gruen_9
#endif

#ifndef skip_g10
        btfsc R_STATUS_2,1
         call gruen_10
#endif

#ifndef skip_g11
        btfsc R_STATUS_2,2
         call gruen_11
#endif

#ifndef skip_g12
        btfsc R_STATUS_2,3
         call gruen_12
#endif

#ifndef skip_g13
        btfsc R_STATUS_2,4
         call gruen_13
#endif

#ifndef skip_g14
        btfsc R_STATUS_2,5
         call gruen_14
#endif

#ifndef skip_g15
        btfsc R_STATUS_2,6
         call gruen_15
#endif

#ifndef skip_g16
        btfsc R_STATUS_2,7
         call gruen_16
#endif
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'11111111'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw h'00'
        movwf R_EEADR                  ; Address: h'00'
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save_buffer        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_paged_adressing
#ifdef use_buffer_register
save_buffer_p1
        movlw h'40'
        movwf R_EEADR                  ; Address: h'40'
        movf R_STATUS_ON_P1_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_ON_P1_buffer
        call saveroutine

        movlw h'50'
        movwf R_EEADR                  ; Address: h'50'
        movf R_STATUS_FL_P1_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_FL_P1_buffer
        call saveroutine

        return

save_buffer_p2
        movlw h'60'
        movwf R_EEADR                  ; Address: h'60'
        movf R_STATUS_ON_P2_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_ON_P2_buffer
        call saveroutine

        movlw h'70'
        movwf R_EEADR                  ; Address: h'70'
        movf R_STATUS_FL_P2_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_FL_P2_buffer
        call saveroutine

        return
#endif ; use_buffer_register
#endif ; use_paged_adressing

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: load_buffer        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_paged_adressing
#ifdef use_buffer_register
load_buffer
        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'40'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_ON_P1_buffer    ; ..into R_STATUS_ON_P1_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'50'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_FL_P1_buffer    ; ..into R_STATUS_FL_P1_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'60'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_ON_P2_buffer    ; ..into R_STATUS_ON_P2_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'70'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_FL_P2_buffer    ; ..into R_STATUS_FL_P2_buffer

        return
#endif ; use_buffer_register
#endif ; use_paged_adressing

;*******************************************
; +++ for debug +++
;*******************************************
;      movf R_ADR_3,W ; test debug
;      movwf R_TESTTEMP
;      call show_testtemp

show_testtemp
; for DEBUG  btfss R_TESTTEMP
         bcf LED_01
        btfsc R_TESTTEMP,0
         bsf LED_01

        btfss R_TESTTEMP,1
         bcf LED_02
        btfsc R_TESTTEMP,1
         bsf LED_02

        btfss R_TESTTEMP,2
         bcf LED_03
        btfsc R_TESTTEMP,2
         bsf LED_03

        btfss R_TESTTEMP,3
         bcf LED_04
        btfsc R_TESTTEMP,3
         bsf LED_04

        btfss R_TESTTEMP,4
         bcf LED_05
        btfsc R_TESTTEMP,4
         bsf LED_05

        btfss R_TESTTEMP,5
         bcf LED_06
        btfsc R_TESTTEMP,5
         bsf LED_06

        btfss R_TESTTEMP,6
         bcf LED_07
        btfsc R_TESTTEMP,6
         bsf LED_07

        btfss R_TESTTEMP,7
         bcf LED_08
        btfsc R_TESTTEMP,7
         bsf LED_08

        return

;***************************************
;***************************************
;***************************************
;***************************************
;** Routine: statemachine_signal      **
;** state manager signal              **
;** Aufruf alle 2-3 ms                **
;** needs <    us                     **
;***************************************
;***************************************
;***************************************
;***************************************
;   ------------------
;   --- Signal     ---
;   ------------------
;R_STATEx   *              *           
;------------\             :  : /--------  LEDxA (red)
;           : \----------------/ :       
;           :  :        :  :       
;#1         :#2 #3             #4:#1  
;
;Time *: receive order to switch    
;
;PWM detail, here Fade in (Fade out = inverted Signal):
;  0        1        2        3        4        5       6       7 Value of R_COUNT
;  :-:      :--:     :---:    :----:   :-----:  :-----: :------::-----
;  : :      :  :     :   :    :    :   :     :  :     : :      ::
; -: :------:  :-----:   :----:    :---:     :--:     :-:      ::
;  1 2      3  2     3   2    3    2   3     2  3     2 3       3
;@1: LED on, R_COUNT=0
;@2: LED off
;@3: LED on, R_COUNT increment every 2ms

statemachine_signal
;   -------------------------
;   --- create 3ms-Raster ---
;   -------------------------
#ifndef test
        movf TMR1H,F 
        btfsc STATUS,Z                 ; if TMR1H <> ZERO
         return
#endif
        clrf TMR1H                     ; .. then reset to ZERO

        movlw CONST_FADE_01            ;
        movwf TMR1L;                   ; fr schnelleres Blinken / Fading

        ; Counter, um State-Change zu verlangsamen
        incfsz R_COUNT,F               ; if R_COUNT = 0
         goto signal2_1                ; .. then change R_STATE 
        incf R_COUNT,F                 ; R_COUNT starts with '1'

#ifdef use_flash_syncro
        ; *********************************************
        ; *** neu in V3.58: Blinken syncronisieren  ***
        ; *********************************************

        ; zunchst prfen, ob in PRE neuen Flash-Outputs definiert sind
        btfss F_NEW_FLASH_SITUATION
         goto FLASH_SYNCRO_OK
        ; in AKT die aktiven Bits lschen, die in PRE nicht mehr aktiv sind
        movf R_STATUS_ON_S1_pre,W
        andwf R_STATUS_ON_S1_akt,F
        movf R_STATUS_ON_S2_pre,W
        andwf R_STATUS_ON_S2_akt,F
        ; in PRE die Bits lschen, die in AKT bereits aktiv sind
        movf R_STATUS_ON_S1_akt,W
        xorwf R_STATUS_ON_S1_pre,F
        movf R_STATUS_ON_S2_akt,W
        xorwf R_STATUS_ON_S2_pre,F
        ; Flag lschen
        bcf F_NEW_FLASH_SITUATION

FLASH_SYNCRO_OK

        ; gibt es ein "neues" Flashing? Dann liegt ein Bit in ..._pre vor, d.h check ob ..._pre =0 ZERO
        movf R_STATUS_FL_S1_pre,W
        iorwf R_STATUS_FL_S2_pre,W
        btfsc STATUS,Z
         goto STATUS_REG_1

        ; es gibt also ein "neues" Flashing? Gibt es ein "altes" flashing? Dann liegt ein Bit in ..._akt vor, d.h check ob ..._akt =0 ZERO
        movf R_STATUS_FL_S1_akt,W
        iorwf R_STATUS_FL_S2_akt,W
        btfss STATUS,Z
         goto FLASH_SYNCRO             ; es gibt ein "neues" und ein "altes" Flashing --> Syncro

        ; es gibt also ein "neues" Flashing und kein "altes" flashing. Somit wird PRE zu AKT.
        movf R_STATUS_FL_S1_pre,W
        movwf R_STATUS_FL_S1_akt
        clrf R_STATUS_FL_S1_pre

        movf R_STATUS_FL_S2_pre,W
        movwf R_STATUS_FL_S2_akt
        clrf R_STATUS_FL_S2_pre

        goto STATUS_REG_1              ; goto State Machine

FLASH_SYNCRO ; flashing syncronisieren
        ; Suche: wer blinkt gerade
        btfsc SET_OUTPUT1_FL           ; ist OUT1 schon am Blinken?
         movf R_STATE1,W               ; .. dann R_STATE1 laden
        btfsc SET_OUTPUT2_FL           ; ist OUT2 schon am Blinken?
         movf R_STATE2,W               ; .. dann R_STATE2 laden
        btfsc SET_OUTPUT3_FL           ; ist OUT3 schon am Blinken?
         movf R_STATE3,W               ; .. dann R_STATE3 laden
        btfsc SET_OUTPUT4_FL           ; ist OUT4 schon am Blinken?
         movf R_STATE4,W               ; .. dann R_STATE4 laden
        btfsc SET_OUTPUT5_FL           ; ist OUT5 schon am Blinken?
         movf R_STATE5,W               ; .. dann R_STATE5 laden
        btfsc SET_OUTPUT6_FL           ; ist OUT6 schon am Blinken?
         movf R_STATE6,W               ; .. dann R_STATE6 laden
        btfsc SET_OUTPUT7_FL           ; ist OUT7 schon am Blinken?
         movf R_STATE7,W               ; .. dann R_STATE7 laden
        btfsc SET_OUTPUT8_FL           ; ist OUT8 schon am Blinken?
         movf R_STATE8,W               ; .. dann R_STATE8 laden
        btfsc SET_OUTPUT9_FL           ; ist OUT9 schon am Blinken?
         movf R_STATE9,W               ; .. dann R_STATE9 laden
        btfsc SET_OUTPUT10_FL          ; ist OUT10 schon am Blinken?
         movf R_STATE10,W              ; .. dann R_STATE10 laden
        movwf R_STATE_FLASHSYNCRO      ; Wert in R_STATE_FLASHSYNCRO schreiben

        ; Suche: wer will blinken?
FLASH_SYNCRO_1
        btfss SET_OUTPUT1_FL_PRE       ; soll OUT1 blinken?
        goto FLASH_SYNCRO_2
         movf R_STATE1,W               ; .. dann R_STATE1 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT1_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT1_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_2
        btfss SET_OUTPUT2_FL_PRE       ; soll OUT2 blinken?
        goto FLASH_SYNCRO_3
         movf R_STATE2,W               ; .. dann R_STATE2 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT2_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT2_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_3
        btfss SET_OUTPUT3_FL_PRE       ; soll OUT3 blinken?
        goto FLASH_SYNCRO_4
         movf R_STATE3,W               ; .. dann R_STATE3 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT3_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT3_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_4
        btfss SET_OUTPUT4_FL_PRE       ; soll OUT4 blinken?
        goto FLASH_SYNCRO_5
         movf R_STATE4,W               ; .. dann R_STATE4 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT4_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT4_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_5
        btfss SET_OUTPUT5_FL_PRE       ; soll OUT5 blinken?
        goto FLASH_SYNCRO_6
         movf R_STATE5,W               ; .. dann R_STATE5 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT5_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT5_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_6
        btfss SET_OUTPUT6_FL_PRE       ; soll OUT6 blinken?
        goto FLASH_SYNCRO_7
         movf R_STATE6,W               ; .. dann R_STATE6 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT6_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT6_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_7
        btfss SET_OUTPUT7_FL_PRE       ; soll OUT7 blinken?
        goto FLASH_SYNCRO_8
         movf R_STATE7,W               ; .. dann R_STATE7 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT7_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT7_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_8
        btfss SET_OUTPUT8_FL_PRE       ; soll OUT8 blinken?
        goto FLASH_SYNCRO_9
         movf R_STATE8,W               ; .. dann R_STATE8 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT8_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT8_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_9
        btfsc SET_OUTPUT9_FL           ; wenn OUT9 bereits blinkt
        goto FLASH_SYNCRO_10           ; .. dann SKIP
        btfss SET_OUTPUT9_FL_PRE       ; soll OUT9 blinken?
        goto FLASH_SYNCRO_10           ; .. dann SKIP
         movf R_STATE9,W               ; .. dann R_STATE9 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT9_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT9_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_10
        btfss SET_OUTPUT10_FL_PRE       ; soll OUT10 blinken?
        goto FLASH_SYNCRO_11
         movf R_STATE10,W               ; .. dann R_STATE10 laden
         subwf R_STATE_FLASHSYNCRO,W   ; STATE von aktiv blinkendem Ausgang identisch mit STATE von zu blinkendem Ausgang?
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bcf SET_OUTPUT10_FL_PRE        ; aus PRE
         btfsc STATUS,Z                ; wenn identisch, dann Z = TRUE
         bsf SET_OUTPUT10_FL            ; .. wird aktive Blinken

FLASH_SYNCRO_11
#endif ;use_flash_syncro

#ifndef use_flash_syncro
        ; es gibt also ein "neues" Flashing und kein "altes" flashing. Somit wird PRE zu AKT.
        movf R_STATUS_FL_S1_pre,W
        movwf R_STATUS_FL_S1_akt
        clrf R_STATUS_FL_S1_pre

        movf R_STATUS_FL_S2_pre,W
        movwf R_STATUS_FL_S2_akt
        clrf R_STATUS_FL_S2_pre
#endif ; not use_flash_syncro

STATUS_REG_1
        ; *********************************************
        ; **************** Change STATE ***************
        ; *********************************************
        ; jeder Ausgang hat seine eigene STATE machine.
        ; in den Abschnitten STATUS_REG_xx wird der Status des jeweiligen Ausganges neu berechnet.
        ; INPUT:  R_STATUS_ON_Sy_akt und R_STATUS_FL_Sy_akt (y = Signal-Nummer 1/2)
        ; OUTPUT: R_STATExx (xx = Ausgang-Nummer)
        ; *********************************************

        ; *** LED1 (Ausgang 1) ***
#ifdef dunkeltast_all_out01 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED1OFF                  ; .. then LED off
#endif ; dunkeltast_all_out01
#ifdef dunkeltast_out01
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED1OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT1
         goto LED1OFF                  ; .. then jump
        btfss SET_OUTPUT1_FL
         goto LED1ON                   ; .. then jump

LED1FLASH ; *** LED1 FLASH ***      
FL1_1   btfss R_STATE1,1               ; check R_STATE1 #1
         goto FL1_2
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2
        goto STATUS_REG_2              ; continue

FL1_2   btfss R_STATE1,2               ; check R_STATE1 #2
         goto FL1_3
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,3                 ; continue R_STATE1#3
        goto STATUS_REG_2              ; continue

FL1_3   btfss R_STATE1,3               ; check R_STATE1 #3
         goto FL1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

FL1_4   btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,1                 ; continue R_STATE1#1
        goto STATUS_REG_2              ; continue

LED1ON  ; *** LED1 ON ***      
ON1_2   btfss R_STATE1,2               ; check R_STATE1 #2
         goto ON1_3
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_3   btfss R_STATE1,3               ; check R_STATE1 #3
         goto ON1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,4                 ; continue R_STATE1#4
        goto STATUS_REG_2              ; continue

ON1_4   btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,1                 ; continue R_STATE1#1
        goto STATUS_REG_2              ; continue
        
LED1OFF ; *** LED1 OFF ***      
OFF1_1  btfss R_STATE1,1               ; check R_STATE1 #1
         goto OFF1_2
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2
         goto STATUS_REG_2             ; continue
        
OFF1_2  btfss R_STATE1,2               ; check R_STATE1 #2
         goto OFF1_4
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,3                 ; continue R_STATE1#3
        goto STATUS_REG_2              ; continue

OFF1_4  btfss R_STATE1,4               ; check R_STATE1 #4
         goto STATUS_REG_2             ; continue
        clrf R_STATE1                  ; clear R_STATE1  
        bsf R_STATE1,2                 ; continue R_STATE1#2



STATUS_REG_2
        ; *** LED2 (Ausgang 2) ***
#ifdef dunkeltast_all_out02 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED2OFF                  ; .. then LED off
#endif ; dunkeltast_all_out02
#ifdef dunkeltast_out02
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED2OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT2
         goto LED2OFF                  ; .. then jump
        btfss SET_OUTPUT2_FL
         goto LED2ON                   ; .. then jump

LED2FLASH ; *** LED2 FLASH ***      
FL2_1   btfss R_STATE2,1               ; check R_STATE2 #1
         goto FL2_2
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2
        goto STATUS_REG_3              ; continue

FL2_2   btfss R_STATE2,2               ; check R_STATE2 #2
         goto FL2_3
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,3                 ; continue R_STATE2#3
        goto STATUS_REG_3              ; continue

FL2_3   btfss R_STATE2,3               ; check R_STATE2 #3
         goto FL2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

FL2_4   btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,1                 ; continue R_STATE2#1
        goto STATUS_REG_3              ; continue
           
LED2ON  ; *** LED2 ON *** 
ON2_2   btfss R_STATE2,2               ; check R_STATE2 #2
         goto ON2_3
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_3   btfss R_STATE2,3               ; check R_STATE2 #3
         goto ON2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,4                 ; continue R_STATE2#4
        goto STATUS_REG_3              ; continue

ON2_4   btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,1                 ; continue R_STATE2#1
        goto STATUS_REG_3              ; continue
        
LED2OFF ; *** LED2 OFF ***      
OFF2_1  btfss R_STATE2,1               ; check R_STATE2 #1
         goto OFF2_2
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2
         goto STATUS_REG_3             ; continue
        
OFF2_2  btfss R_STATE2,2               ; check R_STATE2 #2
         goto OFF2_4
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,3                 ; continue R_STATE2#3
        goto STATUS_REG_3              ; continue

OFF2_4  btfss R_STATE2,4               ; check R_STATE2 #4
         goto STATUS_REG_3             ; continue
        clrf R_STATE2                  ; clear R_STATE2  
        bsf R_STATE2,2                 ; continue R_STATE2#2



STATUS_REG_3
        ; *** LED3 (Ausgang 3) ***
#ifdef dunkeltast_all_out03 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED3OFF                  ; .. then LED off
#endif ; dunkeltast_all_out03
#ifdef dunkeltast_out03
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED3OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT3
         goto LED3OFF                  ; .. then jump
        btfss SET_OUTPUT3_FL
         goto LED3ON                   ; .. then jump
          
LED3FLASH ; *** LED3 FLASH ***      
FL3_1   btfss R_STATE3,1               ; check R_STATE3 #1
         goto FL3_2
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2
        goto STATUS_REG_4              ; continue

FL3_2   btfss R_STATE3,2               ; check R_STATE3 #2
         goto FL3_3
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,3                 ; continue R_STATE3#3
        goto STATUS_REG_4              ; continue

FL3_3   btfss R_STATE3,3               ; check R_STATE3 #3
         goto FL3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

FL3_4   btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,1                 ; continue R_STATE3#1
        goto STATUS_REG_4              ; continue

LED3ON  ; *** LED3 ON ***  
ON3_2   btfss R_STATE3,2               ; check R_STATE3 #2
         goto ON3_3
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_3   btfss R_STATE3,3               ; check R_STATE3 #3
         goto ON3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,4                 ; continue R_STATE3#4
        goto STATUS_REG_4              ; continue

ON3_4   btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,1                 ; continue R_STATE3#1
        goto STATUS_REG_4              ; continue
        
LED3OFF ; *** LED3 OFF ***  
OFF3_1  btfss R_STATE3,1               ; check R_STATE3 #1
         goto OFF3_2
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2
         goto STATUS_REG_4             ; continue
        
OFF3_2  btfss R_STATE3,2               ; check R_STATE3 #2
         goto OFF3_4
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,3                 ; continue R_STATE3#3
        goto STATUS_REG_4              ; continue

OFF3_4  btfss R_STATE3,4               ; check R_STATE3 #4
         goto STATUS_REG_4             ; continue
        clrf R_STATE3                  ; clear R_STATE3  
        bsf R_STATE3,2                 ; continue R_STATE3#2



STATUS_REG_4
        ; *** LED4 (Ausgang 4) ***
#ifdef dunkeltast_all_out04 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED4OFF                  ; .. then LED off
#endif ; dunkeltast_all_out04
#ifdef dunkeltast_out04
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED4OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT4
         goto LED4OFF                  ; .. then jump
        btfss SET_OUTPUT4_FL
         goto LED4ON                   ; .. then jump

LED4FLASH ; *** LED4 FLASH ***      
FL4_1   btfss R_STATE4,1               ; check R_STATE4 #1
         goto FL4_2
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2
        bcf ALTER_FLASH_S1             ; now other LED may also start flashing...
        goto STATUS_REG_5              ; continue

FL4_2  btfss R_STATE4,2                ; check R_STATE4 #2
         goto FL4_3
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,3                 ; continue R_STATE4#3
        goto STATUS_REG_5              ; continue

FL4_3  btfss R_STATE4,3                ; check R_STATE4 #3
         goto FL4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

FL4_4  btfss R_STATE4,4                ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,1                 ; continue R_STATE4#1
        goto STATUS_REG_5              ; continue

LED4ON  ; *** LED4 ON ***      
ON4_2   btfss R_STATE4,2               ; check R_STATE4 #2
         goto ON4_3
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_3   btfss R_STATE4,3               ; check R_STATE4 #3
         goto ON4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,4                 ; continue R_STATE4#4
        goto STATUS_REG_5              ; continue

ON4_4   btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,1                 ; continue R_STATE4#1
        goto STATUS_REG_5              ; continue
        
LED4OFF ; *** LED4 OFF ***      
OFF4_1  btfss R_STATE4,1               ; check R_STATE4 #1
         goto OFF4_2
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2
         goto STATUS_REG_5             ; continue
        
OFF4_2  btfss R_STATE4,2               ; check R_STATE4 #2
         goto OFF4_4
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,3                 ; continue R_STATE4#3
        goto STATUS_REG_5              ; continue

OFF4_4  btfss R_STATE4,4               ; check R_STATE4 #4
         goto STATUS_REG_5             ; continue
        clrf R_STATE4                  ; clear R_STATE4  
        bsf R_STATE4,2                 ; continue R_STATE4#2



STATUS_REG_5
        ; *** LED5 (Ausgang 5) ***
#ifdef dunkeltast_all_out05 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED5OFF                  ; .. then LED off
#endif ; dunkeltast_all_out05
#ifdef dunkeltast_out05
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED5OFF                  ; .. then LED off
#endif
        btfsc ALTER_FLASH_S1           ; if this bit is #1, then no action on this output,
         goto LED5OFF                  ; .. because other LED has first fade-in
        btfss SET_OUTPUT5
         goto LED5OFF                  ; .. then jump
        btfss SET_OUTPUT5_FL
         goto LED5ON                   ; .. then jump

LED5FLASH ; *** LED5 FLASH ***      
FL5_1  btfss R_STATE5,1                ; check R_STATE5 #1
         goto FL5_2
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2
        goto STATUS_REG_6              ; continue

FL5_2  btfss R_STATE5,2                ; check R_STATE5 #2
         goto FL5_3
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,3                 ; continue R_STATE5#3
        goto STATUS_REG_6              ; continue

FL5_3  btfss R_STATE5,3                ; check R_STATE5 #3
         goto FL5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

FL5_4  btfss R_STATE5,4                ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,1                 ; continue R_STATE5#1
        goto STATUS_REG_6              ; continue

LED5ON  ; *** LED5 ON ***      
ON5_2   btfss R_STATE5,2               ; check R_STATE5 #2
         goto ON5_3
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

ON5_3   btfss R_STATE5,3               ; check R_STATE5 #3
         goto ON5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,4                 ; continue R_STATE5#4
        goto STATUS_REG_6              ; continue

ON5_4   btfss R_STATE5,4               ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,1                 ; continue R_STATE5#1
        goto STATUS_REG_6              ; continue
        
LED5OFF ; *** LED5 OFF ***      
OFF5_1  btfss R_STATE5,1               ; check R_STATE5 #1
         goto OFF5_2
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2
         goto STATUS_REG_6             ; continue
        
OFF5_2  btfss R_STATE5,2               ; check R_STATE5 #2
         goto OFF5_4
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,3                 ; continue R_STATE5#3
        goto STATUS_REG_6              ; continue

OFF5_4  btfss R_STATE5,4               ; check R_STATE5 #4
         goto STATUS_REG_6             ; continue
        clrf R_STATE5                  ; clear R_STATE5  
        bsf R_STATE5,2                 ; continue R_STATE5#2



STATUS_REG_6
        ; *** LED6 (Ausgang 6) ***
#ifdef dunkeltast_all_out06 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED6OFF                  ; .. then LED off
#endif ; dunkeltast_all_out06
#ifdef dunkeltast_out06
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED6OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT6
         goto LED6OFF                  ; .. then jump
        btfss SET_OUTPUT6_FL
         goto LED6ON                   ; .. then jump

LED6FLASH ; *** LED6 FLASH ***      
FL6_1  btfss R_STATE6,1                ; check R_STATE6 #1
         goto FL6_2
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2
        goto STATUS_REG_7              ; continue

FL6_2  btfss R_STATE6,2                ; check R_STATE6 #2
         goto FL6_3
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,3                 ; continue R_STATE6#3
        goto STATUS_REG_7              ; continue

FL6_3  btfss R_STATE6,3                ; check R_STATE6 #3
         goto FL6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

FL6_4  btfss R_STATE6,4                ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,1                 ; continue R_STATE6#1
        goto STATUS_REG_7              ; continue

LED6ON  ; *** LED6 ON ***      
ON6_2   btfss R_STATE6,2               ; check R_STATE6 #2
         goto ON6_3
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

ON6_3   btfss R_STATE6,3               ; check R_STATE6 #3
         goto ON6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,4                 ; continue R_STATE6#4
        goto STATUS_REG_7              ; continue

ON6_4   btfss R_STATE6,4               ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,1                 ; continue R_STATE6#1
        goto STATUS_REG_7              ; continue
        
LED6OFF ; *** LED6 OFF ***      
OFF6_1  btfss R_STATE6,1               ; check R_STATE6 #1
         goto OFF6_2
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2
         goto STATUS_REG_7             ; continue
        
OFF6_2  btfss R_STATE6,2               ; check R_STATE6 #2
         goto OFF6_4
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,3                 ; continue R_STATE6#3
        goto STATUS_REG_7              ; continue

OFF6_4  btfss R_STATE6,4               ; check R_STATE6 #4
         goto STATUS_REG_7             ; continue
        clrf R_STATE6                  ; clear R_STATE6  
        bsf R_STATE6,2                 ; continue R_STATE6#2



STATUS_REG_7
        ; *** LED7 (Ausgang 7) ***
#ifdef dunkeltast_all_out07 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED7OFF                  ; .. then LED off
#endif ; dunkeltast_all_out07
#ifdef dunkeltast_out07
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED7OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT7
         goto LED7OFF                  ; .. then jump
        btfss SET_OUTPUT7_FL
         goto LED7ON                   ; .. then jump

LED7FLASH ; *** LED7 FLASH ***      
FL7_1  btfss R_STATE7,1                ; check R_STATE7 #1
         goto FL7_2
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2
        goto STATUS_REG_8              ; continue

FL7_2  btfss R_STATE7,2                ; check R_STATE7 #2
         goto FL7_3
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,3                 ; continue R_STATE7#3
        goto STATUS_REG_8              ; continue

FL7_3  btfss R_STATE7,3                ; check R_STATE7 #3
         goto FL7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

FL7_4  btfss R_STATE7,4                ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,1                 ; continue R_STATE7#1
        goto STATUS_REG_8              ; continue

LED7ON  ; *** LED7 ON ***      
ON7_2   btfss R_STATE7,2               ; check R_STATE7 #2
         goto ON7_3
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

ON7_3   btfss R_STATE7,3               ; check R_STATE7 #3
         goto ON7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,4                 ; continue R_STATE7#4
        goto STATUS_REG_8              ; continue

ON7_4   btfss R_STATE7,4               ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,1                 ; continue R_STATE7#1
        goto STATUS_REG_8              ; continue
        
LED7OFF ; *** LED7 OFF ***      
OFF7_1  btfss R_STATE7,1               ; check R_STATE7 #1
         goto OFF7_2
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2
         goto STATUS_REG_8             ; continue
        
OFF7_2  btfss R_STATE7,2               ; check R_STATE7 #2
         goto OFF7_4
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,3                 ; continue R_STATE7#3
        goto STATUS_REG_8              ; continue

OFF7_4  btfss R_STATE7,4               ; check R_STATE7 #4
         goto STATUS_REG_8             ; continue
        clrf R_STATE7                  ; clear R_STATE7  
        bsf R_STATE7,2                 ; continue R_STATE7#2



STATUS_REG_8
        ; *** LED8 (Ausgang 8) ***
#ifdef dunkeltast_all_out08 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED8OFF                  ; .. then LED off
#endif ; dunkeltast_all_out08
#ifdef dunkeltast_out08
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED8OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT8
         goto LED8OFF                  ; .. then jump
        btfss SET_OUTPUT8_FL
         goto LED8ON                   ; .. then jump

LED8FLASH ; *** LED8 FLASH ***      
FL8_1  btfss R_STATE8,1                ; check R_STATE8 #1
         goto FL8_2
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2
        goto STATUS_REG_9              ; continue

FL8_2  btfss R_STATE8,2                ; check R_STATE8 #2
         goto FL8_3
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,3                 ; continue R_STATE8#3
        goto STATUS_REG_9              ; continue

FL8_3  btfss R_STATE8,3                ; check R_STATE8 #3
         goto FL8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

FL8_4  btfss R_STATE8,4                ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,1                 ; continue R_STATE8#1
        goto STATUS_REG_9              ; continue

LED8ON  ; *** LED8 ON ***      
ON8_2   btfss R_STATE8,2               ; check R_STATE8 #2
         goto ON8_3
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

ON8_3   btfss R_STATE8,3               ; check R_STATE8 #3
         goto ON8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,4                 ; continue R_STATE8#4
        goto STATUS_REG_9              ; continue

ON8_4   btfss R_STATE8,4               ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,1                 ; continue R_STATE8#1
        goto STATUS_REG_9              ; continue
        
LED8OFF ; *** LED8 OFF ***      
OFF8_1  btfss R_STATE8,1               ; check R_STATE8 #1
         goto OFF8_2
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2
         goto STATUS_REG_9             ; continue
        
OFF8_2  btfss R_STATE8,2               ; check R_STATE8 #2
         goto OFF8_4
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,3                 ; continue R_STATE8#3
        goto STATUS_REG_9              ; continue

OFF8_4  btfss R_STATE8,4               ; check R_STATE8 #4
         goto STATUS_REG_9             ; continue
        clrf R_STATE8                  ; clear R_STATE8  
        bsf R_STATE8,2                 ; continue R_STATE8#2



STATUS_REG_9
        ; *** LED9 (Ausgang 9) ***
#ifdef dunkeltast_all_out09 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED9OFF                  ; .. then LED off
#endif ; dunkeltast_all_out09
#ifdef dunkeltast_out09
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED9OFF                  ; .. then LED off
#endif
        btfss SET_OUTPUT9
         goto LED9OFF                  ; .. then jump
        btfss SET_OUTPUT9_FL
         goto LED9ON                   ; .. then jump

LED9FLASH ; *** LED9 FLASH ***      
FL9_1   btfss R_STATE9,1               ; check R_STATE9 #1
         goto FL9_2
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2
        bcf ALTER_FLASH_S2             ; now other LED may also start flashing...
        goto STATUS_REG_10             ; continue

FL9_2   btfss R_STATE9,2               ; check R_STATE9 #2
         goto FL9_3
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,3                 ; continue R_STATE9#3
        goto STATUS_REG_10             ; continue

FL9_3   btfss R_STATE9,3               ; check R_STATE9 #3
         goto FL9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

FL9_4   btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,1                 ; continue R_STATE9#1
        goto STATUS_REG_10             ; continue

LED9ON  ; *** LED9 ON ***      
ON9_2   btfss R_STATE9,2               ; check R_STATE9 #2
         goto ON9_3
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

ON9_3   btfss R_STATE9,3               ; check R_STATE9 #3
         goto ON9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,4                 ; continue R_STATE9#4
        goto STATUS_REG_10             ; continue

ON9_4   btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,1                 ; continue R_STATE9#1
        goto STATUS_REG_10             ; continue
        
LED9OFF ; *** LED9 OFF ***      
OFF9_1  btfss R_STATE9,1               ; check R_STATE9 #1
         goto OFF9_2
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2
         goto STATUS_REG_10            ; continue
        
OFF9_2  btfss R_STATE9,2               ; check R_STATE9 #2
         goto OFF9_4
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,3                 ; continue R_STATE9#3
        goto STATUS_REG_10             ; continue

OFF9_4  btfss R_STATE9,4               ; check R_STATE9 #4
         goto STATUS_REG_10            ; continue
        clrf R_STATE9                  ; clear R_STATE9  
        bsf R_STATE9,2                 ; continue R_STATE9#2



STATUS_REG_10
        ; *** LED10 (Ausgang 10 ***
#ifdef dunkeltast_all_out10 ; Dunkeltastung ALLE (mit Ausnahmen)
        btfsc DUNKELTAST_ALL           ; Dunkeltastung ALLE
         goto LED10OFF                 ; .. then LED off
#endif ; dunkeltast_all_out10
#ifdef dunkeltast_out10
        btfsc DUNKELTAST_AKT           ; Dunkeltastung
         goto LED10OFF                 ; .. then LED off
#endif
        btfsc ALTER_FLASH_S2           ; if this bit is #1, then no action on this output,
         goto LED10OFF                 ; .. because other LED has first fade-in
        btfss SET_OUTPUT10
         goto LED10OFF                 ; .. then jump
        btfss SET_OUTPUT10_FL
         goto LED10ON                  ; .. then jump

LED10FLASH ; *** LED10 FLASH ***      
FL10_1  btfss R_STATE10,1              ; check R_STATE10 #1
         goto FL10_2
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2
        goto STATUS_REG_11             ; continue

FL10_2  btfss R_STATE10,2              ; check R_STATE10 #2
         goto FL10_3
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,3                ; continue R_STATE10#3
        goto STATUS_REG_11             ; continue

FL10_3  btfss R_STATE10,3              ; check R_STATE10 #3
         goto FL10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

FL10_4  btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                  ; clear R_STATE10  
        bsf R_STATE10,1                ; continue R_STATE10#1
        goto STATUS_REG_11             ; continue

LED10ON ; *** LED10 ON ***      
ON10_2  btfss R_STATE10,2              ; check R_STATE10 #2
         goto ON10_3
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

ON10_3  btfss R_STATE10,3              ; check R_STATE10 #3
         goto ON10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,4                ; continue R_STATE10#4
        goto STATUS_REG_11             ; continue

ON10_4  btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,1                ; continue R_STATE10#1
        goto STATUS_REG_11             ; continue
        
LED10OFF; *** LED10 OFF ***      
OFF10_1 btfss R_STATE10,1              ; check R_STATE10 #1
         goto OFF10_2
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2
         goto STATUS_REG_11            ; continue
        
OFF10_2 btfss R_STATE10,2              ; check R_STATE10 #2
         goto OFF10_4
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,3                ; continue R_STATE10#3
        goto STATUS_REG_11             ; continue

OFF10_4 btfss R_STATE10,4              ; check R_STATE10 #4
         goto STATUS_REG_11            ; continue
        clrf R_STATE10                 ; clear R_STATE10  
        bsf R_STATE10,2                ; continue R_STATE10#2



STATUS_REG_11
        ; --- Status speichern ? ---
        call save_status               ; Status speichern, hier ist gerade genug Zeit dafr 

        ; ***********************************************************************
        ; ****** bername von PRE zu AKT (realisiert Dunkelphase) ***************
        ; ***********************************************************************
        ; der LED-Status wird beim Adress-Empfang zunchst in die Register R_STATUS_ON_Sy_pre geschrieben, 
        ; aber im R_STATUS_FL_Sy_akt Register werden die LEDs bereits abgeschaltet, die verndert werden. Dadurch ergibt sich
        ; erst ein STATUS-Takt spter erst das Einblenden der neuen aktiven LEDs und damit die Dunkelphase.
        ; hier nun wird der PRE Status zum AKT Status. Beim nchsten STATUS-Takt werden demnach die
        ; neuen aktiven LEDs eingeblendet, whrend die abgeschalteten LEDs bereits dunkel sind. 
        ; INPUT:  R_STATUS_ON_Sy_pre und R_STATUS_FL_Sy_pre (y = Signal-Nummer 1/2)
        ; OUTPUT: R_STATUS_ON_Sy_akt und R_STATUS_FL_Sy_akt (y = Signal-Nummer 1/2)
        ; ***********************************************************************
        movf R_STATUS_ON_S1_pre,W      ; copy R_STATUS_ON_S1_pre into R_STATUS_ON_S1_akt
        movwf R_STATUS_ON_S1_akt       ; (fuer Dunkelphase)
        movf R_STATUS_ON_S2_pre,W      ; copy R_STATUS_ON_S2_pre into R_STATUS_ON_S2_akt
        movwf R_STATUS_ON_S2_akt       ; (fuer Dunkelphase)

        btfsc DUNKELTAST_PRE           ; hierdurch faded nach Dunkeltastung S2
         bsf DUNKELTAST_AKT            ; .. zur selben Zeit wie S1
        btfss DUNKELTAST_PRE           ; .. ein (nach Dunkelphase)
         bcf DUNKELTAST_AKT            ; .. 

#ifdef use_all_dark
        btfsc DUNKELTAST_ALL_PRE       ; hierdurch faded nach Dunkeltastung S2
         bsf DUNKELTAST_ALL            ; .. zur selben Zeit wie S1
        btfss DUNKELTAST_ALL_PRE       ; .. ein (nach Dunkelphase)
         bcf DUNKELTAST_ALL            ; .. 
#endif ; use_all_dark

signal2_1 
        ; *********************************************
        ; **************** LED steuern ***************
        ; *********************************************
        ; ab hier werden die eigentlichen LEDs angesteuert
        ; die Ausgnge werden entsprchend der Status-Bits in R_STATE mit der passenden PWM angesteuert
        ; INPUT:  R_STATExx (xx = Ausgang-Nummer)
        ; OUTPUT: LED_xx    (xx = Ausgang-Nummer)
        ; *********************************************

        movf R_COUNT,W                 ; get R_COUNT
        movwf R_PWMVALUE               ; .. and put into PWMVALUE

        ;***** PWM-Time ON *****
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE1,W               ; = off or fade out LED1
        btfss STATUS,Z
         bcf LED_01                    ; LED_1 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE2,W               ; = off or fade out LED2
        btfss STATUS,Z
         bcf LED_02                    ; LED_2 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE3,W               ; = off or fade out LED3
        btfss STATUS,Z
         bcf LED_03                    ; LED_3 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE4,W               ; = off or fade out LED4
        btfss STATUS,Z
         bcf LED_04                    ; LED_4 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE5,W               ; = off or fade out LED5
        btfss STATUS,Z
         bcf LED_05                    ; LED_5 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE6,W               ; = off or fade out LED6
        btfss STATUS,Z
         bcf LED_06                    ; LED_6 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE7,W               ; = off or fade out LED7
        btfss STATUS,Z
         bcf LED_07                    ; LED_7 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE8,W               ; = off or fade out LED8
        btfss STATUS,Z
         bcf LED_08                    ; LED_8 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE9,W               ; = off or fade out LED9
        btfss STATUS,Z
         bcf LED_09                    ; LED_9 off
        movlw b'00001100'              ; check status #2,#3
        andwf R_STATE10,W              ; = off or fade out LED10
        btfss STATUS,Z
         bcf LED_10                    ; LED_10 off

        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfsc STATUS,C                 ; if C=1, -> TMR1L > PWM-Value
        goto signal4_2                 ; ... then JUMP

        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE1,W               ; = fade in/on LED1
        btfss STATUS,Z
         bsf LED_01                    ; LED_1 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE2,W               ; = fade in/on LED2
        btfss STATUS,Z
         bsf LED_02                    ; LED_2 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE3,W               ; = fade in/on LED3
        btfss STATUS,Z
         bsf LED_03                    ; LED_3 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE4,W               ; = fade in/on LED4
        btfss STATUS,Z
         bsf LED_04                    ; LED_4 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE5,W               ; = fade in/on LED5
        btfss STATUS,Z
         bsf LED_05                    ; LED_5 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE6,W               ; = fade in/on LED6
        btfss STATUS,Z
         bsf LED_06                    ; LED_6 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE7,W               ; = fade in/on LED7
        btfss STATUS,Z
         bsf LED_07                    ; LED_7 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE8,W               ; = fade in/on LED8
        btfss STATUS,Z
         bsf LED_08                    ; LED_8 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE9,W               ; = fade in/on LED9
        btfss STATUS,Z
         bsf LED_09                    ; LED_9 on
        movlw b'00010010'              ; check status #1,#4
        andwf R_STATE10,W              ; = fade in/on LED10
        btfss STATUS,Z
         bsf LED_10                    ; LED_9 on

        movlw b'11111111'              ; if R_COUNT
        subwf R_COUNT,W                ; .. = 255
        btfsc STATUS,Z
         return                        ; .. then return

signal4_2 ; ** warten auf PWM Off-Time **
        movf TMR1H,F 
        btfss STATUS,Z                 ; if TMR1H <> ZERO
         return                        ; .. then return

        call polarity                  ; check polarity
        movf R_PWMVALUE,W              ; get PWM-Value into W-Register
        subwf TMR1L,W                  ; W = TMR1L - W
        btfss STATUS,C                 ; if C=0, -> TMR1L < PWM-Value
         goto signal4_2                ; ... then JUMP_2

        ; ***** PWM-Time ready *****
        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE1,W               ; = off or fade in LED1
        btfss STATUS,Z
         bcf LED_01                    ; LED_1 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE1,W               ; = on or fade out LED1
        btfss STATUS,Z
         bsf LED_01                    ; LED_1 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE2,W               ; = off or fade in LED2
        btfss STATUS,Z
         bcf LED_02                    ; LED_2 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE2,W               ; = on or fade out LED2
        btfss STATUS,Z
         bsf LED_02                    ; LED_2 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE3,W               ; = off or fade in LED3
        btfss STATUS,Z
         bcf LED_03                    ; LED_3 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE3,W               ; = on or fade out LED3
        btfss STATUS,Z
         bsf LED_03                    ; LED_3 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE4,W               ; = off or fade in LED4
        btfss STATUS,Z
         bcf LED_04                    ; LED_4 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE4,W               ; = on or fade out LED4
        btfss STATUS,Z
         bsf LED_04                    ; LED_4 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE5,W               ; = off or fade in LED5
        btfss STATUS,Z
         bcf LED_05                    ; LED_5 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE5,W               ; = on or fade out LED5
        btfss STATUS,Z
         bsf LED_05                    ; LED_5 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE6,W               ; = off or fade in LED6
        btfss STATUS,Z
         bcf LED_06                    ; LED_6 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE6,W               ; = on or fade out LED6
        btfss STATUS,Z
         bsf LED_06                    ; LED_6 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE7,W               ; = off or fade in LED7
        btfss STATUS,Z
         bcf LED_07                    ; LED_7 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE7,W               ; = on or fade out LED7
        btfss STATUS,Z
         bsf LED_07                    ; LED_7 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE8,W               ; = off or fade in LED8
        btfss STATUS,Z
         bcf LED_08                    ; LED_8 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE8,W               ; = on or fade out LED8
        btfss STATUS,Z
         bsf LED_08                    ; LED_8 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE9,W               ; = off or fade in LED9
        btfss STATUS,Z
         bcf LED_09                    ; LED_9 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE9,W               ; = on or fade out LED9
        btfss STATUS,Z
         bsf LED_09                    ; LED_9 on

        movlw b'00011000'              ; check status #3,#4
        andwf R_STATE10,W              ; = off or fade in LED9
        btfss STATUS,Z
         bcf LED_10                    ; LED_10 off
        movlw b'00000110'              ; check status #1,#2
        andwf R_STATE10,W              ; = on or fade out LED9
        btfss STATUS,Z
         bsf LED_10                    ; LED_10 on

        return                         ; .. then return
