; ********************************************************
; ** Name:    dekoder.asm                               **
; ** Version: V3.76 vom 18/10/2022                      **
version equ h'376'
; ** Project:                                           **
; **                                                    **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; ** Prozessor: 12F629/12F675 / 16F630/16F676 / 16F684  **
; **            1 cycle = 1 us (4 MHz internal Osc)     **
; **                                                    **
; ** IR: RB0 for Input-Signal                           **
; **                                                    **
; ** Timer: 0 for detection of motorola idle-state      **
; **        1 for Timing (Time-Out, SAVE @ DCC)         **
; ********************************************************
; Copyright (C) 2003/04/05/06/07/08/09/10/11/12/13/14/15/16/19/20 Sven Brandt

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License version 2
; as published by the Free Software Foundation.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

#ifndef makefile
; ********************************************************
;#define motorola
#define dcc
;#define nodecoder

;** Variante **
;#define f629_spur1        ; Ansteuerung von einer Weiche mit WeichZwei, Parallelschaltung
;#define f629_dreiwege     ; Ansteuerung von einer 3-Wege Weiche mit WeichZwei
;#define f629_weichei
;#define f629_andreas
;#define f629_weichzwei
;#define f629_schalten        ; 3 Modi 
;#define f629_signal          ; 2x zweibegriffige Signale, optionale Dunkeltastung
;#define f629_signal4         ; 3/4 begriffige Signale (jeder Befehl wirkt auf einen Ausgang)
;#define f629_signal_hl       ; HL-Signale (3 LEDs inkl. Blinken 2er LEDs)
;#define f629_signal_straba   ; Strassenbahn-Signal (wie Ampel ohne Rot/Gelb, zustzliches Kennlicht)
;#define f629_wz_blinkmuster  ; WeichZwei mit konfigurierbaren Blinkmustern
;#define f629_autofuse        ; wie die selbe SW wie WeichEi, aber bei Start GP0 schalten

;#define f684_weiche88
;#define f684_relais8
;#define f684_led_andreas     ; Schranke
;#define f684_led_schalten
;#define f684_led_signal2      ; Signal 5x2
;#define f684_led_signal4
;#define f684_led_signal_sbb
;#define f684_led_blink
;#define f684_led_kirmes
;#define f684_led_gas
;#define f684_led_neon
;#define f684_led_haus
;#define f684_led_feuer
;#define f684_led_ampel
;#define f684_led_blinkmuster   ; LED-Dekoder mit konfigurierbaren Blinkmustern
;#define f684_led_schiebebuhne   ; LED Dekoder fr Schiebebhne (4x Blocksignal, 1x Schalten, 1x Blinklicht)  

;#define f684_led_signal_001    ; 
;#define f684_led_signal_002    ; 
;#define f684_led_signal_003    ; 
;#define f684_led_signal_004    ; 
;#define f684_led_signal_005    ; 
;#define f684_led_signal_006    ; -> ersetzt durch Signal_041
;#define f684_led_signal_007    ; 
;#define f684_led_signal_008    ; 
;#define f684_led_signal_009    ;
;#define f684_led_signal_010    ; SNCF
;#define f684_led_signal_011    ; NS 
;#define f684_led_signal_012    ; Ausfahrsignal mit Kennlicht fr Folgesignal
;#define f684_led_signal_020    ; DB Ausfahrsignal Zs3l
;#define f684_led_signal_021    ; DB Vorsignal Zs3v
;#define f684_led_signal_022    ; Belgien
;#define f684_led_signal_030    ; Skandinavien
;#define f684_led_signal_031    ; Skandinavien
;#define f684_led_signal_032    ; Skandinavien
;#define f684_led_signal_033    ; Skandinavien
;#define f684_led_signal_034    ; Skandinavien
;#define f684_led_signal_040    ; Osteuropa
;#define f684_led_signal_041    ; Osteuropa PKP Polen (ex Signal_006)
#define f684_led_signal_042    ; Osteuropa CSD/CD Slowakai / Tschechien
;#define f684_led_signal_050    ; BB HS+VS
;#define f684_led_signal_051    ; BB VS+VS
;#define f684_led_signal_052    ; BB 2x HS
;#define f684_led_signal_060    ; Neuseeland

;#define f684_sand_ei_1     ; Servo Einzel-Dekoder V1, Impulse invertiert
;#define f684_sand_ei_2     ; Servo Einzel-Dekoder V2, Impulse nicht mehr invertiert
;#define f684_sand_4        ; Servo 4er-Dekoder

; ********************************************************
; ****** fr DEBUG only                        ***********
; ********************************************************
;#define simulation    ; for simulation: SKIP OSCCAL einlesen etc.
;#define demo_mode     ; DemoMode: abrufen aller Adressen automatisch nacheinander
;#define test          ; Testroutinen einschalten - normale Outputs abschalten
;#define test_check    ; Test Befehlsempfang
;#define test_output               ; Test Logik Analyzer Outputs
;#define test_isr      ; Test Interrupt ISR
;#define test_befehlserkennung     ; Test Befehlsempfang                     (DV100: 14pol_cv_Befehlserkennung
;#define test_ablauf   ; Test 
;#define test_oscilator ; Impulse an A0/A1
#define use_mplab        ; MPLAB Pfade

#endif ; makefile

; ********************************************************
; History:
; ********************************************************
; <editor-fold defaultstate="collapsed" desc="History">
; V3.76 - 18.10.2022 Signal Signal_041 / Signal_042 hinzu
; V3.75 - 07.02.2022 led_schiebebuhne hinzu
; V3.74 - 20.03.2020 Signal_060 hinzu
;       - 25.03.2020 Timing ISR-Motorola fr PICs 16f684 mit fehlenden CAL Wert Optimiert / T_FMAX von 110 (war 100)
;       - 17.06.2020 Signal 407: Blinkmuster #14 korrigiert
;       - 20.07.2020 Signal 405: Blinkmuster #2 korrigiert
; V3.73 - 04.01.2020 Korrekturen in state_signal_v2: Probleme mit BLINKEN @ "aufteilung_output_all" und "use_flash_syncro" behoben, Folding hinzu
;                    SandEi: Version #1 und #2 haben unterschiedliche Manipu-IDs bekommen
; V3.72 - 28.11.2019 WeichEi: Restore nach Learning durch lschen von R_ONOFF1_OLD in STATUS_TO_OUTPUT 
; V3.72 - 28.11.2019 Signal_003 - Korrektur Learning und Signalbilder
; V3.72 - 28.11.2019 Signal_052 - hinzu
; V3.71 - 19.09.2019 Autofuse: nur 1 Adresse verwendet, Mode-Umschaltung entfernt
;                    Signal 050/051 hinzu
;                    Korrekturen in state_signal_v2: Mode aus EEprom lesen (Problem Dunkeltastung), Probleme mit BLINKEN behoben
;                    Signal: EEprom Werte in Defines, insbes. R_STATUS/R_STATUS2, sodass Signal-Status korrekt auf Startbedingung gesetzt werden kann (d.h. Signal startet immer mit Begriff #1)
;                    PIC16f630-Varianten (diverse LED-Varianten, Relais, Weich88) werden ab jetzt nicht mehr kompiliert, aus BAT File entfernt
;                    Sand-Ei in Version #1 und #2 aufgeteilt (neue Variante ohne invertierten Impuls-Ausgang, Transistor entfllt)
;                    Sand-Ei: Neue Adress-Zuordung (je R/G fr ein Servo), Blinken bei Befehls-Empfang langsamer, Blinken beim Lernen schneller, Korrektur Kurve 3
;                    Sand4:   Neue Adress-Zuordung (je R/G fr ein Servo), Blinken bei Befehls-Empfang langsamer, Blinken beim Lernen schneller
;                    Folding-Kommentare fr MPLaB hinzu
;                    externe Files fr Register-Konfigurationen "conf_xxx"
; V3.70 - 03.04.2019 weichZwei SCHALTEN: Konfigurationen fr Relais-1 Dekoder entfernt
;                    diverse: bei SAVE wird immer "save_toggle_start" aufgerufen, um dort ggf. ein START Blinkmuster zu laden. Dies ermglicht schnelleres Blinken,
;                     wenn Signalbild 1 / Status 1 / Ausgang 1 aktiv war vor dem Lernen (R_STATUS,0 gesetzt)
; V3.69 - 06.12.2018 Weich88: Lernen: Blinken LED schneller, ABschalten der Antriebe nach MAKESLOW Wert (Timergesteuert), LED geht nur bei gltiger Adresse auf
;                    WeichEi / WeichZwei: Lernen: ABschalten der Antriebe nach MAKESLOW Wert (Timergesteuert)
;                    change in allen HEX, da SAVE Routine umgestellt (2x SUB Routine NEU)
; V3.68 - 29.11.2018 Compiler mit Linker in BATCH File
; V3.67 - 27.11.2018 Adresslerning Wechselblinken bei Haus / Gas / Neon / Feuer (savetoggle_yes gesetzt) 
; V3.66 - 03.03.2018 led_schalten @ Mode: einlesen R_MAKESLOW wurde durch RETURN nicht durchlaufen -> immer nur kurze Impulse
;                    alle DCC: Pausenerkennung hinzu, sonst wurden falsche Adressen angelernt, wenn eine MM Lok aktiv ist (insbes. ECoS) 
;         10.08.2018 PKP (Signal_006): Blinkmuster SAVE korrigiert
; V3.65 - 13.04.2017 led_haus - Shuffle-Funktion auch im IDLE, Laden und schreiben eines Shuffle Wertes aus dem EE 0x70
;                    led_neon - Laden und schreiben eines Shuffle Wertes aus dem EE 0x70
;                    led_gas - Preload "R_VERZx", Korrektur Mode-Umschaltung @ Nodecoder (Einschaltblitz kam nicht)
;                    EE-Daten @ NODECODER: Zelle $00 von "FF" auf "01" (blockierte die Mode-Umschaltung z.B. bei led_gas)
; V3.64 - 29.03.2017 Mode-Umschaltung wartet auf Button-Release & Wartezeit @ PWM (wg. Programm-Problem PicKit2)
;                    led_neon/led_haus - Nodecoder: Korrektur "status_to_output" -> Verarbeiten, wenn Button @ Start gedrckt (Effekt AUS)
; V3.63 - 28.09.2015 Demomode hinzu, led_signal_012 hinzu
; v3.62 - 16.03.2015 led_haus: Langsamer gemacht
; V3.61 - 04.03.2015 led_blinkmuster/NODECODER: Korrektur zu 'change_status_on' und 'change_status_off'
;                    led_blinkmuster: Warteschleife nach LOOP Durchgang fr Kurve 1 ermglicht, Wert aus Zeile 105 (KurvenParameter)
;                    led_signal2: Korrektur Status wieder herstellen (Signal 5 wurde nicht korrekt wieder restored)
;                    led_ampel: Zeiten aus dem EEprom
;                    Signal_009: 4 Adressen (2 je Signal fr Vr1+Zs3v und Vr2 ohne Zs3v) hinzugefgt
;                    Sand-Ei/Sand-4: faktor_min auf 135 (war 160), dadurch ist der minimale Verstellbreich per Justage kleiner (minimales Delta zw. Links und Rechts war 264us, jetzt 48us Impulsbreite)
;                    led_haus: Ausgnge werden jetzt weich ein- und ausgeblendet, verbesserter Zufalls-Generator
;                    WeichEi: Weiches Ein- und Ausblenden der Schalt-Ausgnge
; V3.60 - 05.03.2014 led_schalten, relais8, led_kirmes: Laden von MAKESLOW (ex #define eepromslow_yes) in 'init_dekoder' verschoben, #define eepromslow_yes gelscht
;                    wz_schalten: Laden von MAKESLOW in 'init_dekoder' korrigiert
;                    Signal_009 hinzu
;                    NODECODER: led_kirmes, led_blink, led_blinkmuster, led_ampel: Korrektur zu 'change_status_on' und 'change_status_off'
;                    NODECODER: EEprom $20 war 0x10, korrigiert auf 0x01 (= MAKESLOW)-> Auswirkungen auf led_ampel, led_kirmes, led_haus 
;                    led_blinkmuster: Ablaufzeit wesentlich verlngert (&quot;Stretch-Faktor&quot; kann jetzt bis zu einer Raster-Zeit von 600 sec. eingestellt werden)
;                    weichei, spur1, 3-wege, weich88: Korrektur in Routine "Abschalten": "return" statt "goto loop"
; V3.59 - 26.08.2013 Signal_034 (Schweden): syncrones Blinken
;         27.08.2013 Signal _040 hinzu, aber noch keine Funktion (war gedacht fr Osteuropa)
;         27.08.2013 neue State-Machine "state_signal_v2": optimiert, enthlt alle SUB-Routinen, sodass in "sub_led_signal_xxx" nur noch DEFINES liegen
;         27.08.2013 led_signal_sbb: Augang 4/5 ber die Adresse 9/10 schaltbar
; V3.58 - 16.03.2013 Sand-Ei: Korrektur Taster-Eingnge
;         05.05.2013 WeichEi / WeichZwei / Spur1 / Weich88: Abschalten durch OFF und TIMEOUT kann abgewhlt werden: 
;                    (Wert "*a" in $00 --> kein Abschalten durch TimeOut),(Wert "a*" in $00 --> kein Abschalten durch OFF-Befehl)
;                    
; V3.57 - 04.03.2013 Signal_020 Adresse 9/10 hinzu (fr Mark)
;                    led_weiche - Untervariante "autofuse" hinzu)
; V3.56 - 07.11.2012 Signal_022 (Belgien) hinzu
;                    led_andreas, wz_schalten: Korrektur DUMM-Mode (kein Laden von R_STATUS)
;                    wz_schalten: Laden von MAKESLOW (ex #define eepromslow_yes) in 'init_dekoder' verschoben
; V3.55 - 22.10.2012 alle Dekoder mit 'change_mode_at_start_yes': R_MODE#7 -> Mode-Umschaltung sperren
;                    ( signal, schalten, relais8, led_schalten, led_signal4, led_signal_sbb, led_ampel, led_andreas,
;                     led_signal_001, led_signal_002, led_signal_003, led_signal_005, led_signal_006, led_signal_007)
;                    (SandEi, Sand_4 Bit#7 wird nicht von MANIPULATOR gesetzt, da keine MODE-Umschaltung sondern Justage!)  
;                      
; V3.54 - 27.03.2012 SandEi: X7 kann per Schwellwert geschaltet werde
; V3.53 - 25.01.2012 signal_hl: Abfangen in 'status_to_output', wenn R_STATUS = ZERO / EEprom-Spezifische Register laden in 'init_dekoder' verschoben
;                    signal_021: bei Dunkeltastung werden jetzt alle Ausgnge (auch Kennlicht!) abgeschaltet
;                    blinkmuster / led_blinkmuster: NoDekoder hinzu, EEprom-Spezifische Register laden in 'init_dekoder' verschoben
; V3.52 - SandEi: Tastatur-Eingge auch bei Speisung mit Digitalo-Befehlen (d.h.Impulse am Input) ermglicht
; V3.51 - weichEi / WeichZwei / Spur1 / Weiche88: Abschalt-Befehle knnen ignoriert werden, EEprom-Spezifische Register laden in 'init_dekoder' verschoben
;       - dreiwege: EEprom-Spezifische Register laden in 'init_dekoder' verschoben
;       - LED_HAUS: EEprom-Spezifische Register laden in 'init_dekoder' verschoben
;       - led_signal_020: EEprom-Spezifische Register laden in 'init_dekoder' verschoben
;       - led_signal_021: EEprom-Spezifische Register laden in 'init_dekoder' verschoben
; V3.50 - led_signal_020 / led_signal_021 hinzu
; V3.49 - Sand-4: Relais sinnvoll angesteuert (d.h. Anpassung der vordefinierten Kurven)
; V3.48 - LED_Neon: Ausgnge konfigurierbar, in NODECODER Starten und SToppen per Taster
;       - LED_gas:     in NODECODER Starten und Stoppen per Taster                  , fr DCC/MM durch skip_g2 ff. Platzbedarf reduziert
;       - LED_feuer:   in NODECODER Starten und Stoppen per Taster                  , fr DCC/MM durch skip_g2 ff. Platzbedarf reduziert
;       - LED_haus:    in NODECODER Starten und Stoppen per Taster                  , fr DCC/MM durch skip_g2 ff. Platzbedarf reduziert
;       - LED_kirmes:  in NODECODER 'change_status_on' und 'change_status_off' hinzu, fr DCC/MM durch skip_g5 ff. Platzbedarf reduziert
;       - LED_blink:   in NODECODER 'change_status_on' und 'change_status_off' hinzu
;       - LED_andreas: in NODECODER 'change_status_on' und 'change_status_off' hinzu
;       - LED_ampel:   in NODECODER 'change_status_on' und 'change_status_off' hinzu
;       - andreas:     fr DCC/MM durch skip_g3 ff. Platzbedarf reduziert
; V3.47 - sand-ei: wh. der letzten Impulse aus der Tabelle, damit Servo wirklich in die Endlage kommt.
;       - LED_Signal_011 (NS) hinzu (nedeutet: jetzt bis zu 16 Adressen)
; V3.46 - sand_ei: predefined kurves undefiniert
; V3.45 - LED_Signal_010 hinzu
; V3.44 - sand_ei: korrekturen Taster-Eingnge
; V3.43 - sand_ei: hinzu
; V3.42 - led_ampel: - 2 Modi (deutsch / oesterreich), Korrekturen
; V3.41 - led_ampel: - 2 Modi (deutsch / oesterreich)
;       - led_signal002: - Blinken der Ausgnge beim Lernen korrigiert
; V3.40 - led_ampel: - Fading einnstellbar (Wert in EE-Register $20)
; V3.39 - wz_schalten: - Signalisierung MODE-Umschaltung nur noch mit PortA,1 wg. Problem mit EKGs (Sicherung springt)
;       - LED_Signal_007: - bei Adresse 14 Ausgang 9 eingeschaltet 
;       - LED_Signal_003: - 2 Modi -> unterschiedliche Signalbilder
;                         - berflssige Adressen -> Skip
;                         - debug-Routine "show_testtemp" weg
;       - wz_3wege / wz_spur1 / Weichei / weiche88 - berflssige Adressen -> Skip 
;       - LED_Signal_008: - Korrekturn Signalbild S2
;                         - debug-Routine "show_testtemp" weg
; V3.38 - Sand-4: - Nach Ablauf einer Totzeit wird der erste Impuls 'number_first_imp' mal wiederholt
;       - Sand-4: - lngere Totzeit beim Restore (delay_v_i war '40' jetzt '50')
;       - LED_neon: - Ausgang 10 "Schweisslicht" und 9 "defekt" auch als normale Neon-Ausgnge nutzbar (Flags in EE $20)
;       - WeichZwei: - Time-Out f. Ausgnge unabhngig (wie Relais8)
;       - WeichEi:   - Abschalt-Routine in sub....asm verschoben
;       - Weiche88:  - Abschalt-Routine in sub....asm verschoben
;       - Spur1:     - Abschalt-Routine in sub....asm verschoben
;       - dreiwege:  - Abschalt-Routine in sub....asm verschoben
; V3.37 - Konfiguration ANSEL korrigiert, sodass nutzbar mit 12F675
;       - LED_Signal_001: mehr Adressen fr Vorsignale, ID f. Manipu gendert
;       - LED_Signal_002: mehr Adressen fr Vorsignale, ID f. Manipu gendert
; V3.36 - LED_Signal_007: hinzu
;       - LED_Signal_008: hinzu
; V3.35 - Sand-4: - Fehlerbeseitigung
; V3.34 - Sand-4: - Taster-Erkennung hinzu
; V3.33 - Sand-4: - ROM-Einsparung
;                 - Vorbereitungen zu Taster-Erkennung
;                 - Verzgerung Voltage-Impulse uber Konstante definierbar
;       - Variante "led_signal_006" hinzu
; V3.32 - Sand-4: - Korreturen beim Aufruf (value preload) von "stop" / "break"
;                 - Totzeit zwischen Einschalten Vcc und erstem Inpuls hinzu ("wait"-Flag)
;                 - IR sperren beim Timer lschen fr Inpulse-Generierung & EEPROM-Lesen
; V3.31 - Sand-4: "doppelt_no" f. Sand-4 erlaubt ("#define doppelt_no" weg)
; V3.30 - Korrekturen Sand-4
; V3.29 - Korrekturen Sand-4
; V3.28 - Korrekturen Sand-4
; V3.27 - Erweiterungen zum "SAND"
;       - "LED_Blinkmuster" hinzu
; V3.26 - LED_Signal_005: Out 7/8 sind VS zu Sig. 1, Dunkeltastung hinzu
;       - nderung in der State-Machine "state_signal", jetzt sind alle Outputs auch als Blink konfigurierbar
;         --> dadurch nderung der HEX-File Signal_001...Signal_005
;       - LED_andreas: 'Nachblinken' hinzu, jetzt mit 2 Modi
;       - neues #define: 'restore_in_mode1_only'. Bisher wurde 'doppelt_mode3_yes' dafr missbraucht 
;       - IDs for Maiulator added
;       - HEX-Files fr LED-Dekoder auch fr 16f684 kompiliert (nderungen hierfr nur im MAKE File)
; V3.25 - Sand-Ei hinzu
;       - led_haus: "Gewichtung" der Ausgnge eingefhrt, EEprom-Daten entsprechend angepasst
;       - WeichZwei: "blinkmuster" hinzu
;       - "led_signal_003": Blinkmuster korrigiert. Jetzt 12 Adressen (war 14) 
; V3.24 - LED_Kirmes:  #define eepromslow_yes hinzu (Speed konfigurierbar)
; V3.23 - WeichZwei: Start mit Tri-State (wurde erst nach 255 x 0.5s aufgerufen, PreLoad von R_VERZ)
; V3.22 - neue Signal-Varianten mit 16f684
;       - EEPROM Daten extern ber #include
;       - neues  #define 'use_status2', wenn > 8 Adressen
;       - bis zu 14 Adressen
;       - nderung in der EEPROM Lese-Routine @ Check (2x 2 Befehle umgestellt) -> nderung aller HEX
; V3.21 - WeichZwei "signal_straba" hinzu
; V3.20 - WeichZwei "signal_hl" hinzu 
; V3.19 - LED_signal_2 hinzu, 
;       - neue #defines 'skip_r10' ermglichen ROM sparen. HEX der verschiedenen Dekoder
;         sollten aber unverndert sein
; V3.18 - DCC-Dekoder: ReilCom NOP Befehl setzt Time-Out nicht mehr zurck
; V3.17 - DCC-Dekoder: Verwerfen des RailCom NOP Befehls in der Save Routine
;       - ZweiPol: Abschalt-Verhalten verndert, es kam tw. zu "Phantom-Schaltungen" bei schnellen Befehls-Seq.
; V3.16 - Relais8 -> 4. Mode hinzu (Single Output)
;                 -> 9. Address (all off / on) added
; V3.15 - LED-Dekoder: haus -> neue State Machine (Mindest-Anzahl, langsames Stck-fr-Stck abschalten)
;       - nderungen im EEPROM ($2F @ led_haus, $7F -> Digital-Format info)
;       - LED-Dekoder: andreas -> Fehler in State Machine beseitigt (schnelleres Abschalten)
; V3.14 - LED-Dekoder: andreas -> Fehler in State Machine beseitigt (war tw. Problem bei Abschalten)
; V3.13 - LED-Dekoder: schalten -> 10 Adressen
; V3.12 - new ISR_MOT.ASM (problem with DDW @ Motorola, wrong timing there, support from Siegfried Grob)
; V3.11 - WeichZwei: Tristate fr ZweiPol (Stromaufnahme senken)
;       - wz_signal: Tristate @ Dunkeltastung fr ZweiPol (Stromaufnahme senken)
; V3.10 - LED_signal_sbb hinzu
;       - LED_andreas: adress-learning LED-flashing mode changed (nicht mehr uber SAVEFLASH = Blinkanzahl)
; V3.09 - LED_signal4: vor allen Signalbildern jetzt Dunkelphase
;       - Speicheroptimierung
;       - Neu-Org flags SAVEFLASH and SAVEFLASHALL
;       - address learning -> quicker starting (change of starting conditions)
; V3.08 - LED-Dekoder_blink: bei NODECODER Fehler beim Weiterschalten behoben
; V3.07 - LED-Dekoder_gas: jetzt mit Gasdruckschwankungen (nderungen nur in _SUB und _STATS)
; V3.06 - Weiche88: Signalisierung @Adress-Learning auch ber Blink-Anzahl, Start-Outputs @ learning alligned
;       - LED-Dekoder_blink: 7. Blink-Mode hinzu (Foto)
; V3.05 - Relais 8 erhlt konfigurierbare Einschaltverzgerung in Mode 1/2
; V3.04 - LED-Dekoder_schalten / Relais hat beim Programmieren nicht korrekt signalisiert
; V3.03 - fr Relais-1 dekoder wird "schalten" verwendet, jedochj nur 1 Adresse lernen
;         (Relais 1 hat GPIO5 = Pin2 auf HIGH)
; V3.02 - add configuration for ANSEL (make code compatible to 16F676)
;         (means also change of *.inc file to 16F676)
;       - led_schalten jetzt mit 3 modes
; V3.01 - Relais8: Abschalten von PORTA,1 (extra LED) nach 0.5 sec
;       - WeichEi: Ausgnge schneller togglen bei Lernen der 2. Adresse
; V3.00 - Verschmelzen von 12F629 und 16F630-Code
; V2.30 - berarbeitung des Code bzgl. #ifdef-Organisation
;       - OFF-Befehl DCC wird verarbeitet
;       - WeichZwei, WeichEi, Spur1, 3wege, schalten (Dekoder mit Timeout): 
;         - Time-Out Variable liegt im EEProm
;         - TMR1IF-Flag lschen in CHECK 
;       - langsameres Togglen bei Weichen-Dekodern
; V2.24 - Weiche88: 
;         Time-Out Variable liegt im EEProm
;         bedeutet fr MM: Time-Out, fr DCC: Einschaltverlngerung
;         TMR1IF-Flag lschen in CHECK
;         Ports sind immer OUTPUT
; V2.23 - weiche88: zustzliche Ausschaltverzgerung T_01 / R_COUNT  hinzu
; V2.22 - "gas" fertig gestellt
; V2.21 - bei "haus": Ablaufgeschwindigkeits-Byte im EEPROM
; V2.20 - gas hinzu
; V2.19 - Dekoder-Befehle "3FF" werden werworfen (-> Sonderoption 901)
;       - Hauptprogramm im Speicher -8 Bit (Platzmangel "led_signal"
; V2.18 - andreas fertig gestellt
; V2.17 - ampel bearbeitet
; V2.16 - Signal4: auswhlbarer Mode: mit und ohne Dunkeltastung Vorsignal
; V2.15 - kirmes gendert (neuer Blinkmodus -> init, keine ROTen Tasten)
;       - blink gendert (keine ROTen Tasten, 3ms-Raster in externer Datei)
;       - ampel: Vorarbeiten zu Glhlampen-Effekt
; V2.14 - ampel hinzu, kirmes ergnzt
; V2.13 - einige "equ" ersetzt, haus/neon/feuer/andreas hinzu
; V2.12 - State-MAchines ber #INCLUDE
; V2.11 - Variante "nodecoder" fr "led_blink" hinzu
; V2.10 - DCC hinzu
; V2.09 - KIRMES debugged
; V2.08 - BLINK debugged, WEICHE88 hinzu
; V2.07 - BLINK hinzu, KIRMES hinzu (angefangen); Kommentare hinzu
; V2.06 - OSCCAL wieder rein
; V2.05 - led_signal4 hinzu. CHANGE_MODE nur fr RELAIS8, daher
;         hat sich das HEX fr LED_SCHALTEN gendert.
; V2.04 - led_schalten hinzu
; V2.03 - created from Relais8.asm
; ********************************************************
; </editor-fold>

;** Funktion **
; <editor-fold  defaultstate="collapsed" desc="Funktion">
#ifdef f629_spur1  
#define funktion_spur1
#endif      
#ifdef f629_dreiwege     
#define funktion_dreiwege
#endif      
#ifdef f629_weichei
#define funktion_weichei
#endif      
#ifdef f629_andreas
#define funktion_andreas
#endif      
#ifdef f629_weichzwei
#define funktion_weichzwei
#endif      
#ifdef f629_schalten     
#define funktion_schalten
#endif      
#ifdef f629_signal       
#define funktion_signal
#endif      
#ifdef f629_signal4      
#define funktion_signal4
#endif      
#ifdef f629_signal_hl    
#define funktion_signal_hl
#endif      
#ifdef f629_signal_straba
#define funktion_signal_straba
#endif      
#ifdef f629_wz_blinkmuster
#define funktion_wz_blinkmuster
#endif      
#ifdef f629_autofuse     
 #define funktion_autofuse
 #define funktion_weichei
#endif      

#ifdef f684_weiche88
#define funktion_weiche88
#endif      
#ifdef f684_relais8
#define funktion_relais8
#endif      
#ifdef f684_led_andreas
#define funktion_led_andreas
#endif      
#ifdef f684_led_schalten
#define funktion_led_schalten
#endif      
#ifdef f684_led_signal2
#define funktion_led_signal2
#endif      
#ifdef f684_led_signal4
#define funktion_led_signal4
#endif      
#ifdef f684_led_signal_sbb
#define funktion_led_signal_sbb
#endif      
#ifdef f684_led_blink
#define funktion_led_blink
#endif      
#ifdef f684_led_kirmes
#define funktion_led_kirmes
#endif      
#ifdef f684_led_gas
#define funktion_led_gas
#endif      
#ifdef f684_led_neon
#define funktion_led_neon
#endif      
#ifdef f684_led_haus
#define funktion_led_haus
#endif      
#ifdef f684_led_feuer
#define funktion_led_feuer
#endif      
#ifdef f684_led_ampel
#define funktion_led_ampel
#endif      
#ifdef f684_led_schiebebuhne
#define funktion_led_schiebebuhne
#endif      

#ifdef f684_led_signal_001
#define funktion_led_signal_001
#endif      
#ifdef f684_led_signal_002   
#define funktion_led_signal_002
#endif      
#ifdef f684_led_signal_003   
#define funktion_led_signal_003
#endif      
#ifdef f684_led_signal_004   
#define funktion_led_signal_004
#endif      
#ifdef f684_led_signal_005   
#define funktion_led_signal_005
#endif      
;#ifdef f684_led_signal_006   -> ersetzt durch 041
;#define funktion_led_signal_006
;#endif      
#ifdef f684_led_signal_007   
#define funktion_led_signal_007
#endif      
#ifdef f684_led_signal_008   
#define funktion_led_signal_008
#endif      
#ifdef f684_led_signal_009   
#define funktion_led_signal_009
#endif      
#ifdef f684_led_signal_010  
#define funktion_led_signal_010
#endif      
#ifdef f684_led_signal_011   
#define funktion_led_signal_011
#endif      
#ifdef f684_led_signal_012   
#define funktion_led_signal_012
#endif      
#ifdef f684_led_signal_020   
#define funktion_led_signal_020
#endif      
#ifdef f684_led_signal_021   
#define funktion_led_signal_021
#endif      
#ifdef f684_led_signal_022   
#define funktion_led_signal_022
#endif      
#ifdef f684_led_blinkmuster  
#define funktion_led_blinkmuster
#endif      
#ifdef f684_led_signal_030   
#define funktion_led_signal_030
#endif      
#ifdef f684_led_signal_031   
#define funktion_led_signal_031
#endif      
#ifdef f684_led_signal_032   
#define funktion_led_signal_032
#endif      
#ifdef f684_led_signal_033   
#define funktion_led_signal_033
#endif      
#ifdef f684_led_signal_034   
#define funktion_led_signal_034
#endif      
#ifdef f684_led_signal_040   
#define funktion_led_signal_040
#endif      
#ifdef f684_led_signal_041   
#define funktion_led_signal_041
#endif      
#ifdef f684_led_signal_042   
#define funktion_led_signal_042
#endif      
#ifdef f684_led_signal_050   
#define funktion_led_signal_050
#endif      
#ifdef f684_led_signal_051   
#define funktion_led_signal_051
#endif      
#ifdef f684_led_signal_052   
#define funktion_led_signal_052
#endif      
#ifdef f684_led_signal_060
#define funktion_led_signal_060
#endif      

#ifdef f684_sand_ei_1     
 #define funktion_sand_ei
 #define invert_impulse
#endif      
#ifdef f684_sand_ei_2     
 #define funktion_sand_ei
 #define invert_impulse_no
#endif      
#ifdef f684_sand_4        
 #define funktion_sand_4
#endif      
; </editor-fold>

; ** Prozessor **
; <editor-fold defaultstate="collapsed" desc="Prozessor">
; PIC Type 12f629    ( 8PIN, 1k one Page (0x0-0x3FF), 128 RAM, PORTA,      4MHz)
; PIC Type 16f630/76 (14PIN, 1k one Page (0x0-0x3FF), 128 RAM, PORTA/C,    4MHz)
; PIC Type 16f684    (14PIN, 2k one Page (0x0-0x7FF), 128 RAM, PORTA/C,    4MHz,              TMR0:8Bit, TMR1:16Bit, TMR2:8Bit, PWM via TMR2)
; PIC Type 16f688    (14PIN, 4k one Page (0x0-0xFFF), 256 RAM, PORTA/C,    4MHz, USART-Lissy, TMR0:8Bit, TMR1:16Bit, TMR2:NONE, PWM NONE)
; PIC Type 16F1824   (14PIN, 4k one Page (0x0-0xFFF), 256 RAM, PORTA/C,   32MHz, USART-Lissy, 2x PWM + 2xCCP/PWM = 6x PWM)
; PIC Type 16F18324  (14PIN, 4k Page 0-3 (0x0-0xFFF), 512 RAM, PORTA/C,   32MHz, USART-Lissy, 2x PWM + 4xCCP/PWM = 6x PWM) -> WalD
; PIC Type 16F18344  (20PIN, 4k Page 0-3 (0x0-0xFFF), 512 RAM, PORTA/B/C, 32MHz, USART-Lissy, 2x PWM)
#ifdef f629_spur1  
 #define use16f676
#endif      
#ifdef f629_dreiwege     
 #define use16f676
#endif      
#ifdef f629_weichei
 #define use16f676
#endif      
#ifdef f629_andreas
 #define use16f676
#endif      
#ifdef f629_weichzwei
 #define use16f676
#endif      
#ifdef f629_schalten     
 #define use16f676
#endif      
#ifdef f629_signal       
 #define use16f676
#endif      
#ifdef f629_signal4      
 #define use16f676
#endif      
#ifdef f629_signal_hl    
 #define use16f676
#endif      
#ifdef f629_signal_straba
 #define use16f676
#endif      
#ifdef f629_wz_blinkmuster
 #define use16f676
#endif      
#ifdef f629_autofuse     
 #define use16f676
#endif

#ifdef f684_weiche88
 #define use16f684
#endif      
#ifdef f684_relais8
 #define use16f684
#endif      
#ifdef f684_led_andreas
 #define use16f684
#endif      
#ifdef f684_led_schalten
 #define use16f684
#endif      
#ifdef f684_led_signal2
 #define use16f684
#endif      
#ifdef f684_led_signal4
 #define use16f684
#endif      
#ifdef f684_led_signal_sbb
 #define use16f684
#endif      
#ifdef f684_led_blink
 #define use16f684
#endif      
#ifdef f684_led_kirmes
 #define use16f684
#endif      
#ifdef f684_led_gas
 #define use16f684
#endif      
#ifdef f684_led_neon
 #define use16f684
#endif      
#ifdef f684_led_haus
 #define use16f684
#endif      
#ifdef f684_led_feuer
 #define use16f684
#endif      
#ifdef f684_led_ampel
 #define use16f684
#endif      
#ifdef f684_led_schiebebuhne
 #define use16f684
#endif      

#ifdef f684_led_signal_001
 #define use16f684
#endif      
#ifdef f684_led_signal_002   
 #define use16f684
#endif      
#ifdef f684_led_signal_003   
 #define use16f684
#endif      
#ifdef f684_led_signal_004   
 #define use16f684
#endif      
#ifdef f684_led_signal_005   
 #define use16f684
#endif      
#ifdef f684_led_signal_006   
 #define use16f684
#endif      
#ifdef f684_led_signal_007   
 #define use16f684
#endif      
#ifdef f684_led_signal_008   
 #define use16f684
#endif      
#ifdef f684_led_signal_009   
 #define use16f684
#endif      
#ifdef f684_led_signal_010  
 #define use16f684
#endif      
#ifdef f684_led_signal_011   
 #define use16f684
#endif      
#ifdef f684_led_signal_012   
 #define use16f684
#endif      
#ifdef f684_led_signal_020   
 #define use16f684
#endif      
#ifdef f684_led_signal_021   
 #define use16f684
#endif      
#ifdef f684_led_signal_022   
 #define use16f684
#endif      
#ifdef f684_led_blinkmuster  
 #define use16f684
#endif      
#ifdef f684_led_signal_030   
 #define use16f684
#endif      
#ifdef f684_led_signal_031   
 #define use16f684
#endif      
#ifdef f684_led_signal_032   
 #define use16f684
#endif      
#ifdef f684_led_signal_033   
 #define use16f684
#endif      
#ifdef f684_led_signal_034   
 #define use16f684
#endif      
#ifdef f684_led_signal_040   
 #define use16f684
#endif      
#ifdef f684_led_signal_041   
 #define use16f684
#endif      
#ifdef f684_led_signal_042   
 #define use16f684
#endif      
#ifdef f684_led_signal_050   
 #define use16f684
#endif      
#ifdef f684_led_signal_051   
 #define use16f684
#endif      
#ifdef f684_led_signal_052   
 #define use16f684
#endif      
#ifdef f684_led_signal_060
 #define use16f684
#endif      

#ifdef f684_sand_ei_1     
 #define use16f684
#endif      
#ifdef f684_sand_ei_2     
 #define use16f684
#endif      
#ifdef f684_sand_4        
 #define use16f684
#endif      
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="ERROR Abfragen">
#ifdef use16f684
 #ifdef use16f688
   Error verschiedene Prozessoren 16f684 / 16f688 definiert
 #endif 
 #ifdef use16f1824
   Error verschiedene Prozessoren 16f684 / 16f1824 definiert
 #endif 
 #ifdef use16f18324
   Error verschiedene Prozessoren 16f684 / 16f18324 definiert
 #endif 
 #ifdef use16f18344
   Error verschiedene Prozessoren 16f684 / 16f18344 definiert
 #endif 
#endif 

#ifdef use16f688
 #ifdef use16f684
   Error verschiedene Prozessoren 16f688 / 16f684 definiert
 #endif 
 #ifdef use16f1824
   Error verschiedene Prozessoren 16f688 / 16f1824 definiert
 #endif 
 #ifdef use16f18324
   Error verschiedene Prozessoren 16f688 / 16f18324 definiert
 #endif 
 #ifdef use16f18344
   Error verschiedene Prozessoren 16f688 / 16f18344 definiert
 #endif 
#endif 

#ifdef use16f1824
 #ifdef use16f684
   Error verschiedene Prozessoren 16f1824 / 16f684 definiert
 #endif 
 #ifdef use16f688
   Error verschiedene Prozessoren 16f1824 / 16f688 definiert
 #endif 
 #ifdef use16f18324
   Error verschiedene Prozessoren 16f1824 / 16f18324 definiert
 #endif 
 #ifdef use16f18344
   Error verschiedene Prozessoren 16f1824 / 16f18344 definiert
 #endif 
#endif 

#ifdef use16f18324
 #ifdef use16f684
   Error verschiedene Prozessoren 16f18324 / 16f684 definiert
 #endif 
 #ifdef use16f688
   Error verschiedene Prozessoren 16f18324 / 16f688 definiert
 #endif 
 #ifdef use16f1824
   Error verschiedene Prozessoren 16f18324 / 16f1824 definiert
 #endif 
 #ifdef use16f18344
   Error verschiedene Prozessoren 16f18324 / 16f18344 definiert
 #endif 
#endif 

#ifdef use16f18344
 #ifdef use16f684
   Error verschiedene Prozessoren 16f18344 / 16f684 definiert
 #endif 
 #ifdef use16f688
   Error verschiedene Prozessoren 16f18344 / 16f688 definiert
 #endif 
 #ifdef use16f1824
   Error verschiedene Prozessoren 16f18344 / 16f1824 definiert
 #endif 
 #ifdef use16f18324
   Error verschiedene Prozessoren 16f18344 / 16f18324 definiert
 #endif 
#endif 
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="INCLUDE Files">
#ifdef use12f629
 #include <p12f629.inc>
 LIST p=12f629                       ; Controller
 #define use16f6xx
 #define useOSCCAL
#endif
#ifdef use16f630
 #include <p16f630.inc>
 LIST p=16f630                       ; Controller
 #define use16f6xx
 #define useOSCCAL
#endif
#ifdef use16f676
 #include <p16f676.inc>
 LIST p=16f676                       ; Controller
 #define use16f6xx
 #define useOSCCAL
#endif
#ifdef use16f684
 #include <p16f684.inc>
 LIST p=16f684                       ; Controller
 #define use16f6xx
#endif
#ifdef use16f688
 #include <p16f688.inc>
 LIST p=16f688                       ; Controller
 #define use16f6xx
#endif

#ifdef use16f18324
 #define use16f183x4  ; 16f18324 + 16f18344
 #include <p16f18324.inc>
 LIST p=16f18324                       ; Controller
#endif
#ifdef use16f18344
 #define use16f183x4  ; 16f18324 + 16f18344
 #include <p16f18344.inc>
 LIST p=16f18344                       ; Controller
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines wz_blinkmuster">
#ifdef funktion_wz_blinkmuster
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'f000' | version            ; PIC Device ID
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflashall_yes              ; beim Adress-Learning ber Blinkanzahl aller Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines spur1">
#ifdef funktion_spur1
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'1000' | version            ; PIC Device ID
 #define offcommand_yes                ; check OFF-Command (fr Weichen)
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define save_use_timer1               ; beim Adress-Lernen Timer verwenden (fr Weichen mit Impulsverlngerung MAKESLOW)
; #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'250'                 ; Counter for Toggle
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines signal 5x2">
#ifdef funktion_signal
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'2000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines dreiwege">
#ifdef funktion_dreiwege
CONST_ADRZAHL equ d'3'                 ; Anzahl der Adressen
 __idlocs h'3000' | version            ; PIC Device ID
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'250'                 ; Counter for Toggle
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines signal4">
#ifdef funktion_signal4
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'4000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines signal_hl">
#ifdef funktion_signal_hl
CONST_ADRZAHL equ d'7'                 ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines signal_straba">
#ifdef funktion_signal_straba
CONST_ADRZAHL equ d'2'                 ; Anzahl der Adressen
 __idlocs h'5000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines weichei">
#ifdef funktion_weichei
#ifdef f629_autofuse     
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen Autofuse = 1 (keine LED)
#else    
CONST_ADRZAHL equ d'2'                 ; Anzahl der Adressen WeichEi = 2 (mit LED)
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
#endif      
 __idlocs h'a000' | version            ; PIC Device ID
 ;#define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define offcommand_yes                ; check OFF-Command (fr Weichen)
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define save_use_timer1               ; beim Adress-Lernen Timer verwenden (fr Weichen mit Impulsverlngerung MAKESLOW)
; #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'250'                 ; Counter for Toggle
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines andreas">
#ifdef funktion_andreas
CONST_ADRZAHL equ d'2'                 ; Anzahl der Adressen
 __idlocs h'b000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'050'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines weichzwei">
#ifdef funktion_weichzwei
CONST_ADRZAHL equ d'2'                 ; Anzahl der Adressen
 __idlocs h'c000' | version            ; PIC Device ID
 #define offcommand_yes                ; check OFF-Command (fr Weichen)
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define save_use_timer1               ; beim Adress-Lernen Timer verwenden (fr Weichen mit Impulsverlngerung MAKESLOW)
; #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'250'                 ; Counter for Toggle
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines schalten">
#ifdef funktion_schalten
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'd000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define doppelt_mode3_yes             ; 2x hintereinander check auf gleiche Farbe mglich @Mode#3
 #define restore_in_mode1_only         ; only if R_MODE=0000 0001 the status will be restored
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; ********************************************************

; <editor-fold defaultstate="collapsed" desc="Defines weiche88">
#ifdef funktion_weiche88
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'1000' | version            ; PIC Device ID
 #define offcommand_yes                ; check OFF-Command (fr Weichen)
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
; #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'55'                  ; Counter for Toggle (nur separate LED)
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define save_use_timer1               ; beim Adress-Lernen Timer verwenden (fr Weichen mit Impulsverlngerung MAKESLOW)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines Relais8">
#ifdef funktion_relais8
CONST_ADRZAHL equ d'9'                 ; Anzahl der Adressen
 __idlocs h'8000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define doppelt_mode3_yes             ; 2x hintereinander check auf gleiche Farbe mglich @Mode#3
 #define restore_in_mode1_only         ; only if R_MODE=0000 0001 the status will be restored
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define skip_g9                       ; !! ADR gruen-#9 nur Relais8 -> wird ber Spezial-Routine in CHECK realisiert (da doppelt ausgefhrt werden darf)
 #define skip_r9                       ; !! ADR Rot-#9 nur Relais8 -> wird ber Spezial-Routine in CHECK realisiert (da doppelt ausgefhrt werden darf)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_schalten">
#ifdef funktion_led_schalten
CONST_ADRZAHL equ d'10'                ; Anzahl der Adressen
 __idlocs h'9000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define doppelt_mode3_yes             ; 2x hintereinander check auf gleiche Farbe mglich @Mode#3
 #define restore_in_mode1_only         ; only if R_MODE=0000 0001 the status will be restored
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal2">
#ifdef funktion_led_signal2
CONST_ADRZAHL equ d'5'                 ; Anzahl der Adressen
 __idlocs h'2000' | version            ; PIC Device ID
; #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal4">
#ifdef funktion_led_signal4
CONST_ADRZAHL equ d'7'                 ; Anzahl der Adressen
 __idlocs h'a000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_sbb">
#ifdef funktion_led_signal_sbb
CONST_ADRZAHL equ d'10'                 ; Anzahl der Adressen
 __idlocs h'4000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define use_status2                   ; more than 8 adressen
; #define skip_g9                       ; Skip this Adress (saves ROM)
; #define skip_r9                       ; Skip this Adress (saves ROM)
; #define skip_g10                      ; Skip this Adress (saves ROM)
; #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_andreas">
#ifdef funktion_led_andreas
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'7000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define restore_in_mode1_only         ; only if R_MODE=0000 0001 the status will be restored
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_blink">
#ifdef funktion_led_blink
CONST_ADRZAHL equ d'8'                 ; Anzahl der Adressen
 __idlocs h'b000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflashall_yes              ; beim Adress-Learning ber Blinkanzahl aller Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_kirmes">
#ifdef funktion_led_kirmes
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'c000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflashall_yes              ; beim Adress-Learning ber Blinkanzahl aller Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_neon">
#ifdef funktion_led_neon
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'd000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_haus">
#ifdef funktion_led_haus
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_feuer">
#ifdef funktion_led_feuer
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'f000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_gas">
#ifdef funktion_led_gas
CONST_ADRZAHL equ d'1'                 ; Anzahl der Adressen
 __idlocs h'5000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define savetoggle_yes                ; beim Adress-Learning Ausgnge togglen 
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define skip_g2                       ; Skip this Adress (saves ROM)
 #define skip_r2                       ; Skip this Adress (saves ROM)
 #define skip_g3                       ; Skip this Adress (saves ROM)
 #define skip_r3                       ; Skip this Adress (saves ROM)
 #define skip_g4                       ; Skip this Adress (saves ROM)
 #define skip_r4                       ; Skip this Adress (saves ROM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_ampel">
#ifdef funktion_led_ampel
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'6000' | version            ; PIC Device ID
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflashall_yes              ; beim Adress-Learning ber Blinkanzahl aller Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start (Funktion nur @ 16F684 wg. RAM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_schiebebuhne">
#ifdef funktion_led_schiebebuhne
CONST_ADRZAHL equ d'6'                 ; Anzahl der Adressen
 __idlocs h'2000' | version            ; PIC Device ID
; #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 ;#define skip_g6                       ; Skip this Adress (saves ROM)
 ;#define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle
#endif
; </editor-fold>


; <editor-fold defaultstate="collapsed" desc="Defines SanD">
#ifdef funktion_sand_ei
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'a000' | version            ; PIC Device ID
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflash_portc0              ; beim Adress-Learning ber Blinkanzahl Output C0
 #define change_mode_at_start          ; change Mode if Button is pressed @ start
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif

#ifdef funktion_sand_4
CONST_ADRZAHL equ d'8'                 ; Anzahl der Adressen
 __idlocs h'b000' | version            ; PIC Device ID
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflash_porta5              ; beim Adress-Learning ber Blinkanzahl Outputs A5
 #define change_mode_at_start          ; change Mode if Button is pressed @ start
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_001">
#ifdef funktion_led_signal_001
CONST_ADRZAHL equ d'12'                ; Anzahl der Adressen
 __idlocs h'1000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_002">
#ifdef funktion_led_signal_002
CONST_ADRZAHL equ d'12'                ; Anzahl der Adressen
 __idlocs h'2000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_003">
#ifdef funktion_led_signal_003
CONST_ADRZAHL equ d'12'                ; Anzahl der Adressen
 __idlocs h'3000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_004">
#ifdef funktion_led_signal_004
CONST_ADRZAHL equ d'8'                 ; Anzahl der Adressen
 __idlocs h'4000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 ;#define use_status2                   ; more than 8 adressen
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'11' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_005">
#ifdef funktion_led_signal_005
CONST_ADRZAHL equ d'14'                 ; Anzahl der Adressen
 __idlocs h'5000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_006">
#ifdef funktion_led_signal_006
CONST_ADRZAHL equ d'14'                 ; Anzahl der Adressen
 __idlocs h'6000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_007">
#ifdef funktion_led_signal_007
CONST_ADRZAHL equ d'14'                 ; Anzahl der Adressen
 __idlocs h'7000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_008">
#ifdef funktion_led_signal_008
CONST_ADRZAHL equ d'10'                 ; Anzahl der Adressen
 __idlocs h'8000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'21' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_009">
#ifdef funktion_led_signal_009
CONST_ADRZAHL equ d'12'                ; Anzahl der Adressen
 __idlocs h'9000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_010">
#ifdef funktion_led_signal_010
CONST_ADRZAHL equ d'14'                ; Anzahl der Adressen
 __idlocs h'a000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'01' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_011">
#ifdef funktion_led_signal_011
CONST_ADRZAHL equ d'16'                ; Anzahl der Adressen
 __idlocs h'b000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'01' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_012">
#ifdef funktion_led_signal_012
CONST_ADRZAHL equ d'5'                 ; Anzahl der Adressen
 __idlocs h'2000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 ;#define use_status2                   ; more than 8 adressen
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_020">
#ifdef funktion_led_signal_020
CONST_ADRZAHL equ d'10'                ; Anzahl der Adressen
 __idlocs h'c000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'01' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_021">
#ifdef funktion_led_signal_021
CONST_ADRZAHL equ d'14'                ; Anzahl der Adressen
 __idlocs h'd000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_022">
#ifdef funktion_led_signal_022
CONST_ADRZAHL equ d'16'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_030">
#ifdef funktion_led_signal_030
CONST_ADRZAHL equ d'16'                ; Anzahl der Adressen
 __idlocs h'a000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_031">
#ifdef funktion_led_signal_031
CONST_ADRZAHL equ d'16'                ; Anzahl der Adressen
 __idlocs h'b000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_032">
#ifdef funktion_led_signal_032
CONST_ADRZAHL equ d'14'                ; Anzahl der Adressen
 __idlocs h'c000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_033">
#ifdef funktion_led_signal_033
CONST_ADRZAHL equ d'12'                ; Anzahl der Adressen
 __idlocs h'd000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_034">
#ifdef funktion_led_signal_034
CONST_ADRZAHL equ d'14'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_040">
#ifdef funktion_led_signal_040
CONST_ADRZAHL equ d'20'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'01' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_041">
#ifdef funktion_led_signal_041
CONST_ADRZAHL equ d'14'                 ; Anzahl der Adressen
 __idlocs h'6000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_042">
#ifdef funktion_led_signal_042
CONST_ADRZAHL equ d'14'                 ; Anzahl der Adressen
 __idlocs h'6000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'81' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_050">
#ifdef funktion_led_signal_050
CONST_ADRZAHL equ d'10'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 #define change_mode_at_start_yes      ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'41' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_051">
#ifdef funktion_led_signal_051
CONST_ADRZAHL equ d'10'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'21' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_052">
#ifdef funktion_led_signal_052
CONST_ADRZAHL equ d'10'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'21' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_signal_060">
#ifdef funktion_led_signal_060
CONST_ADRZAHL equ d'14'                ; Anzahl der Adressen
 __idlocs h'e000' | version            ; PIC Device ID
 #define doppelt_no                    ; check nur 1x auf gleiche Farbe mglich
 ;#define change_mode_at_start_yes     ; change Mode if Button is pressed @ start
 #define use_status2                   ; more than 8 adressen
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
CNT_TGL     equ d'150'                 ; Counter for Toggle

; EEprom Werte
#define EE_STAT1  h'21' ; EE Adresse 0x10 = R_STATUS
#define EE_STAT2  h'00' ; EE Adresse 0x30 = R_STATUS_2
#define EE_MODE   h'01' ; EE Adresse 0x00 = R_MODE (Mode-Umschaltung)
#define EE_SLOW   h'ff' ; EE Adresse 0x20 = R_MAKESLOW - not used
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Defines led_blinkmuster">
#ifdef funktion_led_blinkmuster
CONST_ADRZAHL equ d'4'                 ; Anzahl der Adressen
 __idlocs h'f000' | version            ; PIC Device ID
 #define checkred_yes                  ; check auf RED-Command (Adressen paarweise)
 #define saveflash_yes                 ; Adress-Learning ber Blinkanzahl aller Outputs oder einzelner Outputs
 #define saveflashall_yes              ; beim Adress-Learning ber Blinkanzahl aller Outputs
CNT_TGL     equ d'150'                 ; Counter for Toggle
 #define savestatus_on                 ; Save Status ermglichen (nicht bei PWM)
 #define skip_g5                       ; Skip this Adress (saves ROM)
 #define skip_r5                       ; Skip this Adress (saves ROM)
 #define skip_g6                       ; Skip this Adress (saves ROM)
 #define skip_r6                       ; Skip this Adress (saves ROM)
 #define skip_g7                       ; Skip this Adress (saves ROM)
 #define skip_r7                       ; Skip this Adress (saves ROM)
 #define skip_g8                       ; Skip this Adress (saves ROM)
 #define skip_r8                       ; Skip this Adress (saves ROM)
 #define skip_g9                       ; Skip this Adress (saves ROM)
 #define skip_r9                       ; Skip this Adress (saves ROM)
 #define skip_g10                      ; Skip this Adress (saves ROM)
 #define skip_r10                      ; Skip this Adress (saves ROM)
 #define skip_g11                      ; Skip this Adress (saves ROM)
 #define skip_r11                      ; Skip this Adress (saves ROM)
 #define skip_g12                      ; Skip this Adress (saves ROM)
 #define skip_r12                      ; Skip this Adress (saves ROM)
 #define skip_g13                      ; Skip this Adress (saves ROM)
 #define skip_r13                      ; Skip this Adress (saves ROM)
 #define skip_g14                      ; Skip this Adress (saves ROM)
 #define skip_r14                      ; Skip this Adress (saves ROM)
 #define skip_g15                      ; Skip this Adress (saves ROM)
 #define skip_r15                      ; Skip this Adress (saves ROM)
 #define skip_g16                      ; Skip this Adress (saves ROM)
 #define skip_r16                      ; Skip this Adress (saves ROM)
#endif
; </editor-fold>

; ********************************************************
; <editor-fold defaultstate="collapsed" desc="diverse Defines">
#ifdef use16f6xx ; ROM sparend wird nur RP0 gendert
#define bank_EEDAT    bsf STATUS,RP0 
#define bank_EECON1   bsf STATUS,RP0
#define bank_OPTION   bsf STATUS,RP0 
#define bank_TRIS     bsf STATUS,RP0 
#define bank_PIE1     bsf STATUS,RP0
#define bank_RB1_RS232  bsf STATUS,RP0
#endif ; use16f6xx

#ifdef use16f1824
#define bank_EEDAT     banksel EEDAT 
#define bank_EECON1    banksel EECON1
#define bank_OPTION    banksel OPTION_REG  
#define bank_TRIS      banksel TRISA
#define bank_RB1_RS232 banksel RB1_RS232_zeiger
#endif ; use16f1824

#define bank_0        banksel TMR0 ; immer Bank 0  
; </editor-fold>

; ********************************************************
 ERRORLEVEL 0,-302,-306                   ; Errorlevel

; ************************************************************
; ** Files Assignment RAM (locations 0x20 - 0x3F)           **
; ************************************************************
; ** 16F684 ***
; BANK 0 = 020h .. 06fh (070h..07fh = COMMON RAM)
; BANK 1 = 0a0h .. 0bfh (0f0h..0ffh = COMMON RAM)

; ** 16F688 ***
; BANK 0 = 020h .. 06fh (070h..07fh = COMMON RAM)
; BANK 1 = 0a0h .. 0efh (0f0h..0ffh = COMMON RAM)
; BANK 2 = 120h .. 16fh (170h..17fh = COMMON RAM)
; BANK 3 = none         (1f0h..1ffh = COMMON RAM)

; ** 16F18324 ***
; BANK 0 = 020h .. 06fh (070h..07fh = COMMON RAM)
; BANK 1 = 0a0h .. 0efh (0f0h..0ffh = COMMON RAM)
; BANK 2 = 120h .. 16fh (170h..17fh = COMMON RAM)
; BANK 3 = 1a0h .. 1efh (1f0h..1ffh = COMMON RAM)
; BANK 4 = 220h .. 26fh (270h..27fh = COMMON RAM)
; BANK 5 = 2a0h .. 2efh (2f0h..2ffh = COMMON RAM)
; BANK 6 = 320h .. 32fh

; <editor-fold defaultstate="collapsed" desc="Register">
  ; ** ISR-Routine **
W_TEMP        equ  20h        ; saving W-Register
STATUS_TEMP   equ  21h        ; saving status-Register

  ; ** Adresserkennung **
R_ADR_1       equ  22h        ; for Decoder-Adress#1
R_ADR_2       equ  23h        ; for Decoder-Adress#2
R_ADR_3       equ  24h        ; for Decoder-Adress#3 (on)
R_ADR_3a      equ  25h        ; for Decoder-Adress#3 (off)
R_ADR_2a      equ  26h        ; for Decoder-Adress#2 (off)
R_ADR_1a      equ  27h        ; for Decoder-Adress#1 (off)

R_DEKADR_1    equ  28h        ; for read Decoder-Adress#1
R_DEKADR_2    equ  29h        ; for read Decoder-Adress#2
R_DEKADR_3    equ  2ah        ; for read Decoder-Adress#3
R_DEKADR_4    equ  2bh        ; for read Decoder-Adress#4
R_DEKADR_5    equ  2ch        ; for read Decoder-Adress#5
R_DEKADR_1_L  equ  2dh
R_DEKADR_2_L  equ  2eh
R_DEKADR_3_L  equ  2fh
R_DEKADR_1_L_old  equ  30h
#ifdef dcc
R_CNT_TOIF      equ  31h        ; Counter Anzahl TOIF in der Loop. Zur Erkennung von Pausen im Bitstrom -> Verwerfen von MM Befehlen
#endif
#ifdef motorola
R_BITCOUNT    equ  31h        ; counting #read bits
#endif

R_FLAG        equ  32h        ; Register for Flags
F_IDLE          equ  d'0'       ; set if idle detected and ready for receive command
F_READ          equ  d'1'       ; set during reading the bits
F_CMD           equ  d'2'       ; set if command is valid and must checked
R_POL         equ  33h        ; Bit#0 set -> inverse polarity

  ; ** Counter **
R_CNT1_TGL    equ  34h        ; Counter1 for Save-Routine & Button & Change_mode
R_CNT2_TGL    equ  35h        ; Counter2 for Save-Routine & Change_mode
;R_CNT3_TGL    equ  xxh        ; Counter3 for Save-Routine (slow-Toggle @Weichendekoder)
R_CNT_TGL     equ  36h        ; Counter for led flashing @ address save & Change_mode -> Blink-Anzahl 
R_CNT_ADR     equ  37h        ; Counter for address save/read - number & eeprom-adr.
R_TGLA        equ  38h        ; Register fr wechselseitiges signalisieren @ save
R_TGLC        equ  39h        ; Register fr wechselseitiges signalisieren @ save

  ; ** Push-Button **
R_BUTTON      equ  3ah        ; for debouncing push-button

  ; ** EEPROM-Saving **
R_EEDAT       equ  3bh        ; Register used by SAVEROUTINE
R_EEADR       equ  3ch        ; Register used by SAVEROUTINE
R_MODE        equ  3dh        ; for Dekoder-Mode
; #0: 1= Normaler Mode
; #1: 1= Dumm-Mode bzw. Dunkeltast-Mode (fr Signale)
; #2: 1= Automatischer Abfall der Relais
; #3: 1= Single Output Mode
R_STATUS      equ  3eh        ; Register for save on/off status in EEPROM
R_SAVE_F      equ  3fh        ; Register for Flags:
    #define F_SAVE_STATUS    R_SAVE_F,0

; </editor-fold>

; ********************************************************
; **  Bit Assignment                                    **
; ********************************************************
; <editor-fold defaultstate="collapsed" desc="Parameter Konstanten">
; </editor-fold>

; ********************************************************
; **  Reset Vector                                      **
; ********************************************************

  org  0000h                           ; Position (Reset Vector)

; ********************************************************
; **  IDs for Manipulator                               **
; ********************************************************
; <editor-fold defaultstate="collapsed" desc="IDs for Manipulator">
 #ifdef funktion_wz_blinkmuster
  movlw d'01'
  movlw d'1'             ; ab  V3.60
 #endif
 #ifdef funktion_spur1
  movlw d'02'
  movlw d'3'             ; ab  V3.58
 #endif
 #ifdef funktion_signal
  movlw d'03'
 #endif
 #ifdef funktion_dreiwege
  movlw d'04'
 #endif
 #ifdef funktion_signal4
  movlw d'05'
 #endif
 #ifdef funktion_signal_hl
  movlw d'06'
 #endif
 #ifdef funktion_signal_straba
  movlw d'07'
 #endif
 #ifdef funktion_weichei
 #ifndef funktion_autofuse
  movlw d'08'
  movlw d'4'             ; ab  V3.61
 #endif
 #ifdef funktion_autofuse
  movlw d'12'
  movlw d'3'             ; ab  V3.58
 #endif
 #endif
 #ifdef funktion_andreas
  movlw d'09'
 #endif
 #ifdef funktion_weichzwei
  movlw d'10'
  movlw d'3'             ; ab  V3.58
 #endif
 #ifdef funktion_schalten
  movlw d'11'
 #endif

; folgende IDs belegt durch F-Dekoder SW:
; #ifdef funktion_weichzwei
;  movlw d'60'
; #endif
; #ifdef funktion_schalten
;  movlw d'61'
; #endif
;
; #ifdef funktion_led_schalten
;  movlw d'62'
; #endif
; #ifdef funktion_led_blink
;  movlw d'63'
; #endif
; #ifdef funktion_led_neon
;  movlw d'65'   ; war 64, aber da Anzahl der ADressen gendert -> neue ID (d'64' aber nicht wieder verwenden!)
; #endif
;
#ifdef funktion_weiche88
  movlw d'101'
  movlw d'3'             ; ab  V3.58
 #endif
 #ifdef funktion_led_gas
  movlw d'102'
 #endif
 #ifdef funktion_led_ampel
  movlw d'103'
  movlw d'3'             ; ab  V3.61
 #endif
 #ifdef funktion_led_andreas
  movlw d'104'
 #endif
 #ifdef funktion_relais8
  movlw d'105'
 #endif
 #ifdef funktion_led_schalten
  movlw d'106'
 #endif
 #ifdef funktion_led_signal2
  movlw d'107'
 #endif
 #ifdef funktion_led_signal4
  movlw d'108'
 #endif
 #ifdef funktion_led_signal_sbb
  movlw d'109'
  movlw d'1'             ; ab  V3.59
 #endif
 #ifdef funktion_led_blink
  movlw d'110'
 #endif
 #ifdef funktion_led_kirmes
  movlw d'111'
 #endif
 #ifdef funktion_led_neon
  movlw d'112'
  movlw d'2'             ; ab  V3.48
 #endif
 #ifdef funktion_led_haus
  movlw d'113'
  movlw d'1'             ; ab  V3.61
 #endif
 #ifdef funktion_led_feuer
  movlw d'114'
 #endif
 #ifdef funktion_led_schiebebuhne
  movlw d'115'
 #endif

 #ifdef funktion_led_signal_001 
  movlw d'201'
  movlw d'1'             ; ab  V3.37
 #endif
 #ifdef funktion_led_signal_002
  movlw d'202'
  movlw d'1'             ; ab  V3.37
 #endif
 #ifdef funktion_led_signal_003
  movlw d'203'
 #endif
 #ifdef funktion_led_signal_004
  movlw d'204'
 #endif
 #ifdef funktion_led_signal_005
  movlw d'205'
 #endif
 #ifdef funktion_led_signal_006
  movlw d'206'
 #endif
 #ifdef funktion_led_signal_007
  movlw d'207'
 #endif
 #ifdef funktion_led_signal_008
  movlw d'208'
 #endif
 #ifdef funktion_led_signal_010
  movlw d'209'
 #endif
 #ifdef f684_sand_ei_1
  movlw d'210'
  movlw d'1'             ; ab  V3.58
 #endif
 #ifdef f684_sand_ei_2   ; (nicht mehr invertiert)
  movlw d'210'
  movlw d'125'           ; ab  V3.73
 #endif
 #ifdef funktion_sand_4
  movlw d'211'
 #endif
 #ifdef funktion_led_signal_009
  movlw d'212'
  movlw d'1'             ; ab  V3.61
 #endif
 #ifdef funktion_led_signal_012
  movlw d'213'
  movlw d'1'             ; ab  V3.37
 #endif
 #ifdef funktion_led_blinkmuster
  movlw d'221'
  movlw d'1'             ; ab  V3.60
 #endif
 #ifdef funktion_led_signal_011
  movlw d'230'
 #endif
 #ifdef funktion_led_signal_020
  movlw d'231'
  movlw d'1'             ; ab  V3.58
 #endif
 #ifdef funktion_led_signal_021
  movlw d'232'
 #endif
 #ifdef funktion_led_signal_022
  movlw d'233'
 #endif
 #ifdef funktion_led_signal_030
  movlw d'234'
 #endif
 #ifdef funktion_led_signal_031
  movlw d'235'
 #endif
 #ifdef funktion_led_signal_032
  movlw d'236'
 #endif
 #ifdef funktion_led_signal_033
  movlw d'237'
 #endif
 #ifdef funktion_led_signal_034
  movlw d'238'
 #endif
 #ifdef funktion_led_signal_040
  movlw d'239'
 #endif
 #ifdef funktion_led_signal_041
  movlw d'206'
 #endif
 #ifdef funktion_led_signal_042
  movlw d'244'
 #endif
 #ifdef funktion_led_signal_050
  movlw d'240'
 #endif
 #ifdef funktion_led_signal_051
  movlw d'241'
 #endif
 #ifdef funktion_led_signal_052
  movlw d'242'
 #endif
 #ifdef funktion_led_signal_060
  movlw d'243'
 #endif
 #ifdef drehscheibe
  movlw d'255'
 #endif

       goto start
; </editor-fold>

; ********************************************************
; **  Interupt Service Routine ISR                      **
; ********************************************************
; <editor-fold defaultstate="collapsed" desc="ISR">
  org 0004h                            ; Interupt Vector
#ifdef use_mplab
 #ifdef motorola
  #include <isr_mot.asm>
 #endif
 #ifdef dcc
  #include <isr_dcc.asm>
 #endif
#endif
#ifndef use_mplab
 #ifdef motorola
  #include <..\isr_mot.asm>
 #endif
 #ifdef dcc
  #include <..\isr_dcc.asm>
 #endif
#endif

#ifdef nodecoder
; ----- Saving STATUS and W ----------------------
        movwf W_TEMP                   ; Saving W
        swapf STATUS,W                 ; STATUS -> W
        movwf STATUS_TEMP              ; Saving STATUS
        bcf STATUS,RP0                 ; -> BANK 0

; ----- Restore STATUS and W ---------------------
posret  clrf TMR0                      ; clear Timer

        bcf INTCON,INTF                ; clear IR-Flag
        bcf INTCON,T0IF                ; clear IR-Flag
        swapf STATUS_TEMP,W            ; saved STATUS in W
        movwf STATUS                   ; w -> STATUS
        SWAPF W_TEMP,F                 ; W_TEMP -> W, without..
        SWAPF W_TEMP,W                 ; .. changing Z-Flag

        retfie
#endif
; </editor-fold>

; ********************************************************
; **  Programm                                          **
; ********************************************************
; <editor-fold defaultstate="collapsed" desc="Start Adress">
#ifndef test_isr
 org   003ah                           ; Memory-Position
#endif
#ifdef test_isr
 org   004ah                           ; Memory-Position
#endif
; </editor-fold>

;   ----------------------------------------------
;   --- clear memory $20..$5F (..$7F @ 16F684) ---
;   ----------------------------------------------
start
; <editor-fold defaultstate="collapsed" desc="Clear Memory">
        movlw h'20'                    ; memory start adress
        movwf FSR
next_clr
        clrf INDF                      ; clear
        incf FSR,F
        movlw CONST_RAM                ; Anzahl der RAM Zellen
        subwf FSR,W                    ; W = FSR - W = FSR - h'5F'
        btfss STATUS,C                 ; if C=0, -> FSR < h'5F'
         goto next_clr                  ; next bit
; </editor-fold>

;   ------------------------------------
;   --- Configuration / start values ---
;   ------------------------------------
; <editor-fold defaultstate="collapsed" desc="Reg. Confi / Start Values">
#ifdef motorola
        movlw d'18'                    ; read 18 bits
        movwf R_BITCOUNT
#endif ; motorola

;   ----------------------------------------------------------
;   --- INIT: dekoderspezifische Register, Register aus EE ---
;   ----------------------------------------------------------
        call init_dekoder              ; dekoderspezifische Initialisierung

#ifndef simulation
#ifdef useOSCCAL ; OSCCAL nur 12F629/16f630/16f676
        bsf STATUS,RP0                 ; -> Bank 1
        call 3ffh                      ; get the calibration value
        movwf OSCCAL                   ; calibrate
        bank_0                         ; -> Bank 0
#endif ; useOSCCAL
#endif ; simulation

;   ----------------------------------------------------------
;   --- Konfiguration PORTS                                ---
;   ----------------------------------------------------------
#ifdef funktion_sand_4
        clrf PORTA                      ; OE_4094 verhindern
        clrf PORTC                      ; Servo-Impuls verhindern
#endif ; sand_4
#ifdef funktion_sand_ei
        clrf PORTA                      ; OE_4094 verhindern
        clrf PORTC                      ; Servo-Impuls verhindern
#endif ; sand_ei
;#ifdef funktion_schalten                       ; "schalten" wird auch fr Relais-1 Dekdoder verwendet.
        ;bsf STATUS,RP0                 ; -> BANK 1
        ;  bsf TRISA,5                  ; Port 5 Input
        ;bcf STATUS,RP0                 ; -> BANK 0
;        movlw b'00001100'              ; normal configuration
        ;btfsc PORTA,5                  ; Relais-1: hat Pin 2 auf HIGH
        ; movlw b'00101100'             ; Relais-1: A5 muss Input bleiben!
;        bsf STATUS,RP0                 ; -> BANK 1
;          movwf TRISA                  ; set TRIS.Register
;        bcf STATUS,RP0                 ; -> BANK 0
;#else
        bsf STATUS,RP0                 ; -> BANK 1
          movlw b'00001100'            ; #2 #3 Input, Rest ist Output
        andwf TRISA,F
        clrf TRISC                     ; PORTC ist Output
        bcf STATUS,RP0                 ; -> BANK 0
;#endif

#ifdef funktion_sand_4
        ;PDATA als Input schalten
        bsf STATUS,RP0                 ; -> BANK 1
          bsf TRISA,0                  ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; sand_4

#ifdef funktion_sand_ei
        ;PDATA, SW3/4 als Input schalten
        bsf STATUS,RP0                 ; -> BANK 1
          bsf TRISA,0                  ; set TRIS.Register
          bsf TRISC,3                  ; set TRIS.Register
          bsf TRISC,4                  ; set TRIS.Register
        bcf STATUS,RP0                 ; -> BANK 0
#endif ; sand_ei

        clrf PORTA
        clrf PORTC

; ********************************************************
; **  Konfiguration Register                            **
; ********************************************************
#ifndef use_mplab
 #ifdef use12f629
  #include <..\conf_12f629.asm>
 #endif
 #ifdef use16f630
  #include <..\conf_16f630.asm>
 #endif
 #ifdef use16f676
  #include <..\conf_16f676.asm>
 #endif
 #ifdef use16f684
  #include <..\conf_16f684.asm>
 #endif
 #ifdef use16f688
  #include <..\conf_16f688.asm>
 #endif
 #ifdef use16f1824
  #include <..\conf_16f1824.asm>
 #endif
 #ifdef use16f183x4  ; 16f18324 + 16f18344
  #include <..\conf_16f183x4.asm>
 #endif
#endif ;mplab

#ifdef use_mplab
 #ifdef use12f629
  #include <conf_12f629.asm>
 #endif
 #ifdef use16f630
  #include <conf_16f630.asm>
 #endif
 #ifdef use16f676
  #include <conf_16f676.asm>
 #endif
 #ifdef use16f684
  #include <conf_16f684.asm>
 #endif
 #ifdef use16f688
  #include <conf_16f688.asm>
 #endif
 #ifdef use16f1824
  #include <conf_16f1824.asm>
 #endif
 #ifdef use16f183x4  ; 16f18324 + 16f18344
  #include <conf_16f183x4.asm>
 #endif
#endif;mplab
; </editor-fold>

; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  change R_MODE @ STARTUP                           **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
#ifndef simulation
;   -------------------------------------
;   --- Wartezeit                     ---
;   -------------------------------------
;        clrwdt                         ; clear Watchdog-Timer
;        decfsz R_CNT1_TGL,F
;          goto $-2
;        decfsz R_CNT2_TGL,F
;          goto $-4

;   -------------------------------------
;   --- change MODE if BUTTON @ start ---
;   -------------------------------------
; <editor-fold defaultstate="collapsed" desc="Mode Umschaltung">
#ifdef change_mode_at_start_yes        ; change Mode if Button is pressed @ start
        ; R_MODE #7 sperrt Mode-Umschaltung, ansonsten Mode-Umschaltung bei Taste (neu V3.55)
        btfsc R_MODE,7                 ; restoring of outputs
         goto skip_change_mode
        btfsc PORTA,3                  ; if Button
         goto skip_change_mode
        clrwdt
        btfss PORTA,3                  ; if Button
         goto $-2
        call change_mode
skip_change_mode
#endif
#ifdef change_mode_at_start            ; change Mode if Button is pressed @ start (ohne Wartezeit -> Sand)
        ; R_MODE #7 sperrt Mode-Umschaltung, ansonsten Mode-Umschaltung bei Taste (neu V3.55)
        btfsc R_MODE,7                 ; restoring of outputs
         goto skip_change_mode
        btfss PORTA,3                  ; if Button
         call change_mode
skip_change_mode
#endif

#ifdef restore_in_mode1_only 
        btfss R_MODE,0                 ; restoring of relays
         goto init                     ; ..if R_MODE #0 gesetzt (not in Dumm-Mode)
#endif
; </editor-fold>

;   -----------------------------------------
;   --- Ausgnge schalten gemss R_STATUS ---
;   -----------------------------------------
; <editor-fold defaultstate="collapsed" desc="Restore STATUS">
        call status_to_output
; </editor-fold>
#endif; simulation

init
;   ------------------------------------
;   --- Timer rcksetzen, IR Scharf  ---
;   ------------------------------------
; <editor-fold defaultstate="collapsed" desc="Reset Timer / INT">
        clrf TMR0
        clrf TMR1L                     ; clear Timer1 for Time-Out
        clrf TMR1H                     ; clear Timer1 for Time-Out

        bcf INTCON,T0IF                ; clear Flag
        bcf INTCON,INTF                ; clear IR-Flag
        bcf INTCON,RAIF                ; clear IR-Flag
;        bcf PIR1,EEIF                  ; clear Flag
;        bcf PIR1,ADIF                  ; clear Flag
;        bcf PIR1,CMIF                  ; clear Flag
;        bcf PIR1,TMR1IF                ; clear Flag
       clrf PIR1                      ; clear all IR flags
        bsf INTCON,GIE                 ; enable IRs
; </editor-fold>

#ifdef funktion_autofuse
        call gruen_1                   ; bei Start GP0 einschalten, Abschalten durch TimeOut (return vo dort!) 
#endif
#ifdef demo_mode
       ; preload counter
        movlw d'20'                    ;
        movwf R_CNT1_TGL               ; preload R_CNT1_TGL
        movwf R_CNT2_TGL               ; preload R_CNT2_TGL
#endif ; demo_mode

; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  Main Loop                                         **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
loop

; <editor-fold defaultstate="collapsed" desc="Test-Routinen">
;   ----------------------------------------
;   --- test-Routinen, Befehls-Auslsung bei Simulation ---
;   ----------------------------------------
#ifdef simulation
;  clrf R_MODE

;    movf R_FLAGS,W
;    btfsc R_MODE,7
;     call gruen_5
;    btfsc R_FLAGS,2
;     call rot_1
;    btfsc R_DEKADR_1_L,1
;     call gruen_2
;    btfsc R_DEKADR_1_L,2
;     call gruen_3
;    btfsc R_DEKADR_1_L,3
;     call gruen_4
;    clrf R_DEKADR_1_L
 call save
#endif ; simulation

#ifdef test_oscilator
     bsf PORTA,0
     bcf PORTA,0
     bsf PORTA,1
     bcf PORTA,0
     call polarity
     goto loop
#endif;test_oscilator

#ifdef test_output
        call sub_test_output
        goto loop
#endif 
; </editor-fold>

;   ----------------------------------------
;   --- Polaritaets- und Pause-Erkennung ---
;   ----------------------------------------
        call polarity                  ; check polarity

;   ------------------------
;   --- Befehl erkannt ? ---
;   ------------------------
; <editor-fold defaultstate="collapsed" desc="Abarbeitung Befehl">
        btfsc R_FLAG,F_CMD             ; if valid command..
         call check                    ; .. then check adress
; </editor-fold>

;   --------------------------
;   --- Status speichern ? ---
;   --------------------------
; <editor-fold defaultstate="collapsed" desc="Status Speichern">
; wenn egal, wann gespeichert wird (ohne PWM), dann hier
#ifdef savestatus_on
         call save_status
#endif
; </editor-fold>

;   -------------------------
;   --- Read Input BUTTON ---
;   -------------------------
; <editor-fold defaultstate="collapsed" desc="Button">
#ifndef demo_mode
        decfsz R_CNT1_TGL,F            ; increment Counter 1 for debouncing switch
         goto loop_00
        bcf STATUS,C                   ; add "0"
        btfsc PORTA,3                  ; check Button (released = H)
         bsf STATUS,C                  ; .. bei H (Button=released) do this line (add "1")
        rlf R_BUTTON,F                 ;
        movlw b'11110000'              ; 
        subwf R_BUTTON,W
        btfsc STATUS,Z                 ; if the same -> Taster wurde gedrckt
#ifdef nodecoder
        ; Reaktion auf Taste: drcken -> AUS, Loslassen -> AN
         call change_status_off        ; @ Taster wurde gedrckt

        movlw b'00001111'              ; 
        subwf R_BUTTON,W
        btfsc STATUS,Z                 ; if the same -> Taster wurde released
         call change_status_on         ; @ Taster wurde released
#else ; nodecoder
        call save                      ; .. then learning function
#endif ; nodecoder
loop_00
#else ; demo_mode
       call check_demomode
#endif ; demo_mode
; </editor-fold>

;   -------------------------
;   --- State Machine     ---
;   -------------------------
; <editor-fold defaultstate="collapsed" desc="State Machine">
#ifdef funktion_wz_blinkmuster
        call statemachine_blinkmuster
#endif

#ifdef funktion_sand_ei
        call statemachine_sand_ei
#endif

#ifdef funktion_sand_4
        call statemachine_sand_4
#endif

#ifdef funktion_weichzwei
        call abschalten
#endif

#ifdef funktion_weichei
        call statemachine_weichei
#endif

#ifdef funktion_weiche88
        call abschalten
#endif

#ifdef funktion_spur1
        call abschalten
#endif

#ifdef funktion_dreiwege
        call abschalten
#endif

#ifdef funktion_signal
        call statemachine_signal
#endif

#ifdef funktion_signal4
        call statemachine_signal4
#endif

#ifdef funktion_signal_hl
        call statemachine_signal_hl
#endif

#ifdef funktion_signal_straba
        call statemachine_signal_straba
#endif

#ifdef funktion_andreas
        call statemachine_andreas
#endif

#ifdef funktion_schalten
        call abschalten_mode3
#endif

#ifdef funktion_led_ampel
        call statemachine_ampel
#endif

#ifdef funktion_led_andreas
        call statemachine_andreas
#endif

#ifdef funktion_led_schalten
        call abschalten_mode3
#endif

#ifdef funktion_led_signal2
        call statemachine_signal2
#endif

#ifdef funktion_led_signal4
        call statemachine_signal4
#endif

#ifdef funktion_led_signal_sbb
        call statemachine_signal_sbb
#endif

#ifdef funktion_led_blink
        call statemachine_blink
#endif

#ifdef funktion_led_kirmes
        call statemachine_kirmes
#endif

#ifdef funktion_led_haus
        call statemachine_haus
#endif

#ifdef funktion_led_gas
        call statemachine_gas
#endif

#ifdef funktion_led_neon
        call statemachine_neon
#endif

#ifdef funktion_led_feuer
        call statemachine_feuer
#endif

#ifdef funktion_relais8
        call abschalten_mode3
#endif

#ifdef funktion_led_schiebebuhne
        call statemachine_schiebebuhne
#endif

#ifdef funktion_led_signal_001
        call statemachine_signal
#endif

#ifdef funktion_led_signal_002
        call statemachine_signal
#endif

#ifdef funktion_led_signal_003
        call statemachine_signal
#endif

#ifdef funktion_led_signal_004
        call statemachine_signal
#endif

#ifdef funktion_led_signal_005
        call statemachine_signal
#endif

#ifdef funktion_led_signal_006
        call statemachine_signal
#endif

#ifdef funktion_led_signal_007
        call statemachine_signal
#endif

#ifdef funktion_led_signal_008
        call statemachine_signal
#endif

#ifdef funktion_led_signal_009
        call statemachine_signal
#endif

#ifdef funktion_led_signal_010
        call statemachine_signal
#endif

#ifdef funktion_led_signal_011
        call statemachine_signal
#endif

#ifdef funktion_led_signal_012
        call statemachine_signal
#endif

#ifdef funktion_led_signal_020
        call statemachine_signal
#endif

#ifdef funktion_led_signal_021
        call statemachine_signal
#endif

#ifdef funktion_led_signal_022
        call statemachine_signal
#endif

#ifdef funktion_led_signal_030
        call statemachine_signal
#endif

#ifdef funktion_led_signal_031
        call statemachine_signal
#endif

#ifdef funktion_led_signal_032
        call statemachine_signal
#endif

#ifdef funktion_led_signal_033
        call statemachine_signal
#endif

#ifdef funktion_led_signal_034
        call statemachine_signal
#endif

#ifdef funktion_led_signal_040
        call statemachine_signal
#endif

#ifdef funktion_led_signal_041
        call statemachine_signal
#endif

#ifdef funktion_led_signal_042
        call statemachine_signal
#endif

#ifdef funktion_led_signal_050
        call statemachine_signal
#endif

#ifdef funktion_led_signal_051
        call statemachine_signal
#endif

#ifdef funktion_led_signal_052
        call statemachine_signal
#endif

#ifdef funktion_led_signal_060
        call statemachine_signal
#endif

#ifdef funktion_led_blinkmuster
        call statemachine_led_blinkmuster
#endif
; </editor-fold>

        goto loop

; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
; **  SUB - Routines                                    **
; ********************************************************
; ********************************************************
; ********************************************************
; ********************************************************
;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: check_demomode     **
;** DemoMode: automatischer     **
;** Durchlauf der Adressen      **
;** needs <    us               **
;*********************************
;*********************************
;*********************************
;*********************************
; <editor-fold defaultstate="collapsed" desc="demo_mode">
#ifdef demo_mode
check_demomode
        decfsz R_CNT1_TGL,F            ; increment Counter 1 for time delay
         return
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for time delay
         return


       ; preload counter
        movlw d'20'                    ;
        movwf R_CNT2_TGL               ; preload R_CNT2_TGL

        ; DEMO Mode: nchste Adresse aufrufen
        movf R_STATUS,W                ; wenn R_STATUS = 0001xxxx (= war Adresse 5)
        andlw b'11110000'
        sublw b'00010000'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,5               ;
          goto gruen_6                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = 0010xxxx (= war Adresse 5)
        andlw b'11110000'
        sublw b'00100000'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,6               ;
          goto gruen_7                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = 0100xxxx (= war Adresse 5)
        andlw b'11110000'
        sublw b'01000000'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,7               ;
          goto gruen_8                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = 1000xxxx (= war Adresse 5)
        andlw b'11110000'
        sublw b'10000000'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,4               ;
          call gruen_5                 ; CALL -> auch Hauptsignal wird gendert

        movf R_STATUS,W                ; wenn R_STATUS = xxxx0001
        andlw b'00001111'
        sublw b'00000001'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,1               ; STATUS Bit#1 on
          goto gruen_2                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = xxxx0010
        andlw b'00001111'
        sublw b'00000010'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,2               ; STATUS Bit#1 on
          goto gruen_3                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = xxxx0100
        andlw b'00001111'
        sublw b'00000100'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,3               ; STATUS Bit#1 on
          goto gruen_4                 ; RETURN von dort

        movf R_STATUS,W                ; wenn R_STATUS = xxxx1000
        andlw b'00001111'
        sublw b'00001000'
        btfss STATUS,Z
         goto $+3
          bsf R_STATUS,0               ; STATUS Bit#1 on
          goto gruen_1                 ; RETURN von dort
 
         return
#endif ; demo_mode
; </editor-fold>

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: POLARITY           **
;** Check polarity              **
;** needs <    us               **
;*********************************
;*********************************
;*********************************
;*********************************
polarity
; <editor-fold defaultstate="collapsed" desc="polarity MM">
#ifdef motorola
        clrwdt                         ; clear Watchdog-Timer
        btfsc R_FLAG,F_IDLE            ; Routine blocked
         return                        ; .. if already IDLE-State

        btfss INTCON,T0IF              ; if Timer0 overflow
         return
        bcf INTCON,T0IF                ; clear Flag

        ; *** IDLE-State detected ***
        bcf R_FLAG,F_READ
        bsf R_FLAG,F_IDLE              ; change to IDLE-state
        movlw d'18'                    ; ready for reading 18 bits
        movwf R_BITCOUNT

        ; *** check polarity ***
        btfss PORTA,2                  ; check signal
         goto pol_norm

pol_inv ; *** inverse polarity ***
        bsf R_POL,0                    ; polarity invers
        bsf STATUS,RP0                 ; -> BANK 1
         bcf OPTION_REG,INTEDG         ; ..IR on falling edge on RB0
        bcf STATUS,RP0                 ; -> BANK 0
        return

pol_norm ; *** normal polarity ***
        bcf R_POL,0                    ; polarity normal
        bsf STATUS,RP0                 ; -> BANK 1
         bsf OPTION_REG,INTEDG         ; ..IR on rising edge on RB0
        bcf STATUS,RP0                 ; -> BANK 0
        return
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="polarity DCC">
#ifdef dcc
        clrwdt                         ; clear Watchdog-Timer
        bcf R_POL,0                    ; polarity normal

;   -----------------------------------------------------------------------------
;   --- DCC: Erkennung PAUSE zwischen DCC und MM Befehl -> lschen DCC Stream ---
;   -----------------------------------------------------------------------------
; Durchlaufzeit LOOP: >28s (gemessen @ DC und auch ECoS)
; max DCC Impulslnge (ohne Stretched Bit) nicht definiert, bei ECoS 116 us
; ESOS mach eine Pause  (HIGH) von 6ms zwichen DCC und MM (und umgekehrt), dies kann von eine "Stretched 0 Bit" nicht unterschieden werden (bis zu 10ms HIGH).
; in jedem Loop Durchlauf msste daher 1x die ISR aufgerufen werden bei DCC Impulsen. 
; wird nun nach 32 Loop Durchlufen die ISR nicht aufgerufen, dann wurde seit mindestens 960us die ISR nicht aufgerufen (also keine Impuls-Flanke an COMMAND)
; der Counter R_CNT_TOIF wird also bei jedem Durchlauf ++ und in der ISR (bei Falling edge - hier gibt es wenig zu tun in der ISR) zurckgesetzt.
; wird demnach nach 8 Loop Durchlufen ohne fallende Flanke am Command Eingang der DCC Stream gelscht
#ifdef test_isr
        bcf PORTC,4
#endif; test_isr
        incf R_CNT_TOIF,F
        btfss R_CNT_TOIF,6              ; wenn R_CNT_TOIF#6
         goto skip_CNT_TOIF
CNT_TOIF_overrun
#ifdef test_isr
        bsf PORTC,4
#endif; test_isr
        clrf R_DEKADR_5                ; Register erstmal lschen
        clrf R_DEKADR_4                ; Register erstmal lschen
        clrf R_DEKADR_3                ; Register erstmal lschen
        clrf R_DEKADR_2                ; Register erstmal lschen
        clrf R_DEKADR_1                ; Register erstmal lschen
        clrf R_CNT_TOIF
skip_CNT_TOIF

         return                        ; .. if already IDLE-State
#endif ; DCC
; </editor-fold>

#ifdef nodecoder
        clrwdt                         ; clear Watchdog-Timer
        return
#endif

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: CHECK              **
;** Check Adress                **
;** needs < 800 us (@ 8x3 Read  **
;*********************************
;*********************************
;*********************************
;*********************************
check
; <editor-fold defaultstate="collapsed" desc="check Routine">
#ifdef nodecoder
       return
#endif
        bcf R_FLAG,F_CMD               ; Clear Flag
        movlw d'01'                    ; Preload
        movwf R_CNT_ADR                ; .. R_CNT_ADR

#ifdef funktion_relais8
        bsf PORTA,1                    ; LED on
        movf R_VERZ,W
        addlw d'1'                     ; latch vom (R_VERZ + 1)
        movwf R1_ABSCHALT_9            ; .. in R1_ABSCHALT_x
        movf TMR1H,W                   ; latch vom TMR1H in R2_ABSCHALT_x
        movwf R2_ABSCHALT_9            ; abschalten @ [R_VERZ > (R1_ABSCHALT_x + 1)] & (TMR1H > R2_ABSCHALT_x) 
#endif

; <editor-fold defaultstate="collapsed" desc="Test-Routinen">
#ifdef test_befehlserkennung ; C3 Routine CHECK -> Impuls @ Aufruf
       bsf PORTC,3 
       bcf PORTC,3
#endif
; </editor-fold>

;   --------------------------------
;   --- Adresse aus EEPROM holen ---
;   --------------------------------
check_loop
#ifdef test_befehlserkennung ; C1 Impuls bei LOOP durchlauf
       call toggle_c1
#endif

#ifdef dcc
; <editor-fold defaultstate="collapsed" desc="Read from EEprom DCC">
        bcf INTCON,GIE                 ; disable IRs
        movlw h'20'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $20+d16+$31 = $61
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Address: h'20' + R_CNT_ADR
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_3                  ; ..into R_ADR_3

        movlw h'10'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $10+d16+$11 = $51
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_2                  ; ..into R_ADR_2
        movwf R_ADR_2a                 ; ..into R_ADR_2a
        movlw b'00000010'              ; inverting #1
        xorwf R_ADR_2a,F               ; ..from ADR_2a

        movlw h'00'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $00+d16+$11 = $41
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_1                  ; ..into R_ADR_1

        movwf R_ADR_1a                 ; ..into R_ADR_1a
        movlw b'00000001'              ; inverting #0
        xorwf R_ADR_1a,F               ; ..from ADR_1a
        bsf INTCON,GIE                 ; enable IRs
; </editor-fold>
#endif ; dcc

#ifdef motorola
; <editor-fold defaultstate="collapsed" desc="Read from EEprom MM">
        bcf INTCON,GIE                 ; disable IRs
        movlw h'20'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $20+d16+$31 = $61
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Address: h'20' + R_CNT_ADR
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_3                  ; ..into R_ADR_3
        movwf R_ADR_3a                 ; ..into R_ADR_3a
        movlw b'11000000'              ; inverting #7-#6
        xorwf R_ADR_3a,F               ; ..from ADR_3a

        movlw h'10'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $10+d16+$11 = $51
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_2                  ; ..into R_ADR_2

        movlw h'00'
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $00+d16+$11 = $41
#endif ; skip_g16
        bsf STATUS,RP0                 ; -> BANK 1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_ADR_1                  ; ..into R_ADR_1
        bsf INTCON,GIE                 ; enable IRs

;   ------------------------------
;   --- Adresse normieren (MM) ---
;   ------------------------------
        btfss R_POL,0                  ; if not inverse polarity
         goto check1                   ; .. then jump
        comf R_ADR_1,W                 ; if inverse polarity,
        andlw b'00000011'              ; mask unused bits #2-#7
        movwf R_ADR_1
        comf R_ADR_2,F                 ; .. then complement
        comf R_ADR_3,F
        comf R_ADR_3a,F
check1
; </editor-fold>
#endif ; MM

;   ------------------------------
;   --- Check auf Befehl-Grn  ---
;   ------------------------------
; <editor-fold defaultstate="collapsed" desc="Check Adress Byte MM">
#ifdef motorola
        movf  R_ADR_1,W                ; compare R_ADR_1 and R_DEKADR_1
        subwf R_DEKADR_1_L,W
        btfss STATUS,Z                 ; if not the same
         goto check_next               ; .. then check next adress
#ifdef test_befehlserkennung ; C2 Routine CHECK -> Check OK
       bsf PORTC,2 
       bcf PORTC,2
#endif

        movf  R_ADR_2,W                ; compare R_ADR_2 and R_DEKADR_2
        subwf R_DEKADR_2_L,W
        btfss STATUS,Z                 ; if not the same
         goto check_next               ; .. then check next adress
#ifdef test_befehlserkennung ; C2 Routine CHECK -> Check OK
       bsf PORTC,2 
       bcf PORTC,2
#endif

        movf  R_ADR_3,W                ; compare R_ADR_3 and R_DEKADR_3
        subwf R_DEKADR_3_L,W
        btfss STATUS,Z                 ; if not the same
         goto check2                   ; .. then check if RED matches
#ifdef test_befehlserkennung ; C2 Routine CHECK -> Check OK
       bsf PORTC,2 
       bcf PORTC,2
#endif
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Check Adress Byte DCC">
#ifdef dcc
        movf  R_ADR_3,W                ; compare R_ADR_3 and R_DEKADR_3
        subwf R_DEKADR_3_L,W
        btfss STATUS,Z                 ; if not the same
         goto check_next               ; .. then check next adress

        movf  R_ADR_2,W                ; compare R_ADR_2 and R_DEKADR_2
        subwf R_DEKADR_2_L,W
        btfss STATUS,Z                 ; if not the same
         goto check2                   ; .. then check if RED matches

        movf  R_ADR_1,W                ; compare R_ADR_1 and R_DEKADR_1
        subwf R_DEKADR_1_L,W
        btfss STATUS,Z                 ; if not the same
         goto check_next               ; .. then check next adress
#endif
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Auswertung GRUEN">
; *** ADR Gruen-#1 ***
c001    movlw d'01'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c002                     ; .. then check next

#ifdef doppelt_no
 #ifdef funktion_weichei
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,0               ; doppeltes Ausfhren und
         goto c002                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,0                 ; STATUS Bit#0 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_1

; *** ADR Gruen-#2 ***
c002
#ifndef skip_g2
        movlw d'02'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c003                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,1               ; doppeltes Ausfhren und
         goto c003                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,1                 ; STATUS Bit#1 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_2
#endif ;skip_g2

; *** ADR Gruen-#3 ***
c003
#ifndef skip_g3
        movlw d'03'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c004                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,2               ; doppeltes Ausfhren und
         goto c004                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,2                 ; STATUS Bit#2 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_3
#endif ;skip_g3

; *** ADR Gruen-#4 ***
c004
#ifndef skip_g4
        movlw d'04'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c005                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,3               ; doppeltes Ausfhren und
         goto c005                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,3                 ; STATUS Bit#3 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_4
#endif ;skip_g4

; *** ADR Gruen-#5 ***
c005
#ifndef skip_g5
        movlw d'05'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c006                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,4               ; doppeltes Ausfhren und
         goto c006                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,4                 ; STATUS Bit#5 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_5
#endif ;skip_g5

; *** ADR Gruen-#6 ***
c006    
#ifndef skip_g6
        movlw d'06'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c007                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,5               ; doppeltes Ausfhren und
         goto c007                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,5                 ; STATUS Bit#6 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_6
#endif ;skip_g6

; *** ADR Gruen-#7 ***
c007    
#ifndef skip_g7
        movlw d'07'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c008                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,6               ; doppeltes Ausfhren und
         goto c008                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,6                 ; STATUS Bit#7 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_7
#endif ;skip_g7

; *** ADR Gruen-#8 ***
c008    
#ifndef skip_g8
        movlw d'08'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c009                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS,7               ; doppeltes Ausfhren und
         goto c009                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS,7                 ; STATUS Bit#8 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_8
#endif ;skip_g8

; *** ADR Gruen-#9 ***
c009    
#ifndef skip_g9
        movlw d'09'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c010                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,0             ; doppeltes Ausfhren und
         goto c010                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,0               ; STATUS_2 Bit#0 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_9
#endif ;skip_g9

; *** ADR Gruen-#9 nur Relais8 ***
#ifdef funktion_relais8
        movlw d'09'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c015                     ; .. then check next
                                       ; doppeltes Ausfhren immer mglich,
                                       ; .. sonst geht EIN ber #9 und AUS
                                       ; .. ber #1-8 nicht
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_9
#endif ;relais8

; *** ADR Gruen-#10 ***
c010    
#ifndef skip_g10
        movlw d'10'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c011                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,1             ; doppeltes Ausfhren und
         goto c011                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,1               ; STATUS_2 Bit#1 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_10
#endif ;skip_g10

; *** ADR Gruen-#11 ***
c011    
#ifndef skip_g11
        movlw d'11'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c012                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,2             ; doppeltes Ausfhren und
         goto c012                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,2               ; STATUS_2 Bit#2 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_11
#endif ;skip_g11

; *** ADR Gruen-#12 ***
c012    
#ifndef skip_g12
        movlw d'12'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c013                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,3             ; doppeltes Ausfhren und
         goto c013                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,3               ; STATUS_2 Bit#3 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_12
#endif ;skip_g12

; *** ADR Gruen-#13 ***
c013    
#ifndef skip_g13
        movlw d'13'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c014                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,4             ; doppeltes Ausfhren und
         goto c014                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,4               ; STATUS_2 Bit#4 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_13
#endif ;skip_g13

; *** ADR Gruen-#14 ***
c014    
#ifndef skip_g14
        movlw d'14'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c015                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,5             ; doppeltes Ausfhren und
         goto c015                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,5               ; STATUS_2 Bit#5 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_14
#endif ;skip_g14

; *** ADR Gruen-#15 ***
c015    
#ifndef skip_g15
        movlw d'15'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c016                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,6             ; doppeltes Ausfhren und
         goto c016                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,6               ; STATUS_2 Bit#5 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_15
#endif ;skip_g15

; *** ADR Gruen-#16 ***
c016    
#ifndef skip_g16
        movlw d'16'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c017                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfsc R_STATUS_2,7             ; doppeltes Ausfhren und
         goto c017                     ; .. Speichern verhindern
#endif ;doppelt_no

        bsf R_STATUS_2,7               ; STATUS_2 Bit#5 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call gruen_16
#endif ;skip_g16

c017
; </editor-fold>

;   ------------------------------
;   --- Check auf Befehl-Rot   ---
;   ------------------------------
check2
#ifdef checkred_yes
; <editor-fold defaultstate="collapsed" desc="Erkennung ROT">

#ifdef motorola
#ifdef test_befehlserkennung ; C0 Routine CHECK -> Check2
       bsf PORTC,0 
       bcf PORTC,0
#endif
        movf  R_ADR_3a,W               ; compare R_ADR_3a and R_DEKADR_3
        subwf R_DEKADR_3_L,W
        btfss STATUS,Z                 ; if not the same
 #ifdef offcommand_yes 
         goto check3                   ; .. then check off-comand
 #else
         goto check_next               ; .. then check next adress
 #endif
#endif ;motorola

#ifdef dcc
        movf  R_ADR_2a,W               ; compare R_ADR_2a and R_DEKADR_2
        subwf R_DEKADR_2_L,W
        btfss STATUS,Z                 ; if not the same
 #ifdef offcommand_yes 
         goto check3                   ; .. then check off-comand
 #else
         goto check_next               ; .. then check next adress
 #endif

        movf  R_ADR_1a,W               ; compare R_ADR_2a and R_DEKADR_2
        subwf R_DEKADR_1_L,W
        btfss STATUS,Z                 ; if not the same
         goto check_next               ; .. then check next adress
#endif ;dcc
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Auswertung ROT">
; *** ADR Rot-#1 ***
c101    movlw d'01'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c102                     ; .. then check next

#ifdef doppelt_no
 #ifdef funktion_weichei
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,0               ; doppeltes Ausfhren und
         goto c102                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,0                 ; STATUS Bit#1 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_1

; *** ADR Rot-#2 ***
c102    
#ifndef skip_r2
        movlw d'02'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c103                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,1               ; doppeltes Ausfhren und
         goto c103                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,1                 ; STATUS Bit#2 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_2
#endif ;skip_r2

; *** ADR Rot-#3 ***
c103    
#ifndef skip_r3
        movlw d'03'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c104                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,2               ; doppeltes Ausfhren und
         goto c104                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,2                 ; STATUS Bit#3 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_3
#endif ;skip_r3

; *** ADR Rot-#4 ***
c104    
#ifndef skip_r4
        movlw d'04'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c105                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,3               ; doppeltes Ausfhren und
         goto c105                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,3                 ; STATUS Bit#4 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_4
#endif ;skip_r4

; *** ADR Rot-#5 ***
c105    
#ifndef skip_r5
        movlw d'05'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c106                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,4               ; doppeltes Ausfhren und
         goto c106                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,4                 ; STATUS Bit#5 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_5
#endif ;skip_r5

; *** ADR Rot-#6 ***
c106    
#ifndef skip_r6
        movlw d'06'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c107                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,5               ; doppeltes Ausfhren und
         goto c107                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,5                 ; STATUS Bit#6 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_6
#endif ;skip_r6

; *** ADR Rot-#7 ***
c107    
#ifndef skip_r7
        movlw d'07'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c108                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,6               ; doppeltes Ausfhren und
         goto c108                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,6                 ; STATUS Bit#7 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_7
#endif ;skip_r7

; *** ADR Rot-#8 ***
c108    
#ifndef skip_r8
        movlw d'08'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c109                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS,7               ; doppeltes Ausfhren und
         goto c109                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS,7                 ; STATUS Bit#8 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_8
#endif ;skip_r8

; *** ADR Rot-#9 ***
c109    
#ifndef skip_r9
        movlw d'09'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c110                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS_2,0             ; doppeltes Ausfhren und
         goto c110                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS_2,0               ; STATUS_2 Bit#0 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_9
#endif ;skip_r9

; *** ADR Rot-#9 nur Relais8 ***
#ifdef funktion_relais8
        movlw d'09'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c111                     ; .. then check next
                                       ; doppeltes Ausfhren immer mglich,
                                       ; .. sonst geht EIN ber #9 und AUS
                                       ; .. ber #1-8 nicht
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_9
#endif ;relais8

; *** ADR Rot-#10 ***
c110    
#ifndef skip_r10
        movlw d'10'                    ;
        subwf R_CNT_ADR,W
        btfss STATUS,Z                 ; if not the same
         goto c111                     ; .. then check next

#ifdef doppelt_no
 #ifdef doppelt_mode3_yes
        btfsc R_MODE,2
         goto $+5                      ; doppeltes Ausfhren mglich
 #endif
        btfss R_STATUS_2,1             ; doppeltes Ausfhren und
         goto c111                     ; .. Speichern verhindern
#endif ;doppelt_no

        bcf R_STATUS_2,1               ; STATUS_2 Bit#1 on
        bsf F_SAVE_STATUS              ; set Flag for save
        call rot_10
#endif ;skip_r10

c111    goto check_next
; </editor-fold>
#endif ; checkred_yes

; <editor-fold defaultstate="collapsed" desc="OFF Befehl">
;   ------------------------------
;   --- Check auf Befehl OFF   ---
;   ------------------------------
#ifdef offcommand_yes
check3
        ; ist ein Abarbeiten des OFF-Befehl zugelassen (neu v3.51)??? Wenn NEIN, dann ABBRUCH
        movf R_OFF_OK,W
        andlw b'11110000'              ; neu V3.58: nur Bit 7-4 betrachten
        sublw h'a0'                    ; nur wenn dieses Bitmuster in EE-Zelle steht (neu V3.58: nur Bit 7-4 betrachten)
        btfsc STATUS,Z                 ; .. wird abgebrochen
         goto check_next               ; .. (d.h. check_next)

#ifdef motorola
        btfsc R_POL,0                  ; if inverse polarity
        goto check3a                   ; ...the jump
        btfsc R_DEKADR_3_L,0           ; if bit #0 not 0
         goto check_next               ; .. then check next adress
        goto c301
check3a
        btfss R_DEKADR_3_L,0             ; if bit #0 not 1
         goto check_next               ; .. then check next adress
        goto c301
#endif
#ifdef dcc
        btfsc R_DEKADR_2_L,4           ; if bit #4 not 0
         goto check_next               ; .. then check next adress
        goto c301
#endif

; *** ADR Off-Command ***
c301    call offcommand
#endif ; offcommand_yes
; </editor-fold>

; <editor-fold defaultstate="collapsed" desc="Next">
;   ------------------------
;   --- read next adress ---
;   ------------------------
check_next
#ifdef test_befehlserkennung ; C5 Routine CHECK -> Check_next
       bsf PORTC,5 
       bcf PORTC,5
#endif

        incf R_CNT_ADR,F               ; next Adress
        movlw CONST_ADRZAHL+1          ; Anzahl der Adressen
        subwf R_CNT_ADR,W              ; W = R_CNT_ADR - W = R_CNT_ADR - CONST_ADRZAHL
        btfsc STATUS,C                 ; if C=1, -> R_CNT_ADR >= CONST_ADRZAHL
         return                        ; .. then return
        goto check_loop
; </editor-fold>
; </editor-fold>

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save               **
;** wait for adress and save in **
;** EEPROM                      **
;*********************************
;*********************************
;*********************************
;*********************************
save
; <editor-fold defaultstate="collapsed" desc="Save Routine">
#ifdef funktion_sand_4 ; Taster-Funktion nicht fr Adress-Lerning definiert
        btfss R_MODE,0                 ; wenn R_MODE#0 nicht gesetzt
         return                        ; .. keine Lern-Funktion fr SW1 (Port A,3)
#endif ;sand_4
#ifdef funktion_sand_ei ; Taster-Funktion nicht fr Adress-Lerning definiert
        btfss R_MODE,0                 ; wenn R_MODE#0 nicht gesetzt
         return                        ; .. keine Lern-Funktion fr SW1 (Port A,3)
#endif ;sand_ei

        movlw d'01'                    ; alle auf "1" -> Durchlauf aller Routinen beinm ersten Aufruf
        movwf R_CNT_TGL                ; preload R_CNT_TGL wg. Blinken
        movwf R_CNT1_TGL               ; preload R_CNT1_TGL
        movwf R_CNT2_TGL               ; preload R_CNT2_TGL
;        movwf R_CNT3_TGL               ; preload R_CNT3_TGL
        movwf R_CNT_ADR                ; preload R_CNT_ADR

;#ifdef funktion_schalten
;        movlw h'04'                    ; "schalten" wird auch fr Relais-1 Dekoder verwendet.
;        bsf STATUS,RP0                 ; -> BANK 1
;        btfsc TRISA,5                  ; hier jedoch nur eine Adresse (die 4.) lernen
;         movwf R_CNT_ADR               ; TRISA#5 ist bei Relais-1 =1 (Input)
;        bcf STATUS,RP0                 ; -> BANK 0
;#endif

        clrf PORTA
        clrf PORTC
        bcf R_FLAG,7                   ; Clear Flag fr Ausblenden von Abschaltbefehl
                                       ; und Befehls-Wiederholer
;#ifdef savetoggle_yes
        call save_toggle_start         ; erstes Ausgangsmuster laden
;#endif ; savetoggle_yes

#ifdef save_use_timer1
        call save_toggle_start         ; erstes Ausgangsmuster laden (nur R_TGLA/R_TGLC, nicht in die Ports)
        call save_signalisierung       ; Timer laden und aus R_TGLA/R_TGLC die Ports auch ansteuern
#endif ; save_use_timer1

;   --------------------------------------------
;   --- Schleife fr CONST_ADRZAHL Adressen  ---
;   --------------------------------------------
save_loop
        call polarity

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle (FF to ZERO) -> ~ 4.1 ms
         goto save_bit_check

;   --------------------------------------------
;   --- Durchlauf alle 4 ms                  ---
;   --------------------------------------------
;   -------------------------
;   --- Read Input BUTTON ---
;   -------------------------
        bcf STATUS,C
        btfsc PORTA,3                  ; if Button
         bsf STATUS,C
        rlf R_BUTTON,F                 ;
        movlw b'11110000'              ;
        subwf R_BUTTON,W
        btfsc STATUS,Z                 ; if the same
         goto save_next                ; .. then next adress

#ifdef savetoggle_yes                  ; beim Adress-Learning Ausgnge togglen 
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> LEDs off
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTC                    ; .. -> LEDs off
#endif

#ifdef saveflash_porta5 ; Signalisierung nur ber ein OUT
        movlw d'80'                    ; auf "80" -> schnelleres blinken
        movwf R_CNT1_TGL               ; preload R_CNT1_TGL
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf PORTA,5                   ; .. -> LEDs off
#endif ;saveflash_porta5

#ifdef saveflash_portc0 ; Signalisierung nur ber ein OUT
        movlw d'80'                    ; auf "80" -> schnelleres blinken
        movwf R_CNT1_TGL               ; preload R_CNT1_TGL
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf PORTC,0                   ; .. -> LEDs off
#endif ;saveflash_portc0

#ifdef saveflashall_yes
        movlw d'80'                    ; auf "80" -> schnelleres blinken
        movwf R_CNT1_TGL               ; preload R_CNT1_TGL
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> LEDs off
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTC                    ; .. -> LEDs off
#endif ;saveflashall_yes

#ifdef funktion_relais8
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         bcf PORTA,1                   ; .. -> LEDs off
#endif
#ifdef funktion_weiche88
        btfss R_CNT2_TGL,5             ; if CNT2 < 64
         bcf PORTA,1                   ; .. -> LED2 off
#endif

#ifdef save_use_timer1
        btfss PIR1,TMR1IF              ; if Timer1 overflow
         goto save_t1_skip
        bcf PIR1,TMR1IF                ; clear Flag
        incf R_VERZ,F                  ; R_VERZ = R_VERZ + 1

        movf R_MAKESLOW,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R_MAKESLOW
        btfsc STATUS,Z                 ; if Z=1, -> R_VERZ = R_MAKESLOW
         goto save_t1_OFF              ; ...then jump

        incf R_MAKESLOW,W
        subwf R_VERZ,W                 ; W = R_VERZ - W = R_VERZ - R_MAKESLOW +1
        btfsc STATUS,Z                 ; if Z=1, -> R_VERZ = R_MAKESLOW + 1
         call save_signalisierung      ; nchste Weichenansteuerung

 #ifdef funktion_weichei ; bei Weichei - 2. Adresse schnell togglen
        movlw h'02'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_signalisierung      ; .. then jump
 #endif
        goto save_t1_skip

        ; (Weichen)-Ausgang abschalten
 #ifdef funktion_weiche88
save_t1_OFF
        movf PORTA,W                    ; Weichen AUS, aber nicht LED2 (PORTA,1)
        andlw b'00000010'
        movwf PORTA
        clrf PORTC                     ; Weichen AUS
 #endif
 #ifdef funktion_weichei
save_t1_OFF
        clrf PORTA
        clrf PORTC                     ; Weichen AUS
 #endif
 #ifdef funktion_weichzwei
save_t1_OFF
        clrf PORTA
        clrf PORTC                     ; Weichen AUS
 #endif
 #ifdef funktion_spur1
save_t1_OFF
        clrf PORTA
        clrf PORTC                     ; Weichen AUS
 #endif
save_t1_skip
#endif ; save_use_timer1

        decfsz R_CNT2_TGL,F            ; decrement Counter 2 for Toggle (CNT_TGL to ZERO)
         goto save_bit_check

;   -------------------------------------------------
;   --- Counter2 Laden (abhngig von Dekoder-Typ) ---
;   -------------------------------------------------
        movlw CNT_TGL                  ; Preload
        movwf R_CNT2_TGL               ; .. Counter 2 R_CNT2_TGL
 #ifdef funktion_weichei ; bei Weichei - 2. Adresse schnell togglen
        movlw h'02'                    ;
        subwf R_CNT_ADR,W
        movlw d'150'                   ; 
        btfsc STATUS,Z                 ; if same
         movwf R_CNT2_TGL              ; .. then Counter 2 R_CNT2_TGL = 150
 #endif

 #ifdef funktion_signal_hl ; bei HL: 3. / 4. Adresse schnell togglen
        movlw h'03'                    ;
        subwf R_CNT_ADR,W
        movlw d'50'                   ; 
        btfsc STATUS,Z                 ; if same
         movwf R_CNT2_TGL              ; .. then Counter 2 R_CNT2_TGL = 150

        movlw h'04'                    ;
        subwf R_CNT_ADR,W
        movlw d'50'                   ; 
        btfsc STATUS,Z                 ; if same
         movwf R_CNT2_TGL              ; .. then Counter 2 R_CNT2_TGL = 150
 #endif

        ;   ---------------------------------------
        ;   --- Signalisierung Ausgnge togglen ---
        ;   ---------------------------------------
#ifndef saveflashall_yes
#ifndef saveflash_portc0 ; Signalisierung nur ber ein OUT
#ifndef saveflash_porta5 ; Signalisierung nur ber ein OUT
#ifndef save_use_timer1
        call save_signalisierung       ; nchste Ansteuerung
#endif ; not save_use_timer1
#endif ; not saveflash_porta5 ; Signalisierung nur ber ein OUT
#endif ; not saveflash_portc0 ; Signalisierung nur ber ein OUT
#endif ; not saveflashall_yes

;   -----------------------------------------
;   --- Signalisierung durch Blinken      ---
;   -----------------------------------------
#ifdef saveflash_yes                         
        call save_flash
#endif ; saveflash_yes



;   -------------------------------------------------
;   --- Check auf Adresse und ansonsten SAVE_LOOP ---
;   -------------------------------------------------
save_bit_check
        btfss R_FLAG,F_CMD             ; if not valid command
         goto save_loop                ; ... then stay in loop
        bcf R_FLAG,F_CMD               ; clear flag

;   -------------------------
;   --- Adresse normieren ---
;   -------------------------
        btfss R_POL,0                  ; if not inverse polarity
         goto save_norm               ; .. then jump
        ; *** if inverse then invert R_DEKADR_x_L - Register ***
        comf R_DEKADR_1_L,W            ; if inverse polarity,
        andlw b'00000011'              ; mask unused bits #2-#7
        movwf R_DEKADR_1_L
        comf R_DEKADR_2_L,F            ; .. then complement
        comf R_DEKADR_3_L,F

;   ----------------------------------------------
;   --- Wiederholer & Abschaltbefehl verwerfen ---
;   ----------------------------------------------
save_norm; ** ab keine invertierten Signale mehr **
#ifdef motorola
        btfss R_DEKADR_3_L,0           ; clear Flag "Done", if this is a
         bcf R_FLAG,7                  ; .. "switch off" command xx xx xx 00
        btfss R_DEKADR_3_L,0           ; Jump, if this is a
         goto save_loop                ; .. "switch off" command xx xx xx 00
        btfsc R_FLAG,7                 ; if Flag#7 "Done" is set, them the command is
         goto save_loop                ; .. already done and Jumo
        btfsc R_DEKADR_2_L,0           ; Jump, if this is a "3FF" = DCC-misunderstood
         goto save_loop                ; .. again
        bsf R_FLAG,7                   ; else proceed and set Flag#7 "Done"
#endif
#ifdef dcc
        btfss R_DEKADR_3_L,0           ; NOP-Befehl verhindern
         goto save_loop                ; .. ignorieren (muss 0 sein)

        movf R_DEKADR_1_L,W
        subwf R_DEKADR_1_L_old,W
        btfsc STATUS,Z
         goto save_loop                ; wait again if 1_L was same in last command

        movf R_DEKADR_1_L,W
        movwf R_DEKADR_1_L_old

        btfss R_DEKADR_2_L,4           ; Abspeichern von Abschaltbefehlen
         goto save_loop                ; .. verhindern (muss 1 sein)
#endif
;   ---------------------------
;   --- Adresse abspeichern ---
;   ---------------------------
        bcf INTCON,GIE                 ; disable IRs
;       **** write R_DEKADR_3_L ****
        movlw h'20'                    ; "big offset"
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $20+d16+$31 = $61
#endif ; skip_g16
        movwf R_EEADR                  ; Address: h'20' + R_CNT_ADR
        movf R_DEKADR_3_L,W
        movwf R_EEDAT                  ; Data: R_DEKADR_3_L
        call saveroutine

;       **** write R_DEKADR_2_L ****
        movlw h'10'                    ; "big offset"
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $20+d16+$31 = $61
#endif ; skip_g16
        movwf R_EEADR                  ; Address: h'10' + R_CNT_ADR
        movf R_DEKADR_2_L,W
        movwf R_EEDAT                  ; Data: R_DEKADR_2_L
        call saveroutine

;       **** write R_DEKADR_1_L ****
        movlw h'00'                    ; "big offset"
        addwf R_CNT_ADR,W              ; "base address"
#ifndef skip_g16
        btfsc R_CNT_ADR,4
        addlw h'31'                    ; @ #16: $20+d16+$31 = $61
#endif ; skip_g16
        movwf R_EEADR                  ; Address: h'00' + R_CNT_ADR
        movf R_DEKADR_1_L,W
        movwf R_EEDAT                  ; Data: R_DEKADR_1_L
        call saveroutine
        bsf INTCON,GIE                 ; enable IRs

;   ------------------------
;   --- read next adress ---
;   ------------------------
save_next
; fr Dekoder, die im SAVE-Modus mit 2 Ausgngen alternierend togglen,
; muss hier fr die entsprechende Adress-Nummer jeweils ein Ausgang
; eingeschaltet werden
.
        clrf PORTA
        clrf PORTC
#ifdef savetoggle_yes
        call save_toggle_next
#endif        
#ifdef save_use_timer1
        call save_toggle_next          ; nchstes Ausgangsmuster laden (nur R_TGLA/R_TGLC, nicht in die Ports)
        ;call save_signalisierung       ; Timer laden und aus R_TGLA/R_TGLC die Ports auch ansteuern
#endif ; save_use_timer1


#ifdef funktion_relais8 ; for Address #9 toggle all outputs alternating
        movlw CONST_ADRZAHL-1        ; Anzahl der Adressen
        subwf R_CNT_ADR,W              ; W = R_CNT_ADR - W = R_CNT_ADR - CONST_ADRZAHL
        movlw b'00100000'              ; Relais K4/                           (Z not affected)
        btfsc STATUS,Z                 ; if Z=0, -> R_CNT_ADR = CONST_ADRZAHL (Z not affected)
         movwf PORTA                   ; .. do it                             (Z not affected)
        movlw b'00101010'              ; Relais K8/ /K6/ /K2/                 (Z not affected)
        btfsc STATUS,Z                 ; if Z=0, -> R_CNT_ADR = CONST_ADRZAHL
         movwf PORTC                   ; .. do it
#endif ;relais8     

        incf R_CNT_ADR,F               ; next Adress
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        movlw d'1'                     ; Preload
        movwf R_CNT2_TGL               ; .. Counter 2 R_CNT2_TGL

        movlw CONST_ADRZAHL+1          ; Anzahl der Adressen
        subwf R_CNT_ADR,W              ; W = R_CNT_ADR - W = R_CNT_ADR - CONST_ADRZAHL
        btfss STATUS,C                 ; if C=0, -> R_CNT_ADR < CONST_ADRZAHL
         goto save_loop                ; .. then loop again

;   -----------------------------------------
;   --- Ausgnge schalten gemss R_STATUS ---
;   -----------------------------------------
#ifdef funktion_dreiwege
        clrf R_STATE
        return                        ; nicht fr 3-Wege
#endif
        call status_to_output

        return
; </editor-fold>

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save_flash         **
;** Signalisierung durch        **
;** Blink-Anzahl                **
;** Input: R_CNT_ADR 
;**        R_CNT_TGL (changed)
;** Output; PORTA / POST C
;*********************************
;*********************************
;*********************************
;*********************************
;   -----------------------------------------
;   --- Signalisierung durch Blink-Anzahl ---
;   -----------------------------------------
; <editor-fold defaultstate="collapsed" desc="Save Signalisierung Blink-Anzahl">
#ifdef saveflash_yes
save_flash
        movf R_CNT_ADR,W               ; if R_CNT_TGL (Anzahl BLinken)
        movf R_CNT_TGL,F               ; .. = ZERO
        btfsc STATUS,Z                 ; .. then load
         movwf R_CNT_TGL               ; .. R_CNT_ADR (# Adresse) into it
        btfsc STATUS,Z                 ; .. 
         return                        ; .. and jump (dies generiert die PAUSE)

  ;   --- Signalisierung Blinken EINES OUT  ---
 #ifdef funktion_relais8  ; toggelt Adress-individuelle OUT und Blink-Anzahl auf A1
        bsf PORTA,1                    ; if R_CNT_TGL > ZERO -> LED on
 #endif
 #ifdef funktion_weiche88 ; toggelt Adress-individuelle OUT und Blink-Anzahl auf A1
        bsf PORTA,1                    ; if R_CNT_TGL > ZERO -> LED on
 #endif
 #ifdef saveflash_porta5   ; Signalisierung nur via Blink-Anzahl auf einem OUT (A5)
        bsf PORTA,5                    ; if R_CNT_TGL > ZERO -> LED on
 #endif ;saveflash_porta5
 #ifdef saveflash_portc0   ; Signalisierung nur via Blink-Anzahl auf einem OUT (A5)
        bsf PORTC,0                    ; if R_CNT_TGL > ZERO -> LED on
 #endif ;saveflash_portc0

  ;   --- Signalisierung Blinken ALLER OUT  ---
 #ifdef saveflashall_yes
        movlw b'00010001'
        movwf PORTA                    ; if R_CNT_TGL > ZERO -> LED on
        movlw b'00010011'
        movwf PORTC                    ; if R_CNT_TGL > ZERO -> LED on
 #endif ;saveflashall_yes

  ;   --- Counter DEC fr Anzahl-BLinken    ---
        decf R_CNT_TGL,F               ; counter fr ANZAHL-Blinke --
        return
#endif ;saveflash_yes
; </editor-fold>

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save_signalisierung**
;** Umschalten auf nchstes     **
;** Ausgangsmuster              **
;** (Togglen der Ausgnge)      **
;*********************************
;*********************************
;*********************************
;*********************************
; <editor-fold defaultstate="collapsed" desc="Save Signalisierung">
#ifndef saveflashall_yes
#ifndef saveflash_portc0 ; Signalisierung nur ber ein OUT
#ifndef saveflash_porta5 ; Signalisierung nur ber ein OUT

save_signalisierung 
#ifdef save_use_timer1                 ; benutzt TIMER fr Ausgnge abschalten
        clrf TMR1L
        clrf TMR1H                     ; clear Time Out Timer
        clrf R_VERZ
        bcf PIR1,TMR1IF                ; clear Flag
#endif ; save_use_timer1

        movlw h'01'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_01                  ; .. then check

#ifndef skip_g2
        movlw h'02'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_02                  ; .. then check
#endif

#ifndef skip_g3
        movlw h'03'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_03                  ; .. then check
#endif

#ifndef skip_g4
        movlw h'04'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_04                  ; .. then check
#endif

#ifndef skip_g5
        movlw h'05'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_05                  ; .. then check
#endif

#ifndef skip_g6
        movlw h'06'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_06                  ; .. then check
#endif

#ifndef skip_g7
        movlw h'07'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_07                  ; .. then check
#endif
 
#ifndef skip_g8
        movlw h'08'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_08                  ; .. then check
#endif

#ifndef skip_g9
        movlw h'09'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_09                  ; .. then check
#endif
 
#ifdef funktion_relais8
        movlw h'09'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_09                  ; .. then check
#endif ;relais8

#ifndef skip_g10
        movlw h'0a'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_10                  ; .. then check
#endif

#ifndef skip_g11
        movlw h'0b'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_11                  ; .. then check
#endif

#ifndef skip_g12
        movlw h'0c'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_12                  ; .. then check
#endif

#ifndef skip_g13
        movlw h'0d'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_13                  ; .. then check
#endif

#ifndef skip_g14
        movlw h'0e'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_14                  ; .. then check
#endif

#ifndef skip_g15
        movlw h'0f'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_15                  ; .. then check
#endif

#ifndef skip_g16
        movlw h'10'                    ;
        subwf R_CNT_ADR,W
        btfsc STATUS,Z                 ; if same
         call save_16                  ; .. then check
#endif

        return

#endif ; not saveflash_porta5 ; Signalisierung nur ber ein OUT
#endif ; not saveflash_portc0 ; Signalisierung nur ber ein OUT
#endif ; not saveflashall_yes
; </editor-fold>

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save_status        **
;** save relay-positions        **
;** into EEPROM $10             **
;*********************************
;*********************************
;*********************************
;*********************************
save_status
; <editor-fold defaultstate="collapsed" desc="Save Status">
        btfss F_SAVE_STATUS            ; check for "do-save"-flag
         return
        bcf F_SAVE_STATUS              ; clear DoSave-Flag
        movlw h'10'
        movwf R_EEADR                  ; Address: h'10'
        movf R_STATUS,W
        movwf R_EEDAT                  ; Data: R_STATUS
        call saveroutine

#ifdef use_status2
        movlw h'30'
        movwf R_EEADR                  ; Address: h'30'
        movf R_STATUS_2,W
        movwf R_EEDAT                  ; Data: R_STATUS
        call saveroutine
#endif

        return
; </editor-fold>

;*******************************************
;*******************************************
;*******************************************
;*******************************************
;** Routine: saveroutine                  **
;** routine for saving in EEPROM          **
;** uses  R_EEADR (Address)               **
;** and   R_EEDAT (Data)                  **
;*******************************************
;*******************************************
;*******************************************
;*******************************************
saveroutine
#ifdef use16f6xx
; <editor-fold defaultstate="collapsed" desc="SaveRoutine 6xx">
        bcf INTCON,GIE                 ; disable IRs
        movf R_EEDAT,W
        bsf STATUS,RP0                 ; -> BANK 1
        bsf EECON1,WREN                ; enable write
        movwf EEDATA                   ; R_EEDAT into EEDATA
#ifdef use16f684
        bcf STATUS,RP0                 ; -> BANK 0
#endif
        movf R_EEADR,W
#ifdef use16f684
        bsf STATUS,RP0                 ; -> BANK 1
#endif
        movwf EEADR                    ; R_EEADR into EEDATA
        movlw 0x55
        movwf EECON2
        movlw 0xAA
        movwf EECON2
        bsf EECON1,WR                  ; start writing
        clrwdt
        btfsc EECON1,WR
         goto $-2                      ; wait for finish
        bcf EECON1,WREN                ; disable write
        bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; enable IRs
        return
; </editor-fold>
#endif ; use16f6xx

;*********************************
;*********************************
;*********************************
;*********************************
;** INCLUDE-Files               **
;*********************************
;*********************************
;*********************************
;*********************************
; <editor-fold defaultstate="collapsed" desc="INCLUDE-Files">
#ifdef use_mplab

 #ifdef use16f684
  #include <sub_test_16f684.asm>
 #endif

 #ifdef funktion_wz_blinkmuster
  #include <sub_blinkmuster.asm>
  #include <state_blinkmuster.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_spur1
  #include <sub_wz_spur1.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_signal
  #include <sub_wz_signal.asm>
  #include <state_wz_signal.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_dreiwege
  #include <sub_wz_dreiwege.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_signal4
  #include <sub_wz_signal4.asm>
  #include <state_wz_signal4.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_signal_hl
  #include <sub_wz_signal_hl.asm>
  #include <state_wz_signal_hl.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_signal_straba
  #include <sub_wz_signal_straba.asm>
  #include <state_wz_signal_straba.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_weichei
  #include <sub_we_weichei.asm>
  #include <state_we_weichei.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_andreas
  #include <sub_we_andreas.asm>
  #include <state_we_andreas.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_weichzwei
  #include <sub_wz_weichzwei.asm>
  #include <eeprom_standard.asm>
 #endif
 #ifdef funktion_schalten
  #include <sub_wz_schalten.asm>
  #include <eeprom_standard.asm>
 #endif
#endif ;use_mplab

#ifndef use_mplab
 #ifdef funktion_wz_blinkmuster
  #include <..\sub_blinkmuster.asm>
  #include <..\state_blinkmuster.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_spur1
  #include <..\sub_wz_spur1.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_signal
  #include <..\sub_wz_signal.asm>
  #include <..\state_wz_signal.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_dreiwege
  #include <..\sub_wz_dreiwege.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_signal4
  #include <..\sub_wz_signal4.asm>
  #include <..\state_wz_signal4.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_signal_hl
  #include <..\sub_wz_signal_hl.asm>
  #include <..\state_wz_signal_hl.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_signal_straba
  #include <..\sub_wz_signal_straba.asm>
  #include <..\state_wz_signal_straba.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_weichei
  #include <..\sub_we_weichei.asm>
  #include <..\state_we_weichei.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_andreas
  #include <..\sub_we_andreas.asm>
  #include <..\state_we_andreas.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_weichzwei
  #include <..\sub_wz_weichzwei.asm>
  #include <..\eeprom_standard.asm>
 #endif
 #ifdef funktion_schalten
  #include <..\sub_wz_schalten.asm>
  #include <..\eeprom_standard.asm>
 #endif
#endif ;mplab

;*********************************
;*********************************
;*********************************
;*********************************
;** INCLUDE-Files 16F630        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_mplab
 #ifdef funktion_weiche88
  #include <sub_weiche88.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_gas
  #include <sub_led_gas.asm>
  #include <state_led_gas.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_ampel
  #include <sub_led_ampel.asm>
  #include <state_led_ampel.asm>
  #include <eeprom_ampel.asm>
 #endif
 
 #ifdef funktion_led_andreas
  #include <sub_led_andreas.asm>
  #include <state_led_andreas.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_relais8
  #include <sub_relais8.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_schalten
  #include <sub_led_schalten.asm>
  #include <eeprom_led_schalten.asm>
 #endif
 
 #ifdef funktion_led_signal2
  #include <sub_led_signal2.asm>
  #include <state_led_signal2.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_signal4
  #include <sub_led_signal4.asm>
  #include <state_led_signal4.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_signal_sbb
  #include <sub_led_signal_sbb.asm>
  #include <state_led_signal_sbb.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_blink
  #include <sub_led_blink.asm>
  #include <state_led_blink.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_kirmes
  #include <sub_led_kirmes.asm>
  #include <state_led_kirmes.asm>
  #include <eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_neon
  #include <sub_led_neon.asm>
  #include <state_led_neon.asm>
  #include <eeprom_led_neon.asm>
 #endif
 
 #ifdef funktion_led_haus
  #include <sub_led_haus.asm>
  #include <state_led_haus.asm>
  #include <eeprom_led_haus.asm>
 #endif
 
 #ifdef funktion_led_feuer
  #include <sub_led_feuer.asm>
  #include <state_led_feuer.asm>
  #include <eeprom_standard.asm>
 #endif

 #ifdef funktion_led_schiebebuhne
  #include <sub_led_schiebebuhne.asm>
  #include <state_led_schiebebuhne.asm>
  #include <eeprom_standard.asm>
 #endif
#endif ;mplab
asjdhg
#ifndef use_mplab
 #ifdef funktion_weiche88
  #include <..\sub_weiche88.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_gas
  #include <..\sub_led_gas.asm>
  #include <..\state_led_gas.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_ampel
  #include <..\sub_led_ampel.asm>
  #include <..\state_led_ampel.asm>
  #include <..\eeprom_ampel.asm>
 #endif
 
 #ifdef funktion_led_andreas
  #include <..\sub_led_andreas.asm>
  #include <..\state_led_andreas.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_relais8
  #include <..\sub_relais8.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_schalten
  #include <..\sub_led_schalten.asm>
  #include <..\eeprom_led_schalten.asm>
 #endif
 
 #ifdef funktion_led_signal2
  #include <..\sub_led_signal2.asm>
  #include <..\state_led_signal2.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_signal4
  #include <..\sub_led_signal4.asm>
  #include <..\state_led_signal4.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_signal_sbb
  #include <..\sub_led_signal_sbb.asm>
  #include <..\state_led_signal_sbb.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_blink
  #include <..\sub_led_blink.asm>
  #include <..\state_led_blink.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_kirmes
  #include <..\sub_led_kirmes.asm>
  #include <..\state_led_kirmes.asm>
  #include <..\eeprom_standard.asm>
 #endif
 
 #ifdef funktion_led_neon
  #include <..\sub_led_neon.asm>
  #include <..\state_led_neon.asm>
  #include <..\eeprom_led_neon.asm>
 #endif
 
 #ifdef funktion_led_haus
  #include <..\sub_led_haus.asm>
  #include <..\state_led_haus.asm>
  #include <..\eeprom_led_haus.asm>
 #endif
 
 #ifdef funktion_led_feuer
  #include <..\sub_led_feuer.asm>
  #include <..\state_led_feuer.asm>
  #include <..\eeprom_standard.asm>
 #endif

 #ifdef funktion_led_schiebebuhne
  #include <..\sub_led_schiebebuhne.asm>
  #include <..\state_led_schiebebuhne.asm>
  #include <..\eeprom_standard.asm>
 #endif
#endif ;mplab

;*********************************
;*********************************
;*********************************
;*********************************
;** INCLUDE-Files 16F684        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_mplab
 #ifdef funktion_led_signal_001
  #include <sub_led_signal_v1_001.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_002
  #include <sub_led_signal_v3_002.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_003
  #include <sub_led_signal_v1_003.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_004
  #include <sub_led_signal_v1_004.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_005
  #include <sub_led_signal_v1_005.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_006
  #include <sub_led_signal_v1_006.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_007
  #include <sub_led_signal_v1_007.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_008
  #include <sub_led_signal_v1_008.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_009
  #include <sub_led_signal_v3_009.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_010
  #include <sub_led_signal_v1_010.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_011
  #include <sub_led_signal_v1_011.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_012
  #include <sub_led_signal_v2_012.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

#ifdef funktion_led_signal_020
  #include <sub_led_signal_v2_020.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_021
  #include <sub_led_signal_v2_021.asm>
  #include <state_signal.asm>
  #include <eeprom_signal_021.asm>     ; Buffer $40/50/60/70 verwendet
 #endif

 #ifdef funktion_led_signal_022
  #include <sub_led_signal_v2_022.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_030
  #include <sub_led_signal_v2_030.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_031
  #include <sub_led_signal_v2_031.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_032
  #include <sub_led_signal_v2_032.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_033
  #include <sub_led_signal_v2_033.asm>
  #include <state_signal.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_034
  #include <sub_led_signal_v3_034.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_040
  #include <sub_led_signa_v3_040.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_041
  #include <sub_led_signal_v3_041.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_042
  #include <sub_led_signal_v3_042.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_050
  #include <sub_led_signal_v3_050.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_051
  #include <sub_led_signal_v3_051.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_052
  #include <sub_led_signal_v3_052.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_060
  #include <sub_led_signal_v3_060.asm>
  #include <state_signal_v2.asm>
  #include <eeprom_signal.asm>   
 #endif

 #ifdef funktion_sand_ei
  #include <sub_sand_ei.asm>
  #include <state_sand_ei.asm>
  #include <eeprom_servo.asm>
 #endif

 #ifdef funktion_sand_4
  #include <sub_sand_4.asm>
  #include <state_sand_4.asm>
  #include <eeprom_servo.asm>
 #endif

 #ifdef funktion_led_blinkmuster
  #include <sub_led_blinkmuster.asm>
  #include <state_led_blinkmuster.asm>
  #include <eeprom_standard.asm>
 #endif
#endif ;mplab

#ifndef use_mplab
 #ifdef funktion_led_signal_001
  #include <..\sub_led_signal_v1_001.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_002
  #include <..\sub_led_signal_v3_002.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_003
  #include <..\sub_led_signal_v1_003.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_004
  #include <..\sub_led_signal_v1_004.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_005
  #include <..\sub_led_signal_v1_005.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_006
  #include <..\sub_led_signal_v1_006.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>  
 #endif

  #ifdef funktion_led_signal_007
  #include <..\sub_led_signal_v1_007.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_008
  #include <..\sub_led_signal_v1_008.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_009
  #include <..\sub_led_signal_v3_009.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_010
  #include <..\sub_led_signal_v1_010.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_011
  #include <..\sub_led_signal_v1_011.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_012
  #include <..\sub_led_signal_v2_012.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_020
  #include <..\sub_led_signal_v2_020.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>
 #endif

 #ifdef funktion_led_signal_021
  #include <..\sub_led_signal_v2_021.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal_021.asm>  ; Buffer $40/50/60/70 verwendet
 #endif

 #ifdef funktion_led_signal_022
  #include <..\sub_led_signal_v2_022.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>  
 #endif

 #ifdef funktion_led_signal_030
  #include <..\sub_led_signal_v2_030.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_031
  #include <..\sub_led_signal_v2_031.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_032
  #include <..\sub_led_signal_v2_032.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_033
  #include <..\sub_led_signal_v2_033.asm>
  #include <..\state_signal.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_034
  #include <..\sub_led_signal_v3_034.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_040
  #include <..\sub_led_signal_v3_040.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_041
  #include <..\sub_led_signal_v3_041.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_042
  #include <..\sub_led_signal_v3_042.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_050
  #include <..\sub_led_signal_v3_050.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_051
  #include <..\sub_led_signal_v3_051.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_052
  #include <..\sub_led_signal_v3_052.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

 #ifdef funktion_led_signal_060
  #include <..\sub_led_signal_v3_060.asm>
  #include <..\state_signal_v2.asm>
  #include <..\eeprom_signal.asm>   
 #endif

#ifdef funktion_sand_ei
  #include <..\sub_sand_ei.asm>
  #include <..\state_sand_ei.asm>
  #include <..\eeprom_servo.asm>
 #endif

 #ifdef funktion_sand_4
  #include <..\sub_sand_4.asm>
  #include <..\state_sand_4.asm>
  #include <..\eeprom_servo.asm>
 #endif

 #ifdef funktion_led_blinkmuster
  #include <..\sub_led_blinkmuster.asm>
  #include <..\state_led_blinkmuster.asm>
  #include <..\eeprom_standard.asm>
 #endif
#endif ;mplab
; </editor-fold>

      END

