; ********************************************************
; ** Version: V3.57 vom 04/03/2013                      **
; ********************************************************
; ** Include-File zu dekoder.asm                        **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; Copyright (C) 2008/09/10/11/12/13 Sven Brandt
; ********************************************************
; History:
; V3.57 - Adresse 9/10 hinzu
; V3.50 - 'all_dark_by...' hinzu, ermglicht Dunkeltastung des gesamten Signals durch Befehle
;       - Save-Blinkmuster ber #defines
;       - Fahrstrassenabhngige Adressen mglich
;       - Fahrstrassenabhngige Signalbilder mglich
; V3.47 - LED_Signal_011 hinzu, alle Outputs knnen zu einem Signal gehren (jetzt ber #defines), 16 Adressen mglich
; V3.45 - LED_Signal_010 hinzu, alle Outputs knnen zu einem Signal gehren
; V3.41 - corrections Save-Outputs
; V3.22 - first Version

;Mit diesem Include-File knnen Signale konfiguriert werden:
; - Fr LED-Dekoder (idR. mit 16F684)
; - 2 unabhngige Signale mglich
; - jedes Signalbild reagiert auf einen Digital-Befehl (keine Paar-Bildung)
; - 2. Signal kann vom 1. dunkelgetastet werden
; - Blinken aller Ausgnge mglich
; - Wechselblinken Ausgang 4/5 und 9/10 mglich
; - Signal 1 kann auf Ausgnge 1-8  zugreifen
; - Signal 2 kann auf Ausgnge 3-10 zugreifen
; - alle Ausgnge knnen zu einem Signal zugeordnet werden
; - max. 16 Adressen
; - konfigurierbar, ob das erste Signal auf 4/5/6/7 oder 8 Befehle reagiert
; - Dunkeltastung gesamtes Signal mglich, Ein- und Ausschalten ber eigene Befehle

; ================> Adresse
; -{CHECK}--------> R_STATUS ->R_STATUS_ON_S1/2_pre
; -{STATEMACHINE}-> R_STATUS_ON_S1/2_akt
; -{STATEMACHINE}-> R_STATEx
; -{LOOP}---------> LEDs

; in R_STATUS sind die Signalstellungen abgelegt.
; in R_STATUS_ON_S1_xxx bzw. R_STATUS_ON_S2_xxx sind die LED-Stellungen abgelegt.

; In R_STATUS_ON_S1/2_pre/akt und R_STATUS_ON_S1/2_pre/akt sind die
; Ausgnge als Flags vergeben:

; In dieser SW wird (wie bei der "LED_SIGNAL2") mit den Zwischen-Registern
; R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gearbeitet. In dieses zeigen die Flags an, ob
; ein Ausgang gesetzt / blinkend / aus sein soll.
; Im Gegensatz zu LED_SIGNAL2 wird hier jedoch nicht mit ROTen und GRNen Adressen
; gearbeitet, sondern es werden bis zu 14 GRNe Adressen verwendet.
; Die Signal-Bilder werden in R_STATUS und R_STATUS_2 abgelegt (8 Adressen = 8 Bits). In R_STATUS / R_STATUS_2
; liegt demnach die letzte bettigte Adresse und damit das anzuzeigene Signal-Bild.

; In den Routinen "gruen_1" usw. (oder "status_to_output" beim Prog-Start) werden dann
; die Flags in R_STATUS_ON_S1_pre und R_STATUS_ON_S2_pre gesetzt. Hier findet also die
; Umsetzung von Signal-Bild zu den LED-Ausgngen statt.
; Die STATE-Machine schaut sich dann nur die Flags in R_STATUS_ON_S1_akt an und entscheidet
; dann, ob die LED ein-/ausgeblendet oder auch blinken.
;

;----------------------------------------------------------------------
; Definition: welche Befehle (nicht OUTPUTs!) gruen_1..10 gehren zu welchem Signal
; - bei Signal-Dunkeltastung (use_all_dark) mssen die entsprechenden Befehle zu S2 zugeordnet werden (ansonsten ist R_STATUS nach Dunkel-Befehl nicht korrekt)
; - bei fahrwegabhngigen Adressen (gruenx_pagex_aktiv) sollten die Page-Befehle zu S2 gehren, damit Out9/10 ber die Page-Befehle verwendet werden knnen
; - bei fahrwegabhngigen Signalbildern (gruenx_depends_on_page) Adressen mssen die Page-Befehle zu S2 gehren (ansonsten ist R_STATUS nach Page-Wechsel nicht korrekt)
;----------------------------------------------------------------------
;#define aufteilung4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
;#define aufteilung6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
#define aufteilung8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2
;#define aufteilung_all ; keine Trennung in S1/S2. Alle Befehle gehren zum selben Signal

;----------------------------------------------------------------------
; Definition: welche Outputs (nicht Befehle!) Out_1..10 gehren zu welchem Signal
; - bei 'all_in_one' wird nichts aktiviert!
; - bei fahrwegabhngigen Adressen mssen diese Signalbilder zu S1 gehren (sonst geht die Buffer-Lsung nicht)!
;----------------------------------------------------------------------
;#define aufteilung_output4_x  ; gruen 1-4 gehrt zu S1, Rest zu S2
;#define aufteilung_output5_x  ; gruen 1-5 gehrt zu S1, Rest zu S2
;#define aufteilung_output6_x  ; gruen 1-6 gehrt zu S1, Rest zu S2
;#define aufteilung_output7_x  ; gruen 1-7 gehrt zu S1, Rest zu S2
;#define aufteilung_output8_x  ; gruen 1-8 gehrt zu S1, Rest zu S2
#define aufteilung_output_all ; alle Outputs gehren zu S1, die S2-Befehle machen Dunkeltastung, Page-Umschaltung etc.

;----------------------------------------------------------------------
; Definition: welche Outputs bei welcher Adresse im Wechsel blinken sollen
;----------------------------------------------------------------------
;#define  gruen5_flash_alternating_4_5  ; alternate flashing out 4/5  on green5
;#define  gruen6_flash_alternating_4_5  ; alternate flashing out 4/5  on green6
;#define  gruen7_flash_alternating_4_5  ; alternate flashing out 4/5  on green7
;#define  gruen8_flash_alternating_9_10 ; alternate flashing out 9/10 on green8
;#define  gruen9_flash_alternating_9_10 ; alternate flashing out 9/10 on green9
;#define gruen10_flash_alternating_9_10 ; alternate flashing out 9/10 on green10
;#define gruen11_flash_alternating_9_10 ; alternate flashing out 9/10 on green11
;#define gruen12_flash_alternating_9_10 ; alternate flashing out 9/10 on green12
;#define gruen13_flash_alternating_9_10 ; alternate flashing out 9/10 on green13
;#define gruen14_flash_alternating_9_10 ; alternate flashing out 9/10 on green14
;#define gruen15_flash_alternating_9_10 ; alternate flashing out 9/10 on green15
;#define gruen16_flash_alternating_9_10 ; alternate flashing out 9/10 on green16


;----------------------------------------------------
; Definition der Signal-Bilder
; diese Bits geben an, welches Flag in R_STATUS_ON_S1_pre/akt bzw. in R_STATUS_ON_S2_pre/akt
; gesetzt wird.
;----------------------------------------------------

#ifndef aufteilung_output_all
;----------------------------------------------------
; wenn Aufteilung der Outputs auf 2 Signale
;----------------------------------------------------
; *** Signalbilder S1 ein (1. Signal oder Hauptsignal)
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_S1_01  b'00000000'  ; @ Gruen #1
#define SIG_BILD_S1_02  b'00000000'  ; @ Gruen #2
#define SIG_BILD_S1_03  b'00000000'  ; @ Gruen #3
#define SIG_BILD_S1_04  b'00000000'  ; @ Gruen #4
#define SIG_BILD_S1_05  b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S1
#define SIG_BILD_S1_06  b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S1
#define SIG_BILD_S1_07  b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1
#define SIG_BILD_S1_08  b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1
;                         +--------- #7: Set Output #8
;                          +-------- #6: Set Output #7
;                           +------- #5: Set Output #6
;                            +------ #4: Set Output #5
;                             +----- #3: Set Output #4
;                              +---- #2: Set Output #3
;                               +--- #1: Set Output #2
;                                +-- #0: Set Output #1

; *** Signalbilder S2 (2. Signal oder Vorsignal)
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_S2_05  b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2
#define SIG_BILD_S2_06  b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2
#define SIG_BILD_S2_07  b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S2
#define SIG_BILD_S2_08  b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S2
#define SIG_BILD_S2_09  b'00000000'  ; @ Gruen #9
#define SIG_BILD_S2_10  b'00000000'  ; @ Gruen #10
#define SIG_BILD_S2_11  b'00000000'  ; @ Gruen #11
#define SIG_BILD_S2_12  b'00000000'  ; @ Gruen #12
#define SIG_BILD_S2_13  b'00000000'  ; @ Gruen #13
#define SIG_BILD_S2_14  b'00000000'  ; @ Gruen #14
#define SIG_BILD_S2_15  b'00000000'  ; @ Gruen #15
#define SIG_BILD_S2_16  b'00000000'  ; @ Gruen #16
;                         +--------- #7: Set Output #10
;                          +-------- #6: Set Output #9
;                           +------- #5: Set Output #8
;                            +------ #4: Set Output #7
;                             +----- #3: Set Output #6
;                              +---- #2: Set Output #5
;                               +--- #1:
;                                +-- #0:

; *** Signalbilder S1 flash (1. Signal oder Hauptsignal)
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_S1_01F b'00000000'  ; @ Gruen #1
#define SIG_BILD_S1_02F b'00000000'  ; @ Gruen #2
#define SIG_BILD_S1_03F b'00000000'  ; @ Gruen #3
#define SIG_BILD_S1_04F b'00000000'  ; @ Gruen #4
#define SIG_BILD_S1_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S1
#define SIG_BILD_S1_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S1
#define SIG_BILD_S1_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S1
#define SIG_BILD_S1_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S1
;                         +--------- #7: Flash Output #8
;                          +-------- #6: Flash Output #7
;                           +------- #5: Flash Output #6
;                            +------ #4: Flash Output #5
;                             +----- #3: Flash Output #4
;                              +---- #2: Flash Output #3
;                               +--- #1: Flash Output #2
;                                +-- #0: Flash Output #1

; *** Signalbilder S2 flash (2. Signal oder Vorsignal)
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_S2_05F b'00000000'  ; @ Gruen #5  ; if Green 5 is defined to S2
#define SIG_BILD_S2_06F b'00000000'  ; @ Gruen #6  ; if Green 6 is defined to S2
#define SIG_BILD_S2_07F b'00000000'  ; @ Gruen #7  ; if Green 7 is defined to S2
#define SIG_BILD_S2_08F b'00000000'  ; @ Gruen #8  ; if Green 8 is defined to S2
#define SIG_BILD_S2_09F b'00000000'  ; @ Gruen #9
#define SIG_BILD_S2_10F b'00000000'  ; @ Gruen #10
#define SIG_BILD_S2_11F b'00000000'  ; @ Gruen #11
#define SIG_BILD_S2_12F b'00000000'  ; @ Gruen #12
#define SIG_BILD_S2_13F b'00000000'  ; @ Gruen #13
#define SIG_BILD_S2_14F b'00000000'  ; @ Gruen #14
#define SIG_BILD_S2_15F b'00000000'  ; @ Gruen #15
#define SIG_BILD_S2_16F b'00000000'  ; @ Gruen #16
;                         +--------- #7: Flash Output #10
;                          +-------- #6: Flash Output #9
;                           +------- #5: Flash Output #8
;                            +------ #4: Flash Output #7
;                             +----- #3: Flash Output #6
;                              +---- #2: Flash Output #5
;                               +--- #1:
;                                +-- #0:
#endif  ;#ifndef aufteilung_output_all

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
;----------------------------------------------------
; wenn KEINE Aufteilung der Ausgnge auf 2 Signale
;----------------------------------------------------
; *** Signalbilder ein Out 1..8
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
; werden Adress-Page spezifische Signalbilder verwendet (d.h. es erscheinen Fahrweg-Abhngige Signal-Bilder)
; dann stellen die hier definierten Signalbilder die Page1 dar.
#define SIG_BILD_01_a      b'00010001'  ; @ Gruen #1
#define SIG_BILD_02_a      b'01000010'  ; @ Gruen #2
#define SIG_BILD_03_a      b'00000110'  ; @ Gruen #3
#define SIG_BILD_04_a      b'00011000'  ; @ Gruen #4
#define SIG_BILD_05_a      b'00110000'  ; @ Gruen #5
#define SIG_BILD_06_a      b'00110000'  ; @ Gruen #6
#define SIG_BILD_07_a      b'00000000'  ; @ Gruen #7
#define SIG_BILD_08_a      b'00000000'  ; @ Gruen #8
#define SIG_BILD_09_a      b'00000000'  ; @ Gruen #9
#define SIG_BILD_10_a      b'00000000'  ; @ Gruen #10
#define SIG_BILD_11_a      b'00000000'  ; @ Gruen #11
#define SIG_BILD_12_a      b'00000000'  ; @ Gruen #12
#define SIG_BILD_13_a      b'00000000'  ; @ Gruen #13
#define SIG_BILD_14_a      b'00000000'  ; @ Gruen #14
#define SIG_BILD_15_a      b'00000000'  ; @ Gruen #15
#define SIG_BILD_16_a      b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02_a_p2   b'00000010'  ; @ Gruen #2
#define SIG_BILD_03_a_p2   b'10000110'  ; @ Gruen #3
;                            +--------- #7: Set   Output #8
;                             +-------- #6: Set   Output #7
;                              +------- #5: Set   Output #6
;                               +------ #4: Set   Output #5
;                                +----- #3: Set   Output #4
;                                 +---- #2: Set   Output #3
;                                  +--- #1: Set   Output #2
;                                   +-- #0: Set   Output #1

; *** Signalbilder ein Out 9..10
; eine '1' definiert, dass der Ausgang bei der antsprechenden Adresse eingeschaltet werden soll
#define SIG_BILD_01_b      b'00000000'  ; @ Gruen #1
#define SIG_BILD_02_b      b'01000000'  ; @ Gruen #2
#define SIG_BILD_03_b      b'01000000'  ; @ Gruen #3
#define SIG_BILD_04_b      b'00000000'  ; @ Gruen #4
#define SIG_BILD_05_b      b'00000000'  ; @ Gruen #5
#define SIG_BILD_06_b      b'00000000'  ; @ Gruen #6
#define SIG_BILD_07_b      b'01000000'  ; @ Gruen #7
#define SIG_BILD_08_b      b'10000000'  ; @ Gruen #8
#define SIG_BILD_09_b      b'00000000'  ; @ Gruen #9
#define SIG_BILD_10_b      b'00000000'  ; @ Gruen #10
#define SIG_BILD_11_b      b'00000000'  ; @ Gruen #11
#define SIG_BILD_12_b      b'00000000'  ; @ Gruen #12
#define SIG_BILD_13_b      b'00000000'  ; @ Gruen #13
#define SIG_BILD_14_b      b'00000000'  ; @ Gruen #14
#define SIG_BILD_15_b      b'00000000'  ; @ Gruen #15
#define SIG_BILD_16_b      b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02_b_p2   b'10000000'  ; @ Gruen #2
#define SIG_BILD_03_b_p2   b'10000000'  ; @ Gruen #3
;                            +--------- #7: Set   Output #10
;                             +-------- #6: Set   Output #9



; *** Signalbilder S1 flash
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag muss auch fr "ein" gesetzt werden
; *** Signalbilder Flash Out 1..8
#define SIG_BILD_01F_a     b'00000000'  ; @ Gruen #1
#define SIG_BILD_02F_a     b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_a     b'00000000'  ; @ Gruen #3
#define SIG_BILD_04F_a     b'00000000'  ; @ Gruen #4
#define SIG_BILD_05F_a     b'00000000'  ; @ Gruen #5
#define SIG_BILD_06F_a     b'00100000'  ; @ Gruen #6
#define SIG_BILD_07F_a     b'00000000'  ; @ Gruen #7
#define SIG_BILD_08F_a     b'00000000'  ; @ Gruen #8
#define SIG_BILD_09F_a     b'00000000'  ; @ Gruen #9
#define SIG_BILD_10F_a     b'00000000'  ; @ Gruen #10
#define SIG_BILD_11F_a     b'00000000'  ; @ Gruen #11
#define SIG_BILD_12F_a     b'00000000'  ; @ Gruen #12
#define SIG_BILD_13F_a     b'00000000'  ; @ Gruen #13
#define SIG_BILD_14F_a     b'00000000'  ; @ Gruen #14
#define SIG_BILD_15F_a     b'00000000'  ; @ Gruen #15
#define SIG_BILD_16F_a     b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02F_a_p2  b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_a_p2  b'00000000'  ; @ Gruen #3
;                            +--------- #7: Flash Output #8
;                             +-------- #6: Flash Output #7
;                              +------- #5: Flash Output #6
;                               +------ #4: Flash Output #5
;                                +----- #3: Flash Output #4
;                                 +---- #2: Flash Output #3
;                                  +--- #1: Flash Output #2
;                                   +-- #0: Flash Output #1

; *** Signalbilder Flash Out 9..10
; - Ausgang blinkt, wen hier eine '1' fr die entsprechende Adresse definiert wird
; - entsprechendes Flag unter 'Signalbilder' muss auch fr "ein" gesetzt werden
#define SIG_BILD_01F_b     b'00000000'  ; @ Gruen #1
#define SIG_BILD_02F_b     b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_b     b'00000000'  ; @ Gruen #3
#define SIG_BILD_04F_b     b'00000000'  ; @ Gruen #4
#define SIG_BILD_05F_b     b'00000000'  ; @ Gruen #5
#define SIG_BILD_06F_b     b'00000000'  ; @ Gruen #6
#define SIG_BILD_07F_b     b'00000000'  ; @ Gruen #7
#define SIG_BILD_08F_b     b'00000000'  ; @ Gruen #8
#define SIG_BILD_09F_b     b'00000000'  ; @ Gruen #9
#define SIG_BILD_10F_b     b'00000000'  ; @ Gruen #10
#define SIG_BILD_11F_b     b'00000000'  ; @ Gruen #11
#define SIG_BILD_12F_b     b'00000000'  ; @ Gruen #12
#define SIG_BILD_13F_b     b'00000000'  ; @ Gruen #13
#define SIG_BILD_14F_b     b'00000000'  ; @ Gruen #14
#define SIG_BILD_15F_b     b'00000000'  ; @ Gruen #15
#define SIG_BILD_16F_b     b'00000000'  ; @ Gruen #16
; *** alternative Signalbilder fr Adress-Page 2
#define SIG_BILD_02F_b_p2  b'00000000'  ; @ Gruen #2
#define SIG_BILD_03F_b_p2  b'00000000'  ; @ Gruen #3
;                            +--------- #7: Flash Output #10
;                             +-------- #6: Flash Output #9
#endif  ;#ifdef aufteilung_output_all

;----------------------------------------------------------------------
; Definition, ob bei bestimmten Adressen keine nderung des Signal-Bildes stattfinden soll
;----------------------------------------------------------------------
; - insbesondere, wenn Adresse Dunkeltastung aktivieren / deaktivieren soll
; - auch, wenn Adresse den Wechsel der Adress-Page hervorruft
;#define adr1_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr2_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr3_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr4_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr5_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr6_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr7_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr8_no_change  ; diese Adresse beeinflusst Signalbild nicht
#define adr9_no_change  ; diese Adresse beeinflusst Signalbild nicht
#define adr10_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr11_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr12_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr13_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr14_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr15_no_change  ; diese Adresse beeinflusst Signalbild nicht
;#define adr16_no_change  ; diese Adresse beeinflusst Signalbild nicht

;---------------------------------------------------------------------
; Definition Dunkeltastung S2 -> Durch welche Befehle S2 (Vorsignal) Dunkel wird
;----------------------------------------------------------------------
;#define s2_dark_by_1 ; Dunkeltastung von S2 durch Grn 1
;#define s2_dark_by_2 ; Dunkeltastung von S2 durch Grn 2
;#define s2_dark_by_3 ; Dunkeltastung von S2 durch Grn 3
;#define s2_dark_by_4 ; Dunkeltastung von S2 durch Grn 4
;#define s2_dark_by_5 ; Dunkeltastung von S2 durch Grn 5
;#define s2_dark_by_6 ; Dunkeltastung von S2 durch Grn 6
;#define s2_dark_by_7 ; Dunkeltastung von S2 durch Grn 7
;#define s2_dark_by_8 ; Dunkeltastung von S2 durch Grn 8

;----------------------------------------------------------------------
; Definition Dunkeltastung S2 -> Welche Outputs dunkel werden
;----------------------------------------------------------------------
; Definition, weclhe Ausgnge durch 's2_dark_by_..' dunkel werden
;#define dunkeltast_out01      ; hier wird definiert,
;#define dunkeltast_out02      ; .. welche
;#define dunkeltast_out03      ; .. Ausgnge
;#define dunkeltast_out04      ; .. auf die
;#define dunkeltast_out05      ; .. Dunkeltastung (des Signal2)
;#define dunkeltast_out06      ; .. reagieren und bei
;#define dunkeltast_out07      ; .. Dunkeltastung
;#define dunkeltast_out08      ; .. dann auch
;#define dunkeltast_out09      ; .. abschalten
;#define dunkeltast_out10      ; ..

;----------------------------------------------------------------------
; Definition Dunkeltastung ALLE -> Durch welche Befehle ALLES Dunkel wird
;----------------------------------------------------------------------
; - wird hier fr ein Befehl aktiviert, ndert die entsprechende Adresse das Signalbild nicht
; - die Befehle sollten zu S2 gehren

;#define use_all_dark   ; Dunkeltastung ALL verwenden

;#define all_dark_by_1  ; Dunkeltastung des gesamten Signal durch Grn 1
;#define all_dark_by_2  ; Dunkeltastung des gesamten Signal durch Grn 2
;#define all_dark_by_3  ; Dunkeltastung des gesamten Signal durch Grn 3
;#define all_dark_by_4  ; Dunkeltastung des gesamten Signal durch Grn 4
;#define all_dark_by_5  ; Dunkeltastung des gesamten Signal durch Grn 5
;#define all_dark_by_6  ; Dunkeltastung des gesamten Signal durch Grn 6
;#define all_dark_by_7  ; Dunkeltastung des gesamten Signal durch Grn 7
;#define all_dark_by_8  ; Dunkeltastung des gesamten Signal durch Grn 8
;#define all_dark_by_9  ; Dunkeltastung des gesamten Signal durch Grn 9
;#define all_dark_by_10 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_11 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_12 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_13 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_14 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_15 ; Dunkeltastung des gesamten Signal durch Grn 10
;#define all_dark_by_16 ; Dunkeltastung des gesamten Signal durch Grn 10

; hier wird definiert, welche Befehle die Dukeltastung ALLE wieder aufgehoben wird.
; - wird hier fr ein Befehl aktiviert, ndert die entsprechende Adresse das Signalbild nicht
; - die Befehle knnen zu S1 oder S2 gehren
;#define quit_dark_by_1  ; Dunkeltastung des gesamten Signals durch Grn 1  beenden
;#define quit_dark_by_2  ; Dunkeltastung des gesamten Signals durch Grn 2  beenden
;#define quit_dark_by_3  ; Dunkeltastung des gesamten Signals durch Grn 3  beenden
;#define quit_dark_by_4  ; Dunkeltastung des gesamten Signals durch Grn 4  beenden
;#define quit_dark_by_5  ; Dunkeltastung des gesamten Signals durch Grn 5  beenden
;#define quit_dark_by_6  ; Dunkeltastung des gesamten Signals durch Grn 6  beenden
;#define quit_dark_by_7  ; Dunkeltastung des gesamten Signals durch Grn 7  beenden
;#define quit_dark_by_8  ; Dunkeltastung des gesamten Signals durch Grn 8  beenden
;#define quit_dark_by_9  ; Dunkeltastung des gesamten Signals durch Grn 9  beenden
;#define quit_dark_by_10 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_11 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_12 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_13 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_14 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_15 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden
;#define quit_dark_by_16 ; Dunkeltastung des gesamten Signals durch Grn 10 beenden

; Definition, weclhe Ausgnge durch 'all_dark_by_..' dunkel werden
#define dunkeltast_all_out01      ; hier wird definiert,
#define dunkeltast_all_out02      ; .. welche
#define dunkeltast_all_out03      ; .. Ausgnge
#define dunkeltast_all_out04      ; .. auf die
#define dunkeltast_all_out05      ; .. Dunkeltastung (des Signal2)
#define dunkeltast_all_out06      ; .. reagieren und bei
#define dunkeltast_all_out07      ; .. Dunkeltastung
#define dunkeltast_all_out08      ; .. dann auch
;#define dunkeltast_all_out09      ; .. abschalten
;#define dunkeltast_all_out10      ; ..

;----------------------------------------------------------------------
; Definition Adress-Page: gibt es eine Umschaltung zwischen 2 Adress-Pages
;----------------------------------------------------------------------
; ist dies aktiviert, wird mittels eines Befehles zwischen 2 Adress-Pages umgeschaltet.
; Es kann damit das Signal auf verschiedene Adressen reagieren und auch unterschiedliche Signalbilder anzeigen
; z.B. in Abhngigkeit der folgenden Weiche.
; - Wenn die Adresse nicht zu einer Adress-Page zugeordnet werden soll, dann wird hier weder gruenx_page1_aktiv noch gruenx_page2_aktiv aktiviert.
; - hier wird also nur aktiviert, wenn nur auf eine Adress-Page reagiert werden soll.
; - es werden die EE-Zellen 0x40 0x50 0x60 0x70 genutzt, um die Buffer zu speichern, sodass diese nach einem RESET wieder zur Verfgung stehen

#define use_paged_adressing ;  = Page_Adressen verwenden
;#define use_buffer_register ;  muss verwendet werden, wenn fahrwegabhngige Adressen, nicht bei fahrwegabhngigen Signalbildern!

; hier wird definiert, durch welche Adresse zwischen den Adress-Pages gewechselt wird
;#define gruen1_page1_aktiv
;#define gruen2_page1_aktiv
;#define gruen3_page1_aktiv
;#define gruen4_page1_aktiv
;#define gruen5_page1_aktiv
;#define gruen6_page1_aktiv
;#define gruen7_page1_aktiv
;#define gruen8_page1_aktiv
#define gruen9_page1_aktiv
;#define gruen10_page1_aktiv
;#define gruen11_page1_aktiv
;#define gruen12_page1_aktiv
;#define gruen13_page1_aktiv
;#define gruen14_page1_aktiv
;#define gruen15_page1_aktiv
;#define gruen16_page1_aktiv

;#define gruen1_page2_aktiv
;#define gruen2_page2_aktiv
;#define gruen3_page2_aktiv
;#define gruen4_page2_aktiv
;#define gruen5_page2_aktiv
;#define gruen6_page2_aktiv
;#define gruen7_page2_aktiv
;#define gruen8_page2_aktiv
;#define gruen9_page2_aktiv
#define gruen10_page2_aktiv
;#define gruen11_page2_aktiv
;#define gruen12_page2_aktiv
;#define gruen13_page2_aktiv
;#define gruen14_page2_aktiv
;#define gruen15_page2_aktiv
;#define gruen16_page2_aktiv

; hier wird definiert, ob auf eine Adresse nur bei einer bestimmten Adress-Page reagiert werden darf.
; (d.h. fahrwegabhngige Adressen - im Gegensatz zu fahrwegabhngigen Signalbildern!)
;#define  gruen1_if_page1
;#define  gruen2_if_page1
;#define  gruen3_if_page1
;#define  gruen4_if_page1
;#define  gruen5_if_page1
;#define  gruen6_if_page1
;#define  gruen7_if_page1
;#define  gruen8_if_page1
;#define  gruen9_if_page1
;#define gruen10_if_page1
;#define gruen11_if_page1
;#define gruen12_if_page1
;#define gruen13_if_page1
;#define gruen14_if_page1
;#define gruen15_if_page1
;#define gruen16_if_page1

;#define  gruen1_if_page2
;#define  gruen2_if_page2
;#define  gruen3_if_page2
;#define  gruen4_if_page2
;#define  gruen5_if_page2
;#define  gruen6_if_page2
;#define  gruen7_if_page2
;#define  gruen8_if_page2
;#define  gruen9_if_page2
;#define gruen10_if_page2
;#define gruen11_if_page2
;#define gruen12_if_page2
;#define gruen13_if_page2
;#define gruen14_if_page2
;#define gruen15_if_page2
;#define gruen16_if_page2

; hier wird definiert, ob ein Adress-Page spezifisches Signalbild angezeigt werden soll.
; (d.h. fahrwegabhngige Signalbilder - im Gegensatz zu fahrwegabhngigen Adressen)
;#define  gruen1_depends_on_page
#define  gruen2_depends_on_page
#define  gruen3_depends_on_page
;#define  gruen4_depends_on_page
;#define  gruen5_depends_on_page
;#define  gruen6_depends_on_page
;#define  gruen7_depends_on_page
;#define  gruen8_depends_on_page
;#define  gruen9_depends_on_page
;#define gruen10_depends_on_page
;#define gruen11_depends_on_page
;#define gruen12_depends_on_page
;#define gruen13_depends_on_page
;#define gruen14_depends_on_page
;#define gruen15_depends_on_page
;#define gruen16_depends_on_page


;----------------------------------------------------------------------
; Definition Blinkmuster f. SAVE-Blinken
;----------------------------------------------------------------------
; eine '1' bedeutet, dass der Ausgang beim Lernen der entsprechenden Adresse blinkt
#define save_01_port_a  b'00010000'    ; save Adresse 01
#define save_02_port_a  b'00100000'    ; save Adresse 02
#define save_03_port_a  b'00100001'    ; save Adresse 03
#define save_04_port_a  b'00000010'    ; save Adresse 04
#define save_05_port_a  b'00000000'    ; save Adresse 05
#define save_06_port_a  b'00000000'    ; save Adresse 06
#define save_07_port_a  b'00000000'    ; save Adresse 07
#define save_08_port_a  b'00000000'    ; save Adresse 08
#define save_09_port_a  b'00110011'    ; save Adresse 09
#define save_10_port_a  b'00110011'    ; save Adresse 10
#define save_11_port_a  b'00000000'    ; save Adresse 11
#define save_12_port_a  b'00000000'    ; save Adresse 12
#define save_13_port_a  b'00000000'    ; save Adresse 13
#define save_14_port_a  b'00000000'    ; save Adresse 14
#define save_15_port_a  b'00000000'    ; save Adresse 15
#define save_16_port_a  b'00000000'    ; save Adresse 16
;                           +------- #5: Flash Output #2
;                            +------ #4: Flash Output #1
;                               +--- #1: Flash Output #4
;                                +-- #0: Flash Output #3

#define save_01_port_c  b'00000010'  ; save Adresse 01
#define save_02_port_c  b'00011001'  ; save Adresse 02
#define save_03_port_c  b'00111000'  ; save Adresse 03
#define save_04_port_c  b'00000010'  ; save Adresse 04
#define save_05_port_c  b'00000110'  ; save Adresse 05
#define save_06_port_c  b'00000110'  ; save Adresse 06
#define save_07_port_c  b'00010000'  ; save Adresse 07
#define save_08_port_c  b'00001000'  ; save Adresse 08
#define save_09_port_c  b'00010111'  ; save Adresse 09
#define save_10_port_c  b'00101110'  ; save Adresse 10
#define save_11_port_c  b'00000000'  ; save Adresse 11
#define save_12_port_c  b'00000000'  ; save Adresse 12
#define save_13_port_c  b'00000000'  ; save Adresse 13
#define save_14_port_c  b'00000000'  ; save Adresse 14
#define save_15_port_c  b'00000000'  ; save Adresse 15
#define save_16_port_c  b'00000000'  ; save Adresse 16
;                           +------- #5: Flash Output #8
;                            +------ #4: Flash Output #9
;                             +----- #3: Flash Output #10
;                              +---- #2: Flash Output #6
;                               +--- #1: Flash Output #5
;                                +-- #0: Flash Output #7

;----------------------------------------------------
; Parameter
;----------------------------------------------------
CONST_FADE_01          equ d'120'        ; Fade Geschwindigkeit @ Signalbild-Wechsel

;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;-- Ab hier muss man idR. nichts mehr anpassen !!
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
; wenn Adresse Dunkeltastung EIN oder AUS schaltet-> kein Signalbild verndern
;----------------------------------------------------------------------
#ifdef all_dark_by_1
 #define adr1_no_change
#endif
#ifdef all_dark_by_2
 #define adr2_no_change
#endif
#ifdef all_dark_by_3
 #define adr3_no_change
#endif
#ifdef all_dark_by_4
 #define adr4_no_change
#endif
#ifdef all_dark_by_5
 #define adr5_no_change
#endif
#ifdef all_dark_by_6
 #define adr6_no_change
#endif
#ifdef all_dark_by_7
 #define adr7_no_change
#endif
#ifdef all_dark_by_8
 #define adr8_no_change
#endif
#ifdef all_dark_by_9
 #define adr9_no_change
#endif
#ifdef all_dark_by_10
 #define adr10_no_change
#endif
#ifdef all_dark_by_11
 #define adr11_no_change
#endif
#ifdef all_dark_by_12
 #define adr12_no_change
#endif
#ifdef all_dark_by_13
 #define adr13_no_change
#endif
#ifdef all_dark_by_14
 #define adr14_no_change
#endif
#ifdef all_dark_by_15
 #define adr15_no_change
#endif
#ifdef all_dark_by_16
 #define adr16_no_change
#endif

#ifdef quit_dark_by_1
 #define adr1_no_change
#endif
#ifdef quit_dark_by_2
 #define adr2_no_change
#endif
#ifdef quit_dark_by_3
 #define adr3_no_change
#endif
#ifdef quit_dark_by_4
 #define adr4_no_change
#endif
#ifdef quit_dark_by_5
 #define adr5_no_change
#endif
#ifdef quit_dark_by_6
 #define adr6_no_change
#endif
#ifdef quit_dark_by_7
 #define adr7_no_change
#endif
#ifdef quit_dark_by_8
 #define adr8_no_change
#endif
#ifdef quit_dark_by_9
 #define adr9_no_change
#endif
#ifdef quit_dark_by_10
 #define adr10_no_change
#endif
#ifdef quit_dark_by_11
 #define adr11_no_change
#endif
#ifdef quit_dark_by_12
 #define adr12_no_change
#endif
#ifdef quit_dark_by_13
 #define adr13_no_change
#endif
#ifdef quit_dark_by_14
 #define adr14_no_change
#endif
#ifdef quit_dark_by_15
 #define adr14_no_change
#endif
#ifdef quit_dark_by_16
 #define adr14_no_change
#endif

;----------------------------------------------------------------------
; alternative Zuordnungen, hier nichts ndern!
;----------------------------------------------------------------------
; select, if Output 5/6/7/8 belongs to S1/S2
#ifdef aufteilung_output4_x
 #define out5_to_s2  ; Out 5 belongs to S2
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output5_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s2  ; Out 6 belongs to S2
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output6_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s2  ; Out 7 belongs to S2
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output7_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s2  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output8_x
 #define out5_to_s1  ; Out 5 belongs to S1
 #define out6_to_s1  ; Out 6 belongs to S1
 #define out7_to_s1  ; Out 7 belongs to S1
 #define out8_to_s1  ; Out 8 belongs to S2
#endif
#ifdef aufteilung_output_all
 #define out5_to_s1  ; Out 5 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out6_to_s1  ; Out 6 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out7_to_s1  ; Out 7 belongs to S1 (S1 ist hier kein eigenstndiges Signal!)
 #define out8_to_s1  ; Out 8 belongs to S2 (S1 ist hier kein eigenstndiges Signal!)
#endif

; select, if Grn5/6/7/8 belongs to S1/S2 (for filter R_STATUS / -_2)
#ifdef aufteilung4_x
 #define gruen5_to_s2  ; Green 5 belongs to S2
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11110001'
status_g02_a equ b'11110010'
status_g03_a equ b'11110100'
status_g04_a equ b'11111000'
status_g05_a equ b'00011111'
status_g06_a equ b'00101111'
status_g07_a equ b'01001111'
status_g08_a equ b'10001111'
status_g09_a equ b'00001111'
status_g10_a equ b'00001111'
status_g11_a equ b'00001111'
status_g12_a equ b'00001111'
status_g13_a equ b'00001111'
status_g14_a equ b'00001111'
status_g15_a equ b'00001111'
status_g16_a equ b'00001111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'00000000'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung5_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s2  ; Green 6 belongs to S2
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11100001'
status_g02_a equ b'11100010'
status_g03_a equ b'11100100'
status_g04_a equ b'11101000'
status_g05_a equ b'11110000'
status_g06_a equ b'00111111'
status_g07_a equ b'01011111'
status_g08_a equ b'10011111'
status_g09_a equ b'00011111'
status_g10_a equ b'00011111'
status_g11_a equ b'00011111'
status_g12_a equ b'00011111'
status_g13_a equ b'00011111'
status_g14_a equ b'00011111'
status_g15_a equ b'00011111'
status_g16_a equ b'00011111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung6_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s2  ; Green 7 belongs to S2
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'11000001'
status_g02_a equ b'11000010'
status_g03_a equ b'11000100'
status_g04_a equ b'11001000'
status_g05_a equ b'11010000'
status_g06_a equ b'11100000'
status_g07_a equ b'01111111'
status_g08_a equ b'10111111'
status_g09_a equ b'00111111'
status_g10_a equ b'00111111'
status_g11_a equ b'00111111'
status_g12_a equ b'00111111'
status_g13_a equ b'00111111'
status_g14_a equ b'00111111'
status_g15_a equ b'00111111'
status_g16_a equ b'00111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung7_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s2  ; Green 8 belongs to S2
status_g01_a equ b'10000001'
status_g02_a equ b'10000010'
status_g03_a equ b'10000100'
status_g04_a equ b'10001000'
status_g05_a equ b'10010000'
status_g06_a equ b'10100000'
status_g07_a equ b'11000000'
status_g08_a equ b'11111111'
status_g09_a equ b'01111111'
status_g10_a equ b'01111111'
status_g11_a equ b'01111111'
status_g12_a equ b'01111111'
status_g13_a equ b'01111111'
status_g14_a equ b'01111111'
status_g15_a equ b'01111111'
status_g16_a equ b'01111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'11111111'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung8_x
 #define gruen5_to_s1  ; Green 5 belongs to S1
 #define gruen6_to_s1  ; Green 6 belongs to S1
 #define gruen7_to_s1  ; Green 7 belongs to S1
 #define gruen8_to_s1  ; Green 8 belongs to S2
status_g01_a equ b'00000001'
status_g02_a equ b'00000010'
status_g03_a equ b'00000100'
status_g04_a equ b'00001000'
status_g05_a equ b'00010000'
status_g06_a equ b'00100000'
status_g07_a equ b'01000000'
status_g08_a equ b'10000000'
status_g09_a equ b'11111111'
status_g10_a equ b'11111111'
status_g11_a equ b'11111111'
status_g12_a equ b'11111111'
status_g13_a equ b'11111111'
status_g14_a equ b'11111111'
status_g15_a equ b'11111111'
status_g16_a equ b'11111111'
status_g01_b equ b'11111111'
status_g02_b equ b'11111111'
status_g03_b equ b'11111111'
status_g04_b equ b'11111111'
status_g05_b equ b'11111111'
status_g06_b equ b'11111111'
status_g07_b equ b'11111111'
status_g08_b equ b'11111111'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif
#ifdef aufteilung_all
status_g01_a equ b'00000001'
status_g02_a equ b'00000010'
status_g03_a equ b'00000100'
status_g04_a equ b'00001000'
status_g05_a equ b'00010000'
status_g06_a equ b'00100000'
status_g07_a equ b'01000000'
status_g08_a equ b'10000000'
status_g09_a equ b'00000000'
status_g10_a equ b'00000000'
status_g11_a equ b'00000000'
status_g12_a equ b'00000000'
status_g13_a equ b'00000000'
status_g14_a equ b'00000000'
status_g15_a equ b'00000000'
status_g16_a equ b'00000000'
status_g01_b equ b'00000000'
status_g02_b equ b'00000000'
status_g03_b equ b'00000000'
status_g04_b equ b'00000000'
status_g05_b equ b'00000000'
status_g06_b equ b'00000000'
status_g07_b equ b'00000000'
status_g08_b equ b'00000000'
status_g09_b equ b'00000001'
status_g10_b equ b'00000010'
status_g11_b equ b'00000100'
status_g12_b equ b'00001000'
status_g13_b equ b'00010000'
status_g14_b equ b'00100000'
status_g15_b equ b'01000000'
status_g16_b equ b'10000000'
#endif

;----------------------------------------------------
; Mapping: Flag -> Output, hier nichts ndern!
;----------------------------------------------------
; hier wird definiert, bei welchem Flag von R_STATUS_ON_S1_akt bzw, R_STATUS_ON_S2_akt
; der entsprechende Output reagiert. Die Zuordnung S1/S2 wird durch die #defines geregelt,
; daher sind in diesem Abschnitt keine nderungen zur Anpassung ntig.

; *** S1
#define SET_OUTPUT1          R_STATUS_ON_S1_akt,0
#define SET_OUTPUT1_FL       R_STATUS_FL_S1_akt,0    ;
#define SET_OUTPUT2          R_STATUS_ON_S1_akt,1
#define SET_OUTPUT2_FL       R_STATUS_FL_S1_akt,1    ;
#define SET_OUTPUT3          R_STATUS_ON_S1_akt,2
#define SET_OUTPUT3_FL       R_STATUS_FL_S1_akt,2    ;
#define SET_OUTPUT4          R_STATUS_ON_S1_akt,3
#define SET_OUTPUT4_FL       R_STATUS_FL_S1_akt,3    ;
#ifdef out5_to_s1
  #define SET_OUTPUT5        R_STATUS_ON_S1_akt,4
  #define SET_OUTPUT5_FL     R_STATUS_FL_S1_akt,4    ;
#endif
#ifdef out6_to_s1
  #define SET_OUTPUT6        R_STATUS_ON_S1_akt,5
  #define SET_OUTPUT6_FL     R_STATUS_FL_S1_akt,5    ;
#endif
#ifdef out7_to_s1
  #define SET_OUTPUT7        R_STATUS_ON_S1_akt,6
  #define SET_OUTPUT7_FL     R_STATUS_FL_S1_akt,6    ;
#endif
#ifdef out8_to_s1
  #define SET_OUTPUT8        R_STATUS_ON_S1_akt,7
  #define SET_OUTPUT8_FL     R_STATUS_FL_S1_akt,7    ;
#endif

; *** S2
#ifdef out5_to_s2
  #define SET_OUTPUT5        R_STATUS_ON_S2_akt,2
  #define SET_OUTPUT5_FL     R_STATUS_FL_S2_akt,2    ;
#endif
#ifdef out6_to_s2
  #define SET_OUTPUT6        R_STATUS_ON_S2_akt,3
  #define SET_OUTPUT6_FL     R_STATUS_FL_S2_akt,3    ;
#endif
#ifdef out7_to_s2
  #define SET_OUTPUT7        R_STATUS_ON_S2_akt,4
  #define SET_OUTPUT7_FL     R_STATUS_FL_S2_akt,4    ;
#endif
#ifdef out8_to_s2
  #define SET_OUTPUT8        R_STATUS_ON_S2_akt,5
  #define SET_OUTPUT8_FL     R_STATUS_FL_S2_akt,5    ;
#endif
#define SET_OUTPUT9          R_STATUS_ON_S2_akt,6
#define SET_OUTPUT9_FL       R_STATUS_FL_S2_akt,6    ;
#define SET_OUTPUT10         R_STATUS_ON_S2_akt,7
#define SET_OUTPUT10_FL      R_STATUS_FL_S2_akt,7    ;


;----------------------------------------------------
; Register, hier nichts ndern!
;----------------------------------------------------
R_STATUS_ON_S1_pre    equ  40h
R_STATUS_ON_S2_pre    equ  41h
R_STATUS_FL_S1_pre    equ  42h
R_STATUS_FL_S2_pre    equ  43h
R_STATUS_ON_S1_akt    equ  44h
R_STATUS_ON_S2_akt    equ  45h
R_STATUS_FL_S1_akt    equ  46h
R_STATUS_FL_S2_akt    equ  47h

R_COUNT               equ  48h          ; Counter fuer PWM - LED1
R_PWMVALUE            equ  49h          ; PWM-Value, bernahme von R_COUNT
R_STATE1              equ  4ah          ; Status-Flags 1 - LED1
R_STATE2              equ  4bh          ; Status-Flags 2 - LED2
R_STATE3              equ  4ch          ; Status-Flags 3 - LED3
R_STATE4              equ  4dh          ; Status-Flags 4 - LED4
R_STATE5              equ  4eh          ; Status-Flags 5 - LED5
R_STATE6              equ  4fh          ; Status-Flags 6 - LED6
R_STATE7              equ  50h          ; Status-Flags 7 - LED7
R_STATE8              equ  51h          ; Status-Flags 8 - LED8
R_STATE9              equ  52h          ; Status-Flags 9 - LED9
R_STATE10             equ  53h          ; Status-Flags 10 - LED10

R_STATUS_ON_P1_buffer equ  54h          ; Buffer Register Page 1 ON
R_STATUS_FL_P1_buffer equ  55h          ; Buffer Register Page 1 FL
R_STATUS_ON_P2_buffer equ  56h          ; Buffer Register Page 2 ON
R_STATUS_FL_P2_buffer equ  57h          ; Buffer Register Page 2 FL

R_TESTTEMP            equ  5ch
;R_FADEVALUE           equ  5dh          ; contains valid fade constant
R_STATUS_2            equ  5eh          ; for >8 Adresses
R_FLAG_001            equ  5fh          ; various Flags
    #define DUNKELTAST_PRE        R_FLAG_001,0
    #define DUNKELTAST_AKT        R_FLAG_001,1
    #define DUNKELTAST_ALL_PRE    R_FLAG_001,2
    #define DUNKELTAST_ALL        R_FLAG_001,3
    #define F_PAGE1_AKTIV         R_FLAG_001,4 ; set if Adress-Page 1 is activ
    #define F_PAGE2_AKTIV         R_FLAG_001,5 ; set if Adress-Page 2 is activ
    #define ALTER_FLASH_S1        R_FLAG_001,6 ; if set alternate Flashing S1 out 4/5
    #define ALTER_FLASH_S2        R_FLAG_001,7 ; if set alternate Flashing S2 out 9/10


;----------------------------------------------------
; Mapping LED-Nummer zu Port
;----------------------------------------------------
#define LED_01     PORTA,4        ;LED-Dekoder Layout Definition
#define LED_02     PORTA,5        ;LED-Dekoder Layout Definition
#define LED_03     PORTA,0        ;LED-Dekoder Layout Definition
#define LED_04     PORTA,1        ;LED-Dekoder Layout Definition
#define LED_05     PORTC,1        ;LED-Dekoder Layout Definition
#define LED_06     PORTC,2        ;LED-Dekoder Layout Definition
#define LED_07     PORTC,0        ;LED-Dekoder Layout Definition
#define LED_08     PORTC,5        ;LED-Dekoder Layout Definition
#define LED_09     PORTC,4        ;LED-Dekoder Layout Definition
#define LED_10     PORTC,3        ;LED-Dekoder Layout Definition




;   --------------------------------------------------------------------------
;   --- EEprom Locations
;   --------------------------------------------------------------------------
#define EE_ADR_MODE  h'00'           ; EEprom Adresse: R_MODE
#define EE_ADR_S1    h'10'           ; EEprom Adresse: R_STATUS_1: Status #1
#define EE_ADR_S2    h'30'           ; EEprom Adresse: R_STATUS_2: Status #2

; ********************
; *** INIT-Dekoder ***
; ********************
init_dekoder
;   -------------------------------------
;   --- read R_STATUS from EEPROM $10 ---
;   -------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S1
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS                 ; ..into R_STATUS

;   --------------------------------------
;   --- read R_STATUS_2 from EEPROM $30 ---
;   --------------------------------------
        bsf STATUS,RP0                 ; -> BANK 1
        movlw EE_ADR_S2
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_2               ; ..into R_STATUS_2

;   --------------------------------------
;   --- preload bytes                  ---
;   --------------------------------------
        movlw h'ff'                    ; call Statemachine signal4 immediately
        movwf R_COUNT

        bsf R_STATE1,3                 ; Grundzustand "off"
        bsf R_STATE2,3                 ; Grundzustand "off"
        bsf R_STATE3,3                 ; Grundzustand "off"
        bsf R_STATE4,3                 ; Grundzustand "off"
        bsf R_STATE5,3                 ; Grundzustand "off"
        bsf R_STATE6,3                 ; Grundzustand "off"
        bsf R_STATE7,3                 ; Grundzustand "off"
        bsf R_STATE8,3                 ; Grundzustand "off"
        bsf R_STATE9,3                 ; Grundzustand "off"
        bsf R_STATE10,3                ; Grundzustand "off"
        return

; ********************
; *** ADR Gruen-#1 ***
; ********************
#ifndef skip_g1
gruen_1
        ; Signal-Status
        movlw status_g01_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g01_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen1_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen1_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen1_if_page2
  #ifdef gruen1_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen1_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen1_if_page2
#endif ; use_paged_adressing'

#ifdef adr1_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen1_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_01_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_01F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_01_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_01F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen1_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_01F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen1_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_1
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif ; s2_dark_by_1
#ifndef s2_dark_by_1
        bcf DUNKELTAST_PRE
#endif ; s2_dark_by_1

gruen1_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_1
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif ; all_dark_by_1
#ifdef quit_dark_by_1
        bcf DUNKELTAST_ALL_PRE
#endif ; quit_dark_by_1

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen1_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen1_page1_aktiv

  #ifdef gruen1_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen1_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen1_if_page1
gruen1_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen1_if_page1

 #ifdef gruen1_if_page2
gruen1_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_01           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_01F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen1_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g1

; ********************
; *** ADR Gruen-#2 ***
; ********************
#ifndef skip_g2
gruen_2
        ; Signal-Status
        movlw status_g02_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g02_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen2_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen2_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen2_if_page2
  #ifdef gruen2_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen2_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen2_if_page2
#endif ; use_paged_adressing'

#ifdef adr2_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen2_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_02_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_02F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_02_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_02F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen2_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_02F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen2_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_2
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_2
        bcf DUNKELTAST_PRE
#endif

gruen2_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_2
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_2
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen2_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen2_page1_aktiv

  #ifdef gruen2_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen2_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen2_if_page1
gruen2_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen2_if_page1

 #ifdef gruen2_if_page2
gruen2_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_02           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_02F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen2_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g2

; ********************
; *** ADR Gruen-#3 ***
; ********************
#ifndef skip_g3
gruen_3
        ; Signal-Status
        movlw status_g03_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g03_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen3_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen3_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen3_if_page2
  #ifdef gruen3_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen3_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen3_if_page2
#endif ; use_paged_adressing'

#ifdef adr3_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen3_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_03_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_03F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_03_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_03F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen3_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_03F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen3_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_3
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif
#ifndef s2_dark_by_3
        bcf DUNKELTAST_PRE
#endif

gruen3_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_3
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_3
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen3_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen3_page1_aktiv

  #ifdef gruen3_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen3_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen3_if_page1
gruen3_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen3_if_page1

 #ifdef gruen3_if_page2
gruen3_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_03           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_03F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen3_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g3

; ********************
; *** ADR Gruen-#4 ***
; ********************
#ifndef skip_g4
gruen_4
        ; Signal-Status
        movlw status_g04_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g04_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen4_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen4_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen4_if_page2
  #ifdef gruen4_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen4_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen4_if_page2
#endif ; use_paged_adressing'

#ifdef adr4_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen4_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; not 'aufteilung_output_all'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_04_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_04F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_04_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_04F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen4_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_04F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen4_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

        ; Dunkeltastung S2?
#ifdef s2_dark_by_4
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_PRE            ; Dunkeltastung EIN
        btfsc R_MODE,1                 ; Mode mit Dunkeltastung
         bsf DUNKELTAST_AKT            ; Dunkeltastung EIN
#endif ; s2_dark_by_4
#ifndef s2_dark_by_4
        bcf DUNKELTAST_PRE
#endif ; s2_dark_by_4

gruen4_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_4
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif ; all_dark_by_4
#ifdef quit_dark_by_4
        bcf DUNKELTAST_ALL_PRE
#endif ; quit_dark_by_4

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen4_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen4_page1_aktiv

  #ifdef gruen4_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen4_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen4_if_page1
gruen4_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen4_if_page1

 #ifdef gruen4_if_page2
gruen4_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_04           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_04F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen4_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g4

; ********************
; *** ADR Gruen-#5 ***
; ********************
#ifndef skip_g5
gruen_5
        ; Signal-Status
        movlw status_g05_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g05_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen5_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen5_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen5_if_page2
  #ifdef gruen5_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen5_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen5_if_page2
#endif ; use_paged_adressing

#ifdef adr5_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen5_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen5_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
 #endif ; gruen5_to_s1

 #ifdef gruen5_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
 #endif ; gruen5_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_05_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_05F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_05_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_05F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen5_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_05F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen5_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen5_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_5
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_5
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen5_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_5
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_5
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen5_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen5_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen5_page1_aktiv

  #ifdef gruen5_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen5_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen5_if_page1
gruen5_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen5_if_page1

 #ifdef gruen5_if_page2
gruen5_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_05           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_05F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen5_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g5

; ********************
; *** ADR Gruen-#6 ***
; ********************
#ifndef skip_g6
gruen_6
        ; Signal-Status
        movlw status_g06_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g06_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen6_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen6_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen6_if_page2
  #ifdef gruen6_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen6_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen6_if_page2
#endif ; use_paged_adressing

#ifdef adr6_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen6_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen6_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
 #endif ; gruen6_to_s1

 #ifdef gruen6_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
 #endif ; gruen6_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_06_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_06F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_06_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_06F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen6_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_06F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen6_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen6_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_6
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_6
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen6_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_6
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_6
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen6_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen6_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen6_page1_aktiv

  #ifdef gruen6_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen6_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen6_if_page1
gruen6_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen6_if_page1

 #ifdef gruen6_if_page2
gruen6_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_06           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_06F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen6_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g6

; ********************
; *** ADR Gruen-#7 ***
; ********************
#ifndef skip_g7
gruen_7
        ; Signal-Status
        movlw status_g07_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g07_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen7_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen7_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen7_if_page2
  #ifdef gruen7_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen7_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen7_if_page2
#endif ; use_paged_adressing

#ifdef adr7_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen7_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen7_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
 #endif ; gruen7_to_s1

 #ifdef gruen7_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
 #endif ; gruen7_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_07_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_07F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_07_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_07F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen7_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_07F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen7_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen7_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_7
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_7
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen7_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_7
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_7
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen7_flash_alternating_4_5
        bsf ALTER_FLASH_S1
        bcf SET_OUTPUT4                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT5                ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen7_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen7_page1_aktiv

  #ifdef gruen7_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen7_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen7_if_page1
gruen7_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen7_if_page1

 #ifdef gruen7_if_page2
gruen7_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_07           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_07F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen7_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g7

; ********************
; *** ADR Gruen-#8 ***
; ********************
#ifndef skip_g8
gruen_8
        ; Signal-Status
        movlw status_g08_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g08_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen8_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen8_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
  #ifdef gruen8_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen8_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
#endif ; use_paged_adressing

#ifdef adr8_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen8_no_change
#endif

#ifndef aufteilung_output_all          ; Aufteilung in S1 / S2
 #ifdef gruen8_to_s1                   ; Signalbilder, wenn S1 (Hauptsignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
 #endif ; gruen8_to_s1

 #ifdef gruen8_to_s2                   ; Signalbilder, wenn S2 (Nebensignal)
        ; Signalbild statisch:
        movlw SIG_BILD_S2_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
 #endif ; gruen8_to_s2
#endif ; NOT aufteilung_output_all         ; Aufteilung in S1 / S2

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_08_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_08F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_08_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_08F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen8_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_08F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen8_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

#ifdef gruen8_to_s1       ; Dunkeltastung, nur wenn zu S1 gehrig
  ; Dunkeltastung S2?
  #ifdef s2_dark_by_8
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_PRE          ; Dunkeltastung EIN
          btfsc R_MODE,1               ; Mode mit Dunkeltastung
           bsf DUNKELTAST_AKT          ; Dunkeltastung EIN
  #endif
  #ifndef s2_dark_by_8
          bcf DUNKELTAST_PRE
  #endif
#endif

gruen8_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_8
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_8
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen8_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen8_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen8_page1_aktiv

  #ifdef gruen8_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen8_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen8_if_page1
gruen8_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen8_if_page1

 #ifdef gruen8_if_page2
gruen8_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_08           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_08F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen8_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g8

; ********************
; *** ADR Gruen-#9 ***
; ********************
#ifndef skip_g9
gruen_9
        ; Signal-Status
        movlw status_g09_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g09_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen8_if_page1               ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen8_buffer_p1       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
  #ifdef gruen8_if_page2               ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen8_buffer_p2       ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen8_if_page2
#endif ; use_paged_adressing'

#ifdef adr9_no_change                  ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen9_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_09_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_ON_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_09F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_FL_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_09_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_09F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen9_depends_on_page         ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_09F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen9_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen9_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_9
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_9
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen9_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen9_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen9_page1_aktiv

  #ifdef gruen9_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen9_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen9_if_page1
gruen9_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen9_if_page1

 #ifdef gruen9_if_page2
gruen9_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_09           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_09F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen9_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g9

; ********************
; *** ADR Gruen-#10***
; ********************
#ifndef skip_g10
gruen_10
        ; Signal-Status
        movlw status_g10_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g10_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen10_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen10_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen10_if_page2
  #ifdef gruen10_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen10_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen10_if_page2
#endif ; use_paged_adressing'

#ifdef adr10_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen10_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_10_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_ON_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_10F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S1_akt,0     ; unchanged status: Out 1
         xorlw b'00000001'             ; .. will be transfered to current signal screen
        btfsc R_STATUS_FL_S1_akt,1     ; unchanged status: Out 2
         xorlw b'00000010'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_10_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_10F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen10_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_10F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen10_depends_on_page
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen10_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_10
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_10
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen10_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen10_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen10_page1_aktiv

  #ifdef gruen10_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen10_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen10_if_page1
gruen10_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen10_if_page1

 #ifdef gruen10_if_page2
gruen10_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_10           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_10F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen10_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g10

; ********************
; *** ADR Gruen-#11***
; ********************
#ifndef skip_g11
gruen_11
        ; Signal-Status
        movlw status_g11_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g11_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen11_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen11_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen11_if_page2
  #ifdef gruen11_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen11_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen11_if_page2
#endif ; use_paged_adressing'

#ifdef adr11_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen11_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_11_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_11F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_11_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_11F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen11_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_11F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen11_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen11_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_11
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_11
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen11_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen11_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen11_page1_aktiv

  #ifdef gruen11_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen11_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen11_if_page1
gruen11_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen11_if_page1

 #ifdef gruen11_if_page2
gruen11_buffer_p2                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_11           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_11F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen11_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g11

; ********************
; *** ADR Gruen-#12***
; ********************
#ifndef skip_g12
gruen_12
        ; Signal-Status
        movlw status_g12_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g12_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen12_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen12_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen12_if_page2
  #ifdef gruen12_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen12_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen12_if_page2
#endif ; use_paged_adressing'

#ifdef adr12_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen12_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_12_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_12F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_12_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_12F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen12_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_12F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen12_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen12_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_12
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_12
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen12_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen12_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen12_page1_aktiv

  #ifdef gruen12_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen12_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen12_if_page1
gruen12_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen12_if_page1

 #ifdef gruen12_if_page2
gruen12_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_12           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_12F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen12_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g12

; ********************
; *** ADR Gruen-#13***
; ********************
#ifndef skip_g13
gruen_13
        ; Signal-Status
        movlw status_g13_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g13_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen13_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen13_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen13_if_page2
  #ifdef gruen13_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen13_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen13_if_page2
#endif ; use_paged_adressing'

#ifdef adr13_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen13_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_13_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_13F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_13_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_13F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen13_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_13F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen13_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen13_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_13
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_13
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen13_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen13_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen13_page1_aktiv

  #ifdef gruen13_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen13_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen13_if_page1
gruen13_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen13_if_page1

 #ifdef gruen13_if_page2
gruen13_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_13           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_13F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen13_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g13

; ********************
; *** ADR Gruen-#14***
; ********************
#ifndef skip_g14
gruen_14
        ; Signal-Status
        movlw status_g14_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g14_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen14_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen14_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen14_if_page2
  #ifdef gruen14_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen14_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen14_if_page2
#endif ; use_paged_adressing'

#ifdef adr14_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen14_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_14_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_14F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_14_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_14F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen14_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_14F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen14_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen14_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_14
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_14
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen14_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen14_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen14_page1_aktiv

  #ifdef gruen14_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen14_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen14_if_page1
gruen14_buffer_p1                      ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen14_if_page1

 #ifdef gruen14_if_page2
gruen14_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_14           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_14F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen14_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g14

; ********************
; *** ADR Gruen-#15***
; ********************
#ifndef skip_g15
gruen_15
        ; Signal-Status
        movlw status_g15_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g15_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen15_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen15_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen15_if_page2
  #ifdef gruen15_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen15_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen15_if_page2
#endif ; use_paged_adressing'

#ifdef adr15_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen15_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_15_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_15F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_15_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_15F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen15_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_15F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen15_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen15_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_15
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_15
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen15_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen15_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen15_page1_aktiv

  #ifdef gruen15_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen15_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen15_if_page1
gruen15_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen15_if_page1

 #ifdef gruen15_if_page2
gruen15_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_15           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_15F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen15_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g15

; ********************
; *** ADR Gruen-#16***
; ********************
#ifndef skip_g16
gruen_16
        ; Signal-Status
        movlw status_g16_a             ; die Bits in R_STATUS, die zum Selben Schirm (z.B. HS)
        andwf R_STATUS,F               ; .. gehren, mssen genullt werden.
        movlw status_g16_b             ; Hier also eine exclusive '1' fr den eigenen Status und jeweils
        andwf R_STATUS_2,F             ; .. eine '1' fr alle unchanged Status-Bits

        ; Paged Adressing: Check ob Signalbild buffern oder ausfhren
#ifdef use_paged_adressing
  #ifdef gruen16_if_page1              ; Befehl nur ausfhren, wenn Page 1 aktiv ist
          btfss F_PAGE1_AKTIV
           goto gruen16_buffer_p1      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen16_if_page2
  #ifdef gruen16_if_page2              ; Befehl nur ausfhren, wenn Page 2 aktiv ist
          btfss F_PAGE2_AKTIV
           goto gruen16_buffer_p2      ; Sprung, wenn Signalbild nicht gendert werden soll
  #endif ;gruen16_if_page2
#endif ; use_paged_adressing'

#ifdef adr16_no_change                 ; keine nderung des Signal-Bild, nur Dunkeltastung
        goto gruen16_no_change
#endif

#ifndef aufteilung_output_all
        ; Signalbild statisch:
        movlw SIG_BILD_S2_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing:
        movlw SIG_BILD_S2_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif ; not 'all_in_one'

#ifdef aufteilung_output_all           ; keine Aufteilung in S1 / S2
        ; OUT 1..8 statisch:
        movlw SIG_BILD_16_a            ; hier steht drinne, welche Outputs_A eingeschaltet werden
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16_a_p2        ; hier steht drinne, welche Outputs_A eingeschaltet werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 1..8 Signalbild flashing:
        movlw SIG_BILD_16F_a           ; hier steht drinne, welche Outputs_A blinken sollen
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16F_a_p2       ; hier steht drinne, welche Outputs_A blinken werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt

        ; OUT 9..10 statisch:
        movlw SIG_BILD_16_b            ; hier steht drinne, welche Outputs_B eingeschaltet werden
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16_b_p2        ; hier steht drinne, welche Outputs_B eingeschaltet werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_ON_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        andwf R_STATUS_ON_S2_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S2_pre       ; neues Signalbild (nach der Dunkelphase)

        ; OUT 9..10 Signalbild flashing:
        movlw SIG_BILD_16F_b           ; hier steht drinne, welche Outputs_B blinken sollen
 #ifdef gruen16_depends_on_page        ; wenn bei Page2 ein alternatives Signalbild kommen soll
        btfsc F_PAGE2_AKTIV
         movlw SIG_BILD_16F_b_p2       ; hier steht drinne, welche Outputs_B blinken werden (nur bei Page 2)
 #endif ; gruen16_depends_on_page
 #ifdef led_signal_010
        btfsc R_STATUS_FL_S2_akt,7     ; unchanged status: Out 10
         xorlw b'10000000'             ; .. will be transfered to current signal screen
 #endif ; led_signal_010
        movwf R_STATUS_FL_S2_pre
        movwf R_STATUS_FL_S2_akt
#endif

gruen16_no_change
        ; Dunkeltastung ALLE?
#ifdef all_dark_by_16
        bsf DUNKELTAST_ALL_PRE         ; Dunkeltastung alle EIN
        bsf DUNKELTAST_ALL             ; Dunkeltastung alle EIN
#endif
#ifdef quit_dark_by_16
        bcf DUNKELTAST_ALL_PRE
#endif

#ifdef gruen16_flash_alternating_9_10
        bsf ALTER_FLASH_S2
        bcf SET_OUTPUT9                ; Blink-Outputs erst einmal "aus"
        bcf SET_OUTPUT10               ; Blink-Outputs erst einmal "aus"
#endif

#ifdef use_paged_adressing
        ; Paged Adressing: Page umschalten
  #ifdef gruen16_page1_aktiv
        goto switch_to_page_1          ; Switch to page 1 - GOTO! RETURN von dort!
  #endif ;gruen16_page1_aktiv

  #ifdef gruen16_page2_aktiv
        goto switch_to_page_2          ; Switch to page 2 - GOTO! RETURN von dort!
  #endif ;gruen16_page2_aktiv

        ; Paged Adressing: Signalbild buffern, wenn falsche Adress-age aktiv ist
 #ifdef gruen16_if_page1
gruen16_buffer_p1                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P1 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P1_buffer    ; ... in Buffer f. Page 1

        ; Signalbild flashing:
        movlw SIG_BILD_S1_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P1_buffer    ; ... in Buffer FL f. Page 1
        goto save_buffer_p1            ; Save it. GOTO! Return von Dort!
 #endif ; gruen16_if_page1

 #ifdef gruen16_if_page2
gruen16_buffer_p2                       ; Signalbild nicht ndern, aber Bitmuster in Buffer P2 schreiben
        ; Signalbild statisch:
        movlw SIG_BILD_S1_16           ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_ON_P2_buffer    ; ... in Buffer f. Page 2

        ; Signalbild flashing:
        movlw SIG_BILD_S1_16F          ; hier steht drinne, welche Outputs blinken sollen
        movwf R_STATUS_FL_P2_buffer    ; ... in Buffer FL f. Page 2
        goto save_buffer_p2            ; Save it. GOTO! Return von Dort!
 #endif ; gruen16_if_page2
#endif ; use_paged_adressing'

        return
#endif ; skip_g16

;*********************************
;** Routine: switch_to_page_1   **
;** goto Page1,                 **
;** restore Signalbild          **
;*********************************
#ifdef use_paged_adressing
switch_to_page_1
        ; check, ob schon page 1 aktiv ist
        btfsc F_PAGE1_AKTIV            ; wenn Page1 aktiv,
         return                        ; .. dann nichts machen

        ; Flags umschalten
        bsf F_PAGE1_AKTIV
        bcf F_PAGE2_AKTIV

#ifdef use_buffer_register
        ; wenn fahrwegabhngige Adress-Reaktion, dann muss jetzt das gespeicherte Bild des anderen Fahrweges eingeblendet werden

        ; Signalbild statisch, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 1):
        movf R_STATUS_ON_P1_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 1):
        movf R_STATUS_FL_P1_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; use_buffer_register

#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige Signalbilder
        goto restore_after_pagechange
#endif  ; NOT use_buffer_register

        return
#endif ; use_paged_adressing

;*********************************
;** Routine: switch_to_page_2   **
;** goto Page2,                 **
;** restore Signalbild          **
;*********************************
#ifdef use_paged_adressing
switch_to_page_2
        ; check, ob schon page 2 aktiv ist
        btfsc F_PAGE2_AKTIV            ; wenn Page1 aktiv,
         return                        ; .. dann nichts machen

        ; Flags umschalten
        bsf F_PAGE2_AKTIV
        bcf F_PAGE1_AKTIV

#ifdef use_buffer_register
        ; wenn fahrwegabhngige Adress-Reaktion, dann muss jetzt das gespeicherte Bild des anderen Fahrweges eingeblendet werden

        ; Signalbild statisch, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 2):
        movf R_STATUS_ON_P2_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        andwf R_STATUS_ON_S1_akt,F     ; LEDs, die im aktiven und neuen Signalbild vorkommen, bleiben
        movwf R_STATUS_ON_S1_pre       ; neues Signalbild (nach der Dunkelphase)

        ; Signalbild flashing, Laden des darzustellenden Signalbildes (entsprechend des letzten Befehles der Adress-Page 2):
        movf R_STATUS_FL_P2_buffer,W   ; hier steht drinne, welche Outputs eingeschaltet werden
        movwf R_STATUS_FL_S1_pre
        movwf R_STATUS_FL_S1_akt
#endif ; use_buffer_register

#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige Signalbilder
        goto restore_after_pagechange
#endif  ; NOT use_buffer_register

        return
#endif ; use_paged_adressing

;*****************************************
;** Routine: restore_after_pagechange   **
;*****************************************
#ifndef use_buffer_register            ; d.h. Fahrstrassenabhngige  Signalbilder
restore_after_pagechange
 #ifdef gruen1_depends_on_page
        btfsc R_STATUS,0
         call gruen_1
 #endif
 #ifdef gruen2_depends_on_page
        btfsc R_STATUS,1
         call gruen_2
 #endif
 #ifdef gruen3_depends_on_page
        btfsc R_STATUS,2
         call gruen_3
 #endif
 #ifdef gruen4_depends_on_page
        btfsc R_STATUS,3
         call gruen_4
 #endif
 #ifdef gruen5_depends_on_page
        btfsc R_STATUS,4
         call gruen_5
 #endif
 #ifdef gruen6_depends_on_page
        btfsc R_STATUS,5
         call gruen_6
 #endif
 #ifdef gruen7_depends_on_page
        btfsc R_STATUS,6
         call gruen_7
 #endif
 #ifdef gruen8_depends_on_page
        btfsc R_STATUS,7
         call gruen_8
 #endif
 #ifdef gruen9_depends_on_page
        btfsc R_STATUS_2,0
         call gruen_9
 #endif
 #ifdef gruen10_depends_on_page
        btfsc R_STATUS_2,1
         call gruen_10
 #endif
 #ifdef gruen11_depends_on_page
        btfsc R_STATUS_2,2
         call gruen_11
 #endif
 #ifdef gruen12_depends_on_page
        btfsc R_STATUS_2,3
         call gruen_12
 #endif
 #ifdef gruen13_depends_on_page
        btfsc R_STATUS_2,4
         call gruen_13
 #endif
 #ifdef gruen14_depends_on_page
        btfsc R_STATUS_2,5
         call gruen_14
 #endif
 #ifdef gruen15_depends_on_page
        btfsc R_STATUS_2,6
         call gruen_15
 #endif
 #ifdef gruen16_depends_on_page
        btfsc R_STATUS_2,7
         call gruen_16
 #endif
#endif ; NOT use_buffer_register
        return


;*********************************
;** Routine: save               **
;** Signalisierung mit jew.     **
;** Ausgang                     **
;*********************************
; ********************
; *** Adresse #1   ***
; ********************
#ifndef skip_g1
save_01
        movlw save_01_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_01_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_g1

; ********************
; *** Adresse #2   ***
; ********************
#ifndef skip_g2
save_02
        movlw save_02_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_02_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #3   ***
; ********************
#ifndef skip_g3
save_03
        movlw save_03_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_03_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #4   ***
; ********************
#ifndef skip_g4
save_04
        movlw save_04_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_04_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #5   ***
; ********************
#ifndef skip_g5
save_05
        movlw save_05_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_05_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #6   ***
; ********************
#ifndef skip_g6
save_06
        movlw save_06_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_06_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #7   ***
; ********************
#ifndef skip_g7
save_07
        movlw save_07_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_07_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #8   ***
; ********************
#ifndef skip_g8
save_08
        movlw save_08_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_08_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #9   ***
; ********************
#ifndef skip_g9
save_09
        movlw save_09_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_09_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #10  ***
; ********************
#ifndef skip_g10
save_10
        movlw save_10_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_10_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #11  ***
; ********************
#ifndef skip_g11
save_11
        movlw save_11_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_11_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #12  ***
; ********************
#ifndef skip_g12
save_12
        movlw save_12_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_12_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #13  ***
; ********************
#ifndef skip_g13
save_13
        movlw save_13_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_13_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx


; ********************
; *** Adresse #14  ***
; ********************
#ifndef skip_g14
save_14
        movlw save_14_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_14_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #15  ***
; ********************
#ifndef skip_g15
save_15
        movlw save_15_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_15_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ********************
; *** Adresse #16  ***
; ********************
#ifndef skip_g16
save_16
        movlw save_16_port_a          ; Bitmuster f. Toggle Port A
        xorwf PORTA,F

        movlw save_16_port_c          ; Bitmuster f. Toggle Port C
        xorwf PORTC,F
        return
#endif ; skip_gx

; ************************
; *** save_toggle_start **
; ************************
save_toggle_start
        ; wenn OUT1 eingeschaltet war, dann jetzt wieder einschalten (ergibt ein schnelleres Togglen der LED, wenn LED1 eingeschaltet beim Button)
        btfsc R_STATUS,0
         call save_01          ; Signalbild 1 einschalten
        return

;*******************************************
;** Routine: status_to_output             **
;** beim Start und nach dem Speichern     **
;** Ausgnge schalten gemss R_STATUS     **
;*******************************************
status_to_output

#ifdef use_paged_adressing
#ifdef use_buffer_register
        ; Bffer laden
        call load_buffer
#endif ; use_buffer_register

; wenn Adress-Pages verwendet werden, muss das letzte gltige Flag wieder hergestellt werden
        bcf F_PAGE1_AKTIV
        bcf F_PAGE2_AKTIV

 #ifdef gruen1_page2_aktiv
        btfsc R_STATUS,0
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen2_page2_aktiv
        btfsc R_STATUS,1
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen3_page2_aktiv
        btfsc R_STATUS,2
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen4_page2_aktiv
        btfsc R_STATUS,3
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen5_page2_aktiv
        btfsc R_STATUS,4
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen6_page2_aktiv
        btfsc R_STATUS,5
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen7_page2_aktiv
        btfsc R_STATUS,6
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen8_page2_aktiv
        btfsc R_STATUS,7
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen9_page2_aktiv
        btfsc R_STATUS_2,0
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen10_page2_aktiv
        btfsc R_STATUS_2,1
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen11_page2_aktiv
        btfsc R_STATUS_2,2
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen12_page2_aktiv
        btfsc R_STATUS_2,3
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen13_page2_aktiv
        btfsc R_STATUS_2,4
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen14_page2_aktiv
        btfsc R_STATUS_2,5
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen15_page2_aktiv
        btfsc R_STATUS_2,6
         bsf F_PAGE2_AKTIV
 #endif

 #ifdef gruen16_page2_aktiv
        btfsc R_STATUS_2,7
         bsf F_PAGE2_AKTIV
 #endif
        ; wenn nicht Page2, dann Page1 (eine Page muss aktiv sein, im Zweifel: Page1!)
        btfss F_PAGE2_AKTIV
         bsf F_PAGE1_AKTIV
#endif ; use_paged_adressing


;   -----------------------------------------
;   --- Test auf gltiges R_STATUS        ---
;   -----------------------------------------
    ; in R_STATUS sind die Signalstellungen abgelegt.
    ; Durch Aufruf der Routine "gruen_1" etc. werden diese in die
    ; Signalbilder umgewandelt.
#ifndef skip_g1
        btfsc R_STATUS,0
         call gruen_1
#endif

#ifndef skip_g2
        btfsc R_STATUS,1
         call gruen_2
#endif

#ifndef skip_g3
        btfsc R_STATUS,2
         call gruen_3
#endif

#ifndef skip_g4
        btfsc R_STATUS,3
         call gruen_4
#endif

#ifndef skip_g5
        btfsc R_STATUS,4
         call gruen_5
#endif

#ifndef skip_g6
        btfsc R_STATUS,5
         call gruen_6
#endif

#ifndef skip_g7
        btfsc R_STATUS,6
         call gruen_7
#endif

#ifndef skip_g8
        btfsc R_STATUS,7
         call gruen_8
#endif

#ifndef skip_g9
        btfsc R_STATUS_2,0
         call gruen_9
#endif

#ifndef skip_g10
        btfsc R_STATUS_2,1
         call gruen_10
#endif

#ifndef skip_g11
        btfsc R_STATUS_2,2
         call gruen_11
#endif

#ifndef skip_g12
        btfsc R_STATUS_2,3
         call gruen_12
#endif

#ifndef skip_g13
        btfsc R_STATUS_2,4
         call gruen_13
#endif

#ifndef skip_g14
        btfsc R_STATUS_2,5
         call gruen_14
#endif

#ifndef skip_g15
        btfsc R_STATUS_2,6
         call gruen_15
#endif

#ifndef skip_g16
        btfsc R_STATUS_2,7
         call gruen_16
#endif
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: change_mode        **
;** Shift_l R_MODE and save     **
;** into EEPROM $00             **
;*********************************
;*********************************
;*********************************
;*********************************
change_mode
        clrf R_CNT_TGL                 ; reset R_CNT_ADR wg. Blinken
        clrf R_CNT1_TGL                ; Counter fr LED-Signalisierung laden
        clrf R_CNT2_TGL

;   --------------------
;   --- Flag-Wechsel ---
;   --------------------
        btfss R_MODE,0                 ; change from #0 -> #1
         goto $+4
         clrf R_MODE
         bsf R_MODE,1
         goto change_mode_001

        clrf R_MODE                    ; unzulssige FLAGs werden
        bsf R_MODE,0                   ; hier abgefangen!

;   ----------------------
;   --- Signalisierung ---
;   ----------------------
change_mode_001
        movf R_MODE,W                  ;
        movwf R_CNT_TGL
change_mode_002
        clrwdt
        goto $+1
        goto $+1
        goto $+1
        goto $+1
        goto $+1

        movlw b'11111111'
        btfsc R_CNT2_TGL,7             ; if CNT2 > 128
         movwf PORTA                   ; .. -> on
        btfss R_CNT2_TGL,7             ; if CNT2 < 128
         clrf PORTA                    ; .. -> off

        decfsz R_CNT1_TGL,F            ; increment Counter 1 for Toggle
         goto change_mode_002
        decfsz R_CNT2_TGL,F            ; increment Counter 2 for Toggle
         goto change_mode_002

        bcf STATUS,C
        RRF R_CNT_TGL,F
        movf R_CNT_TGL,F               ; Wenn ZERO
        btfss STATUS,Z                 ; dann ende der Signalisierung
         goto change_mode_002

;   -----------------
;   --- Speichern ---
;   -----------------
        movlw h'00'
        movwf R_EEADR                  ; Address: h'00'
        movf R_MODE,W
        movwf R_EEDAT                  ; Data: R_MODE
        call saveroutine
        return

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: save_buffer        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_paged_adressing
#ifdef use_buffer_register
save_buffer_p1
        movlw h'40'
        movwf R_EEADR                  ; Address: h'40'
        movf R_STATUS_ON_P1_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_ON_P1_buffer
        call saveroutine

        movlw h'50'
        movwf R_EEADR                  ; Address: h'50'
        movf R_STATUS_FL_P1_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_FL_P1_buffer
        call saveroutine

        return

save_buffer_p2
        movlw h'60'
        movwf R_EEADR                  ; Address: h'60'
        movf R_STATUS_ON_P2_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_ON_P2_buffer
        call saveroutine

        movlw h'70'
        movwf R_EEADR                  ; Address: h'70'
        movf R_STATUS_FL_P2_buffer,W
        movwf R_EEDAT                  ; Data: R_STATUS_FL_P2_buffer
        call saveroutine

        return
#endif ; use_buffer_register
#endif ; use_paged_adressing

;*********************************
;*********************************
;*********************************
;*********************************
;** Routine: load_buffer        **
;*********************************
;*********************************
;*********************************
;*********************************
#ifdef use_paged_adressing
#ifdef use_buffer_register
load_buffer
        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'40'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_ON_P1_buffer    ; ..into R_STATUS_ON_P1_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'50'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_FL_P1_buffer    ; ..into R_STATUS_FL_P1_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'60'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_ON_P2_buffer    ; ..into R_STATUS_ON_P2_buffer

        bsf STATUS,RP0                 ; -> BANK 1
        movlw h'70'
        movwf EEADR                    ; Adress to read
        bsf EECON1,RD                  ; initiate reading
        movf EEDATA,W                  ; EEDATA into W
        bcf STATUS,RP0                 ; -> BANK 0
        movwf R_STATUS_FL_P2_buffer    ; ..into R_STATUS_FL_P2_buffer

        return
#endif ; use_buffer_register
#endif ; use_paged_adressing

;*******************************************
; +++ for debug +++
;*******************************************
;      movf R_ADR_3,W ; test debug
;      movwf R_TESTTEMP
;      call show_testtemp

show_testtemp
; for DEBUG  btfss R_TESTTEMP
         bcf LED_01
        btfsc R_TESTTEMP,0
         bsf LED_01

        btfss R_TESTTEMP,1
         bcf LED_02
        btfsc R_TESTTEMP,1
         bsf LED_02

        btfss R_TESTTEMP,2
         bcf LED_03
        btfsc R_TESTTEMP,2
         bsf LED_03

        btfss R_TESTTEMP,3
         bcf LED_04
        btfsc R_TESTTEMP,3
         bsf LED_04

        btfss R_TESTTEMP,4
         bcf LED_05
        btfsc R_TESTTEMP,4
         bsf LED_05

        btfss R_TESTTEMP,5
         bcf LED_06
        btfsc R_TESTTEMP,5
         bsf LED_06

        btfss R_TESTTEMP,6
         bcf LED_07
        btfsc R_TESTTEMP,6
         bsf LED_07

        btfss R_TESTTEMP,7
         bcf LED_08
        btfsc R_TESTTEMP,7
         bsf LED_08

        return
